Package: canceR
Type: Package
Title: A Graphical User Interface for accessing and modeling the Cancer
        Genomics Data of MSKCC
Version: 1.46.0
Date: 2024-02-28
Author: Karim Mezhoud. Nuclear Safety & Security Department. Nuclear
        Science Center of Tunisia.
Maintainer: Karim Mezhoud <kmezhoud@gmail.com>
Description: The package is user friendly interface based on the cgdsr
        and other modeling packages to explore, compare, and analyse
        all available Cancer Data (Clinical data, Gene Mutation, Gene
        Methylation, Gene Expression, Protein Phosphorylation, Copy
        Number Alteration) hosted by the Computational Biology Center
        at Memorial-Sloan-Kettering Cancer Center (MSKCC).
License: GPL-2
LazyLoad: yes
Depends: R (>= 4.3), tcltk, cBioPortalData
Imports: GSEABase, tkrplot, geNetClassifier, RUnit, Formula, rpart,
        survival, Biobase, phenoTest, circlize, plyr, tidyr, dplyr,
        graphics, stats, utils, grDevices, R.oo, R.methodsS3
Suggests: testthat (>= 3.1), knitr, rmarkdown, BiocStyle
SystemRequirements: Tktable, BWidget
VignetteBuilder: knitr
BugReports: https://github.com/kmezhoud/canceR/issues
biocViews: GUI, GeneExpression, Clustering, GO, GeneSetEnrichment,
        KEGG, MultipleComparison
RoxygenNote: 7.3.1
Encoding: UTF-8
Config/pak/sysreqs: bwidget cmake make libbz2-dev libicu-dev
        liblzma-dev libpng-dev libsecret-1-dev libuv1-dev libxml2-dev
        libssl-dev tk-table libx11-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:40:43 UTC
RemoteUrl: https://github.com/bioc/canceR
RemoteRef: RELEASE_3_23
RemoteSha: 359ae9800ccd67c8a09f59936e63bd89aecfc62a
NeedsCompilation: no
Packaged: 2026-04-29 18:40:01 UTC; root
Built: R 4.6.0; ; 2026-04-29 20:25:16 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(cbind,na)
S3method(rbind,na)
export(CGDS)
export(Match_GeneList_MSigDB)
export(Run.GSEA)
export(UnifyRowNames)
export(about)
export(canceR)
export(canceR_Issue)
export(canceR_Vignette)
export(dialogGeneClassifier)
export(dialogMetOption)
export(dialogMut)
export(dialogOptionCircos)
export(dialogOptionGSEAlm)
export(dialogOptionPhenoTest)
export(dialogPlotOption_SkinCor)
export(dialogSamplingGSEA)
export(dialogSelectFiles_GSEA)
export(dialogSpecificMut)
export(dialogSummary_GSEA)
export(dialoggetGeneListMSigDB)
export(displayInTable)
export(getCases)
export(getCasesGenProfs)
export(getCircos)
export(getClinicData_MultipleCases)
export(getClinicalDataMatrix)
export(getFreqMutData)
export(getGCTCLSExample)
export(getGCT_CLSfiles)
export(getGSEAlm_Diseases)
export(getGSEAlm_Variables)
export(getGenProfs)
export(getGeneExpMatrix)
export(getGeneList)
export(getGeneListExample)
export(getGeneListFromMSigDB)
export(getGenesClassifier)
export(getGenesTree_MultipleCases)
export(getGenesTree_SingleCase)
export(getInTable)
export(getMSigDB)
export(getMSigDBExample)
export(getMSigDBfile)
export(getMegaProfData)
export(getMetDataMultipleGenes)
export(getMutData)
export(getPhenoTest)
export(getProfData)
export(getProfilesDataMultipleGenes)
export(getProfilesDataSingleGene)
export(getSpecificMut)
export(getSummaryGSEA)
export(getSurvival)
export(getTextWin)
export(geteSet)
export(modalDialog)
export(plotModel)
export(plot_1Gene_2GenProfs)
export(plot_2Genes_1GenProf)
export(setWorkspace)
export(test.CGDS)
export(testCheckedCaseGenProf)
import(Formula)
import(R.oo)
import(RUnit)
import(cBioPortalData)
import(circlize)
import(plyr)
import(rpart)
import(tcltk)
import(tkrplot)
importFrom(Biobase,ExpressionSet)
importFrom(Biobase,exprs)
importFrom(Biobase,openPDF)
importFrom(Biobase,pData)
importFrom(Biobase,varLabels)
importFrom(GSEABase,GeneSet)
importFrom(GSEABase,GeneSetCollection)
importFrom(R.methodsS3,setMethodS3)
importFrom(geNetClassifier,calculateGenesRanking)
importFrom(geNetClassifier,genesDetails)
importFrom(grDevices,colors)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,savePlot)
importFrom(graphics,axis)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(phenoTest,ExpressionPhenoTest)
importFrom(phenoTest,getFc)
importFrom(phenoTest,getSignif)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.exclude)
importFrom(stats,p.adjust)
importFrom(stats,pnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,window)
importFrom(survival,Surv)
importFrom(survival,cox.zph)
importFrom(survival,coxph)
importFrom(survival,survfit)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,memory.limit)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,vignette)
importFrom(utils,write.table)

# A Graphical User Interface for accessing and modeling the Cancer Genomics Data of MSKCC.

[![Travis-CI Build Status](https://travis-ci.org/kmezhoud/canceR.svg?branch=master)](https://travis-ci.org/kmezhoud/canceR)

The package is user friendly interface based on the cgdsr and other modeling packages to explore, compare, and analyse all available Cancer Data (Clinical data, Gene Mutation, Gene Methylation, Gene Expression, Protein Phosphorylation, Copy Number Alteration) hosted by the Computational Biology Center at Memorial-Sloan-Kettering Cancer Center (MSKCC).


<img src="./vignettes/image/Circos.png">


Circular layout is an interesting way to integrate multi-omics heterogeneous  and big data of cancer disease in the same plot. The goal is to make easy the interpretation and the exploring of the relationships between cancers, genes or dimensions.
There are two R packages (RCircos and Circlize) available for Circular layout but its use remain laborious and needs computational skills.
CanceR package implements "getCircos" function to facilitate the visualization of cancer disease using Circos style. User needs only to select cancers and check which dimensions will be plotted.

User can select a simple gene list from files or examples or can focus on gene list from gene sets selected from MSigDB.
In the following case I  focus my exploring to two gene lists corresponding to gene set involved in DNA repair and Response to oxidative stress. canceR package allows user to select any gene sets from MSigDB without gene duplication.

I selected mRNA (0.3 threshold), CNA (0.85), Met HM450 (0.3), Mutation (Frequency 101). Only genes with significant rates are plotted with corresponding colors. For example Gold for mutation, Orange (Methylation), Green (CNA), red (mRNA expression).

I selected 5 cancers represented with 5 sectors. In the same sector there are 7 tracks (layouts) corresponding respectively (out : in) to: Gene List, Gene Sets, mRNA, CNA, Methylation, Mutation

<img src="./vignettes/image/dialogCircos.png">

# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.


# G S E A -- Gene Set Enrichment Analysis

# Auxiliary functions and definitions 

GSEA.GeneRanking <- function(A, class.labels, gene.labels, nperm, permutation.type = 0, sigma.correction = "GeneCluster", fraction=1.0, replace= FALSE, reverse.sign= FALSE) { 

# This function ranks the genes according to the signal to noise ratio for the actual phenotype and also random permutations and bootstrap  
# subsamples of both the observed and random phenotypes. It uses matrix operations to implement the signal to noise calculation 
# in stages and achieves fast execution speed. It supports two types of permutations: random (unbalanced) and balanced. 
# It also supports subsampling and bootstrap by using masking and multiple-count variables.  When "fraction" is set to 1 (default)
# the there is no subsampling or boostrapping and the matrix of observed signal to noise ratios will have the same value for 
# all permutations. This is wasteful but allows to support all the multiple options with the same code. Notice that the second 
# matrix for the null distribution will still have the values for the random permutations 
# (null distribution). This mode (fraction = 1.0) is the defaults, the recommended one and the one used in the examples.
# It is also the one that has be tested more thoroughly. The resampling and boostrapping options are intersting to obtain 
# smooth estimates of the observed distribution but its is left for the expert user who may want to perform some sanity 
# checks before trusting the code.
#
# Inputs:
#   A: Matrix of gene expression values (rows are genes, columns are samples) 
#   class.labels: Phenotype of class disticntion of interest. A vector of binary labels having first the 1's and then the 0's 
#   gene.labels: gene labels. Vector of probe ids or accession numbers for the rows of the expression matrix 
#   nperm: Number of random permutations/bootstraps to perform 
#   permutation.type: Permutation type: 0 = unbalanced, 1 = balanced. For experts only (default: 0) 
#   sigma.correction: Correction to the signal to noise ratio (Default = GeneCluster, a choice to support the way it was handled in a previous package) 
#   fraction: Subsampling fraction. Set to 1.0 (no resampling). For experts only (default: 1.0) 
#   replace: Resampling mode (replacement or not replacement). For experts only (default: F) 
#   reverse.sign: Reverse direction of gene list (default = F)
#
# Outputs:
#   s2n.matrix: Matrix with random permuted or bootstraps signal to noise ratios (rows are genes, columns are permutations or bootstrap subsamplings
#   obs.s2n.matrix: Matrix with observed signal to noise ratios (rows are genes, columns are boostraps subsamplings. If fraction is set to 1.0 then all the columns have the same values
#   order.matrix: Matrix with the orderings that will sort the columns of the obs.s2n.matrix in decreasing s2n order
#   obs.order.matrix: Matrix with the orderings that will sort the columns of the s2n.matrix in decreasing s2n order
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

     A <- A + 0.00000001

     N <- length(A[,1])
     Ns <- length(A[1,])

     subset.mask <- matrix(0, nrow=Ns, ncol=nperm)
     reshuffled.class.labels1 <- matrix(0, nrow=Ns, ncol=nperm)
     reshuffled.class.labels2 <- matrix(0, nrow=Ns, ncol=nperm)
     class.labels1 <- matrix(0, nrow=Ns, ncol=nperm)
     class.labels2 <- matrix(0, nrow=Ns, ncol=nperm)

     order.matrix <- matrix(0, nrow = N, ncol = nperm)
     obs.order.matrix <- matrix(0, nrow = N, ncol = nperm)
     s2n.matrix <- matrix(0, nrow = N, ncol = nperm)
     obs.s2n.matrix <- matrix(0, nrow = N, ncol = nperm)

     obs.gene.labels <- vector(length = N, mode="character")
     obs.gene.descs <- vector(length = N, mode="character")
     obs.gene.symbols <- vector(length = N, mode="character")

     M1 <- matrix(0, nrow = N, ncol = nperm)
     M2 <- matrix(0, nrow = N, ncol = nperm)
     S1 <- matrix(0, nrow = N, ncol = nperm)
     S2 <- matrix(0, nrow = N, ncol = nperm)

     gc()

     C <- split(class.labels, class.labels)
     class1.size <- length(C[[1]])
     class2.size <- length(C[[2]])
     class1.index <- seq(1, class1.size, 1)
     class2.index <- seq(class1.size + 1, class1.size + class2.size, 1)

     for (r in 1:nperm) {
        class1.subset <- sample(class1.index, size = ceiling(class1.size*fraction), replace = replace)
        class2.subset <- sample(class2.index, size = ceiling(class2.size*fraction), replace = replace)
        class1.subset.size <- length(class1.subset)
        class2.subset.size <- length(class2.subset)
        subset.class1 <- rep(0, class1.size)
        for (i in 1:class1.size) {
            if (is.element(class1.index[i], class1.subset)) {
                subset.class1[i] <- 1
            }
        }
        subset.class2 <- rep(0, class2.size)
        for (i in 1:class2.size) {
            if (is.element(class2.index[i], class2.subset)) {
                subset.class2[i] <- 1
            }
        }
        subset.mask[, r] <- as.numeric(c(subset.class1, subset.class2))
        fraction.class1 <- class1.size/Ns
        fraction.class2 <- class2.size/Ns

        if (permutation.type == 0) { # random (unbalanced) permutation
           full.subset <- c(class1.subset, class2.subset)
           label1.subset <- sample(full.subset, size = Ns * fraction.class1)
           reshuffled.class.labels1[, r] <- rep(0, Ns)
           reshuffled.class.labels2[, r] <- rep(0, Ns)
           class.labels1[, r] <- rep(0, Ns)
           class.labels2[, r] <- rep(0, Ns)
           for (i in 1:Ns) {
               m1 <- sum(!is.na(match(label1.subset, i)))
               m2 <- sum(!is.na(match(full.subset, i)))
               reshuffled.class.labels1[i, r] <- m1
               reshuffled.class.labels2[i, r] <- m2 - m1
               if (i <= class1.size) {
                 class.labels1[i, r] <- m2
                 class.labels2[i, r] <- 0
               } else {
                  class.labels1[i, r] <- 0
                  class.labels2[i, r] <- m2
               }
           }

        } else if (permutation.type == 1) { # proportional (balanced) permutation

           class1.label1.subset <- sample(class1.subset, size = ceiling(class1.subset.size*fraction.class1))
           class2.label1.subset <- sample(class2.subset, size = floor(class2.subset.size*fraction.class1))
           reshuffled.class.labels1[, r] <- rep(0, Ns)
           reshuffled.class.labels2[, r] <- rep(0, Ns)
           class.labels1[, r] <- rep(0, Ns)
           class.labels2[, r] <- rep(0, Ns)
           for (i in 1:Ns) {
               if (i <= class1.size) {
                  m1 <- sum(!is.na(match(class1.label1.subset, i)))
                  m2 <- sum(!is.na(match(class1.subset, i)))
                  reshuffled.class.labels1[i, r] <- m1
                  reshuffled.class.labels2[i, r] <- m2 - m1
                  class.labels1[i, r] <- m2
                  class.labels2[i, r] <- 0
               } else {
                  m1 <- sum(!is.na(match(class2.label1.subset, i)))
                  m2 <- sum(!is.na(match(class2.subset, i)))
                  reshuffled.class.labels1[i, r] <- m1
                  reshuffled.class.labels2[i, r] <- m2 - m1
                  class.labels1[i, r] <- 0
                  class.labels2[i, r] <- m2
               }
           }
        }
    }

# compute S2N for the random permutation matrix
     
     P <- reshuffled.class.labels1 * subset.mask
     n1 <- sum(P[,1])         
     M1 <- A %*% P
     M1 <- M1/n1      
     gc()
     A2 <- A*A        
     S1 <- A2 %*% P   
     S1 <- S1/n1 - M1*M1    
     S1 <- sqrt(abs((n1/(n1-1)) * S1))   
     gc()
     P <- reshuffled.class.labels2 * subset.mask
     n2 <- sum(P[,1])           
     M2 <- A %*% P           
     M2 <- M2/n2          
     gc()
     A2 <- A*A           
     S2 <- A2 %*% P      
     S2 <- S2/n2 - M2*M2 
     S2 <- sqrt(abs((n2/(n2-1)) * S2))
     rm(P)
     rm(A2)
     gc()

     if (sigma.correction == "GeneCluster") {  # small sigma "fix" as used in GeneCluster
         S2 <- ifelse(0.2*abs(M2) < S2, S2, 0.2*abs(M2))
         S2 <- ifelse(S2 == 0, 0.2, S2)
         S1 <- ifelse(0.2*abs(M1) < S1, S1, 0.2*abs(M1))
         S1 <- ifelse(S1 == 0, 0.2, S1)
         gc()
     }

     M1 <- M1 - M2
     rm(M2)
     gc()
     S1 <- S1 + S2
     rm(S2)
     gc()

     s2n.matrix <- M1/S1

     if (reverse.sign == TRUE) {
        s2n.matrix <- - s2n.matrix
     }
     gc()

     for (r in 1:nperm) {
        order.matrix[, r] <- order(s2n.matrix[, r], decreasing=TRUE)            
     }

# compute S2N for the "observed" permutation matrix

     P <- class.labels1 * subset.mask
     n1 <- sum(P[,1])         
     M1 <- A %*% P
     M1 <- M1/n1      
     gc()
     A2 <- A*A        
     S1 <- A2 %*% P   
     S1 <- S1/n1 - M1*M1    
     S1 <- sqrt(abs((n1/(n1-1)) * S1))   
     gc()
     P <- class.labels2 * subset.mask
     n2 <- sum(P[,1])           
     M2 <- A %*% P           
     M2 <- M2/n2          
     gc()
     A2 <- A*A           
     S2 <- A2 %*% P      
     S2 <- S2/n2 - M2*M2 
     S2 <- sqrt(abs((n2/(n2-1)) * S2))
     rm(P)
     rm(A2)
     gc()

     if (sigma.correction == "GeneCluster") {  # small sigma "fix" as used in GeneCluster
         S2 <- ifelse(0.2*abs(M2) < S2, S2, 0.2*abs(M2))
         S2 <- ifelse(S2 == 0, 0.2, S2)
         S1 <- ifelse(0.2*abs(M1) < S1, S1, 0.2*abs(M1))
         S1 <- ifelse(S1 == 0, 0.2, S1)
         gc()
     } 

     M1 <- M1 - M2
     rm(M2)
     gc()
     S1 <- S1 + S2
     rm(S2)
     gc()

     obs.s2n.matrix <- M1/S1
     gc()

     if (reverse.sign == TRUE) {
        obs.s2n.matrix <- - obs.s2n.matrix
     }

     for (r in 1:nperm) {
        obs.order.matrix[,r] <- order(obs.s2n.matrix[,r], decreasing=TRUE)            
     }

     return(list(s2n.matrix = s2n.matrix, 
                 obs.s2n.matrix = obs.s2n.matrix, 
                 order.matrix = order.matrix,
                 obs.order.matrix = obs.order.matrix))
}

GSEA.EnrichmentScore <- function(gene.list, gene.set, weighted.score.type = 1, correl.vector = NULL) {  
#
# Computes the weighted GSEA score of gene.set in gene.list. 
# The weighted score type is the exponent of the correlation 
# weight: 0 (unweighted = Kolmogorov-Smirnov), 1 (weighted), and 2 (over-weighted). When the score type is 1 or 2 it is 
# necessary to input the correlation vector with the values in the same order as in the gene list.
#
# Inputs:
#   gene.list: The ordered gene list (e.g. integers indicating the original position in the input dataset)  
#   gene.set: A gene set (e.g. integers indicating the location of those genes in the input dataset) 
#   weighted.score.type: Type of score: weight: 0 (unweighted = Kolmogorov-Smirnov), 1 (weighted), and 2 (over-weighted)  
#  correl.vector: A vector with the coorelations (e.g. signal to noise scores) corresponding to the genes in the gene list 
#
# Outputs:
#   ES: Enrichment score (real number between -1 and +1) 
#   arg.ES: Location in gene.list where the peak running enrichment occurs (peak of the "mountain") 
#   RES: Numerical vector containing the running enrichment score for all locations in the gene list 
#   tag.indicator: Binary vector indicating the location of the gene sets (1's) in the gene list 
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

   tag.indicator <- sign(match(gene.list, gene.set, nomatch=0))    # notice that the sign is 0 (no tag) or 1 (tag) 
   no.tag.indicator <- 1 - tag.indicator 
   N <- length(gene.list) 
   Nh <- length(gene.set) 
   Nm <-  N - Nh 
   if (weighted.score.type == 0) {
      correl.vector <- rep(1, N)
   }
   alpha <- weighted.score.type
   correl.vector <- abs(correl.vector**alpha)
   sum.correl.tag    <- sum(correl.vector[tag.indicator == 1])
   norm.tag    <- 1.0/sum.correl.tag
   norm.no.tag <- 1.0/Nm
   RES <- cumsum(tag.indicator * correl.vector * norm.tag - no.tag.indicator * norm.no.tag)      
   max.ES <- max(RES)
   min.ES <- min(RES)
   if (max.ES > - min.ES) {
#      ES <- max.ES
      ES <- signif(max.ES, digits = 5)
      arg.ES <- which.max(RES)
   } else {
#      ES <- min.ES
      ES <- signif(min.ES, digits=5)
      arg.ES <- which.min(RES)
   }
   return(list(ES = ES, arg.ES = arg.ES, RES = RES, indicator = tag.indicator))    
}


OLD.GSEA.EnrichmentScore <- function(gene.list, gene.set) {  
#
# Computes the original GSEA score from Mootha et al 2003 of gene.set in gene.list 
#
# Inputs:
#   gene.list: The ordered gene list (e.g. integers indicating the original position in the input dataset)  
#   gene.set: A gene set (e.g. integers indicating the location of those genes in the input dataset) 
#
# Outputs:
#   ES: Enrichment score (real number between -1 and +1) 
#   arg.ES: Location in gene.list where the peak running enrichment occurs (peak of the "mountain") 
#   RES: Numerical vector containing the running enrichment score for all locations in the gene list 
#   tag.indicator: Binary vector indicating the location of the gene sets (1's) in the gene list 
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

   tag.indicator <- sign(match(gene.list, gene.set, nomatch=0))    # notice that the sign is 0 (no tag) or 1 (tag) 
   no.tag.indicator <- 1 - tag.indicator 
   N <- length(gene.list) 
   Nh <- length(gene.set) 
   Nm <-  N - Nh 

   norm.tag    <- sqrt((N - Nh)/Nh)
   norm.no.tag <- sqrt(Nh/(N - Nh))

   RES <- cumsum(tag.indicator * norm.tag - no.tag.indicator * norm.no.tag)      
   max.ES <- max(RES)
   min.ES <- min(RES)
   if (max.ES > - min.ES) {
      ES <- signif(max.ES, digits=5)
      arg.ES <- which.max(RES)
   } else {
      ES <- signif(min.ES, digits=5)
      arg.ES <- which.min(RES)
   }
   return(list(ES = ES, arg.ES = arg.ES, RES = RES, indicator = tag.indicator))    
}

GSEA.EnrichmentScore2 <- function(gene.list, gene.set, weighted.score.type = 1, correl.vector = NULL) {  
#
# Computes the weighted GSEA score of gene.set in gene.list. It is the same calculation as in 
# GSEA.EnrichmentScore but faster (x8) without producing the RES, arg.RES and tag.indicator outputs.
# This call is intended to be used to asses the enrichment of random permutations rather than the 
# observed one.
# The weighted score type is the exponent of the correlation 
# weight: 0 (unweighted = Kolmogorov-Smirnov), 1 (weighted), and 2 (over-weighted). When the score type is 1 or 2 it is 
# necessary to input the correlation vector with the values in the same order as in the gene list.
#
# Inputs:
#   gene.list: The ordered gene list (e.g. integers indicating the original position in the input dataset)  
#   gene.set: A gene set (e.g. integers indicating the location of those genes in the input dataset) 
#   weighted.score.type: Type of score: weight: 0 (unweighted = Kolmogorov-Smirnov), 1 (weighted), and 2 (over-weighted)  
#  correl.vector: A vector with the coorelations (e.g. signal to noise scores) corresponding to the genes in the gene list 
#
# Outputs:
#   ES: Enrichment score (real number between -1 and +1) 
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

   N <- length(gene.list) 
   Nh <- length(gene.set) 
   Nm <-  N - Nh 

   loc.vector <- vector(length=N, mode="numeric")
   peak.res.vector <- vector(length=Nh, mode="numeric")
   valley.res.vector <- vector(length=Nh, mode="numeric")
   tag.correl.vector <- vector(length=Nh, mode="numeric")
   tag.diff.vector <- vector(length=Nh, mode="numeric")
   tag.loc.vector <- vector(length=Nh, mode="numeric")

   loc.vector[gene.list] <- seq(1, N)
   tag.loc.vector <- loc.vector[gene.set]

   tag.loc.vector <- sort(tag.loc.vector, decreasing = FALSE)

   if (weighted.score.type == 0) {
      tag.correl.vector <- rep(1, Nh)
   } else if (weighted.score.type == 1) {
      tag.correl.vector <- correl.vector[tag.loc.vector]
      tag.correl.vector <- abs(tag.correl.vector)
   } else if (weighted.score.type == 2) {
      tag.correl.vector <- correl.vector[tag.loc.vector]*correl.vector[tag.loc.vector]
      tag.correl.vector <- abs(tag.correl.vector)
   } else {
      tag.correl.vector <- correl.vector[tag.loc.vector]**weighted.score.type
      tag.correl.vector <- abs(tag.correl.vector)
   }

   norm.tag <- 1.0/sum(tag.correl.vector)
   tag.correl.vector <- tag.correl.vector * norm.tag
   norm.no.tag <- 1.0/Nm
   tag.diff.vector[1] <- (tag.loc.vector[1] - 1) 
   tag.diff.vector[2:Nh] <- tag.loc.vector[2:Nh] - tag.loc.vector[1:(Nh - 1)] - 1
   tag.diff.vector <- tag.diff.vector * norm.no.tag
   peak.res.vector <- cumsum(tag.correl.vector - tag.diff.vector)
   valley.res.vector <- peak.res.vector - tag.correl.vector
   max.ES <- max(peak.res.vector)
   min.ES <- min(valley.res.vector)
   ES <- signif(ifelse(max.ES > - min.ES, max.ES, min.ES), digits=5)

   return(list(ES = ES))

}

GSEA.HeatMapPlot <- function(V, row.names = FALSE, col.labels, col.classes, col.names = FALSE, main = " ", xlab=" ", ylab=" ") {
#
# Plots a heatmap "pinkogram" of a gene expression matrix including phenotype vector and gene, sample and phenotype labels
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

       n.rows <- length(V[,1])
       n.cols <- length(V[1,])
       row.mean <- apply(V, MARGIN=1, FUN=mean)
       row.sd <- apply(V, MARGIN=1, FUN=sd)
       row.n <- length(V[,1])
       for (i in 1:n.rows) {
	   if (row.sd[i] == 0) {
    	       V[i,] <- 0
           } else {
	       V[i,] <- (V[i,] - row.mean[i])/(0.5 * row.sd[i])
           }
           V[i,] <- ifelse(V[i,] < -6, -6, V[i,])
           V[i,] <- ifelse(V[i,] > 6, 6, V[i,])
        }

        mycol <- c("#0000FF", "#0000FF", "#4040FF", "#7070FF", "#8888FF", "#A9A9FF", "#D5D5FF", "#EEE5EE", "#FFAADA", "#FF9DB0", "#FF7080", "#FF5A5A", "#FF4040", "#FF0D1D", "#FF0000") # blue-pinkogram colors. The first and last are the colors to indicate the class vector (phenotype). This is the 1998-vintage, pre-gene cluster, original pinkogram color map

        mid.range.V <- mean(range(V)) - 0.1
        heatm <- matrix(0, nrow = n.rows + 1, ncol = n.cols)
        heatm[1:n.rows,] <- V[seq(n.rows, 1, -1),]
        heatm[n.rows + 1,] <- ifelse(col.labels == 0, 7, -7)
        image(1:n.cols, 1:(n.rows + 1), t(heatm), col=mycol, axes=FALSE, main=main, xlab= xlab, ylab=ylab)

        if (length(row.names) > 1) {
            numC <- nchar(row.names)
            size.row.char <- 35/(n.rows + 5)
            size.col.char <- 25/(n.cols + 5)
            maxl <- floor(n.rows/1.6)
            for (i in 1:n.rows) {
               row.names[i] <- substr(row.names[i], 1, maxl)
            }
            row.names <- c(row.names[seq(n.rows, 1, -1)], "Class")
            axis(2, at=1:(n.rows + 1), labels=row.names, adj= 0.5, tick=FALSE, las = 1, cex.axis=size.row.char, font.axis=2, line=-1)
        }

        if (length(col.names) > 1) {
           axis(1, at=1:n.cols, labels=col.names, tick=FALSE, las = 3, cex.axis=size.col.char, font.axis=2, line=-1)
        }

        C <- split(col.labels, col.labels)
        class1.size <- length(C[[1]])
        class2.size <- length(C[[2]])
        axis(3, at=c(floor(class1.size/2),class1.size + floor(class2.size/2)), labels=col.classes, tick=FALSE, las = 1, cex.axis=1.25, font.axis=2, line=-1)

	return()
}

GSEA.Res2Frame <- function(filename = "NULL") { 
#
# Reads a gene expression dataset in RES format and converts it into an R data frame
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

   header.cont <- readLines(filename, n = 1)
   temp <- unlist(strsplit(header.cont, "\t"))
   colst <- length(temp)
   header.labels <- temp[seq(3, colst, 2)]
   ds <- read.delim(filename, header= FALSE, row.names = 2, sep="\t", skip=3, blank.lines.skip=TRUE, comment.char="", as.is=TRUE)
   colst <- length(ds[1,])
   cols <- (colst - 1)/2
   rows <- length(ds[,1])
   A <- matrix(nrow=rows - 1, ncol=cols)
   A <- ds[1:rows, seq(2, colst, 2)]
   table1 <- data.frame(A)
   names(table1) <- header.labels
   return(table1)
}

GSEA.Gct2Frame <- function(filename = "NULL") { 
#
# Reads a gene expression dataset in GCT format and converts it into an R data frame
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.
   ds <- read.delim(filename, header=TRUE, sep="\t", skip=2, row.names=1, blank.lines.skip=TRUE, comment.char="", as.is=TRUE)
   ds <- ds[-1]
   return(ds)
}

GSEA.Gct2Frame2 <- function(filename = "NULL") { 
#
# Reads a gene expression dataset in GCT format and converts it into an R data frame
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.
      content <- readLines(filename)
      content <- content[-1]
      content <- content[-1]
      col.names <- noquote(unlist(strsplit(content[1], "\t")))
      col.names <- col.names[c(-1, -2)]
      num.cols <- length(col.names)
      content <- content[-1]
      num.lines <- length(content)


      row.nam <- vector(length=num.lines, mode="character")
      row.des <- vector(length=num.lines, mode="character")
      m <- matrix(0, nrow=num.lines, ncol=num.cols)

      for (i in 1:num.lines) {
         line.list <- noquote(unlist(strsplit(content[i], "\t")))
         row.nam[i] <- noquote(line.list[1])
         row.des[i] <- noquote(line.list[2])
         line.list <- line.list[c(-1, -2)]
         for (j in 1:length(line.list)) {
            m[i, j] <- as.numeric(line.list[j])
         }
      }
      ds <- data.frame(m)
      names(ds) <- col.names
      row.names(ds) <- row.nam
      return(ds)
}

GSEA.ReadClsFile <- function(file = "NULL") { 
#
# Reads a class vector CLS file and defines phenotype and class labels vectors for the samples in a gene expression file (RES or GCT format)
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

      cls.cont <- readLines(file)
      num.lines <- length(cls.cont)
      class.list <- unlist(strsplit(cls.cont[[3]], " "))
      s <- length(class.list)
      t <- table(class.list)
      l <- length(t)
      phen <- vector(length=l, mode="character")
      phen.label <- vector(length=l, mode="numeric")
      class.v <- vector(length=s, mode="numeric")
      for (i in 1:l) {
         phen[i] <- noquote(names(t)[i])
         phen.label[i] <- i - 1
      }
      for (i in 1:s) {
         for (j in 1:l) {
             if (class.list[i] == phen[j]) {
                class.v[i] <- phen.label[j]
             }
         }
      }
      return(list(phen = phen, class.v = class.v))
}

GSEA.Threshold <- function(V, thres, ceil) { 
#
# Threshold and ceiling pre-processing for gene expression matrix
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

        V[V < thres] <- thres
        V[V > ceil] <- ceil
        return(V)
}

GSEA.VarFilter <- function(V, fold, delta, gene.names = "NULL") { 
#
# Variation filter pre-processing for gene expression matrix
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

        cols <- length(V[1,])
        rows <- length(V[,1])
        row.max <- apply(V, MARGIN=1, FUN=max)
               row.min <- apply(V, MARGIN=1, FUN=min)
        flag <- array(dim=rows)
        flag <- (row.max /row.min > fold) & (row.max - row.min > delta)
        size <- sum(flag)
        B <- matrix(0, nrow = size, ncol = cols)
        j <- 1
        if (gene.names == "NULL") {
           for (i in 1:rows) {
              if (flag[i]) {
                 B[j,] <- V[i,]
                 j <- j + 1
               }
           }
        return(B)
        } else {
            new.list <- vector(mode = "character", length = size)
            for (i in 1:rows) {
              if (flag[i]) {
                 B[j,] <- V[i,]
                 new.list[j] <- gene.names[i]
                 j <- j + 1
              }
            }
        return(list(V = B, new.list = new.list))
        }
}

GSEA.NormalizeRows <- function(V) { 
#
# Stardardize rows of a gene expression matrix
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

        row.mean <- apply(V, MARGIN=1, FUN=mean)
               row.sd <- apply(V, MARGIN=1, FUN=sd)
        row.n <- length(V[,1])
        for (i in 1:row.n) {
             if (row.sd[i] == 0) {
                  V[i,] <- 0
           } else {
              V[i,] <- (V[i,] - row.mean[i])/row.sd[i]
           }
        }
        return(V)
}

GSEA.NormalizeCols <- function(V) { 
#
# Stardardize columns of a gene expression matrix
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

        col.mean <- apply(V, MARGIN=2, FUN=mean)
               col.sd <- apply(V, MARGIN=2, FUN=sd)
        col.n <- length(V[1,])
        for (i in 1:col.n) {
             if (col.sd[i] == 0) {
                  V[i,] <- 0
           } else {
              V[,i] <- (V[,i] - col.mean[i])/col.sd[i]
           }
        }
        return(V)
}

# end of auxiliary functions

# ----------------------------------------------------------------------------------------
# Main GSEA Analysis Function that implements the entire methodology

GSEA <- function(
input.ds, 
input.cls, 
gene.ann = "", 
gs.db, 
gs.ann = "",
output.directory = "", 
doc.string = "GSEA.analysis", 
non.interactive.run = FALSE, 
reshuffling.type = "sample.labels", 
nperm = 1000, 
weighted.score.type = 1, 
nom.p.val.threshold = -1, 
fwer.p.val.threshold = -1, 
fdr.q.val.threshold = 0.25, 
topgs = 10,
adjust.FDR.q.val = FALSE, 
gs.size.threshold.min = 25, 
gs.size.threshold.max = 500, 
reverse.sign = FALSE, 
preproc.type = 0, 
random.seed = 123456, 
perm.type = 0, 
fraction = 1.0, 
replace = FALSE,
save.intermediate.results = FALSE,
OLD.GSEA = FALSE,
use.fast.enrichment.routine = TRUE) {

# This is a methodology for the analysis of global molecular profiles called Gene Set Enrichment Analysis (GSEA). It determines 
# whether an a priori defined set of genes shows statistically significant, concordant differences between two biological 
# states (e.g. phenotypes). GSEA operates on all genes from an experiment, rank ordered by the signal to noise ratio and 
# determines whether members of an a priori defined gene set are nonrandomly distributed towards the top or bottom of the 
# list and thus may correspond to an important biological process. To assess significance the program uses an empirical 
# permutation procedure to test deviation from random that preserves correlations between genes. 
#
# For details see Subramanian et al 2005
#
# Inputs:
#   input.ds: Input gene expression Affymetrix dataset file in RES or GCT format 
#   input.cls:  Input class vector (phenotype) file in CLS format 
#   gene.ann.file: Gene microarray annotation file (Affymetrix Netaffyx *.csv format) (default: none) 
#   gs.file: Gene set database in GMT format 
#   output.directory: Directory where to store output and results (default: .) 
#   doc.string:  Documentation string used as a prefix to name result files (default: "GSEA.analysis") 
#   non.interactive.run: Run in interactive (i.e. R GUI) or batch (R command line) mode (default: F) 
#   reshuffling.type: Type of permutation reshuffling: "sample.labels" or "gene.labels" (default: "sample.labels") 
#   nperm: Number of random permutations (default: 1000) 
#   weighted.score.type: Enrichment correlation-based weighting: 0=no weight (KS), 1=standard weigth, 2 = over-weigth (default: 1) 
#   nom.p.val.threshold: Significance threshold for nominal p-vals for gene sets (default: -1, no thres) 
#   fwer.p.val.threshold: Significance threshold for FWER p-vals for gene sets (default: -1, no thres) 
#   fdr.q.val.threshold: Significance threshold for FDR q-vals for gene sets (default: 0.25) 
#   topgs: Besides those passing test, number of top scoring gene sets used for detailed reports (default: 10) 
#   adjust.FDR.q.val: Adjust the FDR q-vals (default: F) 
#   gs.size.threshold.min: Minimum size (in genes) for database gene sets to be considered (default: 25) 
#   gs.size.threshold.max: Maximum size (in genes) for database gene sets to be considered (default: 500) 
#   reverse.sign: Reverse direction of gene list (pos. enrichment becomes negative, etc.) (default: F) 
#   preproc.type: Preprocessing normalization: 0=none, 1=col(z-score)., 2=col(rank) and row(z-score)., 3=col(rank). (default: 0) 
#   random.seed: Random number generator seed. (default: 123456) 
#   perm.type: Permutation type: 0 = unbalanced, 1 = balanced. For experts only (default: 0) 
#   fraction: Subsampling fraction. Set to 1.0 (no resampling). For experts only (default: 1.0) 
#   replace: Resampling mode (replacement or not replacement). For experts only (default: F) 
#   OLD.GSEA: if TRUE compute the OLD GSEA of Mootha et al 2003
#   use.fast.enrichment.routine: if true it uses a faster version to compute random perm. enrichment "GSEA.EnrichmentScore2"  
#
#   Output:
#    The results of the method are stored in the "output.directory" specified by the user as part of the input parameters. 
#      The results files are:
#    - Two tab-separated global result text files (one for each phenotype). These files are labeled according to the doc 
#      string prefix and the phenotype name from the CLS file: <doc.string>.SUMMARY.RESULTS.REPORT.<phenotype>.txt
#    - One set of global plots. They include a.- gene list correlation profile, b.- global observed and null densities, c.- heat map 
#      for the entire sorted dataset, and d.- p-values vs. NES plot. These plots are in a single JPEG file named 
#      <doc.string>.global.plots.<phenotype>.jpg. When the program is run interactively these plots appear on a window in the R GUI.
#    - A variable number of tab-separated gene result text files according to how many sets pass any of the significance thresholds 
#      ("nom.p.val.threshold," "fwer.p.val.threshold," and "fdr.q.val.threshold") and how many are specified in the "topgs" 
#      parameter. These files are named: <doc.string>.<gene set name>.report.txt. 
#   - A variable number of gene set plots (one for each gene set report file). These plots include a.- Gene set running enrichment
#      "mountain" plot, b.- gene set null distribution and c.- heat map for genes in the gene set. These plots are stored in a 
#      single JPEG file named <doc.string>.<gene set name>.jpg.
#  The format (columns) for the global result files is as follows.
#  GS : Gene set name.
# SIZE : Size of the set in genes.
# SOURCE : Set definition or source.
# ES : Enrichment score.
# NES : Normalized (multiplicative rescaling) normalized enrichment score.
# NOM p-val : Nominal p-value (from the null distribution of the gene set).
# FDR q-val: False discovery rate q-values
# FWER p-val: Family wise error rate p-values.
# Tag %: Percent of gene set before running enrichment peak.
# Gene %: Percent of gene list before running enrichment peak.
# Signal : enrichment signal strength.
# FDR (median): FDR q-values from the median of the null distributions.
# glob.p.val: P-value using a global statistic (number of sets above the set's NES).
# 
# The rows are sorted by the NES values (from maximum positive or negative NES to minimum)
# 
# The format (columns) for the gene set result files is as follows.
# 
# #: Gene number in the (sorted) gene set
# GENE : gene name. For example the probe accession number, gene symbol or the gene identifier gin the dataset.
# SYMBOL : gene symbol from the gene annotation file.
# DESC : gene description (title) from the gene annotation file.
# LIST LOC : location of the gene in the sorted gene list.
# S2N : signal to noise ratio (correlation) of the gene in the gene list.
# RES : value of the running enrichment score at the gene location.
# CORE_ENRICHMENT: is this gene is the "core enrichment" section of the list? Yes or No variable specifying in the gene location is before (positive ES) or after (negative ES) the running enrichment peak.
# 
# The rows are sorted by the gene location in the gene list.
# The function call to GSEA returns a  two element list containing the two global result reports as data frames ($report1, $report2).
# 
# results1: Global output report for first phenotype 
# result2:  Global putput report for second phenotype
#
# The Broad Institute
# SOFTWARE COPYRIGHT NOTICE AGREEMENT
# This software and its documentation are copyright 2003 by the
# Broad Institute/Massachusetts Institute of Technology.
# All rights are reserved.
#
# This software is supplied without any warranty or guaranteed support
# whatsoever. Neither the Broad Institute nor MIT can be responsible for
# its use, misuse, or functionality.

  print(" *** Running GSEA Analysis...")

if (OLD.GSEA == TRUE) {
   print("Running OLD GSEA from Mootha et al 2003")
}

# Copy input parameters to log file

if (output.directory != "")  {

filename <- paste(output.directory, doc.string, "_params.txt", sep="", collapse="")  

time.string <- as.character(as.POSIXlt(Sys.time(),"GMT"))
write(paste("Run of GSEA on ", time.string), file=filename)

if (is.data.frame(input.ds)) {
#      write(paste("input.ds=", quote(input.ds), sep=" "), file=filename, append= TRUE)
} else {
      write(paste("input.ds=", input.ds, sep=" "), file=filename, append= TRUE)
}
if (is.list(input.cls)) {
#      write(paste("input.cls=", input.cls, sep=" "), file=filename, append= TRUE) 
} else {
      write(paste("input.cls=", input.cls, sep=" "), file=filename, append= TRUE) 
}
if (is.data.frame(gene.ann)) {
#    write(paste("gene.ann =", gene.ann, sep=" "), file=filename, append= TRUE) 
} else {
    write(paste("gene.ann =", gene.ann, sep=" "), file=filename, append= TRUE) 
}
 if (regexpr(pattern=".gmt", gs.db[1]) == -1) {
#   write(paste("gs.db=", gs.db, sep=" "), file=filename, append= TRUE) 
} else {
   write(paste("gs.db=", gs.db, sep=" "), file=filename, append= TRUE) 
}
if (is.data.frame(gs.ann)) {
#    write(paste("gene.ann =", gene.ann, sep=" "), file=filename, append= TRUE) 
} else {
    write(paste("gs.ann =", gs.ann, sep=" "), file=filename, append= TRUE) 
}
write(paste("output.directory =", output.directory, sep=" "), file=filename, append= TRUE) 
write(paste("doc.string = ", doc.string, sep=" "), file=filename, append= TRUE) 
write(paste("non.interactive.run =", non.interactive.run, sep=" "), file=filename, append= TRUE) 
write(paste("reshuffling.type =", reshuffling.type, sep=" "), file=filename, append= TRUE) 
write(paste("nperm =", nperm, sep=" "), file=filename, append= TRUE) 
write(paste("weighted.score.type =", weighted.score.type, sep=" "), file=filename, append= TRUE) 
write(paste("nom.p.val.threshold =", nom.p.val.threshold, sep=" "), file=filename, append= TRUE) 
write(paste("fwer.p.val.threshold =", fwer.p.val.threshold, sep=" "), file=filename, append= TRUE) 
write(paste("fdr.q.val.threshold =", fdr.q.val.threshold, sep=" "), file=filename, append= TRUE) 
write(paste("topgs =", topgs, sep=" "), file=filename, append= TRUE)
write(paste("adjust.FDR.q.val =", adjust.FDR.q.val, sep=" "), file=filename, append= TRUE) 
write(paste("gs.size.threshold.min =", gs.size.threshold.min, sep=" "), file=filename, append= TRUE) 
write(paste("gs.size.threshold.max =", gs.size.threshold.max, sep=" "), file=filename, append= TRUE) 
write(paste("reverse.sign =", reverse.sign, sep=" "), file=filename, append= TRUE) 
write(paste("preproc.type =", preproc.type, sep=" "), file=filename, append= TRUE) 
write(paste("random.seed =", random.seed, sep=" "), file=filename, append= TRUE) 
write(paste("perm.type =", perm.type, sep=" "), file=filename, append= TRUE) 
write(paste("fraction =", fraction, sep=" "), file=filename, append= TRUE) 
write(paste("replace =", replace, sep=" "), file=filename, append= TRUE)
}

# Start of GSEA methodology 

  if (.Platform$OS.type == "windows") {
      memory.limit(6000000000)
      memory.limit()
#      print(c("Start memory size=",  memory.size()))
  }

  # Read input data matrix

  set.seed(seed=random.seed, kind = NULL)
  adjust.param <- 0.5

  gc()

  time1 <- proc.time()

  if (is.data.frame(input.ds)) {
     dataset <- input.ds
  } else {
     if (regexpr(pattern=".gct", input.ds) == -1) {
         dataset <- GSEA.Res2Frame(filename = input.ds)
     } else {
#         dataset <- GSEA.Gct2Frame(filename = input.ds)
         dataset <- GSEA.Gct2Frame2(filename = input.ds)
     }
  }
  gene.labels <- row.names(dataset)
  sample.names <- names(dataset)
  A <- data.matrix(dataset)
  dim(A) 
  cols <- length(A[1,])
  rows <- length(A[,1])

# preproc.type control the type of pre-processing: threshold, variation filter, normalization

 if (preproc.type == 1) {  # Column normalize (Z-score)
    A <- GSEA.NormalizeCols(A)
 } else if (preproc.type == 2) { # Column (rank) and row (Z-score) normalize 
    for (j in 1:cols) {  # column rank normalization
        A[,j] <- rank(A[,j])
    }
    A <- GSEA.NormalizeRows(A)
 } else if (preproc.type == 3) { # Column (rank) norm.
    for (j in 1:cols) {  # column rank normalization
        A[,j] <- rank(A[,j])
    }
 }

  # Read input class vector

  if(is.list(input.cls)) {
     CLS <- input.cls
  } else {
     CLS <- GSEA.ReadClsFile(file=input.cls)
  }
  class.labels <- CLS$class.v
  class.phen <- CLS$phen

  if (reverse.sign == TRUE) {
     phen1 <- class.phen[2]
     phen2 <- class.phen[1]
  } else {
     phen1 <- class.phen[1]
     phen2 <- class.phen[2]
  }

  # sort samples according to phenotype
 
 col.index <- order(class.labels, decreasing= FALSE)
 class.labels <- class.labels[col.index]
 sample.names <- sample.names[col.index]
 for (j in 1:rows) {
    A[j, ] <- A[j, col.index]
 }
 names(A) <- sample.names
 
  # Read input gene set database

 if (regexpr(pattern=".gmt", gs.db[1]) == -1) {
      temp <- gs.db
 } else {
      temp <- readLines(gs.db)
 }

      max.Ng <- length(temp)
      temp.size.G <- vector(length = max.Ng, mode = "numeric") 
      for (i in 1:max.Ng) {
          temp.size.G[i] <- length(unlist(strsplit(temp[[i]], "\t"))) - 2
      }

      max.size.G <- max(temp.size.G)      
      gs <- matrix(rep("null", max.Ng*max.size.G), nrow=max.Ng, ncol= max.size.G)
      temp.names <- vector(length = max.Ng, mode = "character")
      temp.desc <- vector(length = max.Ng, mode = "character")
      gs.count <- 1
      for (i in 1:max.Ng) {
          gene.set.size <- length(unlist(strsplit(temp[[i]], "\t"))) - 2
          gs.line <- noquote(unlist(strsplit(temp[[i]], "\t")))
          gene.set.name <- gs.line[1] 
          gene.set.desc <- gs.line[2] 
          gene.set.tags <- vector(length = gene.set.size, mode = "character")
          for (j in 1:gene.set.size) {
              gene.set.tags[j] <- gs.line[j + 2]
          } 
          existing.set <- is.element(gene.set.tags, gene.labels)
          set.size <- length(existing.set[existing.set == TRUE])
          if ((set.size < gs.size.threshold.min) || (set.size > gs.size.threshold.max)) next
          temp.size.G[gs.count] <- set.size
          gs[gs.count,] <- c(gene.set.tags[existing.set], rep("null", max.size.G - temp.size.G[gs.count]))
          temp.names[gs.count] <- gene.set.name
          temp.desc[gs.count] <- gene.set.desc
          gs.count <- gs.count + 1
      } 
      Ng <- gs.count - 1
      gs.names <- vector(length = Ng, mode = "character")
      gs.desc <- vector(length = Ng, mode = "character")
      size.G <- vector(length = Ng, mode = "numeric") 
      gs.names <- temp.names[1:Ng]
      gs.desc <- temp.desc[1:Ng] 
      size.G <- temp.size.G[1:Ng]

  N <- length(A[,1])
  Ns <- length(A[1,])

  print(c("Number of genes:", N))
  print(c("Number of Gene Sets:", Ng))
  print(c("Number of samples:", Ns))
  print(c("Original number of Gene Sets:", max.Ng))
  print(c("Maximum gene set size:", max.size.G))

# Read gene and gene set annotations if gene annotation file was provided

  all.gene.descs <- vector(length = N, mode ="character") 
  all.gene.symbols <- vector(length = N, mode ="character") 
  all.gs.descs <- vector(length = Ng, mode ="character") 

  if (is.data.frame(gene.ann)) {
     temp <- gene.ann
     a.size <- length(temp[,1])
     print(c("Number of gene annotation file entries:", a.size))  
     accs <- as.character(temp[,1])
     locs <- match(gene.labels, accs)
     all.gene.descs <- as.character(temp[locs, "Gene.Title"])
     all.gene.symbols <- as.character(temp[locs, "Gene.Symbol"])
     rm(temp)
  } else  if (gene.ann == "") {
     for (i in 1:N) {
        all.gene.descs[i] <- gene.labels[i]
        all.gene.symbols[i] <- gene.labels[i]
     }
  } else {
     temp <- read.delim(gene.ann, header= TRUE, sep=",", comment.char="", as.is= TRUE)
     a.size <- length(temp[,1])
     print(c("Number of gene annotation file entries:", a.size))  
     accs <- as.character(temp[,1])
     locs <- match(gene.labels, accs)
     all.gene.descs <- as.character(temp[locs, "Gene.Title"])
     all.gene.symbols <- as.character(temp[locs, "Gene.Symbol"])
     rm(temp)
  }

  if (is.data.frame(gs.ann)) {
     temp <- gs.ann
     a.size <- length(temp[,1])
     print(c("Number of gene set annotation file entries:", a.size))  
     accs <- as.character(temp[,1])
     locs <- match(gs.names, accs)
     all.gs.descs <- as.character(temp[locs, "SOURCE"])
     rm(temp)
  } else if (gs.ann == "") {
     for (i in 1:Ng) {
        all.gs.descs[i] <- gs.desc[i]
     }
  } else {
     temp <- read.delim(gs.ann, header= TRUE, sep="\t", comment.char="", as.is= TRUE)
     a.size <- length(temp[,1])
     print(c("Number of gene set annotation file entries:", a.size))  
     accs <- as.character(temp[,1])
     locs <- match(gs.names, accs)
     all.gs.descs <- as.character(temp[locs, "SOURCE"])
     rm(temp)
  }

  
  Obs.indicator <- matrix(nrow= Ng, ncol=N)
  Obs.RES <- matrix(nrow= Ng, ncol=N)

  Obs.ES <- vector(length = Ng, mode = "numeric")
  Obs.arg.ES <- vector(length = Ng, mode = "numeric")
  Obs.ES.norm <- vector(length = Ng, mode = "numeric")

  time2 <- proc.time()

  # GSEA methodology

  # Compute observed and random permutation gene rankings

  obs.s2n <- vector(length=N, mode="numeric")
  signal.strength <- vector(length=Ng, mode="numeric")
  tag.frac <- vector(length=Ng, mode="numeric")
  gene.frac <- vector(length=Ng, mode="numeric")
  coherence.ratio <- vector(length=Ng, mode="numeric")
  obs.phi.norm <- matrix(nrow = Ng, ncol = nperm)
  correl.matrix <- matrix(nrow = N, ncol = nperm)
  obs.correl.matrix <- matrix(nrow = N, ncol = nperm)
  order.matrix <- matrix(nrow = N, ncol = nperm)
  obs.order.matrix <- matrix(nrow = N, ncol = nperm)

   nperm.per.call <- 100
   n.groups <- nperm %/% nperm.per.call
   n.rem <- nperm %% nperm.per.call
   n.perms <- c(rep(nperm.per.call, n.groups), n.rem)
   n.ends <- cumsum(n.perms)
   n.starts <- n.ends - n.perms + 1

   if (n.rem == 0) {
     n.tot <- n.groups
   } else {
     n.tot <- n.groups + 1
   }

 for (nk in 1:n.tot) {
   call.nperm <- n.perms[nk]

   print(paste("Computing ranked list for actual and permuted phenotypes.......permutations: ", n.starts[nk], "--", n.ends[nk], sep=" "))

   O <- GSEA.GeneRanking(A, class.labels, gene.labels, call.nperm, permutation.type = perm.type, sigma.correction = "GeneCluster", fraction=fraction, replace=replace, reverse.sign = reverse.sign)
   gc()

   order.matrix[,n.starts[nk]:n.ends[nk]] <- O$order.matrix
   obs.order.matrix[,n.starts[nk]:n.ends[nk]] <- O$obs.order.matrix
   correl.matrix[,n.starts[nk]:n.ends[nk]] <- O$s2n.matrix
   obs.correl.matrix[,n.starts[nk]:n.ends[nk]] <- O$obs.s2n.matrix
    rm(O)
 }

  obs.s2n <- apply(obs.correl.matrix, 1, median)  # using median to assign enrichment scores
  obs.index <- order(obs.s2n, decreasing= TRUE)            
  obs.s2n   <- sort(obs.s2n, decreasing= TRUE)            

  obs.gene.labels <- gene.labels[obs.index]       
  obs.gene.descs <- all.gene.descs[obs.index]       
  obs.gene.symbols <- all.gene.symbols[obs.index]       

  for (r in 1:nperm) {
      correl.matrix[, r] <- correl.matrix[order.matrix[,r], r]
  }
  for (r in 1:nperm) {
      obs.correl.matrix[, r] <- obs.correl.matrix[obs.order.matrix[,r], r]
  }

  gene.list2 <- obs.index
  for (i in 1:Ng) {
       print(paste("Computing observed enrichment for gene set:", i, gs.names[i], sep=" ")) 
       gene.set <- gs[i,gs[i,] != "null"]
       gene.set2 <- vector(length=length(gene.set), mode = "numeric")
       gene.set2 <- match(gene.set, gene.labels)
       if (OLD.GSEA == FALSE) {
          GSEA.results <- GSEA.EnrichmentScore(gene.list=gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector = obs.s2n)
       } else {
          GSEA.results <- OLD.GSEA.EnrichmentScore(gene.list=gene.list2, gene.set=gene.set2)
       }
       Obs.ES[i] <- GSEA.results$ES
       Obs.arg.ES[i] <- GSEA.results$arg.ES
       Obs.RES[i,] <- GSEA.results$RES
       Obs.indicator[i,] <- GSEA.results$indicator
       if (Obs.ES[i] >= 0) {  # compute signal strength
           tag.frac[i] <- sum(Obs.indicator[i,1:Obs.arg.ES[i]])/size.G[i]
           gene.frac[i] <- Obs.arg.ES[i]/N
       } else {
           tag.frac[i] <- sum(Obs.indicator[i, Obs.arg.ES[i]:N])/size.G[i]
           gene.frac[i] <- (N - Obs.arg.ES[i] + 1)/N
       }
       signal.strength[i] <- tag.frac[i] * (1 - gene.frac[i]) * (N / (N - size.G[i]))
   }

# Compute enrichment for random permutations 

   phi <- matrix(nrow = Ng, ncol = nperm)
   phi.norm <- matrix(nrow = Ng, ncol = nperm)
   obs.phi <- matrix(nrow = Ng, ncol = nperm)

   if (reshuffling.type == "sample.labels") { # reshuffling phenotype labels
      for (i in 1:Ng) {
        print(paste("Computing random permutations' enrichment for gene set:", i, gs.names[i], sep=" ")) 
        gene.set <- gs[i,gs[i,] != "null"]
        gene.set2 <- vector(length=length(gene.set), mode = "numeric")
        gene.set2 <- match(gene.set, gene.labels)
        for (r in 1:nperm) {
            gene.list2 <- order.matrix[,r]
            if (use.fast.enrichment.routine == FALSE) {
               GSEA.results <- GSEA.EnrichmentScore(gene.list=gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=correl.matrix[, r])   
            } else {
               GSEA.results <- GSEA.EnrichmentScore2(gene.list=gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=correl.matrix[, r])   
            }
            phi[i, r] <- GSEA.results$ES
        }
        if (fraction < 1.0) { # if resampling then compute ES for all observed rankings
            for (r in 1:nperm) {
                obs.gene.list2 <- obs.order.matrix[,r]
                if (use.fast.enrichment.routine == FALSE) {
                   GSEA.results <- GSEA.EnrichmentScore(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r]) 
                } else {
                   GSEA.results <- GSEA.EnrichmentScore2(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])
                }
                obs.phi[i, r] <- GSEA.results$ES
            }
        } else { # if no resampling then compute only one column (and fill the others with the same value)
             obs.gene.list2 <- obs.order.matrix[,1]
            if (use.fast.enrichment.routine == FALSE) {
               GSEA.results <- GSEA.EnrichmentScore(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r]) 
            } else {
               GSEA.results <- GSEA.EnrichmentScore2(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])
            }
            obs.phi[i, 1] <- GSEA.results$ES
            for (r in 2:nperm) {
               obs.phi[i, r] <- obs.phi[i, 1]
            }
        }
        gc()
     }

   } else if (reshuffling.type == "gene.labels") { # reshuffling gene labels
      for (i in 1:Ng) {
        gene.set <- gs[i,gs[i,] != "null"]
        gene.set2 <- vector(length=length(gene.set), mode = "numeric")
        gene.set2 <- match(gene.set, gene.labels)
        for (r in 1:nperm) {
            reshuffled.gene.labels <- sample(1:rows)
            if (use.fast.enrichment.routine == FALSE) {
               GSEA.results <- GSEA.EnrichmentScore(gene.list=reshuffled.gene.labels, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.s2n)   
            } else {
               GSEA.results <- GSEA.EnrichmentScore2(gene.list=reshuffled.gene.labels, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.s2n)   
            }
            phi[i, r] <- GSEA.results$ES
        }
        if (fraction < 1.0) { # if resampling then compute ES for all observed rankings
           for (r in 1:nperm) {
              obs.gene.list2 <- obs.order.matrix[,r]
              if (use.fast.enrichment.routine == FALSE) {
                 GSEA.results <- GSEA.EnrichmentScore(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])   
              } else {
                 GSEA.results <- GSEA.EnrichmentScore2(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])   
              }
              obs.phi[i, r] <- GSEA.results$ES
           }
        } else { # if no resampling then compute only one column (and fill the others with the same value)
           obs.gene.list2 <- obs.order.matrix[,1]
           if (use.fast.enrichment.routine == FALSE) {
              GSEA.results <- GSEA.EnrichmentScore(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])   
           } else {
              GSEA.results <- GSEA.EnrichmentScore2(gene.list=obs.gene.list2, gene.set=gene.set2, weighted.score.type=weighted.score.type, correl.vector=obs.correl.matrix[, r])   
           }
           obs.phi[i, 1] <- GSEA.results$ES
           for (r in 2:nperm) {
              obs.phi[i, r] <- obs.phi[i, 1]
           }
        }
        gc()
     }
   }

# Compute 3 types of p-values

# Find nominal p-values       

print("Computing nominal p-values...")

p.vals <- matrix(0, nrow = Ng, ncol = 2)

if (OLD.GSEA == FALSE) {
   for (i in 1:Ng) {
      pos.phi <- NULL
      neg.phi <- NULL
      for (j in 1:nperm) {
         if (phi[i, j] >= 0) {
            pos.phi <- c(pos.phi, phi[i, j]) 
         } else {
            neg.phi <- c(neg.phi, phi[i, j]) 
         }
      }
      ES.value <- Obs.ES[i]
      if (ES.value >= 0) {
         p.vals[i, 1] <- signif(sum(pos.phi >= ES.value)/length(pos.phi), digits=5)
      } else {
         p.vals[i, 1] <- signif(sum(neg.phi <= ES.value)/length(neg.phi), digits=5)
      }
   }
} else {  # For OLD GSEA compute the p-val using positive and negative values in the same histogram
   for (i in 1:Ng) {
      if (Obs.ES[i] >= 0) {
         p.vals[i, 1] <-  sum(phi[i,] >= Obs.ES[i])/length(phi[i,])
         p.vals[i, 1] <-  signif(p.vals[i, 1], digits=5)
      } else {
         p.vals[i, 1] <-  sum(phi[i,] <= Obs.ES[i])/length(phi[i,])
         p.vals[i, 1] <-  signif(p.vals[i, 1], digits=5)
      }
   }
}

# Find effective size 

 erf <- function (x) 
 {
    2 * pnorm(sqrt(2) * x)
 }

 KS.mean <- function(N) { # KS mean as a function of set size N
       S <- 0
       for (k in -100:100) {
          if (k == 0) next
          S <- S + 4 * (-1)**(k + 1) * (0.25 * exp(-2 * k * k * N) - sqrt(2 * pi) *  erf(sqrt(2 * N) * k)/(16 * k * sqrt(N)))
       }
      return(abs(S))
 }

# KS.mean.table <- vector(length=5000, mode="numeric")

# for (i in 1:5000) {
#    KS.mean.table[i] <- KS.mean(i)
# }

# KS.size <-  vector(length=Ng, mode="numeric")

# Rescaling normalization for each gene set null

print("Computing rescaling normalization for each gene set null...")

if (OLD.GSEA == FALSE) {
   for (i in 1:Ng) {
         pos.phi <- NULL
         neg.phi <- NULL
         for (j in 1:nperm) {
            if (phi[i, j] >= 0) {
               pos.phi <- c(pos.phi, phi[i, j]) 
            } else {
               neg.phi <- c(neg.phi, phi[i, j]) 
            }
         }
         pos.m <- mean(pos.phi)
         neg.m <- mean(abs(neg.phi))

#         if (Obs.ES[i] >= 0) {
#            KS.size[i] <- which.min(abs(KS.mean.table - pos.m))
#         } else {
#            KS.size[i] <- which.min(abs(KS.mean.table - neg.m))
#         }

         pos.phi <- pos.phi/pos.m
         neg.phi <- neg.phi/neg.m
         for (j in 1:nperm) {
            if (phi[i, j] >= 0) {
                phi.norm[i, j] <- phi[i, j]/pos.m
            } else {
                phi.norm[i, j] <- phi[i, j]/neg.m
            }
          }
          for (j in 1:nperm) {
             if (obs.phi[i, j] >= 0) {
                obs.phi.norm[i, j] <- obs.phi[i, j]/pos.m
             } else {
                obs.phi.norm[i, j] <- obs.phi[i, j]/neg.m
             }
          }
          if (Obs.ES[i] >= 0) {
             Obs.ES.norm[i] <- Obs.ES[i]/pos.m
          } else {
             Obs.ES.norm[i] <- Obs.ES[i]/neg.m
          }
   }
} else {  # For OLD GSEA does not normalize using empirical scaling 
   for (i in 1:Ng) {
      for (j in 1:nperm) {
          phi.norm[i, j] <- phi[i, j]/400
      }
      for (j in 1:nperm) {
          obs.phi.norm[i, j] <- obs.phi[i, j]/400
      }
      Obs.ES.norm[i] <- Obs.ES[i]/400
   }
}

# Save intermedite results

       if (save.intermediate.results == TRUE) {

          filename <- paste(output.directory, doc.string, ".phi.txt", sep="", collapse="")
          write.table(phi, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")

          filename <- paste(output.directory, doc.string, ".obs.phi.txt", sep="", collapse="")
          write.table(obs.phi, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")

          filename <- paste(output.directory, doc.string, ".phi.norm.txt", sep="", collapse="")
          write.table(phi.norm, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")

          filename <- paste(output.directory, doc.string, ".obs.phi.norm.txt", sep="", collapse="")
          write.table(obs.phi.norm, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")

          filename <- paste(output.directory, doc.string, ".Obs.ES.txt", sep="", collapse="")
          write.table(Obs.ES, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")

          filename <- paste(output.directory, doc.string, ".Obs.ES.norm.txt", sep="", collapse="")
          write.table(Obs.ES.norm, file = filename, quote= FALSE, col.names= FALSE, row.names= FALSE, sep = "\t")
      }

# Compute FWER p-vals

      print("Computing FWER p-values...")

if (OLD.GSEA == FALSE) {
    max.ES.vals.p <- NULL
    max.ES.vals.n <- NULL
    for (j in 1:nperm) {
       pos.phi <- NULL
       neg.phi <- NULL
       for (i in 1:Ng) {
          if (phi.norm[i, j] >= 0) {
             pos.phi <- c(pos.phi, phi.norm[i, j]) 
          } else {
             neg.phi <- c(neg.phi, phi.norm[i, j]) 
          }
       }
       if (length(pos.phi) > 0) {
          max.ES.vals.p <- c(max.ES.vals.p, max(pos.phi))
       }
       if (length(neg.phi) > 0) {
          max.ES.vals.n <- c(max.ES.vals.n, min(neg.phi))
       }
     }
   for (i in 1:Ng) {
       ES.value <- Obs.ES.norm[i]
       if (Obs.ES.norm[i] >= 0) {
          p.vals[i, 2] <- signif(sum(max.ES.vals.p >= ES.value)/length(max.ES.vals.p), digits=5)
       } else {
          p.vals[i, 2] <- signif(sum(max.ES.vals.n <= ES.value)/length(max.ES.vals.n), digits=5)
       }
     }
} else {  # For OLD GSEA compute the FWER using positive and negative values in the same histogram
    max.ES.vals <- NULL
    for (j in 1:nperm) {
       max.NES <- max(phi.norm[,j])
       min.NES <- min(phi.norm[,j])
       if (max.NES > - min.NES) {
          max.val <- max.NES
       } else {
          max.val <- min.NES
       }
       max.ES.vals <- c(max.ES.vals, max.val)
    }
    for (i in 1:Ng) {
       if (Obs.ES.norm[i] >= 0) {
          p.vals[i, 2] <- sum(max.ES.vals >= Obs.ES.norm[i])/length(max.ES.vals)
       } else {
          p.vals[i, 2] <- sum(max.ES.vals <= Obs.ES.norm[i])/length(max.ES.vals)
       }
       p.vals[i, 2] <-  signif(p.vals[i, 2], digits=4)
     }
}

# Compute FDRs 

      print("Computing FDR q-values...")

      NES <- vector(length=Ng, mode="numeric")
      phi.norm.mean  <- vector(length=Ng, mode="numeric")
      obs.phi.norm.mean  <- vector(length=Ng, mode="numeric")
      phi.norm.median  <- vector(length=Ng, mode="numeric")
      obs.phi.norm.median  <- vector(length=Ng, mode="numeric")
      phi.norm.mean  <- vector(length=Ng, mode="numeric")
      obs.phi.mean  <- vector(length=Ng, mode="numeric")
      FDR.mean <- vector(length=Ng, mode="numeric")
      FDR.median <- vector(length=Ng, mode="numeric")
      phi.norm.median.d <- vector(length=Ng, mode="numeric")
      obs.phi.norm.median.d <- vector(length=Ng, mode="numeric")

      Obs.ES.index <- order(Obs.ES.norm, decreasing= TRUE)
      Orig.index <- seq(1, Ng)
      Orig.index <- Orig.index[Obs.ES.index]
      Orig.index <- order(Orig.index, decreasing= FALSE)
      Obs.ES.norm.sorted <- Obs.ES.norm[Obs.ES.index]
      gs.names.sorted <- gs.names[Obs.ES.index]

      for (k in 1:Ng) {
         NES[k] <- Obs.ES.norm.sorted[k]
         ES.value <- NES[k]
         count.col <- vector(length=nperm, mode="numeric")
         obs.count.col <- vector(length=nperm, mode="numeric")
         for (i in 1:nperm) {
            phi.vec <- phi.norm[,i]
            obs.phi.vec <- obs.phi.norm[,i]
            if (ES.value >= 0) {
               count.col.norm <- sum(phi.vec >= 0)
               obs.count.col.norm <- sum(obs.phi.vec >= 0)
               count.col[i] <- ifelse(count.col.norm > 0, sum(phi.vec >= ES.value)/count.col.norm, 0)
               obs.count.col[i] <- ifelse(obs.count.col.norm > 0, sum(obs.phi.vec >= ES.value)/obs.count.col.norm, 0)
            } else {
               count.col.norm <- sum(phi.vec < 0)
               obs.count.col.norm <- sum(obs.phi.vec < 0)
               count.col[i] <- ifelse(count.col.norm > 0, sum(phi.vec <= ES.value)/count.col.norm, 0)
               obs.count.col[i] <- ifelse(obs.count.col.norm > 0, sum(obs.phi.vec <= ES.value)/obs.count.col.norm, 0)
            }
         }
        phi.norm.mean[k] <- mean(count.col)
        obs.phi.norm.mean[k] <- mean(obs.count.col)
        phi.norm.median[k] <- median(count.col)
        obs.phi.norm.median[k] <- median(obs.count.col)
        FDR.mean[k] <- ifelse(phi.norm.mean[k]/obs.phi.norm.mean[k] < 1, phi.norm.mean[k]/obs.phi.norm.mean[k], 1)
        FDR.median[k] <- ifelse(phi.norm.median[k]/obs.phi.norm.median[k] < 1, phi.norm.median[k]/obs.phi.norm.median[k], 1)
      }

# adjust q-values

      if (adjust.FDR.q.val == TRUE) {
         pos.nes <- length(NES[NES >= 0])
         min.FDR.mean <- FDR.mean[pos.nes]
         min.FDR.median <- FDR.median[pos.nes]
         for (k in seq(pos.nes - 1, 1, -1)) {
              if (FDR.mean[k] < min.FDR.mean) {
                  min.FDR.mean <- FDR.mean[k]
              }
              if (min.FDR.mean < FDR.mean[k]) {
                  FDR.mean[k] <- min.FDR.mean
              }
         }

         neg.nes <- pos.nes + 1
         min.FDR.mean <- FDR.mean[neg.nes]
         min.FDR.median <- FDR.median[neg.nes]
         for (k in seq(neg.nes + 1, Ng)) {
             if (FDR.mean[k] < min.FDR.mean) {
                 min.FDR.mean <- FDR.mean[k]
             }
             if (min.FDR.mean < FDR.mean[k]) {
                 FDR.mean[k] <- min.FDR.mean
             }
         }
     }

     obs.phi.norm.mean.sorted <- obs.phi.norm.mean[Orig.index]
     phi.norm.mean.sorted <- phi.norm.mean[Orig.index]
     FDR.mean.sorted <- FDR.mean[Orig.index]
     FDR.median.sorted <- FDR.median[Orig.index]

#   Compute global statistic

    glob.p.vals <- vector(length=Ng, mode="numeric")
    NULL.pass <- vector(length=nperm, mode="numeric")
    OBS.pass <- vector(length=nperm, mode="numeric")

    for (k in 1:Ng) {
      NES[k] <- Obs.ES.norm.sorted[k]
      if (NES[k] >= 0) {
         for (i in 1:nperm) {
             NULL.pos <- sum(phi.norm[,i] >= 0)            
             NULL.pass[i] <- ifelse(NULL.pos > 0, sum(phi.norm[,i] >= NES[k])/NULL.pos, 0)
             OBS.pos <- sum(obs.phi.norm[,i] >= 0)
             OBS.pass[i] <- ifelse(OBS.pos > 0, sum(obs.phi.norm[,i] >= NES[k])/OBS.pos, 0)
         }
      } else {
         for (i in 1:nperm) {
             NULL.neg <- sum(phi.norm[,i] < 0)
             NULL.pass[i] <- ifelse(NULL.neg > 0, sum(phi.norm[,i] <= NES[k])/NULL.neg, 0)
             OBS.neg <- sum(obs.phi.norm[,i] < 0)
             OBS.pass[i] <- ifelse(OBS.neg > 0, sum(obs.phi.norm[,i] <= NES[k])/OBS.neg, 0)
         }
      }
      glob.p.vals[k] <- sum(NULL.pass >= mean(OBS.pass))/nperm
    }
    glob.p.vals.sorted <- glob.p.vals[Orig.index]

# Produce results report

      print("Producing result tables and plots...")

       Obs.ES <- signif(Obs.ES, digits=5)
       Obs.ES.norm <- signif(Obs.ES.norm, digits=5)
       p.vals <- signif(p.vals, digits=4)
       signal.strength <- signif(signal.strength, digits=3)
       tag.frac <- signif(tag.frac, digits=3)
       gene.frac <- signif(gene.frac, digits=3)
       FDR.mean.sorted <- signif(FDR.mean.sorted, digits=5)
       FDR.median.sorted <-  signif(FDR.median.sorted, digits=5)
       glob.p.vals.sorted <- signif(glob.p.vals.sorted, digits=5)

       report <- data.frame(cbind(gs.names, size.G, all.gs.descs, Obs.ES, Obs.ES.norm, p.vals[,1], FDR.mean.sorted, p.vals[,2], tag.frac, gene.frac, signal.strength, FDR.median.sorted, glob.p.vals.sorted))
       names(report) <- c("GS", "SIZE", "SOURCE", "ES", "NES", "NOM p-val", "FDR q-val", "FWER p-val", "Tag \\%", "Gene \\%", "Signal", "FDR (median)", "glob.p.val")
#       print(report)
       report2 <- report
       report.index2 <- order(Obs.ES.norm, decreasing= TRUE)
       for (i in 1:Ng) {
           report2[i,] <- report[report.index2[i],]
       }   
       report3 <- report
       report.index3 <- order(Obs.ES.norm, decreasing= FALSE)
       for (i in 1:Ng) {
           report3[i,] <- report[report.index3[i],]
       }   
       phen1.rows <- length(Obs.ES.norm[Obs.ES.norm >= 0])
       phen2.rows <- length(Obs.ES.norm[Obs.ES.norm < 0])
       report.phen1 <- report2[1:phen1.rows,]
       report.phen2 <- report3[1:phen2.rows,]

if (output.directory != "")  {
       if (phen1.rows > 0) {
          filename <- paste(output.directory, doc.string, ".SUMMARY.RESULTS.REPORT.", phen1,".txt", sep="", collapse="")
          write.table(report.phen1, file = filename, quote=FALSE, row.names=FALSE, sep = "\t")
       }
       if (phen2.rows > 0) {
          filename <- paste(output.directory, doc.string, ".SUMMARY.RESULTS.REPORT.", phen2,".txt", sep="", collapse="")
          write.table(report.phen2, file = filename, quote=FALSE, row.names=FALSE, sep = "\t")
       }
}

# Global plots

if (output.directory != "")  {
      if (non.interactive.run == FALSE) {
           if (.Platform$OS.type == "windows") {
              glob.filename <- paste(output.directory, doc.string, ".global.plots", sep="", collapse="")
              #windows(width = 10, height = 10)
              dev.new(width = 14, height = 6)
           } else if (.Platform$OS.type == "unix") {
               glob.filename <- paste(output.directory, doc.string, ".global.plots.pdf", sep="", collapse="")
               pdf(file=glob.filename, height = 10, width = 10)
           }
      } else {
           if (.Platform$OS.type == "unix") {
              glob.filename <- paste(output.directory, doc.string, ".global.plots.pdf", sep="", collapse="")
              pdf(file=glob.filename, height = 10, width = 10)
           } else if (.Platform$OS.type == "windows") {
              glob.filename <- paste(output.directory, doc.string, ".global.plots.pdf", sep="", collapse="")
              pdf(file=glob.filename, height = 10, width = 10)
           }
      }
}

      nf <- layout(matrix(c(1,2,3,4), 2, 2, byrow= TRUE), c(1,1), c(1,1), TRUE)

# plot S2N correlation profile

     location <- 1:N
     max.corr <- max(obs.s2n)
     min.corr <- min(obs.s2n)

     x <- plot(location, obs.s2n, ylab = "Signal to Noise Ratio (S2N)", xlab = "Gene List Location", main = "Gene List Correlation (S2N) Profile", type = "l", lwd = 2, cex = 0.9, col = 1)            
     for (i in seq(1, N, 20)) {
       lines(c(i, i), c(0, obs.s2n[i]), lwd = 3, cex = 0.9, col = colors()[12]) # shading of correlation plot
     }
     x <- points(location, obs.s2n, type = "l", lwd = 2, cex = 0.9, col = 1)            
     lines(c(1, N), c(0, 0), lwd = 2, lty = 1, cex = 0.9, col = 1) # zero correlation horizontal line
     temp <- order(abs(obs.s2n), decreasing= TRUE)
     arg.correl <- temp[N]
     lines(c(arg.correl, arg.correl), c(min.corr, 0.7*max.corr), lwd = 2, lty = 3, cex = 0.9, col = 1) # zero correlation vertical line

     area.bias <- signif(100*(sum(obs.s2n[1:arg.correl]) + sum(obs.s2n[arg.correl:N]))/sum(abs(obs.s2n[1:N])), digits=3)
     area.phen <- ifelse(area.bias >= 0, phen1, phen2)
     delta.string <- paste("Corr. Area Bias to \"", area.phen, "\" =", abs(area.bias), "\\%", sep="", collapse="")
     zero.crossing.string <- paste("Zero Crossing at location ", arg.correl, " (",  signif(100*arg.correl/N, digits=3), " \\%)")
     leg.txt <- c(delta.string, zero.crossing.string)
     legend(x=N/10, y=max.corr, bty="n", bg = "white", legend=leg.txt, cex = 0.9)

     leg.txt <- paste("\"", phen1, "\" ", sep="", collapse="")
     text(x=1, y=-0.05*max.corr, adj = c(0, 1), labels=leg.txt, cex = 0.9)

     leg.txt <- paste("\"", phen2, "\" ", sep="", collapse="")
     text(x=N, y=0.05*max.corr, adj = c(1, 0), labels=leg.txt, cex = 0.9)

  if (Ng > 1) { # make these plots only if there are multiple gene sets.

    # compute plots of actual (weighted) null and observed

      phi.densities.pos <- matrix(0, nrow=512, ncol=nperm)
      phi.densities.neg <- matrix(0, nrow=512, ncol=nperm)
      obs.phi.densities.pos <- matrix(0, nrow=512, ncol=nperm)
      obs.phi.densities.neg <- matrix(0, nrow=512, ncol=nperm)
      phi.density.mean.pos <- vector(length=512, mode = "numeric")
      phi.density.mean.neg <- vector(length=512, mode = "numeric")
      obs.phi.density.mean.pos <- vector(length=512, mode = "numeric")
      obs.phi.density.mean.neg <- vector(length=512, mode = "numeric")
      phi.density.median.pos <- vector(length=512, mode = "numeric")
      phi.density.median.neg <- vector(length=512, mode = "numeric")
      obs.phi.density.median.pos <- vector(length=512, mode = "numeric")
      obs.phi.density.median.neg <- vector(length=512, mode = "numeric")
      x.coor.pos <-  vector(length=512, mode = "numeric")
      x.coor.neg <-  vector(length=512, mode = "numeric")

      for (i in 1:nperm) {
         pos.phi <- phi.norm[phi.norm[, i] >= 0, i]
         if (length(pos.phi) > 2) {
            temp <- density(pos.phi, adjust=adjust.param, n = 512, from=0, to=3.5)
         } else {
            temp <- list(x = 3.5*(seq(1, 512) - 1)/512, y = rep(0.001, 512))
         }
         phi.densities.pos[, i] <- temp$y
         norm.factor <- sum(phi.densities.pos[, i])
         phi.densities.pos[, i] <- phi.densities.pos[, i]/norm.factor
         if (i == 1) {
            x.coor.pos <- temp$x
         }

         neg.phi <- phi.norm[phi.norm[, i] < 0, i]
         if (length(neg.phi) > 2) {
            temp <- density(neg.phi, adjust=adjust.param, n = 512, from=-3.5, to=0)
         } else {
            temp <- list(x = 3.5*(seq(1, 512) - 1)/512, y = rep(0.001, 512))
         }
         phi.densities.neg[, i] <- temp$y
         norm.factor <- sum(phi.densities.neg[, i])
         phi.densities.neg[, i] <- phi.densities.neg[, i]/norm.factor
         if (i == 1) {
            x.coor.neg <- temp$x
         }
         pos.phi <- obs.phi.norm[obs.phi.norm[, i] >= 0, i]
         if (length(pos.phi) > 2) {
            temp <- density(pos.phi, adjust=adjust.param, n = 512, from=0, to=3.5)
         } else {
            temp <- list(x = 3.5*(seq(1, 512) - 1)/512, y = rep(0.001, 512))
         }
         obs.phi.densities.pos[, i] <- temp$y
         norm.factor <- sum(obs.phi.densities.pos[, i])
         obs.phi.densities.pos[, i] <- obs.phi.densities.pos[, i]/norm.factor

         neg.phi <- obs.phi.norm[obs.phi.norm[, i] < 0, i]
         if (length(neg.phi)> 2) {  
            temp <- density(neg.phi, adjust=adjust.param, n = 512, from=-3.5, to=0)
         } else {
            temp <- list(x = 3.5*(seq(1, 512) - 1)/512, y = rep(0.001, 512))
         }
         obs.phi.densities.neg[, i] <- temp$y
         norm.factor <- sum(obs.phi.densities.neg[, i])
         obs.phi.densities.neg[, i] <- obs.phi.densities.neg[, i]/norm.factor
         
      }
      phi.density.mean.pos <- apply(phi.densities.pos, 1, mean)
      phi.density.mean.neg <- apply(phi.densities.neg, 1, mean)

      obs.phi.density.mean.pos <- apply(obs.phi.densities.pos, 1, mean)
      obs.phi.density.mean.neg <- apply(obs.phi.densities.neg, 1, mean)

      phi.density.median.pos <- apply(phi.densities.pos, 1, median)
      phi.density.median.neg <- apply(phi.densities.neg, 1, median)

      obs.phi.density.median.pos <- apply(obs.phi.densities.pos, 1, median)
      obs.phi.density.median.neg <- apply(obs.phi.densities.neg, 1, median)

      x <- c(x.coor.neg, x.coor.pos)
      x.plot.range <- range(x)
      y1 <- c(phi.density.mean.neg, phi.density.mean.pos)
      y2 <- c(obs.phi.density.mean.neg, obs.phi.density.mean.pos)
      y.plot.range <- c(-0.3*max(c(y1, y2)),  max(c(y1, y2)))

      print(c(y.plot.range, max(c(y1, y2)), max(y1), max(y2)))

      plot(x, y1, xlim = x.plot.range, ylim = 1.5*y.plot.range, type = "l", lwd = 2, col = 2, xlab = "NES", ylab = "P(NES)", main = "Global Observed and Null Densities (Area Normalized)")

     y1.point <- y1[seq(1, length(x), 2)]
     y2.point <- y2[seq(2, length(x), 2)]
     x1.point <- x[seq(1, length(x), 2)]
     x2.point <- x[seq(2, length(x), 2)]

#     for (i in 1:length(x1.point)) {
#       lines(c(x1.point[i], x1.point[i]), c(0, y1.point[i]), lwd = 3, cex = 0.9, col = colors()[555]) # shading 
#     }
#
#     for (i in 1:length(x2.point)) {
#       lines(c(x2.point[i], x2.point[i]), c(0, y2.point[i]), lwd = 3, cex = 0.9, col = colors()[29]) # shading 
#     }

      points(x, y1, type = "l", lwd = 2, col = colors()[555])
      points(x, y2, type = "l", lwd = 2, col = colors()[29])

      for (i in 1:Ng) {
         col <- ifelse(Obs.ES.norm[i] > 0, 2, 3) 
         lines(c(Obs.ES.norm[i], Obs.ES.norm[i]), c(-0.2*max(c(y1, y2)), 0), lwd = 1, lty = 1, col = 1)
      }
      leg.txt <- paste("Neg. ES: \"", phen2, " \" ", sep="", collapse="")
      text(x=x.plot.range[1], y=-0.25*max(c(y1, y2)), adj = c(0, 1), labels=leg.txt, cex = 0.9)
      leg.txt <- paste(" Pos. ES: \"", phen1, "\" ", sep="", collapse="")
      text(x=x.plot.range[2], y=-0.25*max(c(y1, y2)), adj = c(1, 1), labels=leg.txt, cex = 0.9)

      leg.txt <- c("Null Density", "Observed Density", "Observed NES values")
      c.vec <- c(colors()[555], colors()[29], 1)
      lty.vec <- c(1, 1, 1)
      lwd.vec <- c(2, 2, 2)
      legend(x=0, y=1.5*y.plot.range[2], bty="n", bg = "white", legend=leg.txt, lty = lty.vec, lwd = lwd.vec, col = c.vec, cex = 0.9)

      B <- A[obs.index,]
      if (N > 300) {
         C <- rbind(B[1:100,], rep(0, Ns), rep(0, Ns), B[(floor(N/2) - 50 + 1):(floor(N/2) + 50),], rep(0, Ns), rep(0, Ns), B[(N - 100 + 1):N,])
      } 
      rm(B)
      GSEA.HeatMapPlot(V = C, col.labels = class.labels, col.classes = class.phen, main = "Heat Map for Genes in Dataset")

# p-vals plot
      nom.p.vals <- p.vals[Obs.ES.index,1]
      FWER.p.vals <- p.vals[Obs.ES.index,2]
      plot.range <- 1.25*range(NES)
      plot(NES, FDR.mean, ylim = c(0, 1), xlim = plot.range, col = 1, bg = 1, type="p", pch = 22, cex = 0.75, xlab = "NES", main = "p-values vs. NES", ylab ="p-val/q-val")
      points(NES, nom.p.vals, type = "p", col = 2, bg = 2, pch = 22, cex = 0.75)
      points(NES, FWER.p.vals, type = "p", col = colors()[577], bg = colors()[577], pch = 22, cex = 0.75)
      leg.txt <- c("Nominal p-value", "FWER p-value", "FDR q-value")
      c.vec <- c(2, colors()[577], 1)
      pch.vec <- c(22, 22, 22)
      legend(x=-0.5, y=0.5, bty="n", bg = "white", legend=leg.txt, pch = pch.vec, col = c.vec, pt.bg = c.vec, cex = 0.9)
      lines(c(min(NES), max(NES)), c(nom.p.val.threshold, nom.p.val.threshold), lwd = 1, lty = 2, col = 2) 
      lines(c(min(NES), max(NES)), c(fwer.p.val.threshold, fwer.p.val.threshold), lwd = 1, lty = 2, col = colors()[577]) 
      lines(c(min(NES), max(NES)), c(fdr.q.val.threshold, fdr.q.val.threshold), lwd = 1, lty = 2, col = 1) 

      if (non.interactive.run == FALSE) {  
           if (.Platform$OS.type == "windows") {
               savePlot(filename = glob.filename, type ="jpeg", device = dev.cur())
           } else if (.Platform$OS.type == "unix") {
               dev.off()
           }
      } else {
           dev.off()
      }

  } # if Ng > 1

#----------------------------------------------------------------------------
# Produce report for each gene set passing the nominal, FWER or FDR test or the top topgs in each side

      if (topgs > floor(Ng/2)) {
         topgs <- floor(Ng/2)
      }

      for (i in 1:Ng) {
          if ((p.vals[i, 1] <= nom.p.val.threshold) ||
              (p.vals[i, 2] <= fwer.p.val.threshold) ||
              (FDR.mean.sorted[i] <= fdr.q.val.threshold) || 
              (is.element(i, c(Obs.ES.index[1:topgs], Obs.ES.index[(Ng - topgs + 1): Ng])))) {

#  produce report per gene set

            kk <- 1
            gene.number <- vector(length = size.G[i], mode = "character")
            gene.names <- vector(length = size.G[i], mode = "character")
            gene.symbols <- vector(length = size.G[i], mode = "character")
            gene.descs <- vector(length = size.G[i], mode = "character")
            gene.list.loc <- vector(length = size.G[i], mode = "numeric")
            core.enrichment <- vector(length = size.G[i], mode = "character")
            gene.s2n <- vector(length = size.G[i], mode = "numeric")
            gene.RES <- vector(length = size.G[i], mode = "numeric")
            rank.list <- seq(1, N)

            if (Obs.ES[i] >= 0) {
              set.k <- seq(1, N, 1)
              phen.tag <- phen1
              loc <- match(i, Obs.ES.index)
            } else {
              set.k <- seq(N, 1, -1)
              phen.tag <- phen2
              loc <- Ng - match(i, Obs.ES.index) + 1
            }

            for (k in set.k) {
               if (Obs.indicator[i, k] == 1) {
                  gene.number[kk] <- kk
                  gene.names[kk] <- obs.gene.labels[k]
                  gene.symbols[kk] <- substr(obs.gene.symbols[k], 1, 15)
                  gene.descs[kk] <- substr(obs.gene.descs[k], 1, 40)
                  gene.list.loc[kk] <- k
                  gene.s2n[kk] <- signif(obs.s2n[k], digits=3)
                  gene.RES[kk] <- signif(Obs.RES[i, k], digits = 3)
                  if (Obs.ES[i] >= 0) {
                     core.enrichment[kk] <- ifelse(gene.list.loc[kk] <= Obs.arg.ES[i], "YES", "NO")
                  } else {
                     core.enrichment[kk] <- ifelse(gene.list.loc[kk] > Obs.arg.ES[i], "YES", "NO")
                  }
                  kk <- kk + 1
               }
            }

       gene.report <- data.frame(cbind(gene.number, gene.names, gene.symbols, gene.descs, gene.list.loc, gene.s2n, gene.RES, core.enrichment))
       names(gene.report) <- c("#", "GENE", "SYMBOL", "DESC", "LIST LOC", "S2N", "RES", "CORE_ENRICHMENT")

#       print(gene.report)

if (output.directory != "")  {

       filename <- paste(output.directory, doc.string, ".", gs.names[i], ".report.", phen.tag, ".", loc, ".txt", sep="", collapse="")
       write.table(gene.report, file = filename, quote= FALSE, row.names= FALSE, sep = "\t")

       if (non.interactive.run == FALSE) {
           if (.Platform$OS.type == "windows") {
               gs.filename <- paste(output.directory, doc.string, ".", gs.names[i], ".plot.", phen.tag, ".", loc, sep="", collapse="")
               #windows(width = 14, height = 6)
               dev.new(width = 14, height = 6)
               #quartz(width = 14, height = 6)
           } else if (.Platform$OS.type == "unix") {
               gs.filename <- paste(output.directory, doc.string, ".", gs.names[i], ".plot.", phen.tag, ".", loc, ".pdf", sep="", collapse="")
               pdf(file=gs.filename, height = 6, width = 14)
           }
       } else {
           if (.Platform$OS.type == "unix") {
              gs.filename <- paste(output.directory, doc.string, ".", gs.names[i], ".plot.", phen.tag, ".", loc, ".pdf", sep="", collapse="")
              pdf(file=gs.filename, height = 6, width = 14)
           } else if (.Platform$OS.type == "windows") {
              gs.filename <- paste(output.directory, doc.string, ".", gs.names[i], ".plot.", phen.tag, ".", loc, ".pdf", sep="", collapse="")
              pdf(file=gs.filename, height = 6, width = 14)
           }
       }

}

            nf <- layout(matrix(c(1,2,3), 1, 3, byrow= TRUE), 1, c(1, 1, 1), TRUE)
            ind <- 1:N
            min.RES <- min(Obs.RES[i,])
            max.RES <- max(Obs.RES[i,])
            if (max.RES < 0.3) max.RES <- 0.3
            if (min.RES > -0.3) min.RES <- -0.3
            delta <- (max.RES - min.RES)*0.50
            min.plot <- min.RES - 2*delta
            max.plot <- max.RES
            max.corr <- max(obs.s2n)
            min.corr <- min(obs.s2n)
            Obs.correl.vector.norm <- (obs.s2n - min.corr)/(max.corr - min.corr)*1.25*delta + min.plot
            zero.corr.line <- (- min.corr/(max.corr - min.corr))*1.25*delta + min.plot
            col <- ifelse(Obs.ES[i] > 0, 2, 4)

            # Running enrichment plot
    
            sub.string <- paste("Number of genes: ", N, " (in list), ", size.G[i], " (in gene set)", sep = "", collapse="")
            
            main.string <- paste("Gene Set ", i, ":", gs.names[i])
            plot(ind, Obs.RES[i,], main = main.string, sub = sub.string, xlab = "Gene List Index", ylab = "Running Enrichment Score (RES)", xlim=c(1, N), ylim=c(min.plot, max.plot), type = "l", lwd = 2, cex = 1, col = col)
            for (j in seq(1, N, 20)) {
                lines(c(j, j), c(zero.corr.line, Obs.correl.vector.norm[j]), lwd = 1, cex = 1, col = colors()[12]) # shading of correlation plot
            }
            lines(c(1, N), c(0, 0), lwd = 1, lty = 2, cex = 1, col = 1) # zero RES line
            lines(c(Obs.arg.ES[i], Obs.arg.ES[i]), c(min.plot, max.plot), lwd = 1, lty = 3, cex = 1, col = col) # max enrichment vertical line
            for (j in 1:N) {
               if (Obs.indicator[i, j] == 1) {
                  lines(c(j, j), c(min.plot + 1.25*delta, min.plot + 1.75*delta), lwd = 1, lty = 1, cex = 1, col = 1)  # enrichment tags
               }
            }
            lines(ind, Obs.correl.vector.norm, type = "l", lwd = 1, cex = 1, col = 1)
            lines(c(1, N), c(zero.corr.line, zero.corr.line), lwd = 1, lty = 1, cex = 1, col = 1) # zero correlation horizontal line
            temp <- order(abs(obs.s2n), decreasing= TRUE)
            arg.correl <- temp[N]
            lines(c(arg.correl, arg.correl), c(min.plot, max.plot), lwd = 1, lty = 3, cex = 1, col = 3) # zero crossing correlation vertical line

            leg.txt <- paste("\"", phen1, "\" ", sep="", collapse="")
            text(x=1, y=min.plot, adj = c(0, 0), labels=leg.txt, cex = 1.0)

            leg.txt <- paste("\"", phen2, "\" ", sep="", collapse="")
            text(x=N, y=min.plot, adj = c(1, 0), labels=leg.txt, cex = 1.0)

            adjx <- ifelse(Obs.ES[i] > 0, 0, 1)
           
            leg.txt <- paste("Peak at ", Obs.arg.ES[i], sep="", collapse="")
            text(x=Obs.arg.ES[i], y=min.plot + 1.8*delta, adj = c(adjx, 0), labels=leg.txt, cex = 1.0)

            leg.txt <- paste("Zero crossing at ", arg.correl, sep="", collapse="")
            text(x=arg.correl, y=min.plot + 1.95*delta, adj = c(adjx, 0), labels=leg.txt, cex = 1.0)

            # nominal p-val histogram

            sub.string <- paste("ES =", signif(Obs.ES[i], digits = 3), " NES =", signif(Obs.ES.norm[i], digits=3), "Nom. p-val=", signif(p.vals[i, 1], digits = 3),"FWER=", signif(p.vals[i, 2], digits = 3), "FDR=", signif(FDR.mean.sorted[i], digits = 3))
            temp <- density(phi[i,], adjust=adjust.param)
            x.plot.range <- range(temp$x)
            y.plot.range <- c(-0.125*max(temp$y), 1.5*max(temp$y))
            plot(temp$x, temp$y, type = "l", sub = sub.string, xlim = x.plot.range, ylim = y.plot.range, lwd = 2, col = 2, main = "Gene Set Null Distribution", xlab = "ES", ylab="P(ES)")
            x.loc <- which.min(abs(temp$x - Obs.ES[i]))
            lines(c(Obs.ES[i], Obs.ES[i]), c(0, temp$y[x.loc]), lwd = 2, lty = 1, cex = 1, col = 1)
            lines(x.plot.range, c(0, 0), lwd = 1, lty = 1, cex = 1, col = 1)

            leg.txt <- c("Gene Set Null Density", "Observed Gene Set ES value")
            c.vec <- c(2, 1)
            lty.vec <- c(1, 1)
            lwd.vec <- c(2, 2)
            legend(x=-0.2, y=y.plot.range[2], bty="n", bg = "white", legend=leg.txt, lty = lty.vec, lwd = lwd.vec, col = c.vec, cex = 1.0)

            leg.txt <- paste("Neg. ES \"", phen2, "\" ", sep="", collapse="")
            text(x=x.plot.range[1], y=-0.1*max(temp$y), adj = c(0, 0), labels=leg.txt, cex = 1.0)
            leg.txt <- paste(" Pos. ES: \"", phen1, "\" ", sep="", collapse="")
            text(x=x.plot.range[2], y=-0.1*max(temp$y), adj = c(1, 0), labels=leg.txt, cex = 1.0)

            # create pinkogram for each gene set

            kk <- 1

            pinko <- matrix(0, nrow = size.G[i], ncol = cols)
            pinko.gene.names <- vector(length = size.G[i], mode = "character")
            for (k in 1:rows) {
               if (Obs.indicator[i, k] == 1) {
                  pinko[kk,] <- A[obs.index[k],]
                  pinko.gene.names[kk] <- obs.gene.symbols[k]
                  kk <- kk + 1
               }
            }
            GSEA.HeatMapPlot(V = pinko, row.names = pinko.gene.names, col.labels = class.labels, col.classes = class.phen, col.names = sample.names, main =" Heat Map for Genes in Gene Set", xlab=" ", ylab=" ")

      if (non.interactive.run == FALSE) {  
           if (.Platform$OS.type == "windows") {
               savePlot(filename = gs.filename, type ="jpeg", device = dev.cur())
           } else if (.Platform$OS.type == "unix") {
               dev.off()
           }
      } else {
           dev.off()
      }

        } # if p.vals thres

      } # loop over gene sets


  return(list(report1 = report.phen1, report2 = report.phen2))

}  # end of definition of GSEA.analysis


GSEA.write.gct <- function (gct, filename) 
{
    f <- file(filename, "w")
    cat("#1.2", "\n", file = f, append = TRUE, sep = "")
    cat(dim(gct)[1], "\t", dim(gct)[2], "\n", file = f, append = TRUE, sep = "")
    cat("Name", "\t", file = f, append = TRUE, sep = "")
    cat("Description", file = f, append = TRUE, sep = "")
    names <- names(gct)
    cat("\t", names[1], file = f, append = TRUE, sep = "")
    for (j in 2:length(names)) {
        cat("\t", names[j], file = f, append = TRUE, sep = "")
    }
    cat("\n", file = f, append = TRUE, sep = "\t")
    oldWarn <- options(warn = -1)
    m <- matrix(nrow = dim(gct)[1], ncol = dim(gct)[2] +  2)
    m[, 1] <- row.names(gct)
    m[, 2] <- row.names(gct)
    index <- 3
    for (i in 1:dim(gct)[2]) {
        m[, index] <- gct[, i]
        index <- index + 1
    }
    write.table(m, file = f, append = TRUE, quote = FALSE, sep = "\t", eol = "\n", col.names = FALSE, row.names = FALSE)
    close(f)
    options(warn = 0)
    return(gct)
}


GSEA.ConsPlot <- function(V, col.names, main = " ", sub = " ", xlab=" ", ylab=" ") {

# Plots a heatmap plot of a consensus matrix

     cols <- length(V[1,])
     B <- matrix(0, nrow=cols, ncol=cols)
     max.val <- max(V)
     min.val <- min(V)
     for (i in 1:cols) {
         for (j in 1:cols) {
             k <- cols - i + 1
	     B[k, j] <-  max.val - V[i, j] + min.val
          }
     }

 

#     col.map <- c(rainbow(100, s = 1.0, v = 0.75, start = 0.0, end = 0.75), "#BBBBBB", "#333333", "#FFFFFF")
     col.map <- rev(c("#0000FF", "#4040FF", "#7070FF", "#8888FF", "#A9A9FF", "#D5D5FF", "#EEE5EE", "#FFAADA", "#FF9DB0", "#FF7080", "#FF5A5A", "#FF4040", "#FF0D1D"))

#     max.size <- max(nchar(col.names))
     par(mar = c(5, 15, 15, 5))
     image(1:cols, 1:cols, t(B), col = col.map, axes=FALSE, main=main, sub=sub, xlab= xlab, ylab=ylab)

     for (i in 1:cols) {
         col.names[i]  <- substr(col.names[i], 1, 25)
     }
     col.names2 <- rev(col.names)

     size.col.char <- ifelse(cols < 15, 1, sqrt(15/cols))

     axis(2, at=1:cols, labels=col.names2, adj= 0.5, tick=FALSE, las = 1, cex.axis=size.col.char, font.axis=1, line=-1)
     axis(3, at=1:cols, labels=col.names, adj= 1, tick=FALSE, las = 3, cex.axis=size.col.char, font.axis=1, line=-1)

     return()
}

GSEA.HeatMapPlot2 <- function(V, row.names = "NA", col.names = "NA", main = " ", sub = " ", xlab=" ", ylab=" ", color.map = "default") {
#
# Plots a heatmap of a matrix

       n.rows <- length(V[,1])
       n.cols <- length(V[1,])

       if (color.map == "default") {
         color.map <- rev(rainbow(100, s = 1.0, v = 0.75, start = 0.0, end = 0.75))
       }

        heatm <- matrix(0, nrow = n.rows, ncol = n.cols)
        heatm[1:n.rows,] <- V[seq(n.rows, 1, -1),]

        par(mar = c(7, 15, 5, 5))
        image(1:n.cols, 1:n.rows, t(heatm), col=color.map, axes=FALSE, main=main, sub = sub, xlab= xlab, ylab=ylab)

        if (length(row.names) > 1) {
            size.row.char <- ifelse(n.rows < 15, 1, sqrt(15/n.rows))
            size.col.char <- ifelse(n.cols < 15, 1, sqrt(10/n.cols))
#            size.col.char <- ifelse(n.cols < 2.5, 1, sqrt(2.5/n.cols))
            for (i in 1:n.rows) {
               row.names[i] <- substr(row.names[i], 1, 40)
            }
            row.names <- row.names[seq(n.rows, 1, -1)]
            axis(2, at=1:n.rows, labels=row.names, adj= 0.5, tick=FALSE, las = 1, cex.axis=size.row.char, font.axis=1, line=-1)
        }

        if (length(col.names) > 1) {
           axis(1, at=1:n.cols, labels=col.names, tick=FALSE, las = 3, cex.axis=size.col.char, font.axis=2, line=-1)
        }

	return()
}


GSEA.Analyze.Sets <- function(
   directory,
   topgs = "",
   non.interactive.run = FALSE,
   height = 12,
   width = 17) {

   file.list <- list.files(directory)
   files <- file.list[regexpr(pattern = "\\.report\\.", file.list) > 1]
   max.sets <- length(files)

   set.table <- matrix(nrow = max.sets, ncol = 5)

   for (i in 1:max.sets) {
      temp1 <-  strsplit(files[i], split="\\.report\\.")
      temp2 <-  strsplit(temp1[[1]][1], split="\\.")
      s <- length(temp2[[1]])
      prefix.name <- paste(temp2[[1]][1:(s-1)], sep="", collapse="")
      set.name <- temp2[[1]][s]
      temp3 <-  strsplit(temp1[[1]][2], split="\\.")
      phenotype <- temp3[[1]][1]
      seq.number <- temp3[[1]][2]
      dataset <- paste(temp2[[1]][1:(s-1)], sep="", collapse="\\.")

      set.table[i, 1] <- files[i]

      set.table[i, 3] <- phenotype
      set.table[i, 4] <- as.numeric(seq.number)
      set.table[i, 5] <- dataset

#      set.table[i, 2] <- paste(set.name, dataset, sep ="", collapse="")
      set.table[i, 2] <- substr(set.name, 1, 20) 
   }

   print(c("set name=", prefix.name))
   doc.string <- prefix.name

   set.table <- noquote(set.table)
   phen.order <- order(set.table[, 3], decreasing = TRUE)
   set.table <- set.table[phen.order,]
   phen1 <- names(table(set.table[,3]))[1]
   phen2 <- names(table(set.table[,3]))[2]
   set.table.phen1 <- set.table[set.table[,3] == phen1,]
   set.table.phen2 <- set.table[set.table[,3] == phen2,]
 
   seq.order <- order(as.numeric(set.table.phen1[, 4]), decreasing = FALSE)
   set.table.phen1 <- set.table.phen1[seq.order,]
   seq.order <- order(as.numeric(set.table.phen2[, 4]), decreasing = FALSE)
   set.table.phen2 <- set.table.phen2[seq.order,]

#   max.sets.phen1 <- length(set.table.phen1[,1])
#   max.sets.phen2 <- length(set.table.phen2[,1])

   if (topgs == "") {
      max.sets.phen1 <- length(set.table.phen1[,1])
      max.sets.phen2 <- length(set.table.phen2[,1])
   } else {
      max.sets.phen1 <- ifelse(topgs > length(set.table.phen1[,1]), length(set.table.phen1[,1]), topgs) 
      max.sets.phen2 <- ifelse(topgs > length(set.table.phen2[,1]), length(set.table.phen2[,1]), topgs)
   }

   # Analysis for phen1

   leading.lists <- NULL
   for (i in 1:max.sets.phen1) {
      inputfile <- paste(directory, set.table.phen1[i, 1], sep="", collapse="")
      gene.set <- read.table(file=inputfile, sep="\t", header= TRUE, comment.char="", as.is= TRUE)
      leading.set <- as.vector(gene.set[gene.set[,"CORE_ENRICHMENT"] == "YES", "SYMBOL"])
      leading.lists <- c(leading.lists, list(leading.set))
      if (i == 1) {
         all.leading.genes <- leading.set 
      } else{
         all.leading.genes <- union(all.leading.genes, leading.set)
      }
   }
   max.genes <- length(all.leading.genes)
   M <- matrix(0, nrow=max.sets.phen1, ncol=max.genes)
   for (i in 1:max.sets.phen1) {
      M[i,] <- sign(match(all.leading.genes, as.vector(leading.lists[[i]]), nomatch=0))   # notice that the sign is 0 (no tag) or 1 (tag) 
   }

   Inter <- matrix(0, nrow=max.sets.phen1, ncol=max.sets.phen1)
   for (i in 1:max.sets.phen1) {
      for (j in 1:max.sets.phen1) {
         Inter[i, j] <- length(intersect(leading.lists[[i]], leading.lists[[j]]))/length(union(leading.lists[[i]], leading.lists[[j]]))
      }
   }

   Itable <- data.frame(Inter)
   names(Itable) <- set.table.phen1[1:max.sets.phen1, 2]
   row.names(Itable) <- set.table.phen1[1:max.sets.phen1, 2]

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen1, sep="", collapse="")
           #windows(height = width, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        }
   }

   GSEA.ConsPlot(Itable, col.names = set.table.phen1[1:max.sets.phen1, 2], main = " ", sub=paste("Leading Subsets Overlap ", doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ")

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }

   # Save leading subsets in a GCT file

   D.phen1 <- data.frame(M)
   names(D.phen1) <- all.leading.genes
   row.names(D.phen1) <- set.table.phen1[1:max.sets.phen1, 2]
   output <- paste(directory, doc.string, ".leading.genes.", phen1, ".gct", sep="")
   GSEA.write.gct(D.phen1, filename=output)

   # Save leading subsets as a single gene set in a .gmt file

   row.header <- paste(doc.string, ".all.leading.genes.", phen1, sep="")
   output.line <- paste(all.leading.genes, sep="\t", collapse="\t")
   output.line <- paste(row.header, row.header, output.line, sep="\t", collapse="")
   output <- paste(directory, doc.string, ".all.leading.genes.", phen1, ".gmt", sep="")
   write(noquote(output.line), file = output, ncolumns = length(output.line))

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen1, sep="", collapse="")
           #windows(height = height, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   }
   cmap <-  c("#AAAAFF")
   #cmap <-  c("#AAAAFF", "#111166")
   GSEA.HeatMapPlot2(V = data.matrix(D.phen1), row.names = row.names(D.phen1), col.names = names(D.phen1), main = "Leading Subsets Assignment", sub = paste(doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ", color.map = cmap) 

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }

   DT1.phen1 <- data.matrix(t(D.phen1))
   DT2.phen1 <- data.frame(DT1.phen1)
   names(DT2.phen1) <- set.table.phen1[1:max.sets.phen1, 2]
   row.names(DT2.phen1) <- all.leading.genes
#   GSEA.write.gct(DT2.phen1, filename=outputfile2.phen1)

   # Analysis for phen2

   leading.lists <- NULL
   for (i in 1:max.sets.phen2) {
      inputfile <- paste(directory, set.table.phen2[i, 1], sep="", collapse="")
      gene.set <- read.table(file=inputfile, sep="\t", header= TRUE, comment.char="", as.is= TRUE)
      leading.set <- as.vector(gene.set[gene.set[,"CORE_ENRICHMENT"] == "YES", "SYMBOL"])
      leading.lists <- c(leading.lists, list(leading.set))
      if (i == 1) {
         all.leading.genes <- leading.set 
      } else{
         all.leading.genes <- union(all.leading.genes, leading.set)
      }
   }
   max.genes <- length(all.leading.genes)
   M <- matrix(0, nrow=max.sets.phen2, ncol=max.genes)
   for (i in 1:max.sets.phen2) {
      M[i,] <- sign(match(all.leading.genes, as.vector(leading.lists[[i]]), nomatch=0))   # notice that the sign is 0 (no tag) or 1 (tag) 
   }

   Inter <- matrix(0, nrow=max.sets.phen2, ncol=max.sets.phen2)
   for (i in 1:max.sets.phen2) {
     for (j in 1:max.sets.phen2) {
        Inter[i, j] <- length(intersect(leading.lists[[i]], leading.lists[[j]]))/length(union(leading.lists[[i]], leading.lists[[j]]))
     }
   }

   Itable <- data.frame(Inter)
   names(Itable) <- set.table.phen2[1:max.sets.phen2, 2]
   row.names(Itable) <- set.table.phen2[1:max.sets.phen2, 2]

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen2, sep="", collapse="")
           #windows(height = width, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.overlap.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = width, width = width)
        }
   }

   GSEA.ConsPlot(Itable, col.names = set.table.phen2[1:max.sets.phen2, 2], main = " ", sub=paste("Leading Subsets Overlap ", doc.string, " - ", phen2, sep=""), xlab=" ", ylab=" ")

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }

   # Save leading subsets in a GCT file

   D.phen2 <- data.frame(M)
   names(D.phen2) <- all.leading.genes
   row.names(D.phen2) <- set.table.phen2[1:max.sets.phen2, 2]
   output <- paste(directory, doc.string, ".leading.genes.", phen2, ".gct", sep="")
   GSEA.write.gct(D.phen2, filename=output)

   # Save primary subsets as a single gene set in a .gmt file

   row.header <- paste(doc.string, ".all.leading.genes.", phen2, sep="")
   output.line <- paste(all.leading.genes, sep="\t", collapse="\t")
   output.line <- paste(row.header, row.header, output.line, sep="\t", collapse="")
   output <- paste(directory, doc.string, ".all.leading.genes.", phen2, ".gmt", sep="")
   write(noquote(output.line), file = output, ncolumns = length(output.line))

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen2, sep="", collapse="")
           #windows(height = height, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   }
   cmap <-  c("#AAAAFF")
   #cmap <-  c("#AAAAFF", "#111166")
   GSEA.HeatMapPlot2(V = data.matrix(D.phen2), row.names = row.names(D.phen2), col.names = names(D.phen2), main = "Leading Subsets Assignment", sub = paste(doc.string, " - ", phen2, sep=""), xlab=" ", ylab=" ", color.map = cmap) 

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }

   DT1.phen2 <- data.matrix(t(D.phen2))
   DT2.phen2 <- data.frame(DT1.phen2)
   names(DT2.phen2) <- set.table.phen2[1:max.sets.phen2, 2]
   row.names(DT2.phen2) <- all.leading.genes
#   GSEA.write.gct(DT2.phen2, filename=outputfile2.phen2)

   # Resort columns and rows for phen1

   A <- data.matrix(D.phen1)
   A.row.names <- row.names(D.phen1)
   A.names <- names(D.phen1)

   # Max.genes

#   init <- 1
#   for (k in 1:max.sets.phen1) { 
#      end <- which.max(cumsum(A[k,]))
#      if (end - init > 1) {
#         B <- A[,init:end]
#         B.names <- A.names[init:end]
#        dist.matrix <- dist(t(B))
#         HC <- hclust(dist.matrix, method="average")
##         B <- B[,HC$order] + 0.2*(k %% 2)
#        B <- B[,HC$order] 
#         A[,init:end] <- B
#         A.names[init:end] <- B.names[HC$order]
#         init <- end + 1
#     }
#   }

#   windows(width=14, height=10)
#   GSEA.HeatMapPlot2(V = A, row.names = A.row.names, col.names = A.names, sub = "  ", main = paste("Primary Sets Assignment - ", doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ") 

   dist.matrix <- dist(t(A))
   HC <- hclust(dist.matrix, method="average")
   A <- A[, HC$order]
   A.names <- A.names[HC$order]

   dist.matrix <- dist(A)
   HC <- hclust(dist.matrix, method="average")
   A <- A[HC$order,]
   A.row.names <- A.row.names[HC$order]

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen1, sep="", collapse="")
           #windows(height = height, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen1, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   }


   cmap <-  c("#AAAAFF")
#   cmap <-  c("#AAAAFF", "#111166")
#   GSEA.HeatMapPlot2(V = A, row.names = A.row.names, col.names = A.names, main = "Leading Subsets Assignment (clustered)", sub = paste(doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ", color.map = cmap) 

   GSEA.HeatMapPlot2(V = t(A), row.names = A.names, col.names = A.row.names, main = "Leading Subsets Assignment (clustered)", sub = paste(doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ", color.map = cmap) 

    text.filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen1, ".txt", sep="", collapse="")
    line.list <- c("Gene", A.row.names)
    line.header <- paste(line.list, collapse="\t")
    line.length <- length(A.row.names) + 1
    write(line.header, file = text.filename, ncolumns = line.length)
    write.table(t(A), file=text.filename, append = TRUE, quote= FALSE, col.names= FALSE, row.names= TRUE, sep = "\t")

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }






# resort columns and rows for phen2

   A <- data.matrix(D.phen2)
   A.row.names <- row.names(D.phen2)
   A.names <- names(D.phen2)

   # Max.genes

#   init <- 1
#   for (k in 1:max.sets.phen2) { 
#      end <- which.max(cumsum(A[k,]))
#      if (end - init > 1) {
#         B <- A[,init:end]
#         B.names <- A.names[init:end]
#         dist.matrix <- dist(t(B))
#         HC <- hclust(dist.matrix, method="average")
##         B <- B[,HC$order] + 0.2*(k %% 2)
#        B <- B[,HC$order] 
#         A[,init:end] <- B
#         A.names[init:end] <- B.names[HC$order]
#         init <- end + 1
#     }
#   }

#  windows(width=14, height=10)
#  GESA.HeatMapPlot2(V = A, row.names = A.row.names, col.names = A.names, sub = "  ", main = paste("Primary Sets Assignment - ", doc.string, " - ", phen2, sep=""), xlab=" ", ylab=" ") 

   dist.matrix <- dist(t(A))
   HC <- hclust(dist.matrix, method="average")
   A <- A[, HC$order]
   A.names <- A.names[HC$order]

   dist.matrix <- dist(A)
   HC <- hclust(dist.matrix, method="average")
   A <- A[HC$order,]
   A.row.names <- A.row.names[HC$order]

   if (non.interactive.run == FALSE) {
        if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen2, sep="", collapse="")

           #windows(height = height, width = width)
           dev.new(width = 14, height = 6)
        } else if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   } else {
        if (.Platform$OS.type == "unix") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        } else if (.Platform$OS.type == "windows") {
           filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen2, ".pdf", sep="", collapse="")
           pdf(file=filename, height = height, width = width)
        }
   }
   cmap <-  c("#AAAAFF")
#   cmap <-  c("#AAAAFF", "#111166")

#   GSEA.HeatMapPlot2(V = A, row.names = A.row.names, col.names = A.names, main = "Leading Subsets Assignment (clustered)", sub = paste(doc.string, " - ", phen2, sep=""), xlab=" ", ylab=" ", color.map = cmap) 
   GSEA.HeatMapPlot2(V = t(A), row.names =A.names , col.names = A.row.names, main = "Leading Subsets Assignment (clustered)", sub = paste(doc.string, " - ", phen2, sep=""), xlab=" ", ylab=" ", color.map = cmap) 

    text.filename <- paste(directory, doc.string, ".leading.assignment.clustered.", phen2, ".txt", sep="", collapse="")
    line.list <- c("Gene", A.row.names)
    line.header <- paste(line.list, collapse="\t")
    line.length <- length(A.row.names) + 1
    write(line.header, file = text.filename, ncolumns = line.length)
    write.table(t(A), file=text.filename, append = TRUE, quote= FALSE, col.names= FALSE, row.names=TRUE, sep = "\t")

   if (non.interactive.run == FALSE) {  
        if (.Platform$OS.type == "windows") {
            savePlot(filename = filename, type ="jpeg", device = dev.cur())
        } else if (.Platform$OS.type == "unix") {
            dev.off()
        }
   } else {
        dev.off()
   }

}

#' Get  successively trees of genes list for multiple cases
#' @usage getGenesTree_MultipleCases(entryWidth = 10)
#' @param entryWidth 10
#' @export
#' @return plot tree
#' @examples 
#'  q <- readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' readRDS(paste(.libPaths(),"/canceR/data/brca_tcga73genes.rds", sep=""))
#' getGenesTree_MultipleCases(entryWidth = 10)
#' }
#' @import rpart
#' @import Formula
#' @import RUnit
getGenesTree_MultipleCases <- function(entryWidth = 10){
    
             testCheckedCaseGenProf()
            
            ##Checking Genes list, Cases GeneProf
            if(length(ENV$curselectCases)==1||length(ENV$curselectGenProfs)==1){
                msgNoOneStudy = "Select more than one Case/Genetic Profile or use single Case function"
                tkmessageBox(message=msgNoOneStudy, icon="warning")
                stop(msgNoOneStudy)
                
            }
            
            Lchecked_Studies <- ENV$lchecked_Studies_forCases
            Lchecked_Cases <- length(ENV$curselectCases)
            Lchecked_GenProf <- length(ENV$curselectGenProfs)
            ###Starting function of dialoGenesTree
            d <- 0
            ProfDataAll<-0
            ProfData<-0
            LengthGenProfs<-0
            LengthCases<-0
            for (i in 1:length(ENV$CaseChoice)){
                Si <- ENV$checked_StudyIndex[i]
                progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies[Si], min = 0,
                                                          max = Lchecked_GenProf, width = 400)
                
                if(exists('ttGeneTree', envir = ENV)){
                    tkdestroy(ENV$ttGeneTree)
                    
                }
                
                
                LastLengthGenProfs <- LengthGenProfs
                LengthGenProfs <- LengthGenProfs + ENV$LGenProfs[i]+1
                LastLengthCases <- LengthCases
                LengthCases <- LengthCases + ENV$LCases[i]+1
                
                Sys.sleep(0.1)
                setTkProgressBar(progressBar_ProfilesData, i, label=paste( round(i/Lchecked_GenProf*100, 0),
                                                                           "% of Expression Set"))
                ##########
                ttGeneTree <- tktoplevel()
                #tkwm.geometry(ttGeneTree,"180x250")
                tktitle(ttGeneTree) <- paste(ENV$Studies[Si],": Classify genes by variable")
                
                ##Image Horizontal scale option
                textEntryHscale <- tclVar("2")
                textEntryWidget <- tkentry(ttGeneTree, width = paste(entryWidth),
                                           textvariable = textEntryHscale)
                
                txtHscale <- tklabel(ttGeneTree, text = "Horizontal Scale of the plot")
                tkgrid(txtHscale)
                tkgrid.configure(txtHscale, column=1, row=1, sticky="w")
                tkgrid(textEntryWidget)
                tkgrid.configure(textEntryWidget, column=1, row=1, sticky="ne")
                
                ##Image Vertical scale option
                textEntryVscale <- tclVar("1")
                textEntryWidgetV <- tkentry(ttGeneTree, width = paste(entryWidth),
                                            textvariable = textEntryVscale)
                
                txtVscale <- tklabel(ttGeneTree, text = "Vertical Scale of the plot")
                tkgrid(txtVscale)
                tkgrid.configure(txtVscale, column=1, row=2, sticky="w")
                tkgrid(textEntryWidgetV)
                tkgrid.configure(textEntryWidgetV, column=1, row=2, sticky="ne")
                
                
                
                ##Clinical Data list
                
                label1 <- tklabel(ttGeneTree, text= "Clinical Data")
                
                yscr1 <- tkscrollbar(ttGeneTree, repeatinterval=2,
                                     command=function(...)tkyview(tl1,...))
                xscr1 <- tkscrollbar(ttGeneTree, repeatinterval=2,orient="horizontal",
                                     command=function(...)tkxview(tl1,...))
                
                xscr1Info <- tkscrollbar(ttGeneTree, repeatinterval=2,orient="horizontal",
                                         command=function(...)tkxview(tl1info,...))
                
                
                tl1<-tklistbox(ttGeneTree,height=10, width= 40 ,selectmode="single",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
                tl1info<-tklistbox(ttGeneTree,height=1, width= 40,selectmode="single",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
                
                
                #getClinicData_SingleCase()
                Case<-ENV$CasesRefStudies[ENV$curselectCases[i]]
                ClinicalData<-getClinicalData(ENV$cgds,Case)
                
                
                loadVariable <- function()
                {  
                    curselectVariable <- as.numeric(tkcurselection(tl1))+1
                    lcurselectVariable <- length(curselectVariable)
                    
                    ENV$variable <-  names(ClinicalData)[curselectVariable]
                    tkdelete(tl1info,0,1)
                    tkinsert(tl1info,"end",ENV$variable)
                    
                }
                
                
                Variable.but <-tkbutton(ttGeneTree,text="select",command=loadVariable)
                
                tkgrid(label1,tl1,yscr1)
                tkgrid.configure(yscr1,rowspan=20, columnspan=2,sticky="nsw")
                tkgrid(xscr1)
                tkgrid.configure(xscr1,rowspan=2, column=1,sticky="we")
                tkgrid(Variable.but, tl1info, columnspan=1)
                tkgrid(xscr1Info)
                tkgrid.configure(xscr1Info,rowspan=4, column=1,sticky="we")
                
                print(paste("testing which Genenic Profile: ", ENV$curselectGenProfs[i],"<=", LengthGenProfs))
                print(paste("testing last Genenic Profile: ",ENV$curselectGenProfs[i],">",LastLengthGenProfs))
                if (ENV$curselectGenProfs[i] <= LengthGenProfs && ENV$curselectGenProfs[i]>LastLengthGenProfs){    
                    
                    GenProf<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[i]]
                    
                    ProfData<-getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                    
                    ##Convert data frame to numeric structure
#                     print("converting data frame of Profile data to numeric stucture...")
#                     
#                     for(i in 1:ncol(ProfData)){
#                         
#                         ProfData[,i] <- as.numeric(ProfData[,i])
#                     }
                    
                    ## for loop is faster than apply fonction
                    #rnames <- rownames(ProfData)
                    #ProfData <- as.data.frame(apply(ProfData,2 ,function(x) as.numeric(x)))
                    #rownames(ProfData) <- rnames
                    
                   #test if is there a clinical data
                    if(length(ClinicalData[1,])==0){
                        tkdestroy(ttGeneTree)
                        msgNoClinData=paste("No Clinical Data are Available for\n", CaseChoice)
                        tkmessageBox(message=msgNoClinData, title= CaseChoice, icon="info")
                        break
                    }
                    print('Case has Clinical Data...')
                    ## Select only Cases (rownames) that exist in ClinicalDataSub and ProfData
                    merge <- merge(ClinicalData, ProfData, by="row.names")
                    print("merging Samples from Profile and Clinical Data")
                    ClinicalData<- merge[,2:(length(ClinicalData)+1)]
                    
                    
                    ProfData<-merge[,!(merge %in% ClinicalData)]
                    
                    
                    for (j in 1:length(names(ClinicalData))){
                        
                        tkinsert(tl1,"end",names(ClinicalData)[j])
                    }
                    
                    Methods <- c("class","anova","poisson")
                    # Default selections for the two combo boxes
                    defaultMethod <- tclVar("class")
                    favMethod <- tclVar("class")
                    
                    comboBox <- ttkcombobox(ttGeneTree, values=Methods, textvariable=favMethod, state="readonly")                        
                    
                    text <- tklabel(ttGeneTree,text="Select Method:")
                    tkgrid(text, comboBox)  
                    
                    
                    d<-d+1
                    onOK <- function(){
                        if(exists("variable", envir = ENV)){
                            print(paste("d",d))
                            HorScale <- as.numeric(tclvalue(textEntryHscale))
                            VerScale <- as.numeric(tclvalue(textEntryVscale))
                            ENV$ProfData <- cbind(ClinicalData[,ENV$variable], ProfData[,-1])
                            
                            colnames(ENV$ProfData)[1] <- ENV$variable
                            frmla <- paste0(ENV$variable, "~.", sep="")
                            ENV$frmla <- as.formula(frmla)
                            print(ENV$frmla)
                            
                            
                            ##selected mathod
                            selectedMethod <- tclvalue(favMethod)
                            print(paste("Selected Method:", selectedMethod))
                            
                            plotCommand<- function(){
                                
                                
                                fit <- rpart::rpart(ENV$frmla, method=selectedMethod, data=ENV$ProfData)
                                plot(fit, uniform=TRUE, compress=TRUE,main= paste(ENV$StudyChoice[d],"\n ",ENV$GenProfChoice[d],"vs",ENV$variable ))
                                text(fit, use.n=TRUE, all=TRUE, cex=0.6, fancy=FALSE)
                                ##capture print(fit) for editing
                                summary <- capture.output(print(fit))
                                ## Edit summary fit
                                getTextWin(paste(summary,collapse="\n"))
                            }
                            plotModel(plotCommand, title=paste(ENV$checked_Studies[d],":",ENV$CaseChoice[d],"vs" ,ENV$variable, sep=""), vscale=VerScale, hscale=HorScale)
                            tkdestroy(ttGeneTree)
                        }else{
                            msgNoFrmla <- "Select one variable"
                            tkmessageBox(message= msgNoFrmla, icon="info")
                        }
                        
                    }
                    
                    
                    Ok.but <-tkbutton(ttGeneTree,text=" OK ",command=onOK)
                    tkgrid(Ok.but)
                    tkgrid.configure(Ok.but,rowspan=4, column=1,sticky="n")
                    
                    tkwait.window(ttGeneTree)
                    
                    
                } else {
                    tkdestroy(ttGeneTree)
                    close(progressBar_ProfilesData)
                    msgBadCheck <- paste("Choose only one Case/Genetic Profile by Study.")
                    tkmessageBox(message=msgBadCheck, icon="warning")
                }
                
                
                
                close(progressBar_ProfilesData)
            } 
}

#' Dialog Box to  Select MSigDB Files from drive 
#' @usage
#' getMSigDBfile()
#' 
#' @return A path of MSigDB file
#' @export
#'
#' @examples
#' f <- 5+2
#' \dontrun{
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#' geteSet()
#' getMSigDBfile()
#' }
getMSigDBfile <- function(){
    
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    ENV$MSigDBPath <- tclvalue(tkgetOpenFile(filetypes = "{{gmt Files} {.gmt}} {{All files} *}", 
                                             title="Select MSigDB with Symbol Gene")) 
    
    if (!nchar(ENV$MSigDBPath)) {
        tkmessageBox(message = "No file was selected!")
    } else {
        Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        ENV$mSigDB<-readLines(ENV$MSigDBPath)
        tkmessageBox(message = paste("The selected file was", basename(ENV$MSigDBPath)))
    }
    
}

#' dialog box to Specify Mutation using Regular Expression. Search specific mutation using regular expression.
#' @usage
#' getSpecificMut()
#'
#' @return a a dataframe with specific mutation informations
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getSpecificMut()
#' }
dialogSpecificMut <- function(MutData, c) {
    
    Word <- dialogMut(ENV$StudyRefCase[c], "AA change", "G438C|G...C|G.*.")
    if (Word == "ID_CANCEL") return()
    
    
    MutIndex <- grep(Word, MutData$amino_acid_change)
    if(length(MutIndex)==0){
        msgNoMut=paste ("No Mutation found with genes list for", ENV$StudyChoice[c])
        tkmessageBox(message=msgNoMut, icon="info")
        
    } else { 
        
        
        MutList<-cbind(MutData$gene_symbol[MutIndex],MutData$amino_acid_change[MutIndex])
        colnames(MutList) <- c("Gene", "Mutation")
        
        title=paste(ENV$StudyRefCase[c],ENV$CasesStudies[ENV$curselectCases[c]+1], 
                    ENV$GenProfsStudies[ENV$curselectGenProfs[c]+1], sep=": ")
        getInTable(MutList, title=title)
        
        
    }
    
}

#' get dataframe in TK/TCL table
#' @usage
#' getInTable(table,title)
#' @param table Dataframe
#' @param title string a title of the table
#' @return display a Table
#' @export
#'
#' @examples
#' #data(ClinicalData)
#' \dontrun{
#' getInTable(Table= ClinicalData, title= "Clinical Data")
#' }

getInTable <- function(table,title){
matrix <-rbind(colnames(table), table)
matrix1 <- cbind(row.names(matrix), matrix)
matrix1 <- t(t(matrix1))

# Define a Tcl array and initialize it to that matrix :
tclArray1 <- tclArray()
for (i in (1:length(matrix1[,1]))){
    for (j in (1:length(matrix1[1,]))){
        tclArray1[[i-1,j-1]] <- matrix1[i,j]
    }
}
#Table_Title<- paste("SD:",StudyRefCase[k],"_","GenProf:",curselectGenProfs_forStudy[k],"_","CASE:",curselectCases_forStudy[k],".gct")
table1 <- displayInTable(tclArray1,title= title ,nrow=nrow(matrix1),ncol=ncol(matrix1),height=40,width=50)

}

#### This file actually has the lmpergene function
### also:

### pvals from permutation matrix and qRequire (copied over
### from Category package where they are hidden functions)

### GSEA permutation test for one main effect in a multiple regression setting
### (generalization of gseattperm)



#in this function we  compute the linear regressions
#on a pergene basis
#note the NA handling is still not great, as we don't yet
# deal with NA's in the ExpressionSet

lmPerGene <- function(eSet,formula="",
                      na.rm = TRUE,
                      pooled=FALSE) {

  if (formula=="") {

### Intercept-Only Model (default)
      nSamp = ncol(eSet)
      x=matrix(rep(1,nSamp),nrow=nSamp,ncol=1)
      colnames(x) = "Intercept"
      eS = eSet


  } else {
    xvarnames = all.vars(formula)
  badvars = !(xvarnames %in% varLabels(eSet))
  if( any(badvars) )
      stop("variable", xvarnames[badvars],
          "are not variable names in the supplied ExpressionSet")

  nvar <- length(xvarnames)


##drop any with missing values in some covariate
  if( na.rm ) {
      na = rep(FALSE, ncol(eSet))
      for (i in xvarnames)
         na = na | is.na(eSet[[i]])
      eS = eSet[, !na]
  } else
      eS = eSet

    nSamp=ncol(eS)
  xvar <- Biobase::pData(eS)[, xvarnames, drop=FALSE]

    x = model.matrix(formula, data = xvar)
}
  xx <- crossprod(x)
  xxinv <- solve(xx)
  Hmat <- x %*% xxinv %*% t(x)
  dMat = diag(nSamp) - Hmat
  k = ncol(x)

  xy = exprs(eS) %*% x
  res = exprs(eS) %*% dMat
  beta = solve(xx, t(xy))
  varr = rowSums( exprs(eS) * res)/(nSamp - k) ## vector of raw residual variances

  varr2 = varr
  if (pooled) varr2 = mean(varr)

  varbeta = matrix(diag(xxinv), ncol = k, nrow = nrow(eS),
     byrow=TRUE)
  varbeta = apply(varbeta, 2, function(x) x* varr2)


  colnames(varbeta) = colnames(x)

  return(list(eS = eS, x = x, Hmat = Hmat, formula=formula,coefficients = beta, pooled=pooled,
              sigmaSqr = varr, coef.var = t(varbeta),tstat=beta/sqrt(t(varbeta))))
}

### This file has utilities related to incidence matrices


### This function does the normalized aggregation over gene-sets;
### It is now hopefully generic enough to enable any GSEA flavor

GSNormalize <- function(dataset,incidence,
                      gseaFun=crossprod,
                      fun1="/",
                      fun2=sqrt,
                      removeShift=FALSE,
                      removeStat=mean,...){
    
    dataset = as.matrix(dataset)
    
    ### Removal of column-wise mean shift
    
    if (removeShift) {
        
        colStats=apply(dataset,2,removeStat)
        dataset=sweep(dataset,2,STATS=colStats)
    }
    
    if (ncol(incidence) != nrow(dataset)) stop ("GSNormalize: non-conforming matrices")
    
    
    outm = gseaFun(t(incidence),dataset,...)
    
    
    rownames(outm)=rownames(incidence)
    colnames(outm)=colnames(dataset)
    
    normby = fun2(rowSums(incidence))
    
    outm = sweep (outm,1,normby,FUN=fun1)
    
}


##### GSEA inference for main effect using multiple regression and permutation
##### Inference reported only for main effect, which MUST BE
##### THE FIRST VARIABLE IN THE FORMULA
##### This is because label permutations are done within each block defined by
##### level combinations of the other variables

##### This is an extension of "gseattperm"

gsealmPerm=function (eSet,
                     formula="",
                     mat,
                     nperm,
                     na.rm=TRUE,
                     pooled=FALSE,
                     detailed=FALSE,...) {

### For the most part we rely on 'lmPerGene' for formula validation, NA removal, etc. etc.

nSamp=ncol(eSet)

if (formula=="") {
        nvar=0
} else {
    xvarnames = all.vars(formula)
    nvar <- length(xvarnames)
}

### The observed t-values for the main effect

obsRaw = lmPerGene(eSet=eSet,
                   formula=formula,
                   na.rm=na.rm,
                   pooled=pooled)

if (nvar>0) {
    observedStats= GSNormalize(obsRaw$tstat[2,],incidence=mat,...)
} else {
    observedStats= GSNormalize(t(obsRaw$tstat),incidence=mat,fun2=identity,...)
}

### Permutation loop; we do the intercept-only case separately below

perm.eset=eSet

i <- 1L
if (nvar>0) {
    permMat <- matrix(0, nrow = nrow(eSet), ncol = nperm)

    while (i < (nperm + 1)) {

#### The crux (with nvar>=2): label permutation is done *within each covariate-combination level group separately*

        if (nvar>=2) {
            splitter=Biobase::pData(eSet)[,xvarnames[2]]
            if (nvar>2) splitter=as.list(Biobase::pData(eSet)[,xvarnames[2:nvar]])

            label.perm=unsplit(lapply(split(1:nSamp,splitter),sample),splitter)

### Now we only permute the labels of variable 1
            Biobase::pData(perm.eset)[,xvarnames[1]]<-Biobase::pData(eSet)[label.perm,xvarnames[1]]
        } else if (nvar==1) {
            Biobase::pData(perm.eset)[,xvarnames[1]]<-Biobase::pData(eSet)[sample(1:nSamp),xvarnames[1]]
        }
        temp.results<-lmPerGene(eSet=perm.eset,formula=formula,na.rm=na.rm,pooled=pooled)
# (na.rm=FALSE since we already dealt with na's)

### record t-score for permuted variable
        permMat[, i] <- temp.results$tstat[2,]

        i <- i + 1L
  }
  permMat <- GSNormalize(permMat,incidence=mat,...)
  rownames(permMat)=rownames(mat)

} else if (nvar==0) {

### Intercept only - row permutation and using raw expression means, no need for repeated calls to 'lmPerGene'

    permMat <- matrix(0, nrow = nrow(mat), ncol = nperm)
    rownames(permMat)=rownames(mat)

    for (i in 1:nperm)  permMat[,i]=GSNormalize(t(obsRaw$tstat),incidence=mat[,sample(1:ncol(mat))],fun2=identity,...)
}
if (!detailed) {
  return(pvalFromPermMat(observedStats, permMat))
  } else return (list(pvalues=pvalFromPermMat(observedStats,permMat),lmfit=obsRaw,stats=observedStats,perms=permMat))
}


#### unexported functions

pvalFromPermMat <- function(obs, perms) {
    N <- ncol(perms)
    pvals <- matrix(as.double(NA), nrow = nrow(perms), ncol=2)
    dimnames(pvals) <- list(rownames(perms), c("Lower", "Upper"))

    tempObs <- rep(obs, ncol(perms))
    dim(tempObs) <- dim(perms)
    pvals[ , 1] <- (1+rowSums(perms <= tempObs))/(N+1)
    pvals[ , 2] <- (1+rowSums(perms >= tempObs))/(N+1)
    pvals
}

#' get Summary  results from GSEA-R (Broad Institute)
#' @usage getSummaryGSEA()
#' @return Dataframe with summary results
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' Run.GSEA()
#' getSummaryGSEA()
#' }
getSummaryGSEA <- function(){
    
    Variable <- as.data.frame(getClinicData_MultipleCases(getSummaryGSEAExists=1))
    ##remove empty cases
    Variable <- subset(Variable, !Variable[,1]=="")
    #tkwait.variable(ttClinData_cb)
    
    if(ncol(Variable)!=1){
        tkmessageBox(message="Select only ONE Variable that has 2 classes as OS_STATUS or DFS_STATUS", icon="warning" )
        stop("Select only ONE Variable that has 2 classes as OS_STATUS or DFS_STATUS")
    }
    
    FDR<-dialogSummary_GSEA(Variable)
    
    tab1 <-read.table( ENV$location1, header=TRUE, sep="\t")
    
    cnames<- colnames(tab1)
    ##select only signicant pVal 0.01
    tab1 <-tab1[which(tab1$FDR.q.val<FDR),]
    ##Order Biological process by significant
    tab1 <- tab1[order(tab1$NES),]
    
    
    tab2 <-read.table(ENV$location2, header=TRUE, sep="\t")
    
    ##select only signicant pVal 0.01
    tab2 <-tab2[which(tab2$FDR.q.val<FDR),]
    ##Order Biological process by significant
    tab2 <- tab2[order(tab2$NES),]
    
    
    if(nrow(tab1)== 0){
        tab1 <- t(cbind(rep(0,13)))
        rownames(tab1)[1] <- names(table(Variable[,1]))[1]
        print(paste(names(table(Variable[,1]))[1], "= 0 Genes Sets"))
    } else {
        print(paste(names(table(Variable[,1]))[1],"=", nrow(tab1), "Genes Sets"))
        rownames(tab1)[1] <- names(table(Variable[,1]))[1]
    }
    
    if(nrow(tab2)==0){
        tab2 <- t(cbind(rep(0,13)))
        rownames(tab2)[1] <- names(table(Variable[,1]))[2]
        print(paste(names(table(Variable[,1]))[2], "= 0 Genes Sets"))
    } else{
        print(paste(names(table(Variable[,1]))[2],"=", nrow(tab2), " Genes Sets"))
        rownames(tab2)[1] <- names(table(Variable[,1]))[2]
    }    
    table <- rbind.na(tab1, tab2)
    ## Theses 2 colums remain without name???
    colnames(table)[c(1,3)]<- c("GS", "SOURCE")
    title = paste("GSEA SUMMARY RESULTS REPORT: FDR.q.val < ", FDR)
    getInTable( table=table, title = title)
    
}

#' Display matrix in tcltk table
#' @usage
#' displayInTable(tclarray,title="",height=-1,width=-1,nrow=-1,ncol=-1)
#'
#' @param tclarray a dataframe
#' @param title title of the table
#' @param height -1
#' @param width -1
#' @param nrow -1
#' @param ncol -1
#' 
#' @return display a Table
#' @export
#'
#' @examples
#' #data(ClinicalData)
#' \dontrun{
#' getInTable(Table= ClinicalData, title= "Clinical Data")
#' }
displayInTable <- function(tclarray,title="",height=-1,width=-1,nrow=-1,ncol=-1)
{
    #require(tcltk)
    tt <- tktoplevel()
    tkwm.resizable(tt, TRUE, TRUE)
    #tkwm.geometry(tt, "900x900")
    tclRequire("Tktable")
    tkwm.title(tt,title)
    table1 <- tkwidget(tt,"table",rows=nrow,cols=ncol,titlerows=1,titlecols=1,
                       height=height+1,width=width+3,
                       xscrollcommand=function(...) tkset(xscr,...),yscrollcommand=function(...) tkset(yscr,...))
    xscr <-tkscrollbar(tt,orient="horizontal", command=function(...)tkxview(table1,...))
    yscr <- tkscrollbar(tt,command=function(...)tkyview(table1,...))
    
    tkgrid(table1,yscr)
    tkgrid.configure(yscr,sticky="nsw")
    tkgrid(xscr,sticky="new")
    tkconfigure(table1,variable=tclarray,background="white",selectmode="extended")
    tkconfigure(table1,selectmode="extended",rowseparator="\"\n\"",colseparator="\"\t\"")
    
    
    ##To control whether rows and/or columns can be resized
    tkconfigure(table1,resizeborders="both") 
    return (table1)
}

ENV <- new.env(parent = emptyenv())
#' main function
#' @usage
#' canceR()
#'
#' @return open the starting windows with cancer studies
#' @export
#'
#' @examples
#' ENV <- new.env(parent = emptyenv())
#' \dontrun{
#' canceR()
#'}
#' @import tkrplot
#' @import tcltk
#' @import cBioPortalData
#' @import R.oo
#' 
#'@importFrom graphics axis image layout legend lines par plot points text
#'@importFrom stats as.formula cor density dist hclust median na.exclude p.adjust pnorm sd setNames window
#'@importFrom utils browseURL capture.output memory.limit read.table write.table read.delim
#'@importFrom grDevices colors dev.cur dev.off graphics.off jpeg pdf png rainbow dev.new savePlot
#'@importFrom R.methodsS3 setMethodS3
#'

canceR <- function(){
    
    #if (!require("cgdsr")) devtools::install_version("cgdsr",version="1.3")
    
    ## connect to cBioPortal API
    ENV$cgds <- cBioPortalData::cBioPortal(
        hostname = "www.cbioportal.org",
        protocol = "https",
        api = "/api/v2/api-docs"
    )

    ## Get all Cancer Studies using column 2 (description)
    ENV$Studies <- cBioPortalData::getStudies(ENV$cgds) 
    ENV$Studies_name <- ENV$Studies |> pull('name')
    
    ## first dialog START or CANCEL
    ENV$ttMain <- tktoplevel()
    tktitle(ENV$ttMain) <- "Search for Cancer Studies"
    #tkwm.resizable(ENV$ttMain, FALSE, TRUE)
    #tkwm.geometry(ENV$ttMain, "300x170")
    
    topMenu <- tkmenu(ENV$ttMain)           # Create a menu
    tkconfigure(ENV$ttMain, menu = topMenu) # Add it to the 'tt' window
    fileMenu <- tkmenu(topMenu, tearoff = FALSE)
    HelpMenu<- tkmenu(topMenu, tearoff = FALSE)
    
    ## TopMenus
    tkadd(topMenu, "cascade", label = "File", menu = fileMenu)
    tkadd(topMenu, "cascade", label = "Help", menu = HelpMenu)
    
    ## Second Menus
    # GeneMenu<- tkmenu(AnalysisMenu, tearoff = FALSE)
    
    quit <- function(){
        QuitMsg <- tkmessageBox(message="Are you sure want to quit?",
                                icon="question",type="yesnocancel",default="yes")   
        if ( tclvalue(QuitMsg )== "yes") {
            tkdestroy(ENV$ttMain)
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            rm(list=ls(), envir=.GlobalEnv)
        }
        else {tkfocus(ENV$ttMain)}
    }
    
    
    tkadd(fileMenu, "command", label = "Set Workspace", command = function() setWorkspace())
    tkadd(fileMenu, "command", label = "Get Gene Exp", command = function() getGeneExpMatrix())
    tkadd(fileMenu, "command", label = "Get Clinical Data", command = function() getClinicalDataMatrix())
    tkadd(fileMenu, "command", label = "Quit", command = function() quit())
    tkadd(HelpMenu, "command", label= "Vignette", command= function() canceR_Vignette())
    tkadd(HelpMenu, "command", label= "Issue", command = function() canceR_Issue())
    tkadd(HelpMenu, "command", label = "About", command= function() about())
    
    
    
    loadAllStudies <- function(){
        #delete last search by key words
        if(exists("Matched_index", envir = ENV)){
            rm("Matched_index", envir=ENV)
            rm("matched_studies", envir = ENV)
        }
        
        ENV$n_studies <- nrow(ENV$Studies)
        ENV$Studies_name <- ENV$Studies$name
        
        nbrStudies <- paste("Query result: ", ENV$n_studies,
                            " Studies were loaded. Select one or more Studies to get Data Profiles.",
                            sep="")
        #tkgrid(tklabel(ENV$ttMain,text= nbrStudies ))
        ##Insert ListBox
        tkdelete(tlMain,0,600)
        tkdelete(tlInfo,0,1)
        for(s in ENV$Studies_name)
        {
            tkinsert(tlMain,"end", s )
        }
        # Default selection.  Indexing starts at zero.

        tkselection.set(tlMain,{30;86})  #:356, 30
        
        
        tkinsert(tlInfo,"end",text= nbrStudies )
        tkdelete(tlInfo,0)
    }
    
    loadAllStudies.button <- tkbutton(ENV$ttMain,
                                      text = "Load all Studies", 
                                      command = loadAllStudies)
    
    loadMatchStudies <- function(Word){
        #delete last load of all Studies
        if(exists("n_studies", envir = ENV)){
            rm("n_studies", envir=ENV)
            rm("Studies_name", envir=ENV)
        }
        # select raw with matched "string"
        ENV$Matched_index <- grep(Word, ENV$Studies$name, ignore.case=TRUE) 
        
        #ENV$matched_studies <- ENV$Studies[ENV$Matched_index, "name"] |> pull()
        ENV$matched_studies <- ENV$Studies$name[ENV$Matched_index]
        
        ##Count the nomber of Matched Studies and return the number.
       nbrMatchedStudies_msg <- paste("Query result: ",
                                   length(ENV$Matched_index), 
                                   " Studies were Matched. Select one or more to get its features.",
                                   sep="")
        #tkgrid(tklabel(ENV$ttMain,text= nbrMatchedStudies_msg ))
        tkdelete(tlMain,0,600)
        tkdelete(tlInfo,0,1)
        for( m in ENV$matched_studies){ 
            tkinsert(tlMain,"end", m)
        }
        tkselection.set(tlMain,2)  # Default selection.  Indexing starts at zero.
        
        tkinsert(tlInfo,"end",text= nbrMatchedStudies_msg)
        tkdelete(tlInfo,0)
    }
    
    launchDialog <- function() {
        Word <- modalDialog("Search Studies", "Search by Key Word", "")
        if (Word == "ID_CANCEL") return()
        loadMatchStudies(Word)
        
    }
    
    
    loadMatchStudies.button <- tkbutton(ENV$ttMain,
                                        text = "Search by key words", 
                                        command = launchDialog)
    
    tkgrid(loadAllStudies.button,loadMatchStudies.button, column=0)
    tkgrid.configure(loadAllStudies.button, sticky="w")
    tkgrid.configure(loadMatchStudies.button, sticky="e")
    
    
    
    tlInfo<-tklistbox(ENV$ttMain,height=1, width= 65, selectmode="multiple",
                      xscrollcommand=function(...)tkset(xscrInfo,...),
                      background="white", foreground="blue")
    
    xscrInfo <- tkscrollbar(ENV$ttMain, repeatinterval=2,orient="horizontal",
                            command=function(...)tkxview(tlInfo,...))
    
    tkgrid(tlInfo)
    
    ##Define ListBox of Studies
    tlMain<-tklistbox(ENV$ttMain,height=18, width= 65 ,selectmode="multiple",
                      xscrollcommand=function(...)tkset(xscr,...),
                      yscrollcommand=function(...)tkset(yscr,...), background="white")
    yscr <- tkscrollbar(ENV$ttMain, repeatinterval=2,
                        command=function(...)tkyview(tlMain,...))
    xscr <- tkscrollbar(ENV$ttMain, repeatinterval=2,orient="horizontal",
                        command=function(...)tkxview(tlMain,...))
    
    tkgrid(tlMain,yscr)
    tkgrid.configure(yscr,rowspan=20,sticky="nsw")
    tkgrid(xscr)
    tkgrid.configure(xscr,rowspan=1,sticky="ew")
    
    police <- tkfont.create(family="arial", size=11)
    tkconfigure(tlMain, foreground="black", font=police)
    
    ##listbox Info
    
    loadCasesGenProfs <- function(){ 
        if(exists("n_studies", envir = ENV)){
            ENV$StudyChoice <- ENV$Studies_name[as.numeric(tkcurselection(tlMain))+1]
            ENV$checked_StudyIndex <- as.numeric(tkcurselection(tlMain))+1
            
        } else if(exists("matched_studies", envir=ENV)){
            ENV$StudyChoice <- ENV$matched_studies[as.numeric(tkcurselection(tlMain))+1]
            ENV$checked_StudyIndex <- ENV$Matched_index[as.numeric(tkcurselection(tlMain))+1]
        }

        #Identify checked Studies and its Index in cgds
        ENV$checked_Studies_id <- ENV$Studies[ENV$checked_StudyIndex,"studyId"] |> pull()

        ###tkmessageBox if No Study was selected
        if(length(ENV$checked_Studies_id) ==0){
            msgStudies <- paste("You need to select at less one Study")
            tkmessageBox(title = "Greetings from R TclTk", message=msgStudies, icon = "Info" )
        } else {
            
            getCasesGenProfs()
            
        }
    }
    
    getCasesGenProfs.but <-tkbutton(ENV$ttMain,
                                    text="Get Cases and Genetic Profiles for selected Studies",
                                    command=loadCasesGenProfs)
    tkgrid(getCasesGenProfs.but)
    tkfocus(ENV$ttMain)
    
}

#' Testing checked appropriate Cases for appropriate Genetic profiles.
#' @param singleGene specify if the check for querying genetic profile for a specific gene or not (0,1).
#' @usage testCheckedCaseGenProf(singleGene=0)
#' @return dialog box with warning message
#' 
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' testCheckedCaseGenProf(singleGene=0)
#' }
#' @export
testCheckedCaseGenProf <- function(singleGene=0){
    
if(!exists("curselectCases", envir = ENV)){
    msgNoCaseChoice= paste("Select at least ONE Case and ONE Genetic Profile")
    tkmessageBox(message= msgNoCaseChoice)
    stop(msgNoCaseChoice)
    
} else if (!exists("curselectGenProfs", envir = ENV)){
    
    msgNoGenProfChoice= paste("Select at least ONE Genetic Profile")
    tkmessageBox(message= msgNoGenProfChoice)
    stop(msgNoGenProfChoice)
    
} else if (!exists("GeneList", envir = ENV) && singleGene==0){
    msgNoGeneList= paste("Load Gene List (HUGO)")
    tkmessageBox(message= msgNoGeneList, icon="info")
    stop(msgNoGeneList)
    
} else {
    ######## Verify the number of checked cases and checked GenProfs.
    #If they not have the same length, take this message:
    if(length(ENV$curselectCases) != length(ENV$curselectGenProfs))
    {
        msgNotEqual<-"Select EQUAL number of Cases and Genetic Profiles for every Study!"
        tkmessageBox(message=msgNotEqual, icon="warning")
        tkfocus(ENV$ttCasesGenProfs)
        stop("Select EQUAL number of Cases and Genetic Profiles for every Study!")
    } 
    ######### Test if all cases were corresponded to appropriate Gen profs (same Study)
    for (i in seq(ENV$curselectCases)){
        if(ENV$StudyRefCase[i]!= ENV$StudyRefGenProf[i]){
            
            msgBadChoice="Correspond the Genetic Profile to the Case for the same Study"
            tkmessageBox(message=msgBadChoice, icon="warning")
            tkfocus(ENV$ttCasesGenProfs)
            stop("Correspond the Genetic Profile to the Case for the same Study")
        } 
    }
}
}

#' open pdf vignette
#' @usage
#' canceR_Vignette()
#'
#' @return open pdf vignette
#' @export
#'
#' @examples
#' \dontrun{
#' canceR_Vignette()
#'}
#'
#'@importFrom Biobase openPDF
#'@importFrom utils vignette
#'
`canceR_Vignette` <-
    function() {
        tkgrab.release(window)
        #ViggIndex <- file.path(system.file("extdata/canceR.Rmd",package="canceR"),"canceR.pdf")
        #Biobase::openPDF (ViggIndex) # opens the pdf
        browseURL('https://bioconductor.org/packages/3.14/bioc/vignettes/canceR/inst/doc/canceR.html')
        #rmarkdown::render(input = system.file("extdata/canceR.Rmd",package="canceR"), 'pdf_document')
        ## Morgan
        #utils::vignette(package="canceR", "canceR.Rmd")
    }


#' canceR Report Issue
#' @usage canceR_Issue()
#' @return link to github issues
#' @export
#'
#' @examples
#' \dontrun{
#' canceR_Issue()
#'}
#'


`canceR_Issue` <-
    function() {
        tkgrab.release(window)
        browseURL('https://github.com/kmezhoud/canceR/issues')
        
    }

#' get GSEA linear modeling by studies (diseases)
#' @usage
#' getGSEAlm_Diseases()
#'
#' @return a dataframe with annotation (GO, BP)
#' @export
#' @examples
#'  readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#'  \dontrun{
#'  getGSEAlm_Diseases
#' }
#'@importFrom stats model.matrix
#' 
getGSEAlm_Diseases <-function(){
    
    ##Remove "GenesDetails" objectf if exists
    ifrm <- function(obj, env = ENV()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    ifrm(ENV$GenesDetails, ENV)
    
    
    ####Create a new directory "Results/GSEAlm" for output.
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/")){
        dir.create(file.path(paste(getwd(), "/Results", sep="")), showWarnings = FALSE)
        dir.create(file.path(paste(getwd(), "/Results/GSEAlm", sep="")), showWarnings = FALSE)
    } else if (!file.exists("Results/GSEAlm/")){
        dir.create(file.path(paste(getwd(), "/Results/GSEAlm", sep="")), showWarnings = FALSE)
    }
    
    
    testCheckedCaseGenProf()
    
    Lchecked_Studies <- ENV$lchecked_Studies_forCases
    Lchecked_Cases <- length(ENV$curselectCases)
    Lchecked_GenProf <- length(ENV$curselectGenProfs)
    
    #dialog function to select the number of samples
    ENV$ReturnDialogGeneClasses<- dialogGeneClassifier(Lchecked_Cases)
    if(ENV$ReturnDialogGeneClasses[1] == "ID_CANCEL"){
        stop()
    } else{
        
        
        DiseasesType <- 0
        SamplingProfsData<-0 
        ProfData<-0
        LengthGenProfs<-0
        LengthCases=0
        for (i in 1:Lchecked_Studies){
            
            Si =ENV$checked_StudyIndex[i]
            progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies[Si], min = 0,
                                                      max = Lchecked_GenProf, width = 400)
            
            #tkfocus(progressBar_ProfilesData)
            LastLengthGenProfs = LengthGenProfs
            LengthGenProfs = LengthGenProfs + ENV$LGenProfs[i]+1
            LastLengthCases = LengthCases
            LengthCases= LengthCases + ENV$LCases[i]+1
            
            
            
            for (k in 1:Lchecked_Cases){
                
                Sys.sleep(0.1)
                setTkProgressBar(progressBar_ProfilesData, k, label=paste( round(k/Lchecked_GenProf*100, 0),
                                                                           "% of Expression Set"))
                
                if (ENV$curselectGenProfs[k] <= LengthGenProfs && ENV$curselectGenProfs[k]>LastLengthGenProfs){    
                    
                    GenProf<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                    
                    Case<-ENV$CasesRefStudies[ENV$curselectCases[k]]
                    
                    if(length(ENV$GeneList)>500){
                        ProfData <- getMegaProfData(ENV$GeneList,k )
                    } else{
                        ProfData<-getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                    }
                    
                    ProfData <- t(ProfData)
#                     ##Convert data frame to numeric structure
#                     print("converting data frame of Profile data to numeric stucture...")
#                     
#                     cidx <- !(sapply(ProfData, is.numeric))
#                     ProfData[cidx] <- lapply(ProfData[cidx], as.numeric)
#                     
                    #for(p in 1:ncol(ProfData)){
                        
                     #   ProfData[,p] <- as.numeric(ProfData[,p])
                    #}
                    
                    ## for loop is faster than apply fonction
                    #rnames <- rownames(ProfData)
                    #ProfData <- as.data.frame(apply(ProfData,2 ,function(x) as.numeric(x)))
                    #rownames(ProfData) <- rnames
                    
                    if(!exists(deparse(substitute(ENV$GenesDetails)))) {
                        ProfData <- t(t(ProfData))
                        ## Display AssyData with Tcl Table
                        title=paste(ENV$StudyChoice[k],":",ENV$GenProfsStudies[ENV$curselectGenProfs][k])
                        getInTable(ProfData,title)
                        
                        
                    }
                    
                    if(ncol(ProfData)<ENV$ReturnDialogGeneClasses[1]){
                        msgBigSampl <- paste(ENV$StudyRefCase[k], "has only", ncol(ProfData),"samples.","\nSelect at Max: ",ncol(ProfData), "samples")
                        tkmessageBox(message=msgBigSampl, icon="info")
                        break
                        close(progressBar_ProfilesData)
                    }
                    set.seed(1234)
                    
                    
                    SamplingProfData <- t(apply(ProfData, 1,function(x)sample(x,ENV$ReturnDialogGeneClasses[1])))
                    
                    SamplingColnamesProfData <- sample(colnames(ProfData), ENV$ReturnDialogGeneClasses[1])
                    
                    colnames(SamplingProfData) <- SamplingColnamesProfData
                    
                    SamplingProfsData <- cbind(SamplingProfsData,SamplingProfData)
                    print(paste ("sampling data from study:", k))
                    
                    ### ONly for example brca_tcga73genes.RData
                    ##ENV$StudyRefGenProf <- c("brca_tcga", "prad_tcga")

                    ##Extracting Disease Type
                    DiseaseType<- as.matrix(rep(ENV$StudyRefGenProf[k],times=ENV$ReturnDialogGeneClasses[1]))
                    DiseasesType <- c(DiseasesType, DiseaseType)  
                    
                    
                    
                } 
            } 
            close(progressBar_ProfilesData)
        }            
        
        SamplingProfsData<- SamplingProfsData[,-1]
        DiseasesType <-DiseasesType[-1]
        
        DiseasesType <- as.data.frame(DiseasesType)
        rownames(DiseasesType) <- colnames(SamplingProfsData)                
        ENV$DiseasesType <- DiseasesType
        
        ## create labelDescription for columns of phenoData. 
        ## labeldescription is used by Biobase packages
        ## In our case labelDescription is Equal to column names
        
        
        metaData <- data.frame(labelDescription= "DiseasesType", row.names="DiseasesType")        ## Bioconductor’s Biobase package provides a class called AnnotatedDataFrame   
        
        
        ##that conveniently stores and manipulates 
        ##the phenotypic data and its metadata in a coordinated fashion. 
        
        phenoData<-new("AnnotatedDataFrame", data=DiseasesType, varMetadata=metaData) 
        
        ##Assembling an ExpressionSet  
        eSetClassifier<-Biobase::ExpressionSet(assayData=SamplingProfsData, phenoData=phenoData, annotation="GO") 
        
        ##translate Négative to positive value
        
        if(min(Biobase::exprs(eSetClassifier), na.rm=TRUE)<0){
            print("There are negative values. Translating values by adding the absolute of minimum value to all matrix")                  
            Biobase::exprs(eSetClassifier) <- Biobase::exprs(eSetClassifier) +(abs(min(Biobase::exprs(eSetClassifier), na.rm=TRUE)))
        }
        ENV$eSetClassifier <- eSetClassifier
        
        
        ## create MSigDb for eSetClassifier
        getMSigDB(ENV$eSetClassifier,1)
        
        
        #run GSEAlm
        #nperm determines the number of permutations used to estimate the null distribution
        #of the enrichment score
        
        
        dialogOptionGSEAlm(1, ENV$DiseasesType)
        print("dialog Option of  GSEAlm: OK")
        
        ## test if Covariables works together else get message box
#         if (inherits(try(pVals <-GSEAlm::gsealmPerm(ENV$eSetClassifier,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE), silent=TRUE),"try-error"))
#         {
#             msgBadCovariables <- paste("There is incompatible variables. Select an other Formula.")
#             tkmessageBox(message=msgBadCovariables, icon="warning")
#             close(progressBar_ProfilesData)
#             stop(msgBadCovariables)
#         } else{  
#             print("Computing of pVals using gsealmPerm function ...")
#             pVals <-GSEAlm::gsealmPerm(ENV$eSetClassifier,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE)
#         }
        
         print("Computing of pVals using gsealmPerm function ...")
         pVals <- gsealmPerm(ENV$eSetClassifier,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE)



        #print(paste("pVals:", StudyRefCase[k]))
        # we have to correct for multiple testing. In this case we use the method from Benjamini-Hochberg
        pVals <- apply(pVals, 2, p.adjust, method = "BH", n = nrow(pVals))
        
        pVal_Permut <- paste(names(table(pData(ENV$eSetClassifier)))[2])
        colnames(pVals) <- c(paste("Down Regulated in ",pVal_Permut),paste("Up Regulated in ",pVal_Permut,"nperm:",ENV$permutVal,"pVal:",ENV$seuilpVal))
        
        if(exists(deparse(substitute(GenesDetails)), envir = ENV)) {
            assign(paste("pVals", gsub(".RData","",ENV$mSigDB_SubName),sep="_"),pVals, envir=ENV)
            workspace <- getwd()
            setwd("./Results/GSEAlm")
            ######################
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            if(!file.exists(paste(ENV$StudyRefCase, collapse="_VS_"))){
                dir.create(file.path(paste(getwd(),"/",paste(ENV$StudyRefCase, collapse="_VS_") , sep="")), showWarnings = FALSE)
            }
            setwd(paste("./",paste(ENV$StudyRefCase, collapse="_VS_"), sep=""))
            
            #################
            
            write.table(pVals, paste("pVals_", gsub(".RData","",ENV$mSigDB_SubName),".txt", sep=""), sep="\t")
            #set a significance threshold
            THRESHOLD<-ENV$seuilpVal
            #gene sets that are downregulated in the BLC
            downRegulated <- data.frame(sort(pVals[pVals[, 1]< THRESHOLD,1]))
            names(downRegulated) <- paste("Down Regulated in ",names(table(pData(ENV$eSetClassifier)))[2],"\tpVal<",THRESHOLD)
            #colnames(pVals) <- c(paste("Down Regulated\n",pVal_Permut),paste("Up Regulated\n",pVal_Permut))
            
            assign(paste("downRegulated","GeneClass" ,gsub(".RData","",ENV$mSigDB_SubName), sep="_"),downRegulated, envir=ENV)
            #save downRegulted table
            write.table(downRegulated,"_GeneClass_",paste("downRegulated_", gsub(".RData","",mSigDB_SubName),".txt", sep=""), sep="\t") 
            #gene sets that are upregulated in basal breast cancer
            upRegulated <- data.frame(sort(pVals[pVals[, 2] < THRESHOLD, 2]))
            #colnames(upRegulated[1]) <- paste("Up Regulated\n", seuilpVal, pVal_permut, coVariables)
            names(upRegulated) <- paste("Up Regulated in",names(table(pData(ENV$eSetClassifier)))[2],"\tpVal<",THRESHOLD)
            assign(paste("upRegulated","GeneClass" ,gsub(".RData","",ENV$mSigDB_SubName),sep="_"),upRegulated, envir=ENV)
            #save upRegulated table
            write.table(upRegulated,"_GeneClass_" ,paste("upRegulated_", gsub(".RData","",mSigDB_SubName),".txt", sep=""), sep="\t") 
            
            setwd(workspace)
        }else{
            assign(paste("pVals", gsub(".RData","",ENV$mSigDB_SubName),ENV$StudyRefCase[k] ,sep="_"),pVals, envir=ENV)       
            workspace <- getwd()
            setwd("./Results/GSEAlm")
            ################
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            if(!file.exists(paste(ENV$StudyRefCase, collapse="_VS_"))){
                dir.create(file.path(paste(getwd(),"/",paste(ENV$StudyRefCase, collapse="_VS_") , sep="")), showWarnings = FALSE)
            }
            setwd(paste("./",paste(ENV$StudyRefCase, collapse="_VS_"), sep=""))
            ################
            write.table(pVals, paste("pVals_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$StudyRefCase[k],".txt", sep=""), sep="\t")
            #set a significance threshold
            THRESHOLD<-ENV$seuilpVal
            #gene sets that are downregulated in the BLC
            downRegulated <- data.frame(sort(pVals[pVals[, 1]< THRESHOLD,1]))
            names(downRegulated) <- paste("Down Regulated in ",names(table(pData(ENV$eSetClassifier)))[2],"\tpVal<",THRESHOLD)
            #colnames(pVals) <- c(paste("Down Regulated\n",pVal_Permut),paste("Up Regulated\n",pVal_Permut))
            assign(paste("downRegulated", gsub(".RData","",ENV$mSigDB_SubName), ENV$StudyRefCase[k],sep="_"),downRegulated, envir=ENV)
            #save downRegulted table
            write.table(downRegulated, paste("downRegulated_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$StudyRefCase[k],".txt", sep=""), sep="\t") 
            #gene sets that are upregulated in basal breast cancer
            upRegulated <- data.frame(sort(pVals[pVals[, 2] < THRESHOLD, 2]))
            #colnames(upRegulated[1]) <- paste("Up Regulated\n", seuilpVal, pVal_permut, coVariables)
            names(upRegulated) <- paste("Up Regulated in",names(table(pData(ENV$eSetClassifier)))[2],"\tpVal<",THRESHOLD)
            assign(paste("upRegulated", gsub(".RData","",ENV$mSigDB_SubName), ENV$StudyRefCase[k],sep="_"),upRegulated, envir=ENV)
            
            #save upRegulted table
            write.table(upRegulated, paste("upRegulated_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$tudyRefCase[k],".txt", sep=""), sep="\t") 
            
            
            setwd(workspace)
            
            
            
        } 
        
        ##Get results in Tcltk Table
        
        downRegulatedTab <- rbind(colnames(downRegulated), downRegulated)
        downRegulatedTab <- cbind(rownames(downRegulatedTab),downRegulatedTab)
        downRegulatedTab[,1]<- as.character(downRegulatedTab[,1])
        downRegulatedTab[1,1] <- "***** Down Regulated Gene Sets *****" 
        colnames(downRegulatedTab) <- NULL
        
        
        
        upRegulatedTab <- rbind(colnames(upRegulated), upRegulated)
        upRegulatedTab <- cbind(rownames(upRegulatedTab),upRegulatedTab)
        upRegulatedTab[,1]<- as.character(upRegulatedTab[,1])
        upRegulatedTab[1,1] <- "***** Up Regulated Gene Sets *****"
        colnames(upRegulatedTab) <- NULL
        
        UpDownRegulatedTable <- rbind.na(downRegulatedTab,upRegulatedTab)
        getInTable(UpDownRegulatedTable, title=paste(ENV$StudyRefCase[1]," / ",ENV$StudyRefCase[2]))
    }
}

#' get Methylation data for multiple genes
#' @usage
#' getMetDataMultipleGenes()
#'
#' @return a a dataframe with mean and median of methylation rate (threshold of silencing gene)
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getMetDataMultipleGenes()
#' }
getMetDataMultipleGenes <-function(){
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    testCheckedCaseGenProf()
    
    ######### Test if all cases were corresponded to appropriate Gen profs (same Study)
    # for (i in 1:length(ENV$curselectCases)){
    #     if(ENV$StudyRefCase[i]!=ENV$StudyRefGenProf[i]){
    #         
    #         msgBadChoice="Correspond the Genetic Profile to the Case for the same Study"
    #         tkmessageBox(message=msgBadChoice, icon="warning")
    #         tkfocus(ENV$ttCasesGenProfs)
    #         stop("Correspond the Genetic Profile to the Case for the same Study")
    #         
    #     }
    # }
    
    #ProfDataAll =0
    ProfData = 0
    i <- 0
    for (s in ENV$checked_Studies_id){
        i<-i+1
        Si =ENV$checked_StudyIndex[i]
        progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                                  max = length(ENV$curselectGenProfs), width = 400)
        
        #tkfocus(progressBar_ProfilesData)
        
        study_desc_position_in_genProfs <- 0
        for (k in 1:length(ENV$curselectGenProfs)){
            Sys.sleep(0.1)
            setTkProgressBar(progressBar_ProfilesData, k, 
                             label=paste( round(k/length(ENV$curselectGenProfs)*100, 0),
                                                                       "% of Methylation Data"))
            
            
            # Avoid to select study description  
            if (ENV$curselectGenProfs[k] <= ENV$n_GenProfs[i] && 
                ENV$curselectGenProfs[k] > study_desc_position_in_genProfs){ 
                
                study_desc_position_in_genProfs <- study_desc_position_in_genProfs + ENV$n_GenProfs[i]
                
                GenProf <- ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                
                if (length(grep("methylation", GenProf))==0){
                    msgNoMeth <- "Select Methylation data from Genetics Profiles"
                    tkmessageBox(message = msgNoMeth, icon='info')
                    break
                }
                
                # if (length(grep("methylation", s ))==0){
                #     msgNoMeth <- "Select Methylation data from Cases"
                #     tkmessageBox(message = msgNoMeth, icon='info')
                #     break
                # }
                
                
                #ProfData <- getProfileData(ENV$cgds,ENV$GeneList, GenProf,Study_id)
                ProfData <-  cBioPortalData::getDataByGenes(api = ENV$cgds,
                                               studyId = s,
                                               genes = ENV$GeneList,
                                               by = "hugoGeneSymbol",
                                               molecularProfileIds = GenProf) |>
                    unname() |>
                    as.data.frame()|>
                    select("sampleId", "hugoGeneSymbol", "value") |>
                    tidyr::spread("hugoGeneSymbol", "value") |>
                    data.frame(row.names = 1)
                   #tibble::column_to_rownames("sampleId")

                ##convert data frame to numeric structure
                #if( !is.numeric(ProfData[1,1])){
                #    for(i in 1:ncol(ProfData)){
                    
                 #       ProfData[,i] <- as.numeric(ProfData[,i])
                  #  }
                    
                #}
                ##More facter
                cidx <- !(sapply(ProfData, is.numeric) )
                ProfData[cidx] <- lapply(ProfData[cidx], as.numeric)
                
                ProfData <- round(ProfData, digits=3)
                
                dialogMetOption(ProfData,k)
                
            }
        } 
        close(progressBar_ProfilesData)
    }
    
}

#' get Cases and Genetic Profiles of selected Studies.
#' @usage getCasesGenProfs()
#' @return This function is run by the "Get Cases and Genetic Profiles for selected Studies in starting window. This function needs to select at least one study and display Cases and genetic profiles in the main window.
#' @examples
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
#' @export
#' 
getCasesGenProfs <- function(){
    
    #tclRequire("Tktable")
    
    if(exists("ttCasesGenProfs", envir = ENV)){
        tkdestroy(ENV$ttCasesGenProfs)
    }
    
    ENV$ttCasesGenProfs<-tktoplevel()
    tktitle(ENV$ttCasesGenProfs) <- "Cases list and Genetic Profiles"
    #tkwm.geometry(ENV$ttCasesGenProfs, "550x420")
    
    ##MENU
    topMenu <- tkmenu(ENV$ttCasesGenProfs)           # Create a menu
    tkconfigure(ENV$ttCasesGenProfs, menu = topMenu) # Add it to the 'ENV$ttCasesGenProfs' window
    
    OpenMenu <- tkmenu(topMenu, tearoff=FALSE)
    
    ClinMenu <- tkmenu(topMenu, tearoff = FALSE)
    MutMenu<- tkmenu(topMenu, tearoff = FALSE)
    ProfMenu<- tkmenu(topMenu, tearoff = FALSE)
    PhenoMenu <- tkmenu(topMenu, tearoff = FALSE)
    GSEAMenu <- tkmenu(topMenu, tearoff= FALSE)
    ClassMenu<- tkmenu(topMenu, tearoff=FALSE)
    PlotMenu<- tkmenu(topMenu, tearoff = FALSE)
    HelpMenu<- tkmenu(topMenu, tearoff = FALSE)
    GeneListMenu <- tkmenu(OpenMenu, tearoff=FALSE)
    Gene1 <- tkmenu(PlotMenu, tearoff= FALSE)
    Gene2 <- tkmenu(PlotMenu, tearoff= FALSE)
    Survival <- tkmenu(PlotMenu, tearoff= FALSE)
    get <- tkmenu(GSEAMenu, tearoff=FALSE)
    GSEAlm <- tkmenu(GSEAMenu, tearoff=FALSE)
    TreeClass <- tkmenu(ClassMenu, tearoff=FALSE)
    ##TopMenus
    #tkadd(topMenu,"command", label = "Gene List", command= function()getGeneList())
    tkadd(topMenu, "cascade", label = "Load", menu=OpenMenu)
    tkadd(OpenMenu, "cascade", label = "Gene list", menu=GeneListMenu)
    tkadd(topMenu, "cascade", label = "Clinical Data", menu=ClinMenu)
    tkadd(topMenu, "cascade", label = "Mutation", menu= MutMenu)
    tkadd(topMenu,"command", label = "Methylation", command= function()getMetDataMultipleGenes())
    
    tkadd(topMenu, "cascade", label = "Profiles", menu=ProfMenu)
    #tkadd(topMenu, "command", label = "Profile Data", command=function()getProfilesData())
    tkadd(topMenu, "cascade", label = "PhenoTest", menu=PhenoMenu)
    tkadd(topMenu, "cascade", label ="GSEA", menu=GSEAMenu)
    tkadd(topMenu, "cascade", label ="Classification", menu=ClassMenu)
    tkadd(topMenu, "cascade", label = "Plot", menu=PlotMenu)
    tkadd(topMenu, "cascade", label = "Help", menu = HelpMenu)
    tkadd(PlotMenu, "cascade", label= "1Gene", menu=Gene1)
    tkadd(PlotMenu, "cascade", label= "2Genes", menu=Gene2)
    tkadd(PlotMenu, "cascade", label= "Survival", menu=Survival)
    tkadd(PlotMenu, "command", label = "Circos", command = function() getCircos(dimension = "All"))
    
    tkadd(GSEAMenu, "cascade", label = "get", menu=get)
    #tkadd(GSEAMenu, "command", label = "Which MSig for gene list", command= function()Match_GeneList_MSigDB())
    #tkadd(GSEAMenu, "command", label = "get MSigDB for eSet", command= function()getMSigDB())
    tkadd(GSEAMenu, "cascade", label= "linear Model", menu=GSEAlm)
    
    #Second Menu
    tkadd(GeneListMenu, "command", label = "File", command = function() getGeneList())
    tkadd(GeneListMenu, "command", label = "Example", command = function() getGeneListExample())
    tkadd(GeneListMenu, "command", label = "MSigDB", command = function() getGeneListFromMSigDB())
    tkadd(ClinMenu, "command", label = "Multiple Cases", 
          command = function() getClinicData_MultipleCases(getSummaryGSEAExists=0))
    #tkadd(ClinMenu, "command", label = "All", command = function() getClinicData_MultipleCases())
    #tkadd(ClinMenu, "command", label = "Single Case", command = function() getClinicData_SingleCase())
    #tkadd(ClinMenu, "command", label = "Quit", command = function() tkdestroy(ENV$ttCasesGenProfs))
    tkadd(MutMenu, "command", label = "All", command=function() getMutData())
    tkadd(MutMenu, "command", label = "Specific", command= function() getSpecificMut())
    tkadd(ProfMenu, "command", label = "Single Gene", 
          command= function() getProfilesDataSingleGene()) # getProfilesDataMultipleGenes(getSummaryGSEAExists=0)
    tkadd(ProfMenu, "command", label = "Multiple Genes", 
          command= function() getProfilesDataMultipleGenes(getSummaryGSEAExists=0))
    tkadd(ProfMenu, "command", label = "ListProfData", 
          command= function() getListProfData(ENV$checked_Studies_id, ENV$GeneList))
    
    tkadd(Gene1, "command", label = "2 Gen. Profiles",  command = function() plot_1Gene_2GenProfs())
    tkadd(Gene2, "command", label = "1 Gen. Profile", command = function() plot_2Genes_1GenProf())
    tkadd(Survival, "command", label = "Kaplan-Meier", command = function() getSurvival(Coxph = 0))
    tkadd(Survival, "command", label = "CoxPH", command = function() getSurvival(Coxph = 1))
    tkadd(PhenoMenu, "command", label = "eSet", command = function() geteSet())
    tkadd(PhenoMenu, "command", label= "Pheno/Exp", command= function() getPhenoTest())
    
    #tkadd(get, "command", label = "CLS file", command= function()getCLSfile())
    #tkadd(get, "command", label = "GCT file", command= function()getGCTfile())
    tkadd(get, "command", label = "GCT,CLS files", command= function() getGCT_CLSfiles())
    tkadd(get, "command", label= "Results", command= function() Run.GSEA())
    tkadd(get, "command", label= "Summary", command= function() getSummaryGSEA())
    tkadd(GSEAlm, "command", label = "Which MSig for gene list", command= function() Match_GeneList_MSigDB())
    tkadd(GSEAlm, "command", label= "SubMSigDB/eSet", command= function() getMSigDB(ENV$eSet,1))
    tkadd(GSEAlm, "command", label= "Phenotypes into Disease", command= function() getGSEAlm_Variables())
    tkadd(GSEAlm, "command", label= "Disease vs Disease", command= function() getGSEAlm_Diseases())
    
    
    #tkadd(GSEAMenu, "command", label= "GSEAlm", command= function()getGSEAlm())
    tkadd(ClassMenu, "command", label= "Genes/Diseases", command= function() getGenesClassifier())
    # tkadd(ClassMenu, "command", label= "Genes/Pheno Single", command= function() getGenesTree_SingleCase())
    #tkadd(ClassMenu, "command", label= "Genes/Pheno Multiple", command= function() getGenesTree_MultipleCases())
    tkadd(ClassMenu, "cascade", label= "Genes/Pheno Tree", menu=TreeClass)
    tkadd(TreeClass, "command", label= "Single", command= function() getGenesTree_SingleCase())
    tkadd(TreeClass, "command", label= "Multiple", command= function() getGenesTree_MultipleCases())
    
    tkadd(HelpMenu, "command", label="Vignette", command= function() canceR_Vignette())
    tkadd(HelpMenu, "command", label= "Issue", command = function() canceR_Issue())
    tkadd(HelpMenu, "command", label = "About", command= function() about())
    
    
    # Take LABELS in ENV$ttCasesGenProfs
    tklabel0 = tklabel(ENV$ttCasesGenProfs, text= "The Cases (Samples)")
    tklabel1 =tklabel(ENV$ttCasesGenProfs,text="The Genetic Profiles")
    tkgrid(tklabel0,tklabel1, columnspan=2, pady = 10)
    
    #Define Scrolls
    yscr1 <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,
                         command=function(...)tkyview(ENV$tc,...))
    xscr1 <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(ENV$tc,...))
    
    yscr2 <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,
                         command=function(...)tkyview(ENV$tl,...))
    xscr2 <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(ENV$tl,...))
    
    xscr1Info <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tInfoC,...))
    xscr2Info <- tkscrollbar(ENV$ttCasesGenProfs, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tInfoG,...))
    
    
    #policeInfo <-tkfont.create(family="arial", size=10)
    
    
    
    #Define Frame and LISTBOX
    #frameOverall <- tkframe(ENV$ttCasesGenProfs)
    #frame1 <- tkframe(ENV$ttCasesGenProfs,relief="groove",borderwidth=5, height = 100, width = 500)
    ENV$tc<-tklistbox(ENV$ttCasesGenProfs,height=15, width= 68 ,
                              selectmode="multiple",
                              xscrollcommand=function(...)tkset(xscr1,...),
                              yscrollcommand=function(...)tkset(yscr1,...),background="white")
    ENV$tl<-tklistbox(ENV$ttCasesGenProfs,height=15, width= 68,
                              selectmode="multiple",xscrollcommand=function(...)tkset(xscr2,...),
                              yscrollcommand=function(...)tkset(yscr2,...),background="white")
    
    tInfoC<-tklistbox(ENV$ttCasesGenProfs,height=2, width= 68,
                      selectmode="multiple",
                      xscrollcommand=function(...)tkset(xscr1Info,...),
                      background="white")
    #tkconfigure(tInfoC, foreground="blue", font=policeInfo)
    
    tInfoG<-tklistbox(ENV$ttCasesGenProfs,height=2, width= 68,
                      selectmode="multiple",xscrollcommand=function(...)tkset(xscr2Info,...),
                      background="white")
    #tkconfigure(tInfoG, foreground="blue", font=policeInfo)
    
    
    tkgrid(yscr1,ENV$tc,ENV$tl,yscr2, columnspan=1)
    tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
    tkgrid.configure(yscr2,rowspan=20, sticky="nsw")
    
    tkgrid(xscr1,xscr2,columnspan=2)
    tkgrid.configure(xscr1,rowspan=2, columnspan=2,sticky="ew")
    tkgrid.configure(xscr2,rowspan=2, columnspan= 4,sticky="ew")
    
    tklabel2 = tklabel(ENV$ttCasesGenProfs, text= "Selected Cases (Samples)")
    tklabel3 = tklabel(ENV$ttCasesGenProfs,text="Selected Genetic Profiles")
    tkgrid(tklabel2,tklabel3, columnspan=2, pady = 10)
    
    tkgrid(tInfoC, tInfoG, columnspan=2)
    tkgrid(xscr1Info,xscr2Info,columnspan=2)
    tkgrid.configure(xscr1Info,rowspan=2,columnspan=2,sticky="ew")
    tkgrid.configure(xscr2Info,rowspan=2,columnspan= 3,sticky="ew")
    
    
    getCases()
    getGenProfs()
    
    
    
    loadSelectedCases <- function(){  
        curselectCases <- as.numeric(tkcurselection(ENV$tc))+1
        lcurselectCases <- length(curselectCases)
        
        ENV$curselectCases <- curselectCases
        ENV$CaseChoice <- ENV$CasesStudies[curselectCases+1]
                          
        if (lcurselectCases == 0){
            #tkdelete(tInfoC,0,1)
            #ENV$curselectCases <- NULL
            msgSelectCase="Select at least one Case"
            tkmessageBox(message=msgSelectCase)
            
        } else {
            ## correspond each case  to its study
            StudyRefCase = NULL
            for( k in 1: lcurselectCases){
                h <- 1
                while (curselectCases[k] >  (ENV$n_Cases[h] + 1)){
                    curselectCases[k] <- curselectCases[k] - ENV$n_Cases[h] - 1
                    h <- h + 1
                }  
                StudyRefCase <- cbind(StudyRefCase, ENV$checked_Studies_id[h])
            }
            ENV$StudyRefCase <- StudyRefCase
            
            #redefine curselectCases
            ENV$curselectCases <- as.numeric(tkcurselection(ENV$tc)) + 1
            lcurselectCases <- length(ENV$curselectCases)
            
            ##loop converting curselectCases value of case to index value of the same case
            
            n_Cases_sum <- 0
            
            for( j in 1:lcurselectCases){
                for (i in seq(ENV$checked_Studies_id)){
                    
                    if (curselectCases[j] <= (n_Cases_sum + ENV$n_Cases[i] + i)){
                        curselectCases[j] <- curselectCases[j]  - 1
                        break
                    } 
                    n_Cases_sum <- n_Cases_sum + ENV$n_Cases[i] + 1
                }
            }
            ENV$curselectCases <-curselectCases
            tkdelete(tInfoC, 0, 1)
            tkinsert(tInfoC, "end", ENV$curselectCases)
            tkinsert(tInfoC, "end", ENV$CaseChoice)
            tkfocus(ENV$ttCasesGenProfs)
        }
        
    }
    
    loadSelectedGenProfs <- function(){
        curselectGenProfs = as.numeric(tkcurselection(ENV$tl))+1
        lcurselectGenProfs = length(curselectGenProfs)
        
        ENV$curselectGenProfs <- curselectGenProfs
        ENV$GenProfChoice <- ENV$GenProfsStudies[curselectGenProfs+1]
        
        
        if (lcurselectGenProfs ==0){
            #tkdelete(tInfoG,0,1)
            #ENV$curselectGenProfs <- NULL
            msgSelectCase="Select at least one Genetic Profil"; 
            tkmessageBox(message=msgSelectCase)
        } else {
            
            ## group selected Cases by study. To  avoid corresponding Genetic Profile and Case from different Study
            StudyRefGenProf <- NULL
            for( k in 1: lcurselectGenProfs){
                h=1
                
                while (curselectGenProfs[k] >  (ENV$n_GenProfs[h]+1)){
                    curselectGenProfs[k] <- curselectGenProfs[k] - ENV$n_GenProfs[h] - 1
                    h= h+1
                } 
                #Cancer Reference (StudyRefGenProf) were compared before to get Profile Data
                StudyRefGenProf = cbind(StudyRefGenProf, ENV$checked_Studies_id[h]) 
            }
            
            ENV$StudyRefGenProf<-StudyRefGenProf
            
            
            ##loop converting curselectGenProfs value of Genetic profile to index value of the same genetic profile in cgds
            n_GenProfs_sum<-0
            
            for( j in 1: lcurselectGenProfs){
                for (i in seq(length(ENV$checked_Studies_id))){
                    
                    
                    if (curselectGenProfs[j] <= (n_GenProfs_sum + ENV$n_GenProfs[i] + i)){
                        curselectGenProfs[j] <- curselectGenProfs[j] - 1
                        break
                    } 
                    n_GenProfs_sum <- n_GenProfs_sum + ENV$n_GenProfs[i] + 1
                }
                
            }
            ENV$curselectGenProfs <- curselectGenProfs
            
            tkdelete(tInfoG,0,1)
            tkinsert(tInfoG,"end",ENV$curselectGenProfs)
            tkinsert(tInfoG,"end",ENV$GenProfChoice)
            tkfocus(ENV$ttCasesGenProfs)
        }
    }
    
    LoadCases.but <-tkbutton(ENV$ttCasesGenProfs, text="   Load selected Cases   ",
                             command=loadSelectedCases)
    
    LoadGenProf.but <-tkbutton(ENV$ttCasesGenProfs, text="   Load selected Genetic Profiles   ",
                               command=loadSelectedGenProfs)
    
    
    
    
    tkgrid(LoadCases.but,LoadGenProf.but, columnspan=2)
    
    tkgrid(tInfoC)
    
}

#' Dialog  Box for Sampling patients from expression profile data used for GSEA-R (Broad Institute)
#' @param n_checked_GenProf  Number of checked genetic profiles
#' @param entryWidth 10
#' @param returnValOnCancel  "ID_CANCEL"
#' 
#' @usage dialogSamplingGSEA(n_checked_GenProf, entryWidth = 10,returnValOnCancel = "ID_CANCEL")
#' 
#' @return A vector with sampling size
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' Run.GSEA()
#' #dialogSamplingGSEA(1,entryWidth=10,returnValOnCancel = "ID_CANCEL")
#' }
#'  
dialogSamplingGSEA <- function( n_checked_GenProf, entryWidth = 10,
                                returnValOnCancel = "ID_CANCEL") {
    
    dlg <- tktoplevel()
    tkwm.deiconify(dlg)
    tkgrab.set(dlg)
    tkfocus(dlg)
    tkwm.title(dlg, "Sampling from Profile Data matrix")
    
    txtMsg <- tklabel(dlg, text = "")
    tkgrid(txtMsg)
    tkgrid.configure(txtMsg, column=0, row=1,sticky="we")
    
    textEntryVarTcl <- tclVar("50")
    textEntryWidget <- tkentry(dlg, width = paste(entryWidth),
                               textvariable = textEntryVarTcl)
    
    txtNbr <- tklabel(dlg, text = "Number of Samples")
    tkgrid(txtNbr)
    tkgrid.configure(txtNbr, column=0, row=2, sticky="w")
    tkgrid(textEntryWidget)
    tkgrid.configure(textEntryWidget, column=0, row=2, sticky="ne")
    
    frameCases<- tkframe(dlg,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameCases, text="Checked Cases:"))
    tkgrid.configure(frameCases, sticky="new")
    
    for(i in 3:(n_checked_GenProf+2)){
        
        stdy <- tklabel(frameCases, text=ENV$StudyRefCase[i-2])
        tkgrid(stdy)
        tkgrid.configure(stdy, column=0,row=i, sticky="e")
        SamplesNbr <-tklabel(frameCases, text = paste(": " ,ENV$CaseChoice[i-2]))
        tkgrid(SamplesNbr)
        tkgrid.configure(SamplesNbr, column=1, row=i,sticky="w")
    }
    
    ReturnSamplesNbr <- returnValOnCancel
    
    onOK <- function() {
        ENV$ReturnSamplesNbr <- as.numeric(tclvalue(textEntryVarTcl))
        tkgrab.release(dlg)
        tkdestroy(dlg)
        
    }
    onCancel <- function() {
        ENV$ReturnSamplesNbr <- returnValOnCancel
        tkgrab.release(dlg)
        tkdestroy(dlg)
    }
    OK.but <- tkbutton(dlg, text = "   OK   ", command = onOK)
    Cancel.but <- tkbutton(dlg, text = " Cancel ", command = onCancel)
    tkgrid(Cancel.but)
    tkgrid.configure(Cancel.but, column=0,row= (n_checked_GenProf+5), sticky="w")
    tkgrid(OK.but)
    tkgrid.configure(OK.but, column=0,row= (n_checked_GenProf+5), sticky="e")
    
    
    tkfocus(dlg)
    tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg)})
    tkbind(textEntryWidget, "<Return>", onOK)
    tkwait.window(dlg)
    
    return(ENV$ReturnSamplesNbr)
}

#' get Circos Layout for selected studies and selected dimensions
#' @usage
#' getCircos(dimension)
#' @param dimension string (All,mRNA, CNA, Met,RPPA, miRNA, Mut) 
#' @return a plot with Circos style
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
#' \dontrun{
#' getCircos(dimension ="All")
#' }
#' @import circlize
getCircos <- function(dimension){
    #library(circlize)
    circos.clear()
    if(length(ENV$ListProfData$Expression)!=0){
        getCor_ExpCNAMet(ENV$ListProfData$Expression, dimension = "Exp")
    }
    if(length(ENV$ListProfData$CNA)!=0){
        getCor_ExpCNAMet(ENV$ListProfData$CNA, dimension = "CNA")
    }
    
    if(length(ENV$ListMetData$HM450)!=0){
        if (inherits(try(getCor_ExpCNAMet(ENV$ListMetData$HM450, dimension = "MetHM450"), silent=TRUE),"try-error"))
        {
            msgNoHM27 <- paste("There is more than 2 empty matrices from HM450 Data. Skip HM450")
            tkmessageBox(message=msgNoHM27, icon="warning")
            
        } else{  
            print("Computing of Correlaton HM450 ...")
            getCor_ExpCNAMet(ENV$ListMetData$HM450, dimension = "MetHM450")
        }
        
        
    }
    
    
    if(length(ENV$ListMetData$HM27)!=0){
        if (inherits(try(getCor_ExpCNAMet(ENV$ListMetData$HM27, dimension = "MetHM27"), silent=TRUE),"try-error"))
        {
            msgNoHM27 <- paste("There is more than 2 empty matrices from HM27 Data. Skip HM27")
            tkmessageBox(message=msgNoHM27, icon="warning")
            
        } else{  
            print("Computing of Correlaton HM27 ...")
            getCor_ExpCNAMet(ENV$ListMetData$HM27, dimension = "MetHM27")
        }
        
        
    }
    
    if(length(ENV$ListProfData$RPPA)!=0){
        getCor_ExpCNAMet(ENV$ListProfData$RPPA, dimension = "RPPA")
    }
    if(length(ENV$ListProfData$miRNA)!=0){
        getCor_ExpCNAMet(ENV$ListProfData$miRNA, dimension = "miRNA")
    }
    #df1 = read.table("~/CGDS-R//Cor_Exp")
    #df2 = read.table("~/CGDS-R//Cor_CNA")
    
    if(dimension == "Exp"){
        #all_disease <- unique(ENV$Cor_Exp[[2]])
        all_disease <- ENV$checked_Studies
        #all_genes <- unique(ENV$Cor_Exp[[1]])
        all_genes <- ENV$GeneList
        n_gene <- length(all_genes)
        n_disease <- length(all_disease) - 1
        
        
    } else if(dimension=="MetHM450"){
        all_disease <- unique(ENV$Cor_Met$HM450[[2]])
        all_genes <- unique(ENV$Cor_Met$HM450[[1]])
        n_gene <- length(all_genes)
        n_disease <- length(all_disease) - 1
        
    } else if(dimension=="MetHM27") {
        all_disease <- unique(ENV$Cor_Met$HM27[[2]])
        all_genes <- unique(ENV$Cor_Met$HM27[[1]])
        n_gene <- length(all_genes)
        n_disease <- length(all_disease) - 1
        
    } else if (dimension=="CNA"){
        all_disease <- unique(ENV$Cor_CNA[[2]])
        all_genes <- unique(ENV$Cor_CNA[[1]])
        n_gene <- length(all_genes)
        n_disease <- length(all_disease) - 1
    } else if (dimension=="All"){
        all_disease <- unique(ENV$Cor_Exp[[2]])
        #all_disease <- ENV$checked_Studies
        all_genes <- unique(ENV$Cor_Exp[[1]])
        #all_genes <- ENV$GeneList
        n_gene <- length(all_genes)
        n_disease <- length(all_disease) - 1 
        
    }
    
    
    if(exists("GeneListMSigDB", envir = ENV)){
        ## set color for Gene Sets
        gene_set <- ENV$GeneListMSigDB[[1]]
        names(gene_set) <- ENV$GeneListMSigDB[[2]]
        gene_set_col <- rand_color(length(unique(gene_set)))
        names(gene_set_col) <- unique(gene_set)
        
    }
    ## Dialog Option Circos
    dialogOptionCircos()
    
    
    circos.par(cell.padding = c(0, 0, 0, 0), start.degree = 90)
    circos.initialize(factors = all_disease, xlim = c(0, n_gene))
    
    
    ## Disease/genes Track                         
    circos.trackPlotRegion(ylim = c(0, 1), panel.fun = function(x, y) {
        disease = get.cell.meta.data("sector.index")
        xlim = get.cell.meta.data("xlim")
        ylim = get.cell.meta.data("ylim")
        circos.text(mean(xlim), min(ylim)+.2, disease, facing = "bending.inside" , cex= 0.6)
        
        ## Add Gene Symbol
        ## replace 1 by 0 from Data frames 
        if(length(nrow(ENV$Freq_DfMutData))==1){
            ENV$Freq_DfMutData[is.na(ENV$Freq_DfMutData)] <- 0
        }
        if(length(nrow(ENV$Cor_Exp))==1){
            ENV$Cor_Exp[,c(-1,-2)][ENV$Cor_Exp[,c(-1,-2)]==1] <- 0
        }
        if(length(nrow(ENV$Cor_CNA))==1){
            ENV$Cor_CNA[,c(-1,-2)][ENV$Cor_CNA[,c(-1,-2)]==1] <- 0
        }
        if(length(nrow(ENV$Cor_Met$HM450))==1){
            ENV$Cor_Met$HM450[,c(-1,-2)][ENV$Cor_Met$HM450[,c(-1,-2)]==1] <- 0
        }
        if(length(nrow(ENV$Cor_Met$HM27))==1){
            ENV$Cor_Met$HM27[,c(-1,-2)][ENV$Cor_Met$HM27[,c(-1,-2)]==1] <- 0
        }
        if(length(nrow(ENV$ListProfData$RPPA))==1){
            ENV$RPPA[,c(-1,-2)][ENV$RPPA[,c(-1,-2)]==1] <- 0
        }
        ## plot only gene with high Level > Threshold
        for(i in 1:n_gene){
                      
            
            if(ENV$ReturnCBoxThrCircos[7] == 1&&max(ENV$Freq_DfMutData[i,]) > ENV$ReturnThreshCircos[7] ){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5, col="darkgoldenrod3")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040")
            }else if(ENV$ReturnCBoxThrCircos[1] == 1&&max(ENV$Cor_Exp[i,c(-1,-2)])  > ENV$ReturnThreshCircos[1]){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="red3")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040")
            }else if(ENV$ReturnCBoxThrCircos[1] == 1&&max(ENV$Cor_Exp[i,c(-1,-2)])  < -(ENV$ReturnThreshCircos[1])){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="blue")    
            }else if(ENV$ReturnCBoxThrCircos[2] == 1&&max(ENV$Cor_CNA[i,c(-1,-2)])  > ENV$ReturnThreshCircos[2]){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5, col="green4")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040")
                }else if(ENV$ReturnCBoxThrCircos[3] == 1 && max(ENV$Cor_Met$HM450[i,c(-1,-2)])  > ENV$ReturnThreshCircos[3]){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="mediumorchid4")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040") 
                }else if(ENV$ReturnCBoxThrCircos[3] == 1 && max(ENV$Cor_Met$HM450[i,c(-1,-2)])< -(ENV$ReturnThreshCircos[3])){
                 circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="springgreen4")   
            }else if(ENV$ReturnCBoxThrCircos[4] == 1&&max(ENV$Cor_Met$HM27[i,c(-1,-2)])  > ENV$ReturnThreshCircos[4] ){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5, col="mediumorchid4")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040") 
                }else if(ENV$ReturnCBoxThrCircos[4] == 1 && max(ENV$Cor_Met$HM27[i,c(-1,-2)])< -(ENV$ReturnThreshCircos[4])){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="springgreen4")   
            }else if(ENV$ReturnCBoxThrCircos[5] == 1&& max(ENV$Cor_RPPA[i,c(-1,-2)])  > ENV$ReturnThreshCircos[5]){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="red3" )
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040")
                }else if (ENV$ReturnCBoxThrCircos[5] == 1&& max(ENV$Cor_RPPA[i,c(-1,-2)])  < -(ENV$ReturnThreshCircos[5])){
                    circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5,col="blue" ) 
            }else if(ENV$ReturnCBoxThrCircos[6] == 1&&max(ENV$Cor_miRNA[i,c(-1,-2)])  > ENV$ReturnThreshCircos[6]){
                circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5, col="red2")
                #circos.link("brca_tcga", i , "brca_tcga", 365,rou1=0.8,rou2=0.8, col = "#00000040")
                }else if(ENV$ReturnCBoxThrCircos[6] == 1&&max(abs(ENV$Cor_miRNA[i,c(-1,-2)]))  < -(ENV$ReturnThreshCircos[6])){
                    circos.text(min(xlim)+i, max(ylim)+.5, all_genes[i], facing="clockwise", cex = 0.5, col="blue")
            }
        }
        
    }, track.height = 0.05, bg.border = NA)
    
    ## GetSet Track
    if(exists("GeneListMSigDB", envir = ENV)){
        
        circos.trackPlotRegion(ylim = c(0, 1), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease = get.cell.meta.data("sector.index")
            # genes that belong to this disease
            genes = ENV$Cor_Exp[ENV$Cor_Exp[[2]] == disease, 1]
            ## rearrange factor geneList as in all_gene (No alphabetic)
            genes <- factor(genes, levels = all_genes)
            
            for(i in seq_along(genes)) {
                circos.rect(i-0.5, 0, i+0.5, 1, col = gene_set_col[gene_set[genes[i]]], border = NA)
            }
            
            circos.text(min(xlim)+15, max(ylim)+0.5, "Gene Sets", facing= "bending.inside" ,cex = 0.5, nice.facing=TRUE)
            
            legend(x=-1.1, y=-0.8, pch = 15, col = gene_set_col, legend = names(gene_set_col),bty = "n" ,y.intersp = 0.7,pt.cex = 1, cex=0.455)
            
        }, track.height = 0.02, bg.border = "black")
        
        
    }
    
    
    # Track for Gene Expression
    if(length(nrow(ENV$Cor_Exp))==1 && ncol(ENV$Cor_Exp) == n_disease +3&&ENV$ReturnCBoxCircos[1]==1){
        
        print("Getting Track for Gene Expression...")
        
        circos.trackPlotRegion(ylim = c(0, n_disease), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease <- get.cell.meta.data("sector.index")
            mat <- as.matrix(ENV$Cor_Exp[ENV$Cor_Exp[[2]] == disease, 3:(n_disease+3)])
            mat <- mat[, -which(all_disease %in% disease)]
            corr_col_fun <- colorRamp2(c(-1,0 ,1), c("blue","white" ,"red3"))
            
            for(i in seq_len(n_gene)) {
                for(j in seq_len(n_disease)) {
                    circos.rect(i-1, j-1, i, j, col = corr_col_fun(mat[i, j]), border = NA)
                }
            }
            circos.text(min(xlim)+15, max(ylim), "mRNA Exp", facing= "bending.inside" ,cex = 0.5)
            
            legend(x=-1.11, y=1.1, title = "mRNA",pch = 15, col = corr_col_fun(seq(-1,1, by = 0.5)), legend = seq(-1, 1, by = 0.5),bty = "n",y.intersp=0.7,pt.cex = 1, cex=0.5)
            
            
        }, track.height = 0.1)
        
    }
    
    
    # Track for CNA
    if(length(nrow(ENV$Cor_CNA))==1 && ncol(ENV$Cor_CNA) == n_disease +3&&ENV$ReturnCBoxCircos[2]==1){
        print("Getting Track for CNA...")
        
        
        circos.trackPlotRegion(ylim = c(0, n_disease), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease <- get.cell.meta.data("sector.index")
            mat <- as.matrix(ENV$Cor_CNA[ENV$Cor_CNA[[2]] == disease, 3:(n_disease+3)])
            mat <- mat[, -which(all_disease %in% disease)]
            corr_col_fun <- colorRamp2(c( 0.7,0.8 ,1), c("white","greenyellow" ,"green4"))
            
            for(i in seq_len(n_gene)) {
                for(j in seq_len(n_disease)) {
                    circos.rect(i-1, j-1, i, j, col = corr_col_fun(mat[i, j]), border = NA)
                }
            }
            circos.text(min(xlim)+15, max(ylim), "CNA", facing= "bending.inside" ,cex = 0.5)
            
            legend(x=0.6,y=1.1, title = "CNA",pch = 15, col = corr_col_fun(seq(0.8, 1, by = 0.1)), legend = seq(0.8, 1, by = 0.1),bty = "n", y.intersp=0.7, pt.cex = 1, cex=0.5)
        }, track.height = 0.1)
        
    }
    
    ##track for methylation HM450
    if(length(nrow(ENV$Cor_Met$HM450))==1 && ncol(ENV$Cor_Met$HM450) == n_disease +3&&ENV$ReturnCBoxCircos[3]==1){
        
        print("getting Track for methylation HM450...")
        
        circos.trackPlotRegion(ylim = c(0, n_disease), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease = get.cell.meta.data("sector.index")
            mat = as.matrix(ENV$Cor_Met$HM450[ENV$Cor_Met$HM450[[2]] == disease, 3:(n_disease+3)])
            mat = mat[, -which(all_disease %in% disease)]
            corr_col_fun = colorRamp2(c( -1,0 ,1), c("springgreen4","white" ,"mediumorchid4"))
            
            for(i in seq_len(n_gene)) {
                for(j in seq_len(n_disease)) {
                    circos.rect(i-1, j-1, i, j, col = corr_col_fun(mat[i, j]), border = NA)
                }
            }
            circos.text(min(xlim)+15, max(ylim), "Met HM450", facing= "bending.inside" ,cex = 0.5)
            
            legend(x=-0.9,y=1.1, title = "Meth",pch = 15, col = corr_col_fun(seq(-1,1, by = 0.5)), legend = seq(-1, 1, by = 0.5),bty = "n" ,y.intersp=0.7,pt.cex = 1, cex=0.5)        
        }, track.height = 0.1)
        
    }
    
    ##track for methylation HM27
    if(length(nrow(ENV$Cor_Met$HM27))==1 && ncol(ENV$Cor_Met$HM27) == n_disease +3 && ENV$ReturnCBoxCircos[4]==1){
        print("getting Track for methylation HM27...")
        
        circos.trackPlotRegion(ylim = c(0, n_disease), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease = get.cell.meta.data("sector.index")
            mat = as.matrix(ENV$Cor_Met$HM27[ENV$Cor_Met$HM27[[2]] == disease, 3:(n_disease+3)])
            mat = mat[, -which(all_disease %in% disease)]
            corr_col_fun = colorRamp2(c(-1,0 ,1), c("springgreen4","white" ,"mediumorchid4"))
            
            for(i in seq_len(n_gene)) {
                for(j in seq_len(n_disease)) {
                    circos.rect(i-1, j-1, i, j, col = corr_col_fun(mat[i, j]), border = NA)
                }
            }
            circos.text(min(xlim)+15, max(ylim), "Met HM27", facing= "bending.inside" ,cex = 0.5)
            
            legend(x=-0.9,y=1.1, title = "Meth",pch = 15, col = corr_col_fun(seq(-1,1, by = 0.5)), legend = seq(-1, 1, by = 0.5),bty = "n" ,y.intersp=0.7,pt.cex = 1, cex=0.5)        
        }, track.height = 0.1)
        
    }
    
    
    # Track for Reverse Phase Protein Affinity
    if(length(nrow(ENV$Cor_RPPA))==1 && ncol(ENV$Cor_RPPA) == n_disease +3 &&ENV$ReturnCBoxCircos[5]==1){
        
        print("Getting Track for RPPA...")
        
        circos.trackPlotRegion(ylim = c(0, n_disease), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            disease <- get.cell.meta.data("sector.index")
            mat <- as.matrix(ENV$Cor_RPPA[ENV$Cor_Exp[[2]] == disease, 3:(n_disease+3)])
            mat <- mat[, -which(all_disease %in% disease)]
            corr_col_fun <- colorRamp2(c(-1,0 ,1), c("blue","white" ,"red3"))
            
            for(i in seq_len(n_gene)) {
                for(j in seq_len(n_disease)) {
                    circos.rect(i-1, j-1, i, j, col = corr_col_fun(mat[i, j]), border = NA)
                }
            }
            circos.text(min(xlim)+15, max(ylim), "RPPA", facing= "bending.inside" ,cex = 0.5)
            #legend("bottomleft", title = "RPPA",pch = 15, col = corr_col_fun(seq(-1,1, by = 0.5)), legend = seq(-1, 1, by = 0.5), pt.cex = 1, cex=0.6)        
            
        }, track.height = 0.1)
        
    }
    
    ## Track for miRNA
    
    
    ## track for Gene Mutation
    if(length(nrow(ENV$Freq_DfMutData))==1 &&ENV$ReturnCBoxCircos[7]==1){
        print("Getting Track for Gene Mutation Frequency...")
        
        mat <- as.matrix(ENV$Freq_DfMutData)
        mat[is.na(mat)] <- 0
        corr_col_fun <- colorRamp2(c(median(mat),mean(mat),30,70,max(mat)), c("white","white","gold","darkgoldenrod2", "darkgoldenrod4"))
        
        circos.trackPlotRegion(ylim = c(0, 1), panel.fun = function(x, y) {
            xlim = get.cell.meta.data("xlim")
            ylim = get.cell.meta.data("ylim")
            
            disease <- get.cell.meta.data("sector.index")
            mut = mat[, disease]
            names(mut) = rownames(mat)
            
            mut[is.na(mut)] <- 0
            
            for(i in seq_len(n_gene)) {
                circos.rect(i-0.5, 0, i+0.5, 1, col = corr_col_fun(mut[i]), border = NA)
            }
            circos.text(min(xlim)+15, max(ylim), "Mut", facing= "bending.inside" ,cex = 0.5)
        }, track.height = 0.1)
        interval <- (max(mat)-20)%/%3 
        legend(x=0.8,y=1.1, title = "Mutation",pch = 15, col = corr_col_fun(seq(20, max(mat), by = interval)), legend = seq(20, max(mat), by =interval ),bty = "n", y.intersp = 0.7 ,pt.cex = 1, cex=0.5)
        #legend(x=-0.9,y=1.1, title = "Meth",pch = 15, col = corr_col_fun(seq(-1,1, by = 0.5)), legend = seq(-1, 1, by = 0.5),bty = "n" ,y.intersp=0.7,pt.cex = 1, cex=0.5)        
        
    }
    
    circos.clear()
}

#' classify genes in tree for two phenotypes in the same case(disease).
#' @usage getGenesTree_SingleCase()
#' @return tree plot
#' @export
#'
#'@examples
#'readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#' \dontrun{
#' getGenesTree_SingleCase()
#' }
getGenesTree_SingleCase <- function(){
   
    if(length(ENV$curselectCases)!=1||length(ENV$curselectGenProfs)!=1){
        msgOneStudy = "Select only one Case/Genetic Profile or use multiple Cases"
        tkmessageBox(message=msgOneStudy, icon="warning")
        
    } else {
        LengthCases <- 0
        d <- 0
        for (i in 1:length(ENV$checked_Studies)){
            
            LengthCases <- LengthCases + ENV$LCases[i]+1
            d <- d +1 
            if(ENV$curselectCases < LengthCases){
                
                entryWidth <- 10
                ttGeneTree <- tktoplevel()
                #tkwm.geometry(ttGeneTree,"180x250")
                
              
                tktitle(ttGeneTree) <- paste(ENV$StudyChoice[i],": Classifying genes by variable")
                
                ##Image Horizontal scale option
                textEntryHscale <- tclVar("2")
                textEntryWidget <- tkentry(ttGeneTree, width = paste(entryWidth),
                                           textvariable = textEntryHscale)
                
                txtHscale <- tklabel(ttGeneTree, text = "Horizontal Scale of the plot")
                tkgrid(txtHscale)
                tkgrid.configure(txtHscale, column=1, row=1, sticky="w")
                tkgrid(textEntryWidget)
                tkgrid.configure(textEntryWidget, column=1, row=1, sticky="ne")
                
                ##Image Vertical scale option
                textEntryVscale <- tclVar("1")
                textEntryWidgetV <- tkentry(ttGeneTree, width = paste(entryWidth),
                                            textvariable = textEntryVscale)
                
                txtVscale <- tklabel(ttGeneTree, text = "Vertical Scale of the plot")
                tkgrid(txtVscale)
                tkgrid.configure(txtVscale, column=1, row=2, sticky="w")
                tkgrid(textEntryWidgetV)
                tkgrid.configure(textEntryWidgetV, column=1, row=2, sticky="ne")
                
                
                
                ##Clinical Data list
                
                label1 <- tklabel(ttGeneTree, text= "Clinical Data")
                
                yscr1 <- tkscrollbar(ttGeneTree, repeatinterval=2,
                                     command=function(...)tkyview(tl1,...))
                xscr1 <- tkscrollbar(ttGeneTree, repeatinterval=2,orient="horizontal",
                                     command=function(...)tkxview(tl1,...))
                
                xscr1Info <- tkscrollbar(ttGeneTree, repeatinterval=2,orient="horizontal",
                                         command=function(...)tkxview(tl1info,...))
                
                
                tl1<-tklistbox(ttGeneTree,height=10, width= 40 ,selectmode="single",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
                tl1info<-tklistbox(ttGeneTree,height=1, width= 40,selectmode="single",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
                
                
                Case<-ENV$CasesRefStudies[ENV$curselectCases]
                #getClinicData_SingleCase()
                ClinicalData<-getClinicalData(ENV$cgds,Case)
                
                
                loadVariable <- function()
                {  
                    curselectVariable <- as.numeric(tkcurselection(tl1))+1
                    lcurselectVariable <- length(curselectVariable)
                    
                    ENV$variable <-  names(ClinicalData)[curselectVariable]
                    tkdelete(tl1info,0,1)
                    tkinsert(tl1info,"end",ENV$variable)
                    
                }
                
                
                Variable.but <-tkbutton(ttGeneTree,text="select",command=loadVariable)
                
                tkgrid(label1,tl1,yscr1)
                tkgrid.configure(yscr1,rowspan=20, columnspan=2,sticky="nsw")
                tkgrid(xscr1)
                tkgrid.configure(xscr1,rowspan=2, column=1,sticky="we")
                tkgrid(Variable.but, tl1info, columnspan=1)
                tkgrid(xscr1Info)
                tkgrid.configure(xscr1Info,rowspan=4, column=1,sticky="we")
                
                
                GenProf<-ENV$GenProfsRefStudies[ENV$curselectGenProfs]
                
                
                ProfData<-getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                
                
                ##Convert data frame to numeric structure
#                 print("converting data frame of Profile data to numeric stucture...")
#                 
#                 for(i in 1:ncol(ProfData)){
#                     
#                     ProfData[,i] <- as.numeric(ProfData[,i])
#                 }
                
                ##for loop is faster than apply fonction
                #rnames <- rownames(ProfData)
                #ProfData <- as.data.frame(apply(ProfData,2 ,function(x) as.numeric(x)))
                #rownames(ProfData) <- rnames
                
                #test if is there a clinical data
                if(length(ClinicalData[1,])==0){
                    msgNoClinData=paste("No Clinical Data are Available for\n", ENV$CaseChoice)
                    tkmessageBox(message=msgNoClinData, title= ENV$CaseChoice, icon="info")
                    stop()
                }
                print('Clinical Data exists...')
                ## Select only Cases (rownames) that exist in ClinicalDataSub and ProfData
                merge <- merge(ClinicalData, ProfData, by="row.names")
                print("merging samples from Clinical and Profile Data...")
                ClinicalData<- merge[,2:(length(ClinicalData)+1)]
                
                ProfData<-merge[,!(merge %in% ClinicalData)]
                
                
                for (i in 1:length(names(ClinicalData))){
                    
                    tkinsert(tl1,"end",names(ClinicalData[i]))
                }
                
                Methods <- c("class","anova","poisson")
                # Default selections for the two combo boxes
                defaultMethod <- tclVar("class")
                favMethod <- tclVar("class")
                
                comboBox <- ttkcombobox(ttGeneTree, values=Methods, textvariable=favMethod, state="readonly")                        
                
                text <- tklabel(ttGeneTree,text="Select Method:")
                tkgrid(text, comboBox)              
                
                
                onOK <- function(){
                    if(exists("variable", envir = ENV)&&exists("Methods")){
                        
                        HorScale <- as.numeric(tclvalue(textEntryHscale))
                        VerScale <- as.numeric(tclvalue(textEntryVscale))
                        ENV$ProfData <- cbind(ClinicalData[,ENV$variable], ProfData[,-1])
                        
                        colnames(ENV$ProfData)[1] <- ENV$variable
                        
                        
                        frmla <- paste0(ENV$variable, "~.", sep="")
                        ENV$frmla <- as.formula(frmla)
                        
                        print(paste("Selected formula: ", ENV$frmla))
                        
                        
                        ##selected mathod
                        selectedMethod <- tclvalue(favMethod)
                        print(paste("Selected Method:", selectedMethod))
                        
                        plotCommand<- function(){
                            
                            #img <- tree(frmla, data=ProfData)
                            ENV$fit <- rpart::rpart(ENV$frmla, method=selectedMethod, data=ENV$ProfData)
                            plot(ENV$fit, uniform=TRUE, compress=TRUE,margin=0,main= paste(ENV$StudyChoice[d],"\n ",ENV$GenProfChoice," vs ",ENV$variable ))
                            text(ENV$fit, use.n=TRUE, all=TRUE, cex=0.6, fancy=FALSE)
                            
                        
                            
                        }
                        plotModel(plotCommand, title=paste(ENV$checked_Studies[d],":",ENV$CaseChoice,"vs" ,ENV$variable, sep=""), vscale=VerScale, hscale=HorScale)
                        ##capture print(fit) for editing
                        summary <- capture.output(print(ENV$fit))
                        ## Edit summary fitqqddcdsc
                        getTextWin(paste(summary,collapse="\n"))
                        
                        tkdestroy(ttGeneTree)
                    }else{
                        msgNoFrmla <- "Select one variable"
                        tkmessageBox(message= msgNoFrmla, icon="info")
                    }
                    
                    
                }
                
                Ok.but <-tkbutton(ttGeneTree,text=" OK ",command=onOK)
                tkgrid(Ok.but)
                tkgrid.configure(Ok.but,rowspan=4, column=1,sticky="n")
                ## break loop to avoid getting next studies without case GenProf.
                break()
            }else{
                print(paste("Skip Study:", ENV$checked_Studies[i]))
            }
        }
    }
}

#' get text in tcltk windows
#' @usage getTextWin(text)
#' @param text string
#' 
#' @return tcltk windows with text
#' @export
#' @examples 
#' text <- "mytext"
#' \dontrun{
#' getTextWin(text) 
#' }
getTextWin <- function(text){
    #require(tcltk)
    tt  <- tktoplevel()
    xscr <- tkscrollbar(tt, repeatinterval=5,orient="horizontal",
                        command=function(...)tkxview(txt,...))
    yscr <- tkscrollbar(tt, repeatinterval=5,
                        command=function(...)tkyview(txt,...))
    txt <- tktext(tt,bg="white",font="courier",
                  xscrollcommand=function(...)tkset(xscr,...),yscrollcommand=function(...)tkset(yscr,...),
                  wrap="none")
    tkgrid(txt,yscr)
    tkgrid(xscr)
    tkgrid.configure(yscr,sticky="ns")
    tkgrid.configure(xscr,sticky="ew")
    for (i in (1:100)) tkinsert(txt,"end",paste(text))
    tkconfigure(txt, state="")
    tkfocus(txt)
    
}

#' Survival plot
#' @usage getSurvival(Coxph)
#' @param Coxph if Coxph = 0 : plot Kaplan-Meier curves else Coxph= 1 : plot Cox Proportional Hazard Model
#' @return Survival plot
#' @examples 
#' surv <- 11
#' \dontrun{
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
#' getSurvival(Coxph = 1)
#' }
#' @export
#' 
#' @importFrom survival survfit
#' @importFrom survival coxph 
#' @importFrom survival Surv
#' @importFrom survival cox.zph
#' 
getSurvival<- function(Coxph){
    #library(survival)
    
    ## verify checked cases
    if(!exists("curselectCases", envir = ENV) || length(ENV$curselectCases)!=1){
        msgNoCases <- "Select ONE Case"
        tkmessageBox(message=msgNoCases)
        stop(msgNoCases)
    }
    
    
    
    ## Correspond the name study to selectec case in multiple studies choice
    LengthCases <- 0
    
    for (s in 1:length(ENV$checked_Studies)){
        
        LengthCases <- LengthCases + ENV$LCases[s]+1
        
        if(ENV$curselectCases <= LengthCases){
            
            Case<- ENV$CasesRefStudies[ENV$curselectCases]
            ClinicData <- getClinicalData(ENV$cgds,Case)
            
            
            ##Verify if there are Age,OS_STATUS, OS_MONTHS, DFS_STATUS
            if(length(grep("^OS_STATUS$", names(ClinicData), ignore.case = TRUE))==1&&   
                   length(grep("^OS_MONTHS$", names(ClinicData), ignore.case = TRUE))==1 &&
                   length(grep("^DFS_STATUS$", names(ClinicData), ignore.case = TRUE))==1
            ){
                
                ClinicData[ClinicData== "DECEASED"] <- 1
                ClinicData[ClinicData== "Deceased"] <- 1
                ClinicData[ClinicData== "LIVING"] <- 0
                ClinicData[ClinicData== "Living"] <- 0
                ClinicData$OS_STATUS <- as.numeric(ClinicData$OS_STATUS)
                ENV$ClinicData <- ClinicData
                ## Subset plot space into 3
                #layout(matrix(c(1,1,2,3), 2, 2, byrow = FALSE))
                if(Coxph==0){
                    dialogOptionGSEAlm(s,ClinicData)
                    n<- length(names(table(ClinicData[ENV$curselect])))
                    #fit a Kaplan-Meier and plot it
                    par(mfrow=c(1,1))
                    fit <- survival::survfit(ENV$variable, type="kaplan-meier", conf.type="log",data=ClinicData)
                    plot(fit,pch=20,conf.int = FALSE, lty=2:3,lwd=2 ,xlab="OS_MONTHS", ylab="OS_STATUS: Survival", main=ENV$StudyChoice[s], cex.main = 0.8, cex.lab= 0.7, cex.axis= 0.7, col=1:n)
                    legend("topright", c(names(table(ClinicData[ENV$curselect]))), col=1:n,lty = 2:3, cex=0.7, bty = 'n')
                    #                 ## 45, 1.09
                    ############
                    #                 plotCommand<- function(){
                    #                     #fit a Kaplan-Meier and plot it
                    #                     par(mfrow=c(1,1))
                    #                     fit <- survfit(Surv(OS_MONTHS,OS_STATUS)~DFS_STATUS, data=ClinicData)
                    #                     
                    # #                     if (inherits(try(plot(fit,pch=20,lty=2:3, xlab="OS_MONTHS", ylab="OS_STATUS: Survival", main=ENV$StudyChoice[s], cex.main = 0.8, cex.lab= 0.7, cex.axis= 0.7  ), silent=TRUE),"try-error"))
                    # #                     {
                    # #                         msgBadMARGIN <- paste(" Error in plot.new() : figure margins too large ")
                    # #                         tkmessageBox(message=msgBadMARGIN, icon="warning")
                    # #                         stop(msgBadMARGIN)
                    # #                     } else{         
                    #                         plot(fit,pch=20,lty=2:3, xlab="OS_MONTHS", ylab="OS_STATUS: Survival", main=ENV$StudyChoice[s], cex.main = 0.8, cex.lab= 0.7, cex.axis= 0.7  )
                    #                         legend(45, 1.09, c("DiseaseFree", "Recurred/Progressed"), lty = 2:3, cex=0.7, bty = 'n')
                    #                         
                    #                #     }
                    #                 }
                    #                 Title<- paste (ENV$StudyRefCase,"Kaplan-Meier fit", sep=": ")
                    #                 plotModel(plotCommand, title=Title,hscale=1 vscale=1)
                    #                 #plotCommand(s)
                    #                 #######
                    
                    
                    
                }else if(Coxph==1){
                    if(exists("age", envir=ENV)){
                        rm(age, envir = ENV)
                    }
                    dialogOptionGSEAlm(s,ClinicData)
                    n<- length(names(table(ClinicData[ENV$curselect])))
                    
                    #if(length(grep("^AGE$", names(ClinicData),ignore.case = TRUE))==1){
                        
                        ttAge <- tktoplevel()
                        tktitle(ttAge) <- "Choose numeric variable"
                        Age_Entry  = tclVar(median(ClinicData[,ENV$curselect], na.rm=TRUE))
                        ScaleAge <- tkscale(ttAge,length=200,from=min(as.numeric(ClinicData[,ENV$curselect]), na.rm=TRUE),to=max(as.numeric(ClinicData[,ENV$curselect]), na.rm = TRUE),showvalue=TRUE,
                                            variable=Age_Entry,resolution=1,orient='horiz')
                        tkgrid(ScaleAge)
                        
                        OkOn<-function(){
                            
                            #Entry Threshold of miRNA correlation
                            ENV$age <- as.numeric(tclvalue(Age_Entry))
                            tkdestroy(ttAge) 
                            
                        }
                        
                        Ok.but <- tkbutton(ttAge, text= "OK", command= OkOn)
                        
                        tkgrid(Ok.but)
                        tkwait.window(ttAge)
                        #tkgrid.configure(Ok.but, sticky="n", column=0)
                
                        #fit a Cox proportional hazards model and plot the  
                        #predicted survival for a 60 year old 
                        par(mfrow=c(1,2))
                        fit<- survival::coxph(ENV$variable, data= ENV$ClinicData, method="breslow", na.action=na.exclude)
                    
                    #ClinicData <- ENV$ClinicData
                        #title <- paste("Cox: Predicted survival for a",ENV$age, names(ClinicData[ENV$curselect]))
                        plot(survfit(fit, newdata=data.frame(ClinicData[ENV$curselect]== ENV$age)),xlab = "OS_MONTHS", ylab="OS_STATUS: Survival", main=ENV$StudyChoice[s], cex=1, col=1:n, cex.main = 0.8, cex.lab= 0.7, cex.axis= 0.7) 
                        
                        #legend("bottomleft", c(names(table(ClinicData[ENV$curselect]))),col=1:n, lty = 2:3, cex=0.7, bty = 'n')
                        text(60,1.0,paste("Cox: Predicted survival for a",ENV$age, names(ClinicData[ENV$curselect])), cex=0.65)
                        
                        res <- survival::cox.zph(fit)
                        ##capture results 
                        resCap <- capture.output(print(res))
                        plot(res,main= resCap, cex.main = 0.8, cex.lab= 0.7, cex=0.6,cex.axis= 0.7 ) # plot curves
                        #text(0, 0,main= resCap, cex=0.7)
#                     }else{
#                         msgNoAGE <- "There is no AGE data"
#                         tkmessageBox(message=msgNoAGE, icon="warning")
#                         stop(msgNoAGE)
#                     } 
                    #tkwait.window(ttAge)
                }
                
            }else{
                msgNoData <- "There is no \n OS_STATUT or OS_MONTHS or DFS_STATUS data"
                tkmessageBox(message=msgNoData, icon="warning")
                stop(msgNoData)
            }
        }
    }
}

#' get specific Mutation data for multiple genes
#' @usage
#' getSpecificMut()
#'
#' @return a a dataframe with specific mutation informations
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata//ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getSpecificMut()
#' }
#' 
getSpecificMut <- function(){
    
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    
        testCheckedCaseGenProf()
    
    
        
        Lchecked_Studies <- ENV$lchecked_Studies
        Lchecked_Cases <- length(ENV$curselectCases)
        Lchecked_GenProf <- length(ENV$curselectGenProfs)
        ###########################################################
        MutData=0
        MutData_All <-NULL
        MutDataSub<-0
        MutDataSub_All <- NULL
        
        for(c in 1:Lchecked_Cases){
            
            GenProf <- ENV$GenProfsRefStudies[ENV$curselectGenProfs[c]]
            Study_id <- ENV$CasesRefStudies[ENV$curselectCases[c]]
            
            MutData <- getDataByGenes(
                api = ENV$cgds,
                studyId = Study_id,
                genes = ENV$GeneList,
                by = "hugoGeneSymbol",
                molecularProfileIds = GenProf) |>
                unname() |>
                as.data.frame() |>
                select(-c("uniqueSampleKey", "uniquePatientKey", "molecularProfileId", "sampleId", "studyId"))
            
            if(length(MutData[,1])==0){
                msgNoMutData=paste("No Mutation Data are Available for\n", 
                                   ENV$CasesStudies[ENV$curselectCases[c]+1])
                tkmessageBox(message=msgNoMutData, 
                             title= paste(ENV$StudyRefCase[c],
                             ENV$CasesStudies[ENV$curselectCases[c]+1], 
                             ENV$GenProfsStudies[ENV$curselectCases[c]+1], sep=": "))
                
                
            } else{
                
                dialogSpecificMut(MutData, c)
                
                
                
            }
            
        }
    
}

#' Get Genetic Profile from selected Studies
#' @usage getGenProfs()
#' @return dataframe with genetic profil
#' @examples
#' 
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
#' @export
getGenProfs <- function(){
    
    #get Study Index
    #StudiesRef <- ENV$Studies |> pull("studyId")

    ## and we need the cases list of every study
    #ENV$checked_Studies_forGenProf <- ENV$checked_Studies
    #ENV$lchecked_Studies_forGenProf <- length(ENV$checked_Studies_forGenProf)

   police <- tkfont.create(family="arial", size=11)
   tkconfigure(ENV$tl, foreground="black", font=police)

    GenProfsStudies <- 0
    GenProfsRefStudies <- NULL
    for (i in seq(length(ENV$checked_StudyIndex))){

        Si = ENV$checked_StudyIndex[i]

        tkinsert(ENV$tl,"end", paste("***** Study ", Si ," : ", ENV$Studies$name[Si],"******"))
        
        ENV$GenProfs[[i]] <- cBioPortalData::molecularProfiles(ENV$cgds, ENV$checked_Studies_id[i])
        
        ENV$n_GenProfs[i] <- nrow(ENV$GenProfs[[i]])

        print(paste0("There are ", ENV$n_GenProfs[i], " Genetics Profiles in ", ENV$checked_Studies_id[i]))

        # create progress bar
        progressBar_GenProfs <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                              max = ENV$n_GenProfs[i], width = 400)
        j=0
        GenProfsStudy <- 0
        GenProfRefStudy <- NULL
        GenProfsRefStudy <- NULL
        for(j in seq(ENV$n_GenProfs[i])){

            Sys.sleep(0.1)
            setTkProgressBar(progressBar_GenProfs, j,
                             label=paste(round(j/ENV$n_GenProfs[i]*100, 0),
                                          "% of Genetic Profiles"))

            GenProfStudy <- ENV$GenProfs[[i]][["description"]][j]
            #GenProfStudy <- getGeneticProfiles.CGDS(ENV$cgds, ENV$checked_Studies_forGenProf[i])[,2][j]
            GenProfRefStudy <- ENV$GenProfs[[i]][["molecularProfileId"]][j]
            #GenProfRefStudy <- getGeneticProfiles.CGDS(ENV$cgds, ENV$checked_Studies_forGenProf[i])[,1][j]

            tkinsert(ENV$tl,"end", paste(j,":", GenProfStudy))
            
            GenProfsStudy <- cbind(GenProfsStudy, GenProfStudy)
            GenProfsRefStudy <- cbind(GenProfsRefStudy, GenProfRefStudy)
        }
        close(progressBar_GenProfs)

        GenProfsStudies <- cbind(GenProfsStudies, GenProfsStudy)
        GenProfsRefStudies <- cbind(GenProfsRefStudies, GenProfsRefStudy)
    }
    ENV$GenProfsRefStudies <- GenProfsRefStudies
    ENV$GenProfsStudies <- GenProfsStudies
}

#' about canceR
#' @usage about()
#' @return dialig box with text
#' @export
#'
#' @examples
#' \dontrun{
#' about()
#'}
#'


about <-
    function() {
        
        fontinfo <- tkfont.create(family="times",size=11)
        fontinfo2 <- tkfont.create(family="times",size=13,weight="bold")
        
        abt <- tktoplevel()
        text1 <-tkframe(abt ,borderwidth=2)
        text2 <-tkframe(abt ,borderwidth=2)
        text3 <-tkframe(abt ,borderwidth=2)
        
        
        tkwm.title(abt ," About   ")
        tkgrid(tklabel(abt ,text="    About   ",font=fontinfo2))
        
        tkgrid(tklabel(text1 ,text=" The canceR package is developed by Karim Mezhoud Team (kmezhoud@gmail.com) 
 aimed to make easier access to the Cancer Data hosted by the Computational Biology Center 
at Memorial-Sloan-Kettering Cancer Center (MSKCC) at http://www.cbioportal.org/.",font=fontinfo))
        
        tkgrid(tklabel(text2 ,text=" The canceR package is a Graphical User Interface (GUI) developed for users 
with no or limited knowledge about R programming so they can run all functions from cgdsr package
and perform modeling analysis depending on others packages (see depends).",font=fontinfo))
        
        tkgrid(tklabel(text3 ,text=" Use the cbioportal@googlegroups.com mailing list to obtain additional support.",font=fontinfo))
        #tkgrid(tklabel(text3 ,text=" More detail information of the canceR project can be found in: http://www.bioinformatics.tn",font=fontinfo))
        
        tkgrid.configure(text1 ,sticky="new")
        tkgrid.configure(text2 ,sticky="new")
        tkgrid.configure(text3 ,sticky="new")
        
    }

#' get example of .gmt file from MSigDB (Broad Institute)
#' @usage getMSigDBExample()
#' @return path of GMT file
#' @export
#' @examples 
#'  readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#'  \dontrun{
#'  getMSigDBExample()
#'  }
getMSigDBExample <- function(){
    
    ifrm <- function(obj, env) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    ifrm(MSigDB, ENV)
    ifrm(MSigDBFile,ENV)
    ifrm(fname.MSigDB,ENV)
    
    
    police <- tkfont.create(family="arial", size=10)
    ttMSigDB <- tktoplevel()
    
    tktitle(ttMSigDB) <- paste("Select MSigDB")
    
    
    yscr1 <- tkscrollbar(ttMSigDB, repeatinterval=2,
                         command=function(...)tkyview(tl1,...))
    xscr1 <- tkscrollbar(ttMSigDB, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl1,...))
    
    xscr1Info <- tkscrollbar(ttMSigDB, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl1info,...))
    
    
    tl1<-tklistbox(ttMSigDB,height=10, width= 30 ,selectmode="single",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
    tl1info<-tklistbox(ttMSigDB,height=1, width= 30,selectmode="single",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
    tkconfigure(tl1info, foreground="blue", font=police)
    
    
    loadMSigDB <- function()
    {  
        curselectMSigDB <- as.numeric(tkcurselection(tl1))+1
        lcurselectMSigDB <- length(curselectMSigDB)
        
        fname.MSigDB <-  list.files(paste(path.package("canceR"),"/extdata/MSigDB", sep=""))[curselectMSigDB]
        ENV$fname.GMT <- paste(path.package("canceR"),"/extdata/MSigDB/", fname.MSigDB,sep="")
        
        if (lcurselectMSigDB ==0){
            msgSelectMSigDB<-"Select MSigBD"
            tkmessageBox(message=msgSelectMSigDB)
        } else {
            tkdelete(tl1info,0,1)
            tkinsert(tl1info,"end",fname.MSigDB)
            tkinsert(ENV$tlGMT,"end",ENV$fname.GMT)
            tkfocus(ENV$ttDialogGSEA)
            tkdestroy(ttMSigDB)
        }
        
    }
    
    
    Select.but <-tkbutton(ttMSigDB,text="select",command=loadMSigDB)
    
    
    tkgrid(tl1,yscr1)
    tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
    tkgrid(xscr1)
    tkgrid.configure(xscr1,rowspan=1,sticky="we")
    tkgrid(tl1info, columnspan=1)
    tkgrid(xscr1Info)
    tkgrid.configure(xscr1Info,rowspan=4,sticky="we")
    tkgrid(Select.but)
    
    for (i in 1:length(list.files(paste(path.package("canceR"),"/extdata/MSigDB", sep="")))){
        
        tkinsert(tl1,"end",list.files(paste(path.package("canceR"),"/extdata/MSigDB", sep=""))[i])
    }
}

#' Dialog bos to set returned Mutation information
#' @usage
#' dialogMut(title, question, entryInit, entryWidth = 40,returnValOnCancel = "ID_CANCEL")
#' @param title title of the table
#' @param question question
#' @param entryInit entryInit
#' @param entryWidth 40
#' @param returnValOnCancel "ID_CANCEL"
#'
#' @return a check box with mutations variables
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' dialogMut("title", "question", "entryInit", entryWidth = 40, returnValOnCancel = "ID_CANCEL")
#' }
dialogMut <- function(title, question, entryInit, entryWidth = 40,
                      returnValOnCancel = "ID_CANCEL") {
    dlg <- tktoplevel()
    tkwm.deiconify(dlg)
    tkgrab.set(dlg)
    tkfocus(dlg)
    tkwm.title(dlg, title)
    
    textEntryVarTcl <- tclVar(paste(entryInit))
    textEntryWidget <- tkentry(dlg, width = paste(entryWidth),
                               textvariable = textEntryVarTcl)
    tkgrid(tklabel(dlg, text = "       "))
    tkgrid(tklabel(dlg, text = question), textEntryWidget)
    tkgrid(tklabel(dlg, text = " *\n .\n |",justify = "right"), column=0, row=2, sticky="n")
    tkgrid(tklabel(dlg, text = " several time the last character\n any character or number\n or", justify = "left"), column=1,row=2, sticky="w")
    ReturnVal <- returnValOnCancel
    
    onOK <- function() {
        ENV$ReturnVal <- tclvalue(textEntryVarTcl)
        tkgrab.release(dlg)
        tkdestroy(dlg)
        
        #tkfocus(ttMain)
    }
    onCancel <- function() {
        ENV$ReturnVal <- returnValOnCancel
        tkgrab.release(dlg)
        tkdestroy(dlg)
        return(ReturnVal)
        #tkfocus(ttMain)
    }
    OK.but <- tkbutton(dlg, text = "   OK   ", command = onOK)
    Cancel.but <- tkbutton(dlg, text = " Cancel ", command = onCancel)
    tkgrid(Cancel.but,OK.but, column=1, sticky="w")
    tkgrid.configure(Cancel.but, column=1, sticky="n")
    #tkgrid(tklabel(dlg, text = "    "))
    
    tkfocus(dlg)
    tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg); tkfocus(ENV$ttCasesGenProfs)})
    tkbind(textEntryWidget, "<Return>", onOK)
    tkwait.window(dlg)
    
    return(ENV$ReturnVal)
}

#' bind non equal row
#' 
#' @usage rbind.na(..., deparse.level = 1) 
#' @param ...  (generalized) vectors or matrices. 
#' @param deparse.level  integer controlling the construction of labels in 
#' the case of non-matrix-like arguments (for the default method):
#' deparse.level = 0 constructs no labels; the default,
#' deparse.level = 1 or 2 constructs labels from the argument names.
#' 
#'
#' @return a data frame with merged rows
#'
#' @examples
#' \dontrun{
#' row1 <- c("a","b","c","d")
#' row2 <- c("A", "B", "C")
#' row3 <- rbind.na(row1, row2)
#'}
#'
#'@export
#'
rbind.na <- function(..., deparse.level = 1) 
{
    na <- nargs() - (!missing(deparse.level))
    deparse.level <- as.integer(deparse.level)
    stopifnot(0 <= deparse.level, deparse.level <= 2)
    argl <- list(...)
    while (na > 0 && is.null(argl[[na]])) {
        argl <- argl[-na]
        na <- na - 1
    }    
    if (na == 0) 
        return(NULL)
    if (na == 1) {
        if (isS4(..1)) 
            return(rbind2(..1))
        else return(matrix(..., nrow = 1)) ##.Internal(rbind(deparse.level, ...)))
    }
    if (deparse.level) {
        symarg <- as.list(sys.call()[-1L])[1L:na]
        Nms <- function(i) {
            if (is.null(r <- names(symarg[i])) || r == "") {
                if (is.symbol(r <- symarg[[i]]) || deparse.level == 
                  2) 
                  deparse(r)
            }
            else r
        }
    }
    
    ## deactivated, otherwise no fill in with two arguments
    if (na == 0) {
        r <- argl[[2]]
        fix.na <- FALSE
    }
    else {
        nrs <- unname(lapply(argl, ncol))
        iV <- sapply(nrs, is.null)
        fix.na <- identical(nrs[(na - 1):na], list(NULL, NULL))
        ## deactivated, otherwise data will be recycled
        #if (fix.na) {
        #    nr <- max(if (all(iV)) sapply(argl, length) else unlist(nrs[!iV]))
        #    argl[[na]] <- rbind(rep(argl[[na]], length.out = nr), 
        #        deparse.level = 0)
        #}
        if (deparse.level) {
            if (fix.na) 
                fix.na <- !is.null(Nna <- Nms(na))
            if (!is.null(nmi <- names(argl))) 
                iV <- iV & (nmi == "")
            ii <- if (fix.na) 
                2:(na - 1)
            else 2:na
            if (any(iV[ii])) {
                for (i in ii[iV[ii]]) if (!is.null(nmi <- Nms(i))) 
                  names(argl)[i] <- nmi
            }
        }
        
        ## filling with NA's to maximum occuring ncols
        nCol <- as.numeric(sapply(argl, function(x) if (is.null(ncol(x))) length(x)
                                                    else ncol(x)))
        maxCol <- max(nCol, na.rm = TRUE)  
        argl <- lapply(argl, function(x)  if (is.null(ncol(x))) c(x, rep(NA, maxCol - length(x)))
                                          else cbind(x, matrix(, nrow(x), maxCol - ncol(x))))  
        
        ## create a common name vector from the
        ## column names of all 'argl' items
        namesVEC <- rep(NA, maxCol)  
        for (i in 1:length(argl)) {
          CN <- colnames(argl[[i]])          
          m <- !(CN %in% namesVEC)
          namesVEC[m] <- CN[m]          
        }  
        
        ## make all column names from common 'namesVEC'
        for (j in 1:length(argl)) {    
          if (!is.null(ncol(argl[[j]]))) colnames(argl[[j]]) <- namesVEC
        }
        
        r <- do.call(rbind, c(argl[-1L], list(deparse.level = deparse.level)))        
    }
    
    d2 <- dim(r)
    
    ## make all column names from common 'namesVEC'
    colnames(r) <- colnames(argl[[1]])
    
    r <- rbind2(argl[[1]], r)
        
    if (deparse.level == 0) 
        return(r)
    ism1 <- !is.null(d1 <- dim(..1)) && length(d1) == 2L
    ism2 <- !is.null(d2) && length(d2) == 2L && !fix.na
    if (ism1 && ism2) 
        return(r)
    Nrow <- function(x) {
        d <- dim(x)
        if (length(d) == 2L) 
            d[1L]
        else as.integer(length(x) > 0L)
    }
    nn1 <- !is.null(N1 <- if ((l1 <- Nrow(..1)) && !ism1) Nms(1))
    nn2 <- !is.null(N2 <- if (na == 2 && Nrow(..2) && !ism2) Nms(2))
    if (nn1 || nn2 || fix.na) {
        if (is.null(rownames(r))) 
            rownames(r) <- rep.int("", nrow(r))
        setN <- function(i, nams) rownames(r)[i] <- if (is.null(nams)) 
            ""
        else nams
        if (nn1) 
            setN(1, N1)
        if (nn2) 
            setN(1 + l1, N2)
        if (fix.na) 
            setN(nrow(r), Nna)
    }
    r
}

#' get Gene List from examples. User can select one from available gene list
#' @usage getGeneListExample()
#' @return Gene list path of file
#' @export
#' @examples
#' ENV <- new.env(parent = emptyenv())
#' \dontrun{
#' getGeneListExample() 
#' }
getGeneListExample <- function(){
    
    
    ifrm <- function(obj, env = globalenv()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    ifrm(ENV$GeneList)
    ifrm(ENV$GeneListfile)
    ifrm(ENV$fname.GeneList)
    
    
    police <- tkfont.create(family="arial", size=10)
    ttGeneList <- tktoplevel()
    
    
    tktitle(ttGeneList) <- paste("Select Gene list")
    
    
    # Take LABELS in ttGeneList
    label1 <- tklabel(ttGeneList, text= "Survival\n Event(Living/Deceased) \n and Time (Months)")
    
    
    yscr1 <- tkscrollbar(ttGeneList, repeatinterval=2,
                         command=function(...)tkyview(tl1,...))
    xscr1 <- tkscrollbar(ttGeneList, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl1,...))
    
    xscr1Info <- tkscrollbar(ttGeneList, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl1info,...))
    
    
    tl1<-tklistbox(ttGeneList,height=10, width= 30 ,selectmode="single",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
    tl1info<-tklistbox(ttGeneList,height=1, width= 30,selectmode="single",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
    tkconfigure(tl1info, foreground="blue", font=police)
    
    
    loadGeneList <- function()
    {  
        curselectGeneList <- as.numeric(tkcurselection(tl1))+1
        lcurselectGeneList <- length(curselectGeneList)
        
        fname.GeneList <-  list.files(paste(path.package("canceR"),"/extdata/GeneList", sep=""))[curselectGeneList]
        ENV$GeneListfile <- paste(path.package("canceR"),"/extdata/GeneList/", fname.GeneList,sep="")
        if (lcurselectGeneList ==0){
            msgSelectGeneList<-"Select Gene list"
            tkmessageBox(message=msgSelectGeneList)
        } else {
            tkdelete(tl1info,0,1)
            tkinsert(tl1info,"end",fname.GeneList)
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            GeneList<-unique(read.table(ENV$GeneListfile))
            ENV$GeneList <- t(GeneList)
            tkmessageBox(message = paste("The file selected is", basename(fname.GeneList),"with", length(t(GeneList))," genes"),icon="info")
            tkfocus(ttGeneList)
            tkdestroy(ttGeneList)
        }
        
    }
    
    
    Select.but <-tkbutton(ttGeneList,text="select",command=loadGeneList)
    
    
    tkgrid(tl1,yscr1)
    tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
    tkgrid(xscr1)
    tkgrid.configure(xscr1,rowspan=1,sticky="we")
    tkgrid(tl1info, columnspan=1)
    tkgrid(xscr1Info)
    tkgrid.configure(xscr1Info,rowspan=4,sticky="we")
    tkgrid(Select.but)
    
    for (i in 1:length(list.files(paste(path.package("canceR"),"/extdata/GeneList", sep="")))){
        
        tkinsert(tl1,"end",list.files(paste(path.package("canceR"),"/extdata/GeneList", sep=""))[i])
    }
}

#mSigDB<- list(set1=c("a","b","d","x"), set2=c("b","c","q","m"), set3=c("b","f","e","k","q","h"))
## Remove gene in duplicates
#sapply(1:length(MSigDB),function(x){MSigDB[[x]][as.numeric(table(unlist(MSigDB))[MSigDB[[x]]])==1]})
#MSigDB <- readLines("/home/mezhoud/CGDS-R/canceRdev/inst/extdata/MSigDB/c5.bp.v4.0.symbols.gmt")

#' get gene list from MSigDB
#' @usage
#' getGeneListFromMSigDB()
#' 
#' @return a vector with gene list
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcgaGSEAlm1021.rds", sep=""))
#' \dontrun{
#' getGeneListFromMSigDB()
#'}
#'
getGeneListFromMSigDB <- function(){
    if(exists("GeneListMSigDB", envir = ENV)){
        rm("GeneListMSigDB", envir = ENV)
        rm("GeneList", envir = ENV)
    }
    
    getMSigDBfile()
    MSigDB <- ENV$mSigDB
    
    #split each line at the tabs
    MSigDB<-strsplit(MSigDB,'\t')
    #use the first entry of each gene set as name
    names(MSigDB)<-sapply(MSigDB,function(x)x[1])
    #remove the first two entries
    MSigDB<-sapply(MSigDB,function(x)x[3:length(x)])
    
    
    ## Filtring interesting/Wanted GeneSet before 
    ## removing duplicate genes between Gene Sets
    # 
    # regex <- c("DNA_DAMAGE_CHECKPOINT",
    #            "DNA_INTEGRITY_CHECKPOINT",
    #            "MITOTIC_CELL_CYCLE_CHECKPOINT",
    #            "DNA_REPAIR",
    #            "NEGATIVE_REGULATION_OF_PROGRAMMED_CELL_DEATH",
    #            "INDUCTION_OF_APOPTOSIS_BY_INTRACELLULAR_SIGNALS", 
    #            "DNA_DAMAGE_RESPONSESIGNAL_TRANSDUCTION_RESULTING_IN_INDUCTION_OF_APOPTOSIS",
    #             "NEGATIVE_REGULATION_OF_APOPTOSIS",
    #             "RESPONSE_TO_OXIDATIVE_STRESS",
    #            "STRESS_ACTIVATED_PROTEIN_KINASE_SIGNALING_PATHWAY",
    #            "NEGATIVE_REGULATION_OF_CELL_ADHESION",
    #            "REGULATION_OF_CELL_CELL_ADHESION",
    #            "POSITIVE_REGULATION_OF_CELL_ADHESION"
    #            )
    dialoggetGeneListMSigDB(MSigDB)
    regex <- ENV$regex
    
    SubMSigDB <- 0
    for( i in 1:length(regex)){
        
        SubMSigDB_regex <- MSigDB[grep(regex[i],names(MSigDB), ignore.case = TRUE)]
        SubMSigDB <- c( SubMSigDB,SubMSigDB_regex)
    }
    SubMSigDB <- SubMSigDB[-1]
    MSigDB <- SubMSigDB
    
    ## remove duplicates genes from Gene Sets:
    ## any gene must belong to one gene Set. First one.
    GeneListMSigDB <- sapply(1:length(MSigDB),function(x){
        c <- list() 
        for (j in 1:x){
            c <- c(c,MSigDB[[j]])}
        MSigDB[[x]][table(unlist(c))[MSigDB[[x]]]==1]
        
    })
    
    names(GeneListMSigDB) <- names(MSigDB)
    
    GeneListMSigDB <- as.data.frame(unlist(GeneListMSigDB[sapply(GeneListMSigDB,length)>0]))
    
    rnames <- gsub("[[:digit:]]", "", rownames(GeneListMSigDB))
    GeneListMSigDB <- cbind(rnames, GeneListMSigDB)
    rownames(GeneListMSigDB) <- NULL
    colnames(GeneListMSigDB) <- c("GeneSet","Gene")
    ENV$GeneListMSigDB <- GeneListMSigDB
    ENV$GeneList <- t(as.character(GeneListMSigDB[,2]))
}

#' get GSEA linear modeling by variables (phenotype)
#' @usage
#' getGSEAlm_Variables()
#'
#' @return a dataframe with annotation (GO, BP)
#' @export
#'
#' @examples
#'  x <- 3
#' \dontrun{
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' getGSEAlm_Variables()
#' }
getGSEAlm_Variables <-function(){
    
    ##Remove "GenesDetails" objectf if exists
    ifrm <- function(obj, env = ENV()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    
    ####Create a new directory "Results/GSEAlm" for output
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/")){
        dir.create(file.path(paste(getwd(), "/Results", sep="")), showWarnings = FALSE)
        dir.create(file.path(paste(getwd(), "/Results/GSEAlm", sep="")), showWarnings = FALSE)
    } else if (!file.exists("Results/GSEAlm/")){
        dir.create(file.path(paste(getwd(), "/Results/GSEAlm", sep="")), showWarnings = FALSE)
    }
    
    
    
        testCheckedCaseGenProf()
        
        Lchecked_Studies <- ENV$lchecked_Studies_forCases
        Lchecked_Cases <- length(ENV$curselectCases)
        Lchecked_GenProf <- length(ENV$curselectGenProfs)
        
        ProfDataAll=0
        ProfData=0
        LengthGenProfs=0
        LengthCases=0
        for (i in 1:Lchecked_Studies){
            
            Si =ENV$checked_StudyIndex[i]
            progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies[Si], min = 0,
                                                      max = Lchecked_GenProf, width = 400)
            
            
            LastLengthGenProfs <- LengthGenProfs
            LengthGenProfs <- LengthGenProfs + ENV$LGenProfs[i]+1
            LastLengthCases <- LengthCases
            LengthCases<- LengthCases + ENV$LCases[i]+1
            
            
            
            for (k in 1:Lchecked_Cases){
                
                ## Selecting GeneList from getGeneList.R or getGenesClasses.R functions.
                ifExist <- function(obj, env = ENV()) {
                    obj <- deparse(substitute(obj))
                    if(exists(obj, envir = env)) {
                        
                        ENV$GeneList <- rownames(ENV$GenesDetails[[k]])
                        ENV$GeneListfile <- paste(names(ENV$GenesDetails)[k]) 
                    }
                }
                
                
                
                Sys.sleep(0.1)
                setTkProgressBar(progressBar_ProfilesData, k, label=paste( round(k/Lchecked_GenProf*100, 0),
                                                                           "% of Expression Set"))
                
                if (ENV$curselectGenProfs[k] <= LengthGenProfs && ENV$curselectGenProfs[k]>LastLengthGenProfs){    
                    
                    GenProf<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                    
                    Case<-ENV$CasesRefStudies[ENV$curselectCases[k]]
                 
                    
                    if(length(ENV$GeneList)>500){
                        ProfData <- getMegaProfData(ENV$GeneList,k )
                    } else{
                        ProfData<-getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                    }
                    
                    if(ncol(ProfData)==0){
                        tkmessageBox(message=paste("There is no mRNA expression data for", ENV$GenProfsStudies[k]),icon="info")
                        break(paste("There is no mRNA expression data for",ENV$GenProfsStudies[k]))
                        close(progressBar_ProfilesData)
                        
                    }
                    
                    ##Convert data frame to numeric structure
#                     print("converting data frame of Profile data to numeric stucture...")
#                     
#                     cidx <- !(sapply(ProfData, is.numeric))
#                     ProfData[cidx] <- lapply(ProfData[cidx], as.numeric)
                    
                    #for(p in 1:ncol(ProfData)){
                        
                     #   ProfData[,i] <- as.numeric(ProfData[,p])
                    #}
                    
                    
                    if(!exists(deparse(substitute(ENV$GenesDetails)),envir = ENV)) {
                        
                        ## Display AssyData with Tcl Table
                        title <- paste(ENV$StudyRefGenProf[k],":",ENV$GenProfChoice[k])
                        ProfData <- t(t(ProfData))
                        getInTable(ProfData, title)
                        
                        
                        
                    }
                    
                    #####getClinicData_MultipleCases function
                    Case<- ENV$CasesRefStudies[ENV$curselectCases[k]]
                    
                    ClinicalData<-getClinicalData(ENV$cgds,Case)
                    
                    
                    if(length(ClinicalData[1,])==0){
                        msgNoClinData=paste("No Clinical Data are Available for\n", ENV$CasesStudies[ENV$curselectCases[k]+1])
                        tkmessageBox(message=msgNoClinData, title= paste("Study: ",ENV$StudyRefCase[k]))
                        close(progressBar_ProfilesData)
                        break
                    } 
                    
                    
                    if(!exists(deparse(substitute(ENV$GenesDetails)), envir = ENV)) {
                        ## Display AssyData with Tcl Table
                        title<-paste(ENV$StudyRefCase[k],ENV$CasesStudies[ENV$curselectCases[k]+1], sep=": ")
                        getInTable(ClinicalData,title)
                        
                    } 
                    
                    ## Select only Cases (rownames) that exist in ClinicalDataSub and ProfData
                    merge <- merge(ClinicalData, ProfData, by="row.names")
                    print("merging Clincal and Profile Data")
                    ClinicalData<- merge[,1:(length(ClinicalData)+1)]
                    
                    
                    rownames(ClinicalData)<- ClinicalData[,1]
                    ClinicalData <- ClinicalData[-1]
                    ProfData<-merge[,!(merge %in% ClinicalData)]
                    
                    #ProfData <- ProfData[-1]
                    AssayData<- t(ProfData)
                    colnames(AssayData) <- AssayData[1,]
                    AssayData <- AssayData[-1,]
                    rnames <- rownames(AssayData) 
                    AssayData <- as.matrix(apply(AssayData,2 ,function(x) as.numeric(x)))
                    rownames(AssayData) <- rnames
                    
                    ENV$ClinicalData <- ClinicalData
                    ENV$ProfData <- ProfData
                    ENV$AssayData <- AssayData
                    
                    #Test if the same length cases for phenoData and AssayData
                    if (all(rownames(ENV$ClinicalData)==colnames(ENV$AssayData))){
                        
                        
                        
                        ## create labelDescription for columns of phenoData. 
                        ## labeldescription is used by Biobase packages
                        ## In our case labelDescription is Equal to column names
                        metaData <- data.frame(labelDescription= colnames(ENV$ClinicalData), row.names=colnames(ENV$ClinicalData))        ## Bioconductor’s Biobase package provides a class called AnnotatedDataFrame   
                        ##that conveniently stores and manipulates 
                        ##the phenotypic data and its metadata in a coordinated fashion. 
                        phenoData<-new("AnnotatedDataFrame", data=ENV$ClinicalData, varMetadata=metaData)    
                        
                        print("getting eSet for selected Profileand Clinical Data ...")
                        ##Assembling an ExpressionSet  
                        eSet<-Biobase::ExpressionSet(assayData=ENV$AssayData, phenoData=phenoData, annotation="GO")  
                        
                        ##translate Negative to positive value
                        if(min(Biobase::exprs(eSet), na.rm=TRUE)<0){
                            print("There are negative values. Translating values by adding the absolute of minimum value to all matrix")                  
                            Biobase::exprs(eSet) <- Biobase::exprs(eSet) +(abs(min(Biobase::exprs(eSet), na.rm=TRUE)))
                        }
                        ENV$eSet <- eSet
                        
                        
                        ## create MSigDb for eSet
                        getMSigDB(ENV$eSet,1)
                        
                        #run GSEAlm
                        #nperm determines the number of permutations used to estimate the null distribution
                        #of the enrichment score
                        
                        
                        dialogOptionGSEAlm(k, ENV$ClinicalData)
                        print("dialog Option of GSEAlm:  OK")
                        
                        ## test if Covariables works together else get message box
#                         if (inherits(try(pVals <-GSEAlm::gsealmPerm(ENV$eSet,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE), silent=TRUE),"try-error"))
#                         {
#                             msgBadCovariables <- paste("There is incompatible variables. Select an other Formula.")
#                             tkmessageBox(message=msgBadCovariables, icon="warning")
#                             close(progressBar_ProfilesData)
#                             stop(msgBadCovariables)
#                         } else{ 
#                             print("Computing of pVals using gsealmPerm function ...")
#                             pVals <-GSEAlm::gsealmPerm(ENV$eSet,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE)
#                         }
                        
                        print("Computing of pVals using gsealmPerm function ...")
                        pVals <- gsealmPerm(ENV$eSet,ENV$coVariables,ENV$mSigDB_forGeneList,nperm= ENV$permutVal, na.rm=TRUE)
                        

                       print(paste("End computing of pVals of:", ENV$StudyRefCase[k]))
                        # we have to correct for multiple testing. In this case we use the method from Benjamini-Hochberg
                        pVals <- apply(pVals, 2, p.adjust, method = "BH", n = nrow(pVals))
                        
                        #pVal_Permut <- paste(names(table(pData(eSetClassifier)))[2])
                        ##class2 is from dialogOptionGSEAlm
                        colnames(pVals) <- c(paste("Down Regulated in",ENV$coVariables[-1],ENV$class2),paste("Up Regulated in",ENV$coVariables[-1],ENV$class2,"nperm:",ENV$permutVal,"pVal:",ENV$seuilpVal))
                        
                        if(exists(deparse(substitute(ENV$GenesDetails)), envir = ENV)) {
                            print(ENV$GenesDetails)
                            assign(paste("pVals", gsub(".RData","",ENV$mSigDB_SubName),sep="_"),pVals, envir=ENV)
                            workspace <- getwd()
                            setwd("./Results/GSEAlm")
                            
                            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                            if(!file.exists(ENV$StudyRefCase[k])){
                                dir.create(file.path(paste(getwd(),"/",ENV$StudyRefCase[k] , sep="")), showWarnings = FALSE)
                            }
                            setwd(paste("./",ENV$StudyRefCase[k], sep=""))
                            
                            
                            write.table(pVals, paste("pVals_", gsub(".RData","",ENV$mSigDB_SubName),".txt", sep=""), sep="\t")
                            #set a significance threshold
                            THRESHOLD<-ENV$seuilpVal
                            #gene sets that are downregulated in the BLC
                            downRegulated <- data.frame(sort(pVals[pVals[, 1]< THRESHOLD,1]))
                            names(downRegulated) <- paste("Down Regulated in",ENV$coVariables[-1],ENV$class2,"pVal<",THRESHOLD)
                            #colnames(pVals) <- c(paste("Down Regulated\n",pVal_Permut),paste("Up Regulated\n",pVal_Permut))
                            
                            assign(paste("downRegulated","GeneClass" ,gsub(".RData","",ENV$mSigDB_SubName), sep="_"),downRegulated, envir=ENV)
                            #save downRegulted table
                            write.table(downRegulated,"_GeneClass_",paste("downRegulated_", gsub(".RData","",ENV$mSigDB_SubName),".txt", sep=""), sep="\t") 
                            
                            #gene sets that are upregulated in basal breast cancer
                            upRegulated <- data.frame(sort(pVals[pVals[, 2] < THRESHOLD, 2]))
                            #colnames(upRegulated[1]) <- paste("Up Regulated\n", seuilpVal, pVal_permut, coVariables)
                            names(upRegulated) <- paste("Up Regulated in",ENV$coVariables[-1],ENV$class2,"pVal<",THRESHOLD)
                            assign(paste("upRegulated","GeneClass" ,gsub(".RData","",ENV$mSigDB_SubName),sep="_"),upRegulated, envir=ENV)
                            #save upRegulated table
                            write.table(upRegulated,"_GeneClass_" ,paste("upRegulated_", gsub(".RData","",ENV$mSigDB_SubName),".txt", sep=""), sep="\t") 
                            
                            setwd(workspace)
                        }else{
                            assign(paste("pVals", gsub(".RData","",ENV$mSigDB_SubName),ENV$StudyRefCase[k] ,sep="_"),pVals, envir=ENV)       
                            workspace <- getwd()
                            setwd("./Results/GSEAlm")
                            ######################
                            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                            if(!file.exists(ENV$StudyRefCase[k])){
                                dir.create(file.path(paste(getwd(),"/",ENV$StudyRefCase[k] , sep="")), showWarnings = FALSE)
                            }
                            setwd(paste("./",ENV$StudyRefCase[k], sep=""))
                            
                            #################
                            write.table(pVals, paste("pVals_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$StudyRefCase[k],".txt", sep=""), sep="\t")
                            #set a significance threshold
                            THRESHOLD<-ENV$seuilpVal
                            #gene sets that are downregulated in the BLC
                            downRegulated <- data.frame(sort(pVals[pVals[, 1]< THRESHOLD,1]))
                            names(downRegulated) <- paste("Down Regulated in",ENV$coVariables[-1],ENV$class2,"pVal<",THRESHOLD)
                            #colnames(pVals) <- c(paste("Down Regulated\n",pVal_Permut),paste("Up Regulated\n",pVal_Permut))
                            assign(paste("downRegulated", gsub(".RData","",ENV$mSigDB_SubName), ENV$StudyRefCase[k],sep="_"),downRegulated, envir=ENV)
                            #save downRegulted table
                            write.table(downRegulated, paste("downRegulated_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$StudyRefCase[k],".txt", sep=""), sep="\t") 
                            #gene sets that are upregulated in basal breast cancer
                            upRegulated <- data.frame(sort(pVals[pVals[, 2] < THRESHOLD, 2]))
                            #colnames(upRegulated[1]) <- paste("Up Regulated\n", seuilpVal, pVal_permut, coVariables)
                            names(upRegulated) <- paste("Up Regulated in",ENV$coVariables[-1],ENV$class2,"pVal<",THRESHOLD)
                            assign(paste("upRegulated", gsub(".RData","",ENV$mSigDB_SubName), ENV$StudyRefCase[k],sep="_"),upRegulated, envir=ENV)
                            
                            #save upRegulted table
                            write.table(upRegulated, paste("upRegulated_", gsub(".RData","",ENV$mSigDB_SubName),"_",ENV$StudyRefCase[k],".txt", sep=""), sep="\t") 
                            
                            
                            setwd(workspace)
                            
                        }
                        ##Get results in Tcltk Table
                        
                        downRegulatedTab <- rbind(colnames(downRegulated), downRegulated)
                        downRegulatedTab <- cbind(rownames(downRegulatedTab),downRegulatedTab)
                        downRegulatedTab[,1]<- as.character(downRegulatedTab[,1])
                        downRegulatedTab[1,1] <- "***** Down Regulated Gene Sets *****" 
                        colnames(downRegulatedTab) <- NULL
                        
                        upRegulatedTab <- rbind(colnames(upRegulated), upRegulated)
                        upRegulatedTab <- cbind(rownames(upRegulatedTab),upRegulatedTab)
                        upRegulatedTab[,1]<- as.character(upRegulatedTab[,1])
                        upRegulatedTab[1,1] <- "***** Up Regulated Gene Sets *****"
                        colnames(upRegulatedTab) <- NULL
                        
                        UpDownRegulatedTable <- rbind.na(downRegulatedTab,upRegulatedTab)
                        getInTable(UpDownRegulatedTable, title=ENV$tudyRefCase[k])
                        
                    }else {tkmessageBox( message= "The expression Gene Set and the Clinical Data do not have the same samples", icon="warning")
                    }
                } 
            } 
            close(progressBar_ProfilesData)
        } 
    #}
    
    ##Remove GenesDetails objectf if exists
    ifrm(GenesDetails, ENV)
    
    
}

#' User needs to specify which gene is interesting to get genomic cancer data. The gene must be with Symbol and one gene by line.
#' @usage getGeneList()
#' @return Gene list path of file
#' @export
#' @examples
#' ENV <- new.env(parent = emptyenv())
#' \dontrun{
#' getGeneList()
#'  }
getGeneList <- function(){
   if(exists("GeneListMSigDB", envir=ENV)){
       rm("GeneListMSigDB", envir=ENV)
   }
       
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(exists("Genelistfile", envir = ENV)){
        rm(ENV$GeneList)
        rm(ENV$GeneListfile)
    }
    ENV$GeneListfile <- tclvalue(tkgetOpenFile(filetypes = "{{txt Files} {.txt}} {{All files} *}", title="Choose Gene List from File")) 
    print(ENV$GeneListfile)
    if (!nchar(ENV$GeneListfile)) {
        tkmessageBox(message = "No file was selected!")
        tkfocus(ENV$ttCasesGenProfs)
    } else {
        #Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        ENV$GeneList<-unique(read.table(ENV$GeneListfile))
        
       
        ENV$GeneList <- t(ENV$GeneList)
        tkmessageBox(message = paste("The file selected is", basename(ENV$GeneListfile),"with", length(t(ENV$GeneList))," genes"),icon="info")
        tkfocus(ENV$ttCasesGenProfs)
       
    }
    
}

#' Get cases for selected Studies. The Cases are the descrption of the samples from patients. The samples can be subdivided by the type of assays as, sequencing, CNA, Mutation, Methylation.
#' @usage 
#' getCases()
#' @export
#' @return a dataframe with cases
#' 
#' @examples
#' 
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
#'   
getCases <- function(){

    #get Study Index 
    #StudiesRef <- ENV$Studies |> pull("studyId")
    #checked_StudyIndex
    #ENV$checked_StudyIndex_forCases <- ENV$checked_StudyIndex
    
    ## and we need the cases list of every study
    #checked_Studies_forCases <- ENV$checked_Studies_id
    #ENV$checked_Studies_forCases <- checked_Studies_forCases
    #lchecked_Studies_forCases <- length(checked_Studies_forCases)
    #ENV$lchecked_Studies_forCases <- lchecked_Studies_forCases
    
    
    police <- tkfont.create(family="arial", size=11)
    tkconfigure(ENV$tc, foreground="black", font=police)
    
    CasesStudies <- 0
    CasesRefStudies <- NULL
    #ENV$Cases <- NULL
    for (i in seq(length(ENV$checked_StudyIndex))){
        
        Si <- ENV$checked_StudyIndex[i]
        
        tkinsert(ENV$tc,"end", paste("***** Study ", Si ," : ", ENV$Studies$name[Si], "*****"))
        
        ENV$Cases[[i]] <- cBioPortalData::sampleLists(ENV$cgds, ENV$checked_Studies_id[i])
        
        ENV$n_Cases[i] <- nrow(ENV$Cases[[i]])
        
        print(paste("There are", ENV$n_Cases[i], "Cases in", ENV$checked_Studies_id[i], sep=" "))
        
        # create progress bar
        progressBar_Cases <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                           max = ENV$n_Cases[i], width = 400)
        
        j <- 0
        CasesStudy <- 0
        CaseRefStudy <- NULL
        CasesRefStudy <-  NULL
        for (j in seq(ENV$n_Cases[i])){
            Sys.sleep(0.1)
            setTkProgressBar(progressBar_Cases, j, 
                             label=paste(round(j/ENV$n_Cases[i]*100, 0),
                                                                "% of Cases"))
            CaseStudy <- ENV$Cases[[i]][["description"]][j]
            #print(CaseStudy)
            #CaseStudy <- getCaseLists(ENV$cgds,checked_Studies_id[i])[,3][j]
            CaseRefStudy <- ENV$Cases[[i]][["studyId"]][j]
            #print(CaseRefStudy)
            #CaseRefStudy <- getCaseLists(ENV$cgds, checked_Studies_id[i])[,1][j]
    
            tkinsert(ENV$tc,"end", paste(j,":", CaseStudy))
            
            CasesStudy <- c(CasesStudy, CaseStudy)
            CasesRefStudy <- c(CasesRefStudy,CaseRefStudy)
        }
        ##Close ProgressBar_Cases
        close(progressBar_Cases)
        CasesStudies <- c(CasesStudies, CasesStudy)
        CasesRefStudies <- c(CasesRefStudies, CasesRefStudy)
        
    }
    
    ENV$CasesRefStudies <- CasesRefStudies
    ENV$CasesStudies <- CasesStudies
    
}

#' plot correlation of two genes expressions.
#' @usage plot_2Genes_1GenProf()
#' @return plot
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' plot_2Genes_1GenProf()
#' }
plot_2Genes_1GenProf <- function(){
    
    if(!exists("curselectCases", envir = ENV)){
        msgNoCaseChoice= paste("Select at least ONE Case and ONE Genetic Profile")
        tkmessageBox(message= msgNoCaseChoice)
        stop(msgNoCaseChoice)
    } else if (!exists("curselectGenProfs", envir = ENV)){
        
        msgNoGenProfChoice= paste("Select at least ONE Genetic Profile")
        tkmessageBox(message= msgNoGenProfChoice)
        stop(msgNoGenProfChoice)
    } else if(length(ENV$GenProfChoice)!=1 || !exists("GenProfChoice", envir = ENV)) {
        msgLGenProfChoice= "Select ONE Genetic Profiles"
        tkmessageBox(message=msgLGenProfChoice, icon="warning")
        
        stop(msgLGenProfChoice)
        
    } else if(length(ENV$CaseChoice)!= 1 || !exists("CaseChoice", envir = ENV)){
        msgLCaseChoice="Select ONE Case"
        tkmessageBox(message=msgLCaseChoice, icon="warning")
        
        stop(msgLCaseChoice)
        
    } else{
        LengthCases <- 0
        d <- 0
        for (s in 1:length(ENV$checked_Studies)){
            
            LengthCases <- LengthCases + ENV$LCases[s]+1
            d <- d +1 
            if(ENV$curselectCases <= LengthCases){
                
                
                
                
                Lchecked_GenProfs <- length(ENV$curselectGenProfs)
                ######### Test if all cases were corresponded to appropriate Gen profs (same Study)
                for (i in 1:Lchecked_GenProfs){
                    if(ENV$StudyRefCase[i]!=ENV$StudyRefGenProf[i]){
                        
                        msgBadChoice="Correspond the Genetic Profile to the Case for the same Study"
                        tkmessageBox(message=msgBadChoice, icon="warning")
                        
                        #stop("Correpond the Genetic Profile to the Case for the same Study")
                        break
                        
                    }
                }
                
                #dialogPlotOption_SkinCor()
                #tkwait.window(ttDialSkinCor)
                launchDialog1 <- function() {
                    ENV$GENE1 <- modalDialog("CNA/Mut vs mRNA ", "Enter HUGO Gene 1", "MDM4")
                    if (ENV$GENE1 == "ID_CANCEL") return()
                    
                }
                launchDialog1()
                
                launchDialog2 <- function() {
                    GENE2 <- modalDialog("CNA/Mut vs mRNA ", "Enter HUGO Gene 2", "MDM2")
                    if (GENE2 == "ID_CANCEL") return()
                    oneGenProf<- ENV$GenProfsRefStudies[ENV$curselectGenProfs]
                    oneCase <- ENV$CasesRefStudies[ENV$curselectCases]
                    plotCommand<- function(){
                        
                        if (inherits(try(plot(ENV$cgds,ENV$checked_Studies[s],c(ENV$GENE1,GENE2),oneGenProf ,oneCase), silent=TRUE),"try-error"))
                        {
                            msgBadMARGIN <- paste(" Error in plot.new() : figure margins too large ")
                            tkmessageBox(message=msgBadMARGIN, icon="warning")
                            stop(msgBadMARGIN)
                        } else{         
                            plot(ENV$cgds,ENV$checked_Studies[s],c(ENV$GENE1,GENE2),oneGenProf ,oneCase)
                            
                            
                        }
                        
                        
                        
                    }
                    Title<- paste (ENV$StudyRefCase, ENV$CaseChoice, sep=": ")
                    plotModel(plotCommand, title=Title,hscale=1, vscale=1)
                    plotCommand()
                }
                launchDialog2()
                
                
            }
        }
    }
    
}

#' get matrix with clinical from file
#' @usage getClinicalDataMatrix()
#' @export
#' @return dataframe of clinicaldata
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' getClinicalDataMatrix()
#' }
getClinicalDataMatrix <- function(){
 
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(exists("ClinicalDataMatrixfile", envir = ENV)){
        rm(ENV$ClinicalDataMatrixfile)
    }
    ENV$ClinicalDataMatrixfile <- tclvalue(tkgetOpenFile(filetypes = "{{txt Files} {.txt}} {{All files} *}", 
                                                         title="Choose Gene Expression Matrix from File")) 
    
    if (!nchar(ENV$ClinicalDataMatrixfile)) {
        tkmessageBox(message = "No file was selected!")
    } else {
        Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        ENV$ClinicalData<-read.table(ENV$ClinicalDataMatrixfile)
       
        tkmessageBox(message = paste("The file selected was", ENV$ClinicalDataMatrixfile))
        tkfocus(ttMain)
    }
    
}

#' get GCT and CLS example files.
#' @usage getGCTCLSExample()
#' @return GCT and CLS files
#' @export
#' @examples
#' ## Load workspace
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getGCTCLSExample()
#' }
getGCTCLSExample <- function(){
    #require(tcltk)
    
    ifrm <- function(obj, env = ENV()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    ifrm(fname.GCT, ENV)
    ifrm(fname.CLS,ENV)
    
    ENV$fname.GCT <- paste(path.package("canceR"),"/extdata/gct_cls/", "Lung_Bost_collapsed_symbols_common_Mich_Bost.gct",sep="")
    ENV$fname.CLS <- paste(path.package("canceR"),"/extdata/gct_cls/", "Lung_Boston.cls",sep="")
    
    
    
    tkinsert(ENV$tlGCT,"end",ENV$fname.GCT)
    tkinsert(ENV$tlCLS,"end",ENV$fname.CLS)
    tkfocus(ENV$ttDialogGSEA)
    
}

#' Plotting two genetic profiles for  one Gene 
#' @usage plot_1Gene_2GenProfs()
#' @return plot
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
#' \dontrun{
#' plot_1Gene_2GenProfs()
#' }
plot_1Gene_2GenProfs <- function(){
    
    
    
    if(!exists("curselectCases", envir = ENV)){
        msgNoCaseChoice= paste("Select at least ONE Case and TWO Genetic Profiles")
        tkmessageBox(message= msgNoCaseChoice)
        stop(msgNoCaseChoice)
    } else if (!exists("curselectGenProfs", envir = ENV)){
        
        msgNoGenProfChoice= paste("Select at least ONE Genetic Profile")
        tkmessageBox(message= msgNoGenProfChoice)
        stop(msgNoGenProfChoice)
    } else if(length(ENV$GenProfChoice)%%2!=0 || !exists("GenProfChoice", envir = ENV)) {
        msgLGenProfChoice= "Select (x2) Genetic Profiles"
        tkmessageBox(message=msgLGenProfChoice)
        tkfocus(ENV$ttCasesGenProfs)
        stop(msgLGenProfChoice)
        
    } else if(length(ENV$CaseChoice)!= length(ENV$GenProfChoice)/2 || !exists("CaseChoice", envir = ENV)){
        msgLCaseChoice="Associate two Genetic Profiles to only one Case for every Study"
        tkmessageBox(message=msgLCaseChoice)
        tkfocus(ENV$ttCasesGenProfs)
        stop(msgLCaseChoice)
        
    } else{
        ## Test if all Cases were corresponded to TWO appropriate Genetic profiles (same Study)
        for (i in 0:(length(ENV$StudyRefCase)-1)){
            if(ENV$StudyRefCase[i+1] != ENV$StudyRefGenProf[(i*2)+1] || ENV$StudyRefCase[i+1] != ENV$StudyRefGenProf[(i*2)+2]){
                msgBadChoice="Correspond two Genetic Profiles to one Case for the same Study"
                tkmessageBox(message=msgBadChoice, icon="warning")
                stop(msgBadChoice)
                
            }
        }
        
        for(s in 0:(length(ENV$StudyRefCase)-1)){
            
            #Lchecked_GenProfs <- length(curselectGenProfs)
            dialogPlotOption_SkinCor(s)
            tkwait.window(ENV$ttDialSkinCor)
            
            launchDialog <- function() {
                GENE <- modalDialog("CNA/Mut vs mRNA ", "Enter HUGO Gene Symbol", "MDM4")
                if (GENE == "ID_CANCEL") return()
                
                if(ENV$Axes == "0"){
                GenProf1<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[s*2+1]]
                GenProf2<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[s*2+2]]
                } else{
                GenProf2<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[s*2+1]]
                GenProf1<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[s*2+2]]
                    
                }
                
                twoGenProfs<- c(GenProf1,GenProf2)
                oneCase<-ENV$CasesRefStudies[ENV$curselectCases[s+1]]
                ##skin and Correlation are from dialogPlotOption_SkinCor.R
                plotCommand<- function(){
                    if (inherits(try(plot(ENV$cgds,ENV$checked_Studies[s+1],GENE,twoGenProfs ,oneCase, skin= ENV$skin, add.corr=ENV$correlation, legend.pos = 'topright'), silent=TRUE),"try-error"))
                    {
                        msgBadAcces <- paste(" Error in plot.new() : figure margins too large or failed access to cbioportal or genetic profiles are not numeric. For example Mutation variable is not numeric. ")
                     tkmessageBox(message=msgBadAcces, icon="warning")
                     stop(msgBadAcces)
                    } else{
                    plot(ENV$cgds,ENV$checked_Studies[s+1],GENE,twoGenProfs ,oneCase, skin= ENV$skin, add.corr=ENV$correlation, legend.pos = 'topright')
                    }
                }
                Title<- paste(ENV$StudyRefCase[s+1], ENV$CaseChoice[s+1], sep=": ")
                plotModel(plotCommand, title=Title,hscale=1, vscale=1)
            }
            launchDialog()
            
           
        }
    }
}

library(R.oo)
# .onAttach <- function(libname, pkgname){
#     packageStartupMessage('Please send questions to cbioportal@googlegroups.com')
# }
#

#' CGDS  connect object to cBioPortal
#' @description Creates a CGDS connection object from a CGDS endpoint URL. This object must be passed on to the methods which query the server.
#' @param url  A CGDS URL (required).
#' @param token An optional 'Authorization: Bearer' token to connect to cBioPortal instances that require authentication (default NULL)
#' @param verbose A boolean variable specifying verbose output (default FALSE)
#' @param ploterrormsg An optional message to display in plots if an error occurs (default ”)
#'
#' @usage CGDS(url,verbose=FALSE,ploterrormsg='',token=NULL)
#'
#' @export
setConstructorS3("CGDS", function(url='',verbose=FALSE,ploterrormsg='',token=NULL) {
    R.oo::extend(R.oo::Object(), "CGDS",
                 .url=url,
                 .token=token,
                 .verbose=verbose,
                 .ploterrormsg='')
})


#' S3 method to test cBioPortal connection
#'
#'
#' @method test CGDS
#'
#' @param x connection object
#' @param ... not used
#'
#'
#' @export test.CGDS
setMethodS3("test","CGDS", function(x, ...) {
    checkEq = function(a,b) { if (identical(a,b)) "OK\n" else "FAILED!\n" }
    checkGrt = function(a,b) { if (a > b) "OK\n" else "FAILED!\n" }
    checkTrue = function(a) { if (a) "OK\n" else "FAILED!\n" }
    
    cancerstudies = cBioPortalData::getStudies(x) |> select("name", "description", "studyId")
    
    cat('getCancerStudies... ',
        checkEq(colnames(cancerstudies),c("name","description", "studyId")))
    
    ct = cancerstudies[2,3] # should be row 1 instead ...
    
    cat('getCaseLists (1/1) ... ',
        checkEq(#colnames(getCaseLists(x,ct)),
            cBioPortalData::sampleLists(x, ct) |> select("name", "description", "sampleListId") |> colnames(),
            c("name","description","sampleListId")))
    
    # clinical data
    # check colnames
    cat('getClinicalData (1/1) ... ',
        checkTrue("DFS_MONTHS" %in% colnames(cBioPortalData::clinicalData(x, "gbm_tcga_pub"))))
    
    # check one gene, one profile
    cat('getGeneticProfiles (1/1) ... ',
        checkEq(#colnames(getProfileData(x,'NF1','gbm_tcga_mrna','gbm_tcga_all')),
            molecularProfiles(api = x, studyId = "gbm_tcga_pub") |>
                select("name", "description", "molecularProfileId") |> colnames(),
            c("name", "description", "molecularProfileId")))
    # # check many genes, one profile
    # cat('getGeneticProfiles (2/6) ... ',
    #     checkEq(colnames(getProfileData(x,c('MDM2','MDM4'),'gbm_tcga_mrna','gbm_tcga_all')),
    #             c("MDM2","MDM4")))
    # # check one gene, many profile
    # cat('getGeneticProfiles (3/6) ... ',
    #     checkEq(colnames(getProfileData(x,'NF1',c('gbm_tcga_mrna','gbm_tcga_mutations'),'gbm_tcga_all')),
    #             c('gbm_tcga_mrna','gbm_tcga_mutations')))
    # # check 3 cases returns matrix with 3 columns
    # cat('getGeneticProfiles (4/6) ... ',
    #     checkEq(rownames(getProfileData(x,'BRCA1','gbm_tcga_mrna',cases=c('TCGA-02-0001-01','TCGA-02-0003-01'))),
    #             make.names(c('TCGA-02-0001-01','TCGA-02-0003-01'))))
    # # invalid gene names return empty data.frame
    # cat('getGeneticProfiles (5/6) ... ',
    #     checkEq(nrow(getProfileData(x,c('NF10','NF11'),'gbm_tcga_mrna','gbm_tcga_all')),as.integer(0)))
    # # invalid case_list_id returns error
    # cat('getGeneticProfiles (6/6) ... ',
    #     checkEq(colnames(getProfileData(x,'NF1','gbm_tcga_mrna','xxx')),
    #             'Error..Invalid.case_set_id...xxx.'))
    cat('getProfileData (1/1) ... ',
        checkEq(#colnames(getGeneticProfiles(x,ct)),
            cBioPortalData::getDataByGenes(
                api = x,
                studyId = "gbm_tcga_pub",
                genes = c("NF1", "TP53", "ABL1"),
                by = "hugoGeneSymbol",
                molecularProfileIds = "gbm_tcga_pub_mrna")|>
                unname() |>
                as.data.frame() |>
                select("hugoGeneSymbol","patientId", "value") |>
                colnames(),
            c("hugoGeneSymbol","patientId","value")))
})

#' get Profile (GCT file) and Phenotype (CLS file) Data from Disease.
#' @usage getGCT_CLSfiles()
#' @return GCT and CLS files paths
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getGCT_CLSfiles()
#' }
getGCT_CLSfiles <- function(){
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    ## verify the rightness of checked Case Gene. Prof.
    testCheckedCaseGenProf()
    
    ####Create a new directory "Results" for All Profiles Data
    ##Set Results as tempary work directory
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/")){
        dir.create(file.path(paste(getwd(), "/Results", sep="")), showWarnings = FALSE)
        dir.create(file.path(paste(getwd(), "/Results/gct_cls", sep="")), showWarnings = FALSE)
    } else if (!file.exists("Results/gct_cls/")){
        dir.create(file.path(paste(getwd(), "/Results/gct_cls", sep="")), showWarnings = FALSE)
    }
    
    ProfData=0
    i<-0
    for (s in ENV$checked_Studies_id){
        i <- i+1
        Si =ENV$checked_StudyIndex[i]
        progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                                  max = length(ENV$curselectGenProfs), width = 400)
        
        ####Tree ROOT==Studies
        StudyiChoice <- paste("Study",i,"Choice", sep="")
        study_desc_position_in_genProfs <- 0
        
        for (k in 1:length(ENV$curselectGenProfs)){
            Sys.sleep(0.1)
            setTkProgressBar(progressBar_ProfilesData, k, 
                             label=paste(round(k/length(ENV$curselectGenProfs)*100, 0),
                                         "% of Profiles Data"))
            
            # Avoid to select study description  
            if (ENV$curselectGenProfs[k] <= ENV$n_GenProfs[i] && 
                ENV$curselectGenProfs[k] > study_desc_position_in_genProfs){    
                
                study_desc_position_in_genProfs <- study_desc_position_in_genProfs + ENV$n_GenProfs[i]   
                
                GenProf <- ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                
                ReturnDialogSamplingGSEA <- dialogSamplingGSEA(length(ENV$curselectGenProfs))
                
                if(ReturnDialogSamplingGSEA[1] == "ID_CANCEL"){
                    stop()
                }else if(length(ENV$GeneList) > 1000){ 
                    
                    
                    ProfData <- getDataByGenes(
                        api =  ENV$cgds,
                        studyId = s,
                        genes = ENV$GeneList,  #c("NF1", "TP53", "ABL1")
                        by = "hugoGeneSymbol",
                        molecularProfileIds = GenProf)  |>
                        unname() |>
                        as.data.frame() |>
                        select("hugoGeneSymbol","sampleId", "value") |>
                        tidyr::spread("hugoGeneSymbol", "value") 
                        #data.frame(row.names = 1)
                    
                    ProfData <<- ProfData
                    ##Convert data frame to numeric structure
                    #                     print("converting data frame of Profile data to numeric stucture...")
                    #                     for(i in 1:ncol(ProfData)){
                    #                         
                    #                         ProfData[,i] <- as.numeric(ProfData[,i])
                    #                     }
                    
                    ## substitute "NaN" by "NA"
                    if(length(grep("NaN", ProfData))!=0){
                        
                        print("This step takes a while to remove NaN string and
                              convert them to NA (12000 genes takes about 10 min)")
                        for (j in 1:ncol(ProfData)){
                            #ProfData[,i]<- as.numeric(gsub("\\[Not Available\\]","na", ProfData[,i]))
                            #ProfData <- sapply(ProfData, function(x) gsub("NA", "na", x))
                            ProfData[,j]<- as.numeric(gsub("NaN",NA, ProfData[,j]))
                            #ProfData[,i]<- as.numeric(gsub(NA,"na", ProfData[,i]))
                        }
                    }
                    print("Removing genes without data... ")
                    ##remove all NAs rows
                    ProfData <- ProfData[which( apply( !( apply(ProfData,1,is.na) ),2,sum)!=0 ),]
                    print(paste("dimension of Profile Data:"))
                    print(dim(ProfData))
                    ## When matrix has negative values,  simple space are generated before every positive value. 
                    ## This space cause deleting all columns from matrix when we try to remove the columns without values NAs
                    # if(min(ProfData, na.rm = TRUE) < 0){
                    #     for (j in 1:ncol(ProfData)){
                    #         
                    #         ProfData[,j]<- as.numeric(gsub(" ","", ProfData[,j]))
                    #         
                    #     }
                    # }
                    
                    ##remove all NAs columns only for CNA profile Data
                    #if(length(grep("cna",ENV$CaseChoice[k], ignore.case = TRUE))!=0){
                    #ProfData<- ProfData[,-which( apply( !( apply(ProfData,1,is.na) ),1,sum)==0 )]
                    #}
                    
                    if(nrow(ProfData) < ReturnDialogSamplingGSEA){
                        tkmessageBox(message= paste("After cleaning data frame, it remains: ", 
                                                    nrow(ProfData), "samples < ", ReturnDialogSamplingGSEA,
                                                    ". Select",nrow(ProfData), 
                                                    "as the size of sampling and repeat the request." ), icon="info")
                        close(progressBar_ProfilesData)
                        stop(paste("After cleaning data frame, it remains: ",
                                   nrow(ProfData), "samples < ", ReturnDialogSamplingGSEA,
                                   ". Select", nrow(ProfData), "as the size of sampling and repeat the request." ))
                        
                    }
                    ##Sampling Profile Data
                    set.seed(1234)
                    SamplingProfData <- 
                        apply(ProfData, 2,function(x) sample(x, ReturnDialogSamplingGSEA)) |>
                        as.data.frame()
        
                    
                    SamplingProfData <<- SamplingProfData 
                    print("Getting  clinical data...")
                    
                    ClinicalData <- getClinicData_MultipleCases(getSummaryGSEAExists=1)
                    ClinicalData <<- ClinicalData
                    # ClinicalData <- subset(ClinicalData, !ClinicalData[,1]=="NA")
                    # ClinicalData <- subset(ClinicalData, !ClinicalData[,1]=="[Unknown]")
                    # ClinicalData <- subset(ClinicalData, !ClinicalData[,1]=="[Not Available]")
                    # ClinicalData <- subset(ClinicalData, !ClinicalData[,1]=="")
                    
                    # if(ncol(ClinicalData)>1){
                    #     tkmessageBox(message="Select only ONE clinical Data with only TWO classes")
                    #     stop("Select only ONE clinical Data with only TWO classes")
                    # }
                    
                    print("merging profile data and clinical data tables...")
                    ## MERGE Data.frames: ClinincalData and ProfData: 
                    ## Select only Cases (rownames) that exist in ProfData
                    names_Clinical_Data <- colnames(ClinicalData)
                    names_SamplingProfData <- colnames(SamplingProfData)

                    clinical_profData <- ClinicalData |>
                        left_join(SamplingProfData, by="sampleId")


                    AssayData <- clinical_profData |>
                        select(all_of(names_SamplingProfData)) |>
                        data.frame(row.names = 1) |>
                        t() 
                        #as.matrix()
                    
                    ClinicalData <- ClinicalData |>
                        data.frame(row.names = 1)
                    ClinicalData <<- ClinicalData
                    
                    # merge <- merge(ClinicalData, SamplingProfData, by="row.names") |> 
                    #     data.frame(row.names = 1)
                    # ##Ordering by classes in selected variable in ClinicalData
                    # 
                    # merge <- merge[order(merge[,1]),]
                    # merge <- subset(merge, !merge[,1]=="")
                    # print("Merging Clinical and Profile data...")
                     PhenoData <- ClinicalData[["OS_STATUS"]]
                    # 
                    # 
                    # AssayData<-merge[,-1]
                    # print("AssayData")
                    # AssayData <- round(AssayData, digits=2)
                    # print("round Assaydata")
                    ## GSEA-R does no accept negative value
                    ## Translate matrix with minimum negative value
                    
                    if(min(AssayData, na.rm=TRUE) < 0){
                        print("There are negative values. Translating values by adding the absolute 
                              of minimum value to all matrix")                  
                        AssayData <- AssayData +(abs(min(AssayData, na.rm=TRUE)))
                    }
                    
                    
                    Table_Title <- paste("SD_", ENV$StudyRefCase[k],"_","GenProf_",
                                         ENV$curselectGenProfs[k],"_","CASE_",
                                         ENV$curselectCases[k],".gct")
                    getInTable(AssayData, Table_Title)

                    ###Save AssayData to GCT file                            
                    #AssayData <- t(AssayData)
                    ncol <- ncol(AssayData)
                    nrow <- nrow(AssayData)
                    
                    tmp<- cbind(rownames(AssayData), rep("na", nrow))
                    tmp <- rbind("",tmp)
                    tmp[1,1:2] <- c("NAME", "Description")
                    
                    
                    print("Saving GCT et CLS files ...")
                    
                    rownames(AssayData) <- NULL
                    AssayData <-rbind(colnames(AssayData),AssayData)
                    colnames(AssayData) <- NULL
                    AssayData <- cbind(tmp, AssayData)
                    
                    
                    AssayData <- rbind("","", AssayData)
                    AssayData[1,1] <- "#1.2"
                    AssayData[2,1] <- nrow
                    AssayData[2,2] <- ncol
                    fileName <- paste("SD_",ENV$StudyRefCase[k],"_","GenProf_",
                                    ENV$curselectGenProfs[k],"_","CASE_",ENV$curselectCases[k],
                                    "_",gsub(".txt","",basename(ENV$GeneListfile)) ,".gct", sep="")
                    
                    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                    setwd(paste(getwd(),"/Results/gct_cls", sep=""))
                    
                    
                    write.table(AssayData,file=fileName, col.names=FALSE, 
                                quote=FALSE, row.names=FALSE, sep="\t")
                    
                    ## Built and Save CLS file from PhenoData frame
                    if(sum(grep("[A-Za-z]",PhenoData))==0){
                        
                        
                        fileName<-paste("SD_",ENV$StudyRefCase[k],"_","GenProf_",
                                        ENV$curselectGenProfs[k],"_","CASE_",ENV$curselectCases[k],
                                        "_","OS_STATUS","_",gsub(".txt","",basename(ENV$GeneListfile)) ,".cls", sep="")
                        
                        sink(fileName)
                        cat("#numeric")
                        cat("\n")
                        cat("#","OS_STATUS", sep="")
                        cat("\n")
                        cat(PhenoData)
                        cat("\n")
                        sink()
                        
                        setwd("../../")
                        tkmessageBox(message="cls continuous file was generated but is seems not
                                     working with the present version of GSEA.1.0.R")
                    }else{ 
                        
                        nbrOfSample <-length(PhenoData)
                        nbrOfClasses <-length(table(PhenoData))
                        if(nbrOfClasses!= 2){
                            tkmessageBox(message="Select Variable with only two classes")
                            stop("Select Variable with only two classes")
                            close(progressBar_ProfilesData)
                        }else{
                            
                            classes<- 0
                            for(i in 1:nbrOfClasses){
                                
                                classes <- cbind(classes,names(table(PhenoData))[i])
                            }
                            classes[1] <- "#"
                            classes <- sub(" ", "", classes)

                            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                            
                            fileName<-paste("SD_",ENV$StudyRefCase[k],"_","GenProf_",
                                            ENV$curselectGenProfs[k],"_","CASE_",
                                            ENV$curselectCases[k],"_","OS_STATUS",
                                            "_",gsub(".txt","",basename(ENV$GeneListfile)),".cls", sep="")
                            
                            sink(fileName)
                            cat(paste(nbrOfSample, nbrOfClasses, "1", sep=" "))
                            cat("\n")
                            cat(classes)
                            cat("\n")
                            cat(PhenoData)
                            cat("\n")
                            sink()
                            
                            setwd("../../")
                            
                        }
                    }
                    ENV$PhenoData <- PhenoData
                } else {
                    msgSmallGeneList <- paste("The request has only", length(ENV$GeneList),
                                              " genes. The GSEA should be not robust. 
                                                  It is better to run GSEA with 1000 genes or more.",
                                              sep=" ")
                    tkmessageBox(message=msgSmallGeneList, icon="info")
                }
            }
        } 
        close(progressBar_ProfilesData)
        
    }
}

#' get Profiles Data  for a Single Gene.
#' @usage getProfilesDataSingleGene()
#' @return dataframe with profiles data for a single gene
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' ## Select Case from Breast Cancer
#'  ENV <- new.env(parent = emptyenv())
#' ENV$curselectCases <- 9
#' ##Select Genetic Profile from Breast Cancer
#' ENV$curselectGenProfs <- 4
#' ## get Specific Mutation data for 73 Genes list
#' \dontrun{
#' getProfilesDataSingleGene()
#' }
getProfilesDataSingleGene <-function(){
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    testCheckedCaseGenProf(singleGene=1)

    
    for (s in ENV$checked_Studies_id){
        
        for(c in seq(length(ENV$curselectGenProfs))){

                
                launchDialog <- function(){
                    
                    Dialog_Title<- paste("STUDY:", s ,"CASE:", ENV$curselectCases_forStudy[c], sep=" ")
                    GENE <- modalDialog(Dialog_Title, "Enter HUGO Gene Symbol", "MDM4")
                    if (GENE == "ID_CANCEL") return()
                    
                    #ProfDataS<-getProfileData(ENV$cgds, GENE, GenProfS, Study_id)
                    print(ENV$GenProfsRefStudies[ENV$curselectGenProfs[c]])
                    print(s)
                    ProfDataS <- cBioPortalData::getDataByGenes(api =  ENV$cgds,
                                                                studyId = s,
                                                                genes = GENE,
                                                                by = "hugoGeneSymbol", 
                                                                molecularProfileIds = ENV$GenProfsRefStudies[ENV$curselectGenProfs[c]]) |>
                        unname() |>
                        as.data.frame() |>
                        select("hugoGeneSymbol","sampleId", "value") |>
                        tidyr::spread("hugoGeneSymbol", "value") 
                        #data.frame(row.names = 1)
                    
                    
                    ttProfData_cb <- tktoplevel()
                    tktitle(ttProfData_cb) <- paste(ENV$StudyRefCase[c],ENV$CaseChoice[c], sep=": ")
                    #tkwm.geometry(ttProfData_cb,"300x300")
                    
                    cbAll <- tkcheckbutton(ttProfData_cb)
                    cbAllValue <- tclVar("0")
                    tkconfigure(cbAll, variable = cbAllValue)
                    labelAll<- tklabel(ttProfData_cb, text= "All")
                    tkgrid(labelAll, cbAll)
                    
                    cbIValue=0
                    for(i in seq(ncol(ProfDataS))){
                        
                        cbi <- paste ("cb", i, sep="")  
                        cbi <- tkcheckbutton(ttProfData_cb)
                        cbiValue <- paste("cb", i, "Value", sep="")
                        cbIValue[i] <- cbiValue
                        cbIValue[i] <- tclVar("0")
                        
                        tkconfigure(cbi,variable=cbiValue)
                        labeli <- paste ("label", i , sep="") 
                        labelI <- labeli
                        labelI <- tklabel(ttProfData_cb,text= names(ProfDataS[i]))
                        tkgrid(labelI,cbi)
                        
                    }
                    ProfDataSSub<-0
                    OnOK <- function(){
                        
                        cbAllVal <- as.character(tclvalue(cbAllValue))
                        if(cbAllVal =="1"){
                            
                            for (i in 1:length(names(ProfDataS))){
                                ProfDataS[,i]<- gsub("\\[Not Available\\]","NA", ProfDataS[,i])
                            }
                            
                            #ProfDataS <- t(t(ProfDataS))
                            getInTable(ProfDataS,
                                       title= paste0(ENV$StudyRefCase[c],": ",
                                                    ENV$CaseChoice[c]))
 
                        } else{
                            
                            for (i in 1: length(names(ProfDataS))){
                                cbiValue <- paste("cb", i, "Value", sep="")
                                cbIValue <- cbiValue
                                cbiVal <- paste("cb", i, "Val", sep="")
                                cbIVal<-cbiVal
                                
                                cbIVal[i] <- as.character(tclvalue(cbIValue))
                                #tkdestroy(ttProfData_cb)
                                if (cbIVal[i]=="1"){
                                    
                                    ## convert metacharacter "[""]" not supported by tclarray()
                                    ProfDataS[,i] <- gsub("\\[Not Available\\]","NA", ProfDataS[,i])
                                    
                                    ProfDataSSub <- cbind(ProfDataSSub, ProfDataS[i])
                                }
                            }
                            ProfDataSSub <- ProfDataSSub[-1]
                            
                            if(length(ProfDataSSub)==0){
                                tkmessageBox(message= paste("Select at least one data type"), icon="warning")
                                stop("Select at least one data type")
                            }
                            
                            #ProfDataSSub <- t(t(ProfDataSSub))
                            getInTable(ProfDataSSub, 
                                       title= paste0(ENV$StudyRefCase[c],": ",
                                                     ENV$CaseChoice[c]))
                            
                        }
                        tkdestroy(ttProfData_cb)
                    }
                    
                    OK.but <- tkbutton(ttProfData_cb,text="OK",command=OnOK)
                    tkgrid(OK.but)
                    tkfocus(ttProfData_cb)
                    
                    ##Waiting to checkbox before to access to the next clinical data
                    tkwait.window(ttProfData_cb)
                }
                launchDialog()
                
#            }
        }   
        
    }
}

#' Get gene correlation for multiple dimensions. 
#' @usage 
#' getCor_ExpCNAMet(ListMatrix, dimension)
#' 
#' @param ListMatrix  is a List of numeric matrices
#' @param dimension  Exp,CNA, Met , miRNA , RPPA
#' 
#' @return correlation matrix
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
#' \dontrun{
#' getListProfData()
#' getCor_ExpCNAMet(ENV$ListProfData$Expression, dimension="mRNA")
#' head(ENV$Cor_Exp)
#' }
#' @import plyr
#' 
getCor_ExpCNAMet <- function(ListMatrix, dimension){
    
    #library(plyr)
    
    ##Merge data frame by rownames keeping the order of the first one
    MergeRowNames <- function(x,n){
        
        keeping.order <- function(data, fn, ...) { 
            col <- ".sortColumn"
            data[,col] <- 1:nrow(data) 
            out <- fn(data, ...) 
            if (!col %in% colnames(out)) stop("Ordering column not preserved by function") 
            out <- out[order(out[,col]),] 
            out[,col] <- NULL 
            out 
        } 
        
        rownames(x) = make.names(x[,1], unique=TRUE)
        ## ordering genes in MutData as in GeneList
        
        df_GeneList <- as.data.frame(rep(t(ENV$GeneList), n))
        #df_GeneList <- as.data.frame(ENV$GeneList)
        rownames(df_GeneList) <- make.names(df_GeneList[,1], unique =TRUE)
        
        df_merge <- keeping.order(df_GeneList, merge, y=x, by = "row.names")
        x <- df_merge[,c(-1,-2)]
        return(x)
    }
    
#     
#     if(exists("Cor_Exp", envir = ENV)){
#         rm(Cor_Exp, envir=ENV)
#         
#     }    
#     if(exists("Cor_CNA", envir = ENV)){
#         rm(Cor_CNA, envir=ENV)
#         
#     }   
#     if(exists("Cor_Met", envir = ENV)){
#         rm(Cor_Met, envir=ENV)
#         
#     }
#     if(exists("Cor_RPPA", envir = ENV)){
#         rm("Cor_RPPA", envir=ENV)
#         
#     }
#     
    
    
    
    ## Rotate the cube of Matrices with 90°
    ## get the same column of a list of matrices
    getColumn <- function(x, colNum, len = nrow(x)){
        y <- x[,colNum]
        length(y) <- len
        y
    }
    
    ## get Matrices with each same column of all matrices 
    getMatrices <- function(colNums, dataList = x){
        # the number of rows required
        n <- max(sapply(dataList, nrow))
        lapply(colNums, function(x, dat, n) { # iterate along requested columns
            do.call(cbind, lapply(dat, getColumn,x, len=n)) # iterate along input data list
        }, dataList, n)
    }
    
    ## Define Lists
    
    #Lchecked_GenProf <- length(ENV$curselectGenProfs)
    #ListProfData <- vector("list", Lchecked_GenProf)
    #ListProfData$Expression <- ENV$ListProfData$Expression
    #ENV$Cor_Met <- vector("list", 2)
    
    ## Rotate the list of Matrices by 90°
    ListMatrix90 <- getMatrices(c(1:ncol(ListMatrix[[1]])),dataList=ListMatrix)

    ## Replace NA by 0
    ListMatrix90 <- lapply(ListMatrix90, function(x) {x[is.na(x)] <- 0; x })
    
    if(dimension == "CNA"){
        # Ordering value
        ListMatrix90 <- lapply(ListMatrix90,function(x)apply(x, 2, function(x) x[order(x)]))
    }
    ## Convet the list of matrices to Array
    #ArrayMatrix90 <- array(unlist( ListMatrix90), dim = c(nrow( ListMatrix90[[1]]), ncol( ListMatrix90[[1]]), length( ListMatrix90)))
    #dimnames(ArrayMatrix90) <- list(1: dim(ArrayMatrix90)[1],names(ListProfData$Expression),colnames(ListProfData$Expression[[1]]) )
    
    #cor.balance(t(ListMatrix90[[1]][1:136,]),m=1,G=4)
    #cor(ListMatrix90[[1]], method="spearman" , use= "na")
    
    Cor_ListMatrix <- lapply(ListMatrix90,function(x) cor(x,use="na", method="spearman"))
    
    #lapply(Cor_ListProfData$Expression, function(x) x[abs(x)>0.8])
    
    ## convert the list of correlation matrices to Array
    Cor_ArrayMatrix <- array(unlist( Cor_ListMatrix), dim = c(nrow(Cor_ListMatrix[[1]]), ncol(Cor_ListMatrix[[1]]), length(Cor_ListMatrix)))
    dimnames(Cor_ArrayMatrix) <- list(names(ListMatrix), names(ListMatrix), colnames(ListMatrix[[1]]))
    
    #################
    #aaply(Cor_ArrayMatrix, c(3,1), function(x) x[ENV$GeneList,,drop=FALSE])
    
    #ProfData[,(as.factor(ENV$GeneList))][1,1:7]
    #ProfData[,(as.factor(ENV$GeneList))][1,1:7]
    ################
    
    #DF1_CorMatrix <- adply(Cor_ArrayMatrix,1:2)
    
    ## Convert array to df by 3,1 dimensions
    Cor_DfMatrix <- plyr::adply(Cor_ArrayMatrix,c(3,1))
    Cor_DfMatrix[is.na(Cor_DfMatrix)]<-0
    
    ## keeping the order of geneList
    Cor_DfMatrix <-MergeRowNames(Cor_DfMatrix, length(ENV$checked_Studies))

    if(dimension == "Exp"){
        ENV$Cor_Exp <- Cor_DfMatrix        
    } else if(dimension == "MetHM450") {
        ENV$Cor_Met$HM450 <- Cor_DfMatrix
    } else if (dimension=="MetHM27"){
        ENV$Cor_Met$HM27 <- Cor_DfMatrix
    } else if(dimension == "CNA") {
        ENV$Cor_CNA <- Cor_DfMatrix
    } else if(dimension == "RPPA") {
        ENV$Cor_RPPA <- Cor_DfMatrix
        } else if(dimension == "miRNA"){
            ENV$Cor_miRNA <- Cor_DfMatrix
        }
    
    #     ## aggregate only correlation bigger that threshol by rowname
    #     temp1 <-aggregate(temp[,c(-1,-2)], list(temp[,2]), function(x) sum(abs(x)>.2))
    #     rownames(temp1)<-temp1[,1]
    #     temp1 <- temp1[,-1]
    #     ## NA to 0
    #     temp1[is.na(temp1)]<-0
    
    ## get 0 to the dialonal
    #                     for(cn in intersect(rownames(temp1), colnames(temp1))) {
    #                         temp1[cn, cn] = 0
    #                     }
    #                     
    
    #     ###mapping genes by disease
    #     tempGene<-adply(Cor_ArrayMatrix,c(3))
    #     tempGene1 <-aggregate(tempGene[,-1], list(tempGene[,1]), function(x) sum(x>.2&& x<1,na.rm = TRUE))
    #     
    #     rownames(tempGene1) <- tempGene1[,1]
    #     tempGene1 <- tempGene1[-1]
    #     tempGene1[tempGene1[,3]==1,]
    #     
    #     tempGene1[tempGene1[,6]==1,]
    #     L <-apply(tempGene1,2,function(x) x[x==1] )
    
    #     library(circlize)
    #     chordDiagram(as.matrix(temp1),symmetric = TRUE, directional = TRUE,diffHeight = 0.06)
    #     #chordDiagram(cor(temp1),symmetric = TRUE, directional = TRUE,diffHeight = 0.06)
    #     circos.clear()
    
    #                     library(reshape2)
    #                     
    #                     DF2_CorMatrix <- melt(DF1_CorMatrix)
    #                     DF2_CorMatrix <- subset(DF2_CorMatrix, abs(value) <1)
    #                     DF2_CorMatrix <- subset(DF2_CorMatrix, abs(value) >0.2)
    #                     
    #FilterDiseaseCor <- apply(Cor_ArrayMatrix,MARGIN=c(1,2) ,function(x) x[abs(x)>0.7])
    
    
    
    ## Sum the significant correlated gene between studies and get matrix
    ## Studies/Studies/numberOfCorrelatedGenes
    
    #                     mat<-matrix(1:25,5)
    #                     pmean <- function(x,y) (x+y)/2
    #                     mat[] <- pmean(mat, matrix(mat, nrow(mat), byrow=TRUE))
    #                     rownames(mat) <- paste0("Cancer",1:5)
    #                     colnames(mat) <- paste0("Cancer",1:5)
    
    
    
    
}

#' get Clinical Data for Multiple Cases. User needs to select at least one case to run this function. Get clinical data for more one or multiple cases.
#' @usage 
#' getClinicData_MultipleCases(getSummaryGSEAExists)
#' 
#' @param getSummaryGSEAExists  if equal to 0, the clinical data is displayed in table. if the argument is equal to 1, the clinical data is used to summarise GSEA analysis results.
#' 
#' @export
#' @return dataframe with clinical data
#' 
#' @examples 
#' ##Load Session
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' ## Select Case
#' ENV <- new.env(parent = emptyenv())
#' ENV$curselectCases <- 2
#' ## get Clinical data
#' \dontrun{
#' getClinicData_MultipleCases(getSummaryGSEAExists = 0)
#' }
#' 
getClinicData_MultipleCases <- function(getSummaryGSEAExists){
    
    ##getSummaryGSEAExists is an argument for the function getSummaryGSEA(). this function accept only one clinical dats.
    if(is.null(ENV$curselectCases) || length(ENV$curselectCases) ==0){
        msgNoCases <- "Select at less one Case"
        tkmessageBox(message=msgNoCases)
        stop(msgNoCases) 
        
    }else if(getSummaryGSEAExists == 1 && 
             length(ENV$curselectCases) > 1){

            tkmessageBox(message=paste("Multiple Cases are loaded. To get Summary of GSEA Results, 
                                   Select only the Clinical data of study on which are you working."),icon="warning")
            stop("Multiple Cases are loaded. To get Summary of GSEA Results, 
             Select only the Clinical data of study on which are you working.")
            
            print(paste("length(ENV$curselectCases): ", length(ENV$curselectCases)))

        }else if(getSummaryGSEAExists == 0 &&
                 length(ENV$curselectCases) != 0){
            
            ClinicalData <- 0
            
            for(c in seq(ENV$curselectCases)){
                Study_id <- ENV$CasesRefStudies[ENV$curselectCases[c]]
                print(ENV$curselectCases[c])
                print(ENV$CasesRefStudies)
                print(paste0("Study_id: ", Study_id))
                
                #study_id <- gsub('_[a-z]*[0-9]*$', '', Study_id)
                ## extract only abbreviation of studies
                # unique(
                #     grep(paste(
                #         unique(unlist(lapply(selected_by_curser, 
                #                              function(x) unlist(strsplit(x,split = "_"))[1]))),
                #         collapse = "|"), checked_Studies, value=TRUE)
                # )
                
                ClinicalData <- cBioPortalData::clinicalData(ENV$cgds, Study_id) |>
                    select("sampleId", everything()) 
                #data.frame(row.names = 1)

                if(nrow(ClinicalData)==0){
                    msgNoClinData=paste("No Clinical Data are Available for\n", ENV$CasesStudies[ENV$curselectCases[c]+2])
                    tkmessageBox(message=msgNoClinData, title= paste("Study: ", ENV$StudyRefCase[c]))
                } else{
                    
                    ttClin<-tktoplevel()
                    tktitle(ttClin) <- paste0("Clinical Data of"," ",ENV$CasesStudies[ENV$curselectCases[c]+2])
                    tkwm.geometry(ttClin, "430x420")
                    yscr1 <- tkscrollbar(ttClin, repeatinterval=2,
                                         command=function(...)tkyview(ttc,...))
                    xscr1 <- tkscrollbar(ttClin, repeatinterval=2,orient="horizontal",
                                         command=function(...)tkxview(ttc,...))
                    ttc<-tklistbox(ttClin,height=20, width= 50 ,selectmode="multiple",
                                   xscrollcommand=function(...)tkset(xscr1,...),
                                   yscrollcommand=function(...)tkset(yscr1,...),background="white")
                    
                    tkgrid(ttc,yscr1, columnspan=1)
                    tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
                    tkgrid(xscr1,columnspan=2)
                    tkgrid.configure(xscr1,rowspan=2,columnspan=2,sticky="ew")
                    
                    tkinsert(ttc,"end","All")
                    cbIValue=0
                    for(j in 1: ncol(ClinicalData)){
                        tkinsert(ttc,"end",names(ClinicalData)[j])
                    }
                    
                    OnOK <- function(){
                        ENV$curselect <- as.numeric(tkcurselection(ttc))+1
                        
                        if(ENV$curselect[1]=="1"){
                            ENV$ClinicalData <- ClinicalData
                            
                            title <- paste0(ENV$StudyRefCase[c],": ", ENV$CasesStudies[ENV$curselectCases[c]+2])
                            getInTable(ENV$ClinicalData, title)
                            tkdestroy(ttClin)
                        } else{
                            #ENV$ClinicalData <- ClinicalData[!(is.na(ClinicalData[,ENV$curselect]) | ClinicalData[,ENV$curselect]==""), ] 
                            ENV$ClinicalData <- ClinicalData[ENV$curselect-1]
                            
                            title=paste(ENV$StudyRefCase[c],
                                        ENV$CasesStudies[ENV$curselectCases[c]+1], sep=": ")
                            getInTable(ENV$ClinicalData, title)
                            
                            tkdestroy(ttClin)
                        }
                    }
                    OK.but <- tkbutton(ttClin,text="OK",command=OnOK)
                    tkgrid(OK.but,columnspan=2)
                    
                    ##Waiting to checkbox before to access to the next clinical data
                    tkwait.window(ttClin)
                }

                ##getSummaryGSEA funtion needs return(ClinicalSub_All)
                if(getSummaryGSEAExists==1){
                    return(ENV$ClinicalData)
                }
            }
        }
    
    
}

#' Dialog Box to  Select GCT, CLS, GMT and output Files for GSEA-R (Broad Institute) 
#' @usage
#' dialogSelectFiles_GSEA()
#' 
#' @return A vector with files paths
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' dialogSelectFiles_GSEA()
#' }
#' 
#' 
dialogSelectFiles_GSEA <- function(){
    
    ENV$ttDialogGSEA<-tktoplevel()
    
    tktitle(ENV$ttDialogGSEA) <- "Choose files"
    
    
    
    ###select GSEA program file
    tlGSEA<-tklistbox(ENV$ttDialogGSEA,height=1, width= 70 ,background="white")
    
    ### Select phenotype file (.cls)
    
    ENV$tlCLS<-tklistbox(ENV$ttDialogGSEA,height=1, width= 70 ,background="white")
    chooseCLS <- function(){
        if(exists("fname.CLS", envir = ENV)){
            tkdelete(ENV$tlCLS,0,1)
        }
        ENV$fname.CLS <- tclvalue(tkgetOpenFile(filetypes = "{{CLS Files} {.cls}} {{All files} *}", title="load phenotype.cls file")) # Very simple, isn't it?
        #fname.CLS <<- file.choose()
        if(length(grep(".cls$", ENV$fname.CLS))==0){
            tkmessageBox(message="Choose .cls format file", icon="warning")
            stop("Choose .cls format file")
            tkfocus(ENV$ttDialogGSEA)
        } else
            tkinsert(ENV$tlCLS,"end",ENV$fname.CLS)
        tkfocus(ENV$ttDialogGSEA)
    }
    
    chooseCLS.but <- tkbutton(ENV$ttDialogGSEA, text = "browse", command = chooseCLS)
    
    Phenotype.cls.but<- tkbutton(ENV$ttDialogGSEA, text = "Phenotype.cls", command = getGCTCLSExample)
    #tklabelCLS <- tklabel(ENV$ttDialogGSEA,text="Phenotype.cls")
    
    tkgrid(Phenotype.cls.but,ENV$tlCLS,chooseCLS.but, columnspan=1)
    tkgrid.configure(Phenotype.cls.but,rowspan=20, columnspan=1,sticky="nsw")
    
    ### Select gene Expression matrix (.gct file)
    ENV$tlGCT<-tklistbox(ENV$ttDialogGSEA,height=1, width= 70 ,background="white")
    chooseGCT <- function(){
        if(exists("fname.GCT", envir = ENV)){
            tkdelete(ENV$tlGCT,0,1)
        }
        ENV$fname.GCT <- tclvalue(tkgetOpenFile(filetypes = "{{GCT Files} {.gct}} {{All files} *}", title="load Expression.gct file")) # Very simple, isn't it?
        #fname.GCT <<- file.choose()
        if(length(grep(".gct$", ENV$fname.GCT))==0){
            tkmessageBox(message="Choose .gct format file", icon="warning")
            stop("Choose .gct format file")
            tkfocus(ENV$ttDialogGSEA)
        } else
            tkinsert(ENV$tlGCT,"end",ENV$fname.GCT)
        tkfocus(ENV$ttDialogGSEA)
    }
    
    chooseGCT.but <- tkbutton(ENV$ttDialogGSEA, text = "browse", command = chooseGCT)
    
    tklabelGCT <-tklabel(ENV$ttDialogGSEA,text="Expression.gct")
    
    tkgrid(tklabelGCT,ENV$tlGCT,chooseGCT.but, columnspan=1)
    tkgrid.configure(tklabelGCT,rowspan=20, columnspan=1,sticky="nsw")
    
    ###Select gene Set (mSigDB) dataBase (.gmt file)
    
    ENV$tlGMT<-tklistbox(ENV$ttDialogGSEA,height=1, width= 70 ,background="white")
    chooseGMT <- function(){
        if(exists("fname.GMT", envir = ENV)){
            tkdelete(ENV$tlGMT,0,1)
        }
        ENV$fname.GMT <- tclvalue(tkgetOpenFile(filetypes = "{{GMT Files} {.gmt}} {{All files} *}", title="load MSigDB.gmt file")) # Very simple, isn't it?
        #fname.GMT <<- file.choose()
        if(length(grep(".gmt$", ENV$fname.GMT))==0){
            tkmessageBox(message="Choose .gmt format file", icon="warning")
            stop("Choose .gmt format file")
            tkfocus(ENV$ttDialogGSEA)
        } else
            tkinsert(ENV$tlGMT,"end",ENV$fname.GMT)
        tkfocus(ENV$ttDialogGSEA)
    }
    
    chooseGMT.but <- tkbutton(ENV$ttDialogGSEA, text = "browse", command = chooseGMT)
    
    
    MSigDB.but <- tkbutton(ENV$ttDialogGSEA, text = "MSigDB.gmt", command = getMSigDBExample)
    
    tkgrid(MSigDB.but,ENV$tlGMT,chooseGMT.but, columnspan=1)
    tkgrid.configure(MSigDB.but,rowspan=20, columnspan=1,sticky="nsw")
    
    
    ### Select output Directory 
    
    tlOutput<-tklistbox(ENV$ttDialogGSEA,height=1, width= 70 ,background="white")
    
    chooseOutput <- function(){
        if(exists("fname.Output", envir = ENV)){
            tkdelete(tlOutput,0,1)
        }
        prefix<- paste(sub(".cls","" ,basename(ENV$fname.CLS)),sub(".gmt","" ,basename(ENV$fname.GMT)), sep="_")
        ENV$fname.Output <- paste(getwd(),"/Results/GSEA/", prefix,"/", sep="")
        
        tkinsert(tlOutput,"end",ENV$fname.Output)
        
        Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        if(!file.exists("Results/GSEA/")){
            dir.create(file.path(paste(getwd(), "/Results/GSEA", sep="")), showWarnings = FALSE)
            dir.create(file.path(paste(getwd(), "/Results/GSEA", sep="")), showWarnings = FALSE)
        } 
        setwd("./Results/GSEA/")
        if(!file.exists(basename(ENV$fname.Output) )){
            
            dir.create(basename(ENV$fname.Output),showWarnings=TRUE)
            
        }
        setwd("../../")
    }
    
    chooseOutput.but <- tkbutton(ENV$ttDialogGSEA, text = "   Set   ", command = chooseOutput)
    
    tklabelOutput <- tklabel(ENV$ttDialogGSEA,text="Output  folder")
    
    tkgrid(tklabelOutput,tlOutput,chooseOutput.but, columnspan=1)
    tkgrid.configure(tklabelOutput,rowspan=20, columnspan=1,sticky="nsw")
    
    
    OK <- function(){
        
        
        print(ENV$fname.GCT)
        print(ENV$fname.CLS)
        print(ENV$fname.GMT)
        print(ENV$fname.Output)
        
        #setwd(paste(getwd(), "/Results/GSEA", sep=""))
        tkconfigure(ENV$ttDialogGSEA,cursor="watch")
        tkdestroy(ENV$ttDialogGSEA)
    }
    
    Ok.but <- tkbutton(ENV$ttDialogGSEA, text = " OK ", command = OK)
    
    Cancel <- function(){
        
        tkdestroy(ENV$ttDialogGSEA)
        
    }
    Cancel.but <- tkbutton(ENV$ttDialogGSEA, text= "Cancel", command = Cancel)
    tkgrid(Cancel.but, Ok.but)
    tkgrid.configure(Cancel.but, sticky="n",columnspan=2)
    tkgrid.configure(Ok.but, sticky="n",columnspan=2)
    
    tkwait.window(ENV$ttDialogGSEA)
    
}

#' Checkbox to select variables from clinical data
#' @usage dialogOptionPhenoTest(eSet)
#' @param eSet Expression Set
#' 
#' @return vectors: variables to test Survival status, AGE, p-value 
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#' \dontrun{
#' dialogOptionPhenoTest(ENV$eSet)
#' }
#' 
dialogOptionPhenoTest <- function(eSet){
    #require(tcltk)
    
    ifrm <- function(obj, env = globalenv()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    ifrm(EventTime)
    ifrm(Category)
    ifrm(Continu)
    
    ## define font for selected variables
    police <- tkfont.create(family="arial", size=10)
    
    
    ENV$ttPheno <- tktoplevel()
    #tkwm.geometry(ENV$ttPheno,"180x250")
    
    tktitle(ENV$ttPheno) <- paste("Associate between a list of variable and the gene expression")
    
    
    # Take LABELS in ENV$ttPheno
    label1 <- tklabel(ENV$ttPheno, text= "Survival\n Event(Living/Deceased) \n and Time (Months)")
    label2 <- tklabel(ENV$ttPheno,text="Categorical \n select non numeric \n variable")
    label3 <- tklabel(ENV$ttPheno, text= "Continuous \n select numeric variable")
    label4 <- tklabel(ENV$ttPheno,text="p.adjust.method")
    #tkgrid(tklabel0,tklabel1, columnspan=2, pady = 10)
    
    
    yscr1 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,
                         command=function(...)tkyview(tl1,...))
    xscr1 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl1,...))
    
    xscr1Info <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl1info,...))
    
    yscr2 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,
                         command=function(...)tkyview(tl2,...))
    xscr2 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl2,...))
    xscr2Info <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl2info,...))
    
    yscr3 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,
                         command=function(...)tkyview(tl3,...))
    xscr3 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl3,...))
    xscr3Info <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl3info,...))
    yscr4 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,
                         command=function(...)tkyview(tl4,...))
    xscr4 <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl4,...))
    xscr4Info <- tkscrollbar(ENV$ttPheno, repeatinterval=2,orient="horizontal",
                             command=function(...)tkxview(tl4info,...))  
    
    
    tl1<-tklistbox(ENV$ttPheno,height=5, width= 20 ,selectmode="multiple",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
    tl1info<-tklistbox(ENV$ttPheno,height=1, width= 20,selectmode="single",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
    tkconfigure(tl1info, foreground="blue", font=police)
    
    tl2<-tklistbox(ENV$ttPheno,height=5, width= 20 ,selectmode="single",xscrollcommand=function(...)tkset(xscr2,...),yscrollcommand=function(...)tkset(yscr2,...),background="white")
    tl2info<-tklistbox(ENV$ttPheno,height=1, width= 20,selectmode="single",xscrollcommand=function(...)tkset(xscr2Info,...),background="white")
    tkconfigure(tl2info, foreground="blue", font=police)
    
    tl3<-tklistbox(ENV$ttPheno,height=5, width= 20 ,selectmode="single",xscrollcommand=function(...)tkset(xscr3,...),yscrollcommand=function(...)tkset(yscr3,...),background="white")
    tl3info<-tklistbox(ENV$ttPheno,height=1, width= 20,selectmode="single",xscrollcommand=function(...)tkset(xscr3Info,...),background="white")
    tkconfigure(tl3info, foreground="blue", font=police)
    
    tl4<-tklistbox(ENV$ttPheno,height=5, width= 20 ,selectmode="single",xscrollcommand=function(...)tkset(xscr4,...),yscrollcommand=function(...)tkset(yscr4,...),background="white")
    tl4info<-tklistbox(ENV$ttPheno,height=1, width= 20,selectmode="single",xscrollcommand=function(...)tkset(xscr4Info,...),background="white")
    tkconfigure(tl4info, foreground="blue", font=police)
    
    
    loadSurvival <- function()
    {  
        curselectSurvival <- as.numeric(tkcurselection(tl1))+1
        lcurselectSurvival <- length(curselectSurvival)
        
        ENV$EventTime <-  names(Biobase::pData(eSet))[curselectSurvival]
        if (lcurselectSurvival !=2){
            msgSelectSurvival<-"Select two variables for Survival \n Event(OS_STATUT) \n Time(OS_MONTHS"
            tkmessageBox(message=msgSelectSurvival)
        } else {
            tkdelete(tl1info,0,1)
            tkinsert(tl1info,"end",ENV$EventTime)
            tkfocus()
        }
        
    }
    
    
    Survival.but <-tkbutton(ENV$ttPheno,text="select",command=loadSurvival)
    
    tkgrid(label1,tl1,yscr1)
    tkgrid.configure(yscr1,rowspan=20, columnspan=2,sticky="nsw")
    tkgrid(xscr1)
    tkgrid.configure(xscr1,rowspan=2, column=1,sticky="we")
    tkgrid(Survival.but, tl1info, columnspan=1)
    tkgrid(xscr1Info)
    tkgrid.configure(xscr1Info,rowspan=4, column=1,sticky="we")
    
    for (i in 1:length(names(Biobase::pData(eSet)))){
        
        tkinsert(tl1,"end",names(Biobase::pData(eSet))[i])
    }
    
    ###################################
    
    loadCategorical <- function()
    {  
        curselectCategorical <- as.numeric(tkcurselection(tl2))+1
        lcurselectCategorical <- length(curselectCategorical)
        
        ENV$Category <-  names(Biobase::pData(eSet))[curselectCategorical]
        
        if (lcurselectCategorical ==0){
            msgSelectCategorical<-"Select one variable for Category \n not numeric"
            tkmessageBox(message=msgSelectCategorical)
            tkfocus(ENV$ttPheno)
        }else if (all(is.na(Biobase::pData(eSet)[curselectCategorical] ))){
            msgNoData <- paste("No data are avaible for ",Category)
            tkmessageBox(message=msgNoData, icon="info")
        } else if (is.numeric(Biobase::pData(eSet)[curselectCategorical][1,])==TRUE){
            msgBadCategory<-"Select non numeric Variable"
            tkmessageBox(message=msgBadCategory)
        } else {
            tkdelete(tl2info,0,1)
            tkinsert(tl2info,"end",ENV$Category)
            tkfocus()
            ##Remove NA from selected variables 
            #Biobase::pData(eSet)[curselectCategorical] <-as.matrix(na.omit(Biobase::pData(eSet)[curselectCategorical]))
        }
        
    }
    
    
    Categorical.but <-tkbutton(ENV$ttPheno,text="select",command=loadCategorical)
    
    tkgrid(label2,tl2,yscr2)
    tkgrid.configure(yscr2,rowspan=20, columnspan=2,sticky="nsw")
    tkgrid(xscr2)
    tkgrid.configure(xscr2,rowspan=2, column=1,sticky="we")
    tkgrid(Categorical.but, tl2info, columnspan=1)
    tkgrid(xscr2Info)
    tkgrid.configure(xscr2Info,rowspan=4, column=1,sticky="we")
    
    for (i in 1:length(names(Biobase::pData(eSet)))){
        
        tkinsert(tl2,"end",names(Biobase::pData(eSet))[i])
    }
    
    ###################################
    
    loadContinuous <- function()
    {  
        curselectContinuous <- as.numeric(tkcurselection(tl3))+1
        lcurselectContinuous <- length(curselectContinuous)
        
        ENV$Continu <-  names(Biobase::pData(eSet))[curselectContinuous]
        if (lcurselectContinuous ==0){
            msgSelectContinuous<-"Select one or more Continuous variables \n  numeric"
            tkmessageBox(message=msgSelectContinuous)
        }
        if (is.character(Biobase::pData(eSet)[curselectContinuous][1,])==TRUE|is.factor(Biobase::pData(eSet)[curselectContinuous][1,])==TRUE){
            msgBadContinuous<-"Select non character/factor Variable"
            tkmessageBox(message=msgBadContinuous)
        } else {
            tkdelete(tl3info,0,1)
            tkinsert(tl3info,"end",ENV$Continu)
            tkfocus()
            ##Remove NA from selected variables 
            #Biobase::pData(eSet)[curselectContinuous] <-as.matrix(na.omit(Biobase::pData(eSet)[curselectContinuous]))
        }
        
    }
    
    
    Continuous.but <-tkbutton(ENV$ttPheno,text="select",command=loadContinuous)
    
    tkgrid(label3,tl3,yscr3)
    tkgrid.configure(yscr3,rowspan=20, columnspan=2,sticky="nsw")
    tkgrid(xscr3)
    tkgrid.configure(xscr3,rowspan=2, column=1,sticky="we")
    tkgrid(Continuous.but, tl3info, columnspan=1)
    tkgrid(xscr3Info)
    tkgrid.configure(xscr3Info,rowspan=4, column=1,sticky="we")
    
    for (i in 1:length(names(Biobase::pData(eSet)))){
        
        tkinsert(tl3,"end",names(Biobase::pData(eSet))[i])
    }
    
    
    ###################################
    
    tkconfigure(tl4info, foreground="blue", font=police)
    
    loadp.adjust.Method <- function()
    {  
        curselectp.adjust <- as.numeric(tkcurselection(tl4))+1
        lcurselectp.adjust <- length(curselectp.adjust)
        
        p.adjustMethod <- c('none','BH', 'BY','bonferroni' )
        
        if (lcurselectp.adjust ==0){
            msgSelectp.adjust<-"Select p.adjust Method"
            tkmessageBox(message=msgSelectp.adjust)
        } else {
            tkdelete(tl4info,0,1)
            tkinsert(tl4info,"end",p.adjustMethod[curselectp.adjust])
            ENV$p.adjustChoice <- p.adjustMethod[curselectp.adjust]
            tkfocus()
        }
        
    }
    
    
    p.adjust.but <-tkbutton(ENV$ttPheno,text="select",command=loadp.adjust.Method)
    
    tkgrid(label4,tl4,yscr4)
    tkgrid.configure(yscr4,rowspan=20, columnspan=2,sticky="nsw")
    tkgrid(xscr4)
    tkgrid.configure(xscr4,rowspan=2, column=1,sticky="we")
    tkgrid(p.adjust.but, tl4info, columnspan=1)
    tkgrid(xscr4Info)
    tkgrid.configure(xscr4Info,rowspan=4, column=1,sticky="we")
    
    p.adjustMethod <- c('none','BH', 'BY','bonferroni' )
    
    
    
    for (i in 1:length(p.adjustMethod)){
        
        tkinsert(tl4,"end",p.adjustMethod[i])
    }
    
    # Default selection.  Indexing starts at zero.
    tkselection.set(tl4,1) 
    
    onOK <- function(){
        if(exists("EventTime", envir = ENV)){
            survival <- matrix(c(ENV$EventTime[1],ENV$EventTime[2]), ncol=2, byrow=TRUE)  ##Statut: Living vs die, months: number of months
            colnames(survival)<-  c('time','event')  ## colnames of survival matrix 
            
            ENV$var2test <- list(survival=survival,categorical=ENV$Category, continuous=ENV$Continu) 
            
            print(paste("varibales to test:",ENV$var2test, sep=" "))
            print(paste("pVals adujustment choice:",ENV$p.adjustChoice, sep=" "))
        } else{
            
            ENV$var2test <- list(categorical=ENV$Category, continuous=ENV$Continu)
            print(paste("varibales to test:",ENV$var2test, sep=" "))
            print(paste("pVals adujustment choice:",ENV$p.adjustChoice, sep=" "))
        }
        tkdestroy(ENV$ttPheno)
        
    }
    
    Ok.but <-tkbutton(ENV$ttPheno,text=" OK ",command=onOK)
    tkgrid(Ok.but)
    tkgrid.configure(Ok.but,rowspan=4, column=1,sticky="we")
    
    
    
    
}

#' model plotting with tcltk
#' @usage plotModel(plotCommand, title= "TITLE",hscale=1, vscale=1 )
#' @param plotCommand plotcommand
#' @param title title of plot
#' @param hscale horizintal scale
#' @param vscale vertical scale
#' 
#' @return plot
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
#' \dontrun{
#' plot_1Gene_2GenProfs()
#' }
plotModel <- function(plotCommand, title= "TITLE",hscale=1, vscale=1 ){
    
    
    ttplot <- tktoplevel()
    tkwm.title(ttplot,title)
    img <- tkrplot::tkrplot(ttplot,fun=plotCommand,hscale=hscale,vscale=vscale)
    
    
    
    
    
    ####Save Graph Option
    rbValue3 <- tclVar("PNG") # le choix par défaut
    
    frameGraph <- tkframe(ttplot,relief="groove",borderwidth=2)
    tkgrid(frameGraph)
    tkgrid.configure(frameGraph, sticky="new")
    labelGraph<-(tklabel(frameGraph, text=" Specify format file:"))
    rb9 <- tkradiobutton(frameGraph)
    rb10 <- tkradiobutton(frameGraph)
    rb11 <- tkradiobutton(frameGraph)
    
    # config des boutons radio. Une seule variable tcl pour 3 boutons
    tkconfigure(rb9,variable=rbValue3,value="SVG", text="SVG")
    tkconfigure(rb10,variable=rbValue3,value="JPG", text="JPG")
    tkconfigure(rb11,variable=rbValue3,value="PNG", text="PNG")
    
    tkgrid(labelGraph,rb9, rb10, rb11,columnspan=4)
    tkgrid.configure(rb9,rb10,rb11, sticky="e")
    
    
    
    
    
    Save <- function(){
        rbVal3 <- as.character(tclvalue(rbValue3))
        tkdestroy(ttplot)
        if (rbVal3=="SVG")
            #tkmessageBox(message=rbVal2)
            ENV$graph<-paste(rbVal3)
        if (rbVal3=="JPG")
            #tkmessageBox(message=rbVal2)
            ENV$graph<-paste(rbVal3)
        if (rbVal3=="PNG")
            #tkmessageBox(message=rbVal2)
            ENV$graph<-paste(rbVal3)
        
        #fileName <- tclvalue(tkgetSaveFile(initialfile = "plot.svg",filetypes = "{{SVG Files} {.svg}} {{JPEG Files} {.jpg .jpeg}} {{PNG Files} {.png}}  {{All files} *}")) 
        if (ENV$graph=="SVG"){
            fileName <- tclvalue(tkgetSaveFile(initialfile = "plot.svg",filetypes = "{{SVG Files} {.svg}}")) 
        } else if (ENV$graph=="JPG"){
            fileName <- tclvalue(tkgetSaveFile(initialfile = "plot.jpg",filetypes = "{{JPEG Files} {.jpg .jpeg}}")) 
        } else if (ENV$graph=="PNG"){
            fileName <- tclvalue(tkgetSaveFile(initialfile = "plot.png",filetypes = "{{PNG Files} {.png}} ")) 
            
        }
        if (!nchar(fileName)) {
            tkmessageBox(message = "No file was selected!")
        } else {
            tkmessageBox(message = paste("The file selected was", fileName))
        }
        ##ENV$graph file type Option
        if(ENV$graph=="SVG"){
            #library(RSvgDevice) Doesn't work with windows OS
            #RSvgDevice::devSVG(fileName,width = 600 / 300, height = 600 / 300)
            if(Sys.info()['sysname'] != "windows"){
            #require(RSvgDevice)
            devSVG(fileName,width = 600 / 300, height = 600 / 300)
            plotCommand()
            graphics.off()
            }
        } else if (ENV$graph=="JPG"){
            jpeg(fileName, width=7, height=7,units="in", res=300)
            plotCommand()
            dev.off()
        } else if (ENV$graph=="PNG"){
            png(fileName,width=7, height=7,units="in", res=300)
            plotCommand()
            dev.off()
        }
    }
    
    tkgrid(frameGraph)
    tkgrid.configure(frameGraph, sticky="e")
    
    tkgrid(img)
    Save.but <- tkbutton(ttplot,text="Save",command=Save)
    
    tkgrid(Save.but)
    #tkgrid(copy.but, Save.but, column="0")
    #tkgrid.configure(copy.but, sticky="w")
    tkgrid.configure(Save.but,sticky="e")
    
}

#' Dialog Box to set methylation options
#' @usage
#' dialogMetOption(ProfData, k)
#' @param ProfData adataframe with methylation data
#' @param k threshold of silencing gene 0:1
#'
#' @return a dialog box to set methylation option (threshold of silencing gene)
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getMetDataMultipleGenes()
#' #dialogMetOption(ProfData,0.7)
#' }
#' 
#' 
dialogMetOption <- function(ProfData,k){
    
    ttMethData <- tktoplevel()
    #tkwm.geometry(ttMethData,"180x250")
    
    tktitle(ttMethData) <- paste(ENV$StudyRefCase[k],ENV$GenProfChoice[k], sep=" ")
    
    rEntry  = tclVar(.8)
    
    frameTHRESHOLD<- tkframe(ttMethData,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameTHRESHOLD, text="Specify Threshold of correlation rate:"))
    tkgrid(frameTHRESHOLD)
    tkgrid.configure(frameTHRESHOLD, sticky="new")
    
    tkgrid(tkscale(frameTHRESHOLD,from=0.5,to=1,showvalue=TRUE,
                   variable=rEntry,resolution=.05,orient='horiz'))
    
    
    okOn <- function(){
    
        #Compute mean of correlation
        meanProfData <-as.matrix(round(colMeans(ProfData), digits=3))
        #Compute median of correlation
        medianProfData<- as.matrix(apply(ProfData, 2, median))
        #Entry Threshold of correlation
        seuilRVal <- as.numeric(tclvalue(rEntry))
        
        MeanTable <- subset(meanProfData, meanProfData[,1] > seuilRVal)
        MedianTable <- subset(medianProfData, medianProfData[,1] > seuilRVal)
        
        mergeTable <- merge(MeanTable, MedianTable, by="row.names")
        
        colnames(mergeTable) <- c("Gene","Mean", "Median")
        
        title <- paste (ENV$StudyRefCase[k],":",
                        "Correlation of silencing gene expression by Methylation",
                        "r(met/mRNA)>",
                        seuilRVal,ENV$GenProfChoice[k], sep=" " )
        
        getInTable(mergeTable, title=title)
        
        
        tkdestroy(ttMethData)
    }
    
    Ok.but <- tkbutton(ttMethData, text= "OK", command= okOn)
    
    
    tkgrid(Ok.but)
    tkgrid.configure(Ok.but, sticky="n", column=0)
    
    
    tkwait.window(ttMethData)
    
}

#' Dialogue Box for gene classifier setting: sample size and postprob threshold
#' @usage
#' dialogGeneClassifier(Lchecked_Cases,entryWidth = 10,returnValOnCancel = "ID_CANCEL")
#' @param Lchecked_Cases integer with a number of checked cases
#' @param  entryWidth integer default 10
#' @param returnValOnCancel "ID_CANCEL"
#' 
#' @return a dataframe with genes classes
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
#' \dontrun{
#' getGenesClassifier()
#' dialogGeneClassifier(1,10,returnValOnCancel = "ID_CANCEL")
#'}
#'
#'

dialogGeneClassifier <- function(Lchecked_Cases,entryWidth = 10,
                                 returnValOnCancel = "ID_CANCEL") {
    
    dlg <- tktoplevel()
    tkwm.deiconify(dlg)
    tkgrab.set(dlg)
    tkfocus(dlg)
    tkwm.title(dlg, "Sampling...")
    
    
    txtMsg <- tklabel(dlg, text = "Max Nbr of samples does not exceed the smaller samples Nbr of all checked Cases",background="white")
    tkgrid(txtMsg)
    tkgrid.configure(txtMsg, column=0, row=1,sticky="we")
    textEntryVarTcl <- tclVar("50")
    textEntryWidget <- tkentry(dlg, width = paste(entryWidth),
                               textvariable = textEntryVarTcl)
    #tkgrid(tklabel(dlg, text = "       "))
    txtNbr <- tklabel(dlg, text = "Number of Samples")
    tkgrid(txtNbr)
    tkgrid.configure(txtNbr, column=0, row=2, sticky="w")
    tkgrid(textEntryWidget)
    tkgrid.configure(textEntryWidget, column=0, row=2, sticky="ne")
    
    frameCases<- tkframe(dlg,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameCases, text="Checked Cases:"))
    tkgrid.configure(frameCases, sticky="new")
    
    
    
    for(i in 3:(Lchecked_Cases+2)){
        
        stdy <- tklabel(frameCases, text=ENV$StudyRefCase[i-2])
        tkgrid(stdy)
        tkgrid.configure(stdy, column=0,row=i, sticky="e")
        SamplesNbr <-tklabel(frameCases, text = paste(": " ,ENV$CaseChoice[i-2]))
        tkgrid(SamplesNbr)
        tkgrid.configure(SamplesNbr, column=1, row=i,sticky="w")
    }
    
    ReturnSamplesNbr <- returnValOnCancel
    
    textEntrylpThreshold <- tclVar("0.95")
    textEntryWidget2 <- tkentry(dlg, width = paste(entryWidth),
                                textvariable = textEntrylpThreshold)
    #tkgrid(tklabel(dlg, text = "       "))
    TxtPostProb <- tklabel(dlg, text="Posterior Probability Threshold")
    tkgrid(TxtPostProb)
    tkgrid.configure(TxtPostProb, column=0, row= (Lchecked_Cases+3), sticky="w")
    
    txtThres <- tklabel(dlg, text = "lpThreshold")
    tkgrid(txtThres)
    tkgrid.configure(txtThres, column=0, row=(Lchecked_Cases+4), sticky="w")
    tkgrid(textEntryWidget2)
    tkgrid.configure(textEntryWidget2, column=0, row=(Lchecked_Cases+4), sticky="ne")
    
    
    onOK <- function() {
        ENV$ReturnSamplesNbr <- as.numeric(tclvalue(textEntryVarTcl))
        ENV$Threshold <- as.numeric(tclvalue(textEntrylpThreshold))
        
        tkgrab.release(dlg)
        tkdestroy(dlg)
        
    }
    onCancel <- function() {
        #ReturnSamplesNbr <- returnValOnCancel
        tkgrab.release(dlg)
        tkdestroy(dlg)
    }
    OK.but <- tkbutton(dlg, text = "   OK   ", command = onOK)
    Cancel.but <- tkbutton(dlg, text = " Cancel ", command = onCancel)
    tkgrid(Cancel.but)
    tkgrid.configure(Cancel.but, column=0,row= (Lchecked_Cases+5), sticky="w")
    tkgrid(OK.but)
    tkgrid.configure(OK.but, column=0,row= (Lchecked_Cases+5), sticky="e")
    #tkgrid(tklabel(dlg, text = "    "))
    
    tkfocus(dlg)
    tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg)})
    tkbind(textEntryWidget, "<Return>", onOK)
    tkwait.window(dlg)
    
    return(c(ENV$ReturnSamplesNbr, ENV$Threshold))
    
    
}

#' Search and get genetic profiles (CNA,mRNA, Methylation, Mutation...)
#'
#' @details See \url{https://github.com/kmezhoud/bioCancer/wiki}
#'
#' @return A data frame with Genetic profile
#'
#' @usage getProfData(study,genProf, listGenProf, GeneList, Mut)
#' @param study  Study ID
#' @param genProf Genetic Profile id (cancer_study_id_[mutations, cna, methylation, mrna ]).
#' @param listGenProf A list of Genetic Profiles for one study.
#' @param GeneList A list of genes
#' @param Mut Condition to set if the genetic profile is mutation or not (0,1)
#'
#' @examples
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
#'@export
getProfData<-function(study, genProf, listGenProf, GeneList, Mut){
    
    
    if(length(grep(genProf, listGenProf))!= 0){
        
        ProfData_X <- cBioPortalData::getDataByGenes(api =  ENV$cgds,
                                                     studyId = study,
                                                     genes = GeneList,
                                                     by = "hugoGeneSymbol",
                                                     molecularProfileIds = genProf) |>
            unname() |>
            as.data.frame()
        
        # avoid error if no profile data exist for geneList
        if(nrow(ProfData_X)<1){
            #all(dim(ProfData_X)==c(0,1) ||
            #nrow(ProfData_X)==0)== TRUE){ #length(ProfData_X)== 0 ||
            
            #print(paste0("No Profile Data available for:", genProf))
            
            ## built empty data frame with gene Symbol in colnames
            if(Mut==0){
                
                ProfData_X <- as.data.frame(setNames(replicate(length(GeneList),numeric(1),
                                                               simplify = FALSE),
                                                     GeneList[order(GeneList)]))
                return(ProfData_X)
                
            }else if (Mut==1){
                ## built emty data.frame as the same form of MutData
                hugoGeneSymbol <- as.vector(GeneList)
                proteinChange <- rep(character(1), length(GeneList))
                variantType <- rep(character(1), length(GeneList))
                MutData <- data.frame(hugoGeneSymbol, proteinChange, variantType)
                
                return(MutData)
            }
            
        }else if(Mut== 0){
            
            ProfData_X <- ProfData_X |>
                #.[[1]] |>
                select("sampleId", "hugoGeneSymbol", "value") |>
                tidyr::spread("hugoGeneSymbol", "value") |>
                data.frame(row.names = 1)
            ##  check the order of geneList and add gene with empty data NA.
            missing <- setdiff(GeneList[order(GeneList)], names(ProfData_X))
            ProfData_X[missing] <- NA
            ProfData_X |> select(GeneList[order(GeneList)])
            
            return(ProfData_X)
            
        } else if(Mut==1){
            
            #print(paste0("Getting Mutation Data of ", study ," ..."))
            
            MutData <- ProfData_X  |>
                #.[[1]] |>
                select("hugoGeneSymbol", "proteinChange", "variantType")
            
            return(MutData)
            
        }
    }else{
        
        msgNoGenProf= paste("There is no genetic Profiles: ",
                                     genProf )
        
        tkmessageBox(message= msgNoGenProf)
        #stop(msgNoGenProf)
        
        print(paste("There is no genetic Profiles: ", genProf ))
        
        ProfData_X <- as.data.frame(setNames(replicate(
            length(GeneList),numeric(1), simplify = FALSE),
            GeneList[order(GeneList)]))
        
        return(ProfData_X)
    }
}



#' Unify row names in data frame with the same order of gene list.
#' @usage UnifyRowNames(x,geneList)
#' @param x data frame with gene symbol in the row name
#' @param geneList a gene list
#'
#' @return a data frame having the gene in row name ordered as in gene list.
#'
#' @examples
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
#' @export
UnifyRowNames <- function(x, geneList){
    ## compute the ratio of mutation
    df_MutData <-as.data.frame(table(x$hugoGeneSymbol) /sum(table(x$hugoGeneSymbol))*100)
    ## compute le sum of mutation using table function.
    #df_MutData <-as.data.frame(table(x$hugoGeneSymbol))
    rownames(df_MutData) <- df_MutData$Var1  #tibble::column_to_rownames("sampleId")
    ## ordering genes in MutData as in GeneList
    
    df_GeneList <- as.data.frame(t(geneList))
    #df_GeneList <- as.data.frame(GeneList)
    rownames(df_GeneList) <- df_GeneList[,1]
    df_merge <- merge(df_GeneList, df_MutData, by="row.names",all.x=TRUE)
    Freq_Mut <- df_merge[,c(-2,-3)]
    return(Freq_Mut)
}


#' get mutation frequency
#'
#' @usage getFreqMutData(list, GeneList)
#'
#' @param list a list of data frame with mutation data. Each data frame is for one study
#' @param GeneList file name of GeneList examples: "73"
#'
#' @return a data frame with mutation frequency. gene is in rows and study is in column
#' @export
#'
#' @examples
#' cgds <- cBioPortal(
#' hostname = "www.cbioportal.org",
#' protocol = "https",
#' api = "/api/v2/api-docs"
#' )
#' \dontrun{
#' getDataByGenes( api =  cgds,
#' studyId = "gbm_tcga_pub",
#' genes = c("NF1", "TP53", "ABL1"),
#' by = "hugoGeneSymbol",
#' molecularProfileIds = "gbm_tcga_pub_mrna"
#' )
#'}
getFreqMutData <- function(list, GeneList){
    
    #GeneList <- whichGeneList(GeneListLabel)
    
    if(is.null(list)){stop("Select a less one Study.")}
    
    Freq_ListMutData <- lapply(list,
                               function(x) UnifyRowNames(x, GeneList))
    
    ## convert the list of correlation matrices to Array
    Freq_ArrayMutData <- array(unlist(Freq_ListMutData),
                               dim = c(nrow(Freq_ListMutData[[1]]),
                                       ncol( Freq_ListMutData[[1]]),
                                       length(Freq_ListMutData)))
    
    if (inherits(try(dimnames(Freq_ArrayMutData) <-
                     list(Freq_ListMutData[[1]][,1],
                          colnames(Freq_ListMutData[[1]]),
                          names(Freq_ListMutData)),
                     silent=TRUE),"try-error")){
        p("There is a Study without Mutation Data.
      Use Mutation Panel to verify mutations data for selected studies.",
          align="center", style = "color:blue")
    }else{
        dimnames(Freq_ArrayMutData) <-
            list(Freq_ListMutData[[1]][,1],
                 colnames(Freq_ListMutData[[1]]),
                 names(Freq_ListMutData))
    }
    #   ?getListProfData(Genes= empty)
    if(dim(Freq_ArrayMutData)[3]==1){
        Freq_DfMutData <- as.numeric(Freq_ArrayMutData[,2,])
        names(Freq_DfMutData) <- names(Freq_ArrayMutData[,2,])
        ## ordering gene list as in GeneList from MSigDB:
        ## grouping genes with the same biological process or gene Sets
        Freq_DfMutData <- Freq_DfMutData[GeneList]
        Freq_DfMutData <- data.frame(round(Freq_DfMutData,digits=2))
        names(Freq_DfMutData) <- names(Freq_ListMutData)
    }else{
        Freq_DfMutData <- apply(Freq_ArrayMutData[,2,],2,as.numeric)
        rownames(Freq_DfMutData) <- rownames(Freq_ArrayMutData[,2,])
        ## ordering gene list as in GeneList from MSigDB:
        ## grouping genes with the same biological process or gene Sets
        Freq_DfMutData <- Freq_DfMutData[GeneList,,drop=FALSE]
        Freq_DfMutData <- data.frame(round(Freq_DfMutData,digits=2))
    }
    return(Freq_DfMutData)
}



#' Get list of data frame with profiles data (CNA,mRNA, Methylation, Mutation...)
#'
#' @usage getListProfData(checked_Studies, geneList)
#'
#' @param checked_Studies checked studies in corresponding panel (input$StudiesIDCircos, input$StudiesIDReactome).
#' @param geneList  GeneList with Hugo Symbol
#'
#' @return A LIST of profiles data (CNA, mRNA, Methylation, Mutation, miRNA, RPPA).
#'         Each dimension content a list of studies.
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' getListProfData()
#' head(ENV$ProfData$Expression)
#' }
#' 
getListProfData <- function(checked_Studies, geneList){

# if(exists("ListProfData", envir = ENV)){
#     rm("ListProfData", envir=ENV)
# }
# if(exists("ListMetData", envir = ENV)){
#     rm("ListMetData", envir=ENV)
# }
# if(exists("ListMutData", envir = ENV)){
#     rm("ListMutData", envir=ENV)
# }
    
    ENV$ListProfData <- NULL
    ENV$ListMetData <- NULL
    ENV$ListMutData <- NULL
    

    #Lchecked_Studies <- ENV$lchecked_Studies

    #get Study references
    #StudiesRef <- getCancerStudies(ENV$cgds)[,1]


    #LengthGenProfs <- 0
    #LengthCases <- 0
    i <- 0
    for (s in checked_Studies){
        i <- i+1
        #Si = ENV$checked_StudyIndex[i]
        progressBar_ProfilesData <- tkProgressBar(title = paste(s,":"), 
                                                  min = 0,
                                                  max = length(checked_Studies), 
                                                  width = 400)

        Sys.sleep(0.1)
        setTkProgressBar(progressBar_ProfilesData, i, 
                         label=paste(round(i/length(checked_Studies)*100, 0),
                                     "% of Profiles Data"))

        ### get Cases and Genetic Profiles  with cgdsr references
        GenProf_CNA<- paste(s,"_gistic", sep="")
        #Case_CNA   <- paste(s,"_cna", sep="")

        GenProf_Exp<- paste(s,"_rna_seq_v2_mrna", sep="")
        #Case_Exp   <- paste(s,"_rna_seq_v2_mrna", sep="")

        GenProf_Met_HM450<- paste(s,"_methylation_hm450", sep="")
        #Case_Met_HM450   <- paste(s,"_methylation_hm450", sep="")

        GenProf_Met_HM27<- paste(s,"_methylation_hm27", sep="")
        #Case_Met_HM27   <- paste(s,"_methylation_hm27", sep="")

        GenProf_RPPA<- paste(s,"_RPPA_protein_level", sep="")
        #Case_RPPA   <- paste(s,"_rppa", sep="")

        GenProf_miRNA<- paste(s,"_mirna", sep="")
        #Case_miRNA   <- paste(s,"_microrna", sep="")

        GenProf_Mut<- paste(s,"_mutations", sep="")
        #Case_Mut   <- paste(s,"_sequenced", sep="")
        
        
        ## get Genetics Profiles for selected each Study
        GenProfsRefStudies <- unname(unlist(apply(
            as.data.frame(s), 1,
            function(x) molecularProfiles(api = ENV$cgds, studyId = x)[,"molecularProfileId"])))

        if (length(geneList) > 0){
            print(GenProf_Exp)
            
            ProfData_CNA <- getProfData(s, GenProf_CNA,
                                        GenProfsRefStudies, geneList, Mut=0)
            
            ProfData_Exp <- getProfData(s, GenProf_Exp,
                                        GenProfsRefStudies, geneList, Mut=0)
            
            ProfData_Met_HM450 <- getProfData(s, GenProf_Met_HM450,
                                              GenProfsRefStudies, geneList, Mut=0)
            
            ProfData_Met_HM27 <- getProfData(s, GenProf_Met_HM27,
                                             GenProfsRefStudies, geneList, Mut=0)
            
            ProfData_RPPA <- getProfData(s, GenProf_RPPA,
                                         GenProfsRefStudies, geneList, Mut=0)
            
            ProfData_miRNA <- getProfData(s, GenProf_miRNA,
                                          GenProfsRefStudies, geneList, Mut=0)
            
            MutData <- getProfData(s,GenProf_Mut,
                                   GenProfsRefStudies, ENV$GeneList, Mut=1)
            

            # ProfData_CNA<- grepRef(Case_CNA, ENV$CasesRefStudies, GenProf_CNA,
            #                        ENV$GenProfsRefStudies, ENV$GeneList, Mut=0)
            # ProfData_Exp<- grepRef(Case_Exp, ENV$CasesRefStudies, GenProf_Exp,
            #                        ENV$GenProfsRefStudies, ENV$GeneList, Mut=0)
            # ProfData_Met_HM450 <- grepRef(Case_Met_HM450, ENV$CasesRefStudies,  GenProf_Met_HM450,
            #                               ENV$GenProfsRefStudies, ENV$GeneList, Mut=0)
            # ProfData_Met_HM27 <- grepRef(Case_Met_HM27, ENV$CasesRefStudies, 
            #                              GenProf_Met_HM27, ENV$GenProfsRefStudies, ENV$GeneList,Mut=0)
            # ProfData_RPPA<- grepRef(Case_RPPA, ENV$CasesRefStudies,
            #                         GenProf_RPPA, ENV$GenProfsRefStudies, ENV$GeneList,Mut=0)
            # ProfData_miRNA<- grepRef(Case_miRNA, ENV$CasesRefStudies,
            #                          GenProf_miRNA, ENV$GenProfsRefStudies, ENV$GeneList,Mut=0)
            # MutData <- grepRef(Case_Mut,ENV$CasesRefStudies ,GenProf_Mut,
            #                    ENV$GenProfsRefStudies,ENV$GeneList, Mut=1)

        } else {
            tkmessageBox(message= "Load gene List", icon="warning")
            close(progressBar_ProfilesData)
            stop("Load Gene List")
        }

        ENV$ListProfData$CNA[[s]] <- ProfData_CNA
        ENV$ListProfData$Expression[[s]] <- ProfData_Exp
        ENV$ListMetData$HM450[[s]] <- ProfData_Met_HM450
        ENV$ListMetData$HM27[[s]] <- ProfData_Met_HM27
        ENV$ListProfData$RPPA[[s]] <- ProfData_RPPA
        ENV$ListProfData$miRNA[[s]] <- ProfData_miRNA
        ENV$ListMutData[[s]] <- MutData

        print(" End Getting Profiles Data... ")
        close(progressBar_ProfilesData)
    }

    print("Start getting Frequency of Mutation ...")
    
    ENV$Freq_DfMutData <- getFreqMutData(ENV$ListMutData, geneList)

    print("End getting Mutation Frequency...")

}

#' Multi-select choice of gene sets from loaded MSigDB
#' @usage
#' dialoggetGeneListMSigDB(MSigDB)
#' @param MSigDB object with MSigDB. A list of genesets
#' 
#' @return a dataframe with genes classes
#' @export
#'
#' @examples
#' z <- 7
#' \dontrun{
#' ##MSigDB <- readLines(paste(.libPaths(),"/canceR/extdata/MSigDB/c5.bp.v4.0.symbols.gmt", sep=""))
#' dialoggetGeneListMSigDB(MSigDB)
#' }
#'
#'
#'
dialoggetGeneListMSigDB <- function(MSigDB){   

    if(exists("match_GS", envir = ENV)){
        rm("match_GS", envir=ENV)
    }
    
## function load matched Gene Sets   
loadMatchGS <- function(Word){
        
    
    # select row with matched "string"
    GSidx <- grep(Word, names(MSigDB), ignore.case=TRUE) 
    
    ENV$match_GS <- names(MSigDB)[GSidx]
    
    
    ##Count the nomber of Matched Studies and return the number.
    nMatchGS <- paste("Query result: ",length(ENV$match_GS), " Gene Sets were Matched.",sep="")
    #tkgrid(tklabel(ttGeneListMSigDB,text= nMatchGS ))
    tkdelete(tlInfo,0,1)
    tkinsert(tlInfo,"end",nMatchGS)
    
    tkdelete(tl1,0,1500)
    #tkdelete(tl1info,0,1)
    for (i in (1:length(ENV$match_GS)))
    { 
        tkinsert(tl1,"end",ENV$match_GS[i])
    }
    tkselection.set(tl1,2)  # Default selection.  Indexing starts at zero.

    #tkdelete(tl1info,0)
}



launchDialog <- function() {
    Word <- modalDialog("Search Gene Sets", "Search by Key Word", "")
    if (Word == "ID_CANCEL") return()
    loadMatchGS(Word)
    
}



    
    
###############################
## define font for selected variables
police <- tkfont.create(family="arial", size=10)

ttGeneListMSigDB <- tktoplevel()

tktitle(ttGeneListMSigDB) <- paste("Get Gene Set")

loadMatchGS.button <- tkbutton(ttGeneListMSigDB, text = "Search by key words", command = launchDialog)
tlInfo<-tklistbox(ttGeneListMSigDB,height=1, width= 40,selectmode="single",background="white")

tkgrid(tlInfo,loadMatchGS.button)
tkgrid.configure(tlInfo, column=0, sticky="w")
tkgrid.configure(loadMatchGS.button, column=0, sticky="e")
##LABELS 
label1 <- tklabel(ttGeneListMSigDB, text= "Select Gene Sets")

yscr1 <- tkscrollbar(ttGeneListMSigDB, repeatinterval=2,
                     command=function(...)tkyview(tl1,...))
xscr1 <- tkscrollbar(ttGeneListMSigDB, repeatinterval=2,orient="horizontal",
                     command=function(...)tkxview(tl1,...))

yscr1Info <- tkscrollbar(ttGeneListMSigDB, repeatinterval=2,orient="vertical",
                         command=function(...)tkxview(tl1info,...))
xscr1Info <- tkscrollbar(ttGeneListMSigDB, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(tl1info,...))


tl1<-tklistbox(ttGeneListMSigDB,height=10, width= 60 ,selectmode="multiple",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")
tl1info<-tklistbox(ttGeneListMSigDB,height=5, width= 80,selectmode="multiple",xscrollcommand=function(...)tkset(xscr1Info,...),background="white")
tkconfigure(tl1info, foreground="blue", font=police)

### function load Gene Sets
ENV$regex <-0
loadGeneSets <- function()
{  

    if(!exists("match_GS", envir=ENV)){
    #tkdelete(tl1info,0,1)
    curselectGS <- as.numeric(tkcurselection(tl1))+1
    tkinsert(tl1info,"end",names(MSigDB)[curselectGS]) 
    ENV$regex <- c(ENV$regex,names(MSigDB)[curselectGS])
    }else{
        curselectGS <- as.numeric(tkcurselection(tl1))+1
        tkinsert(tl1info,"end",ENV$match_GS[curselectGS]) 
        ENV$regex <- c(ENV$regex,ENV$match_GS[curselectGS])
        
    }
    
}
ENV$regex <- ENV$regex[-1]

OKon <- function(){
    tkdestroy(ttGeneListMSigDB)
    
}
OKOn.but <- tkbutton(ttGeneListMSigDB, text="OK", command=OKon)
loadGS.but <-tkbutton(ttGeneListMSigDB,text="select",command=loadGeneSets)
tkgrid(label1)
tkgrid(tl1,yscr1)
tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
tkgrid(xscr1)
tkgrid.configure(xscr1,rowspan=2, column=0,sticky="we")
tkgrid( tl1info, yscr1Info)
tkgrid.configure(yscr1Info,rowspan=20, columnspan=1,sticky="nsw")
tkgrid(xscr1Info)
tkgrid.configure(xscr1Info,rowspan=4, column=0,sticky="we")
tkgrid(loadGS.but,OKOn.but)
tkgrid.configure(loadGS.but,column=0, sticky="nw")
tkgrid.configure(OKOn.but,column=0, sticky="ne")
for (i in 1:length(names(MSigDB))){
    
    tkinsert(tl1,"end",names(MSigDB)[i])
}

tkwait.window(ttGeneListMSigDB)
}

# to avoid 'no visible binding for global variable' NOTE
globalVariables(c("Li","A","EventTime","Category", "Continu", "Category",
                  "ttMain", "x", "fname.GCT", "fname.CLS", "mSigDB_SubName",
                  "GenesDetails", "CaseChoice", "ListProfData", "ListMetData",
                  "ListMutData", "lenght", "myGlobalEnv", "MSigDB", "MSigDBFile",
                  "fname.MSigDB", "age", "CasesStudies", "curselectCases", "devSVG",
                  "ttCasesGenProfs", "cbind2", "new", "rbind2", "vignette", "pull",
                  "select", "across", "left_join", "all_of", "p", "getProfileData",
                  "getClinicalData", "GSEA", "Run.GSEA", "everything"))

#' Checkbox to select dimensions
#' @usage
#' dialogOptionCircos()
#' 
#' @return a checkbox with all dimensions
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
#' \dontrun{
#' dialogOptionCircos()
#' #getCircos(dimension ="All")
#' }
#' @import circlize
dialogOptionCircos <- function(){

ttCircos <- tktoplevel()
#tkwm.geometry(ttCircos,"180x250")
tktitle(ttCircos) <- paste("Circos plot Option")

## Dimension Frame

frameDimension <- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(frameDimension)
tkgrid.configure(frameDimension, sticky="new")
tkgrid(tklabel(frameDimension, text="Select Dimensions: "), columnspan=3)

## mRNA checkBox
cbValmRNA <- tclVar("0")
cbmRNA <- tkcheckbutton(frameDimension)
tkconfigure(cbmRNA,variable=cbValmRNA, text="mRNA")
## CNA checkBox
cbValCNA <- tclVar("0")
cbCNA <- tkcheckbutton(frameDimension)
tkconfigure(cbCNA,variable=cbValCNA, text="CNA")
## Met HM450 checkBox
cbValHM450 <- tclVar("0")
cbHM450 <- tkcheckbutton(frameDimension)
tkconfigure(cbHM450,variable=cbValHM450, text="HM450")
## Met HM27 checkBox
cbValHM27 <- tclVar("0")
cbHM27 <- tkcheckbutton(frameDimension)
tkconfigure(cbHM27,variable=cbValHM27, text="HM27")
## RPPA checkBox
cbValRPPA <- tclVar("0")
cbRPPA <- tkcheckbutton(frameDimension)
tkconfigure(cbRPPA,variable=cbValRPPA, text="RPPA")
## miRNA checkBox
cbValmiRNA <- tclVar("0")
cbmiRNA <- tkcheckbutton(frameDimension)
tkconfigure(cbmiRNA,variable=cbValmiRNA, text="miRNA")
## Mut checkBox
cbValMut <- tclVar("0")
cbMut <- tkcheckbutton(frameDimension)
tkconfigure(cbMut,variable=cbValMut, text="Mut")

tkgrid(cbmRNA, cbCNA, cbHM450, cbHM27)
tkgrid(cbRPPA, cbmiRNA, cbMut)

## mRNA frame
mRNA_Entry  = tclVar(.8)
frameExp<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameExp, text="Threshold of mRNA correlation :"))
tkgrid(frameExp)
tkgrid.configure(frameExp, sticky="new")

cbValThrmRNA <- tclVar("0")
cbThrmRNA <- tkcheckbutton(frameExp)
tkconfigure(cbThrmRNA,variable=cbValThrmRNA, text="")

ScaleExp <- tkscale(frameExp,length=200,from=0.2,to=1,showvalue=TRUE,
        variable=mRNA_Entry,resolution=.05,orient='horiz')
tkgrid(ScaleExp,cbThrmRNA)
tkgrid.configure(cbThrmRNA,sticky="s", column=1)

## CNA frame
CNA_Entry  = tclVar(.8)
frameCNA<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameCNA, text="Threshold of CNA correlation :"))
tkgrid(frameCNA)
tkgrid.configure(frameCNA, sticky="new")

cbValThrCNA <- tclVar("0")
cbThrCNA <- tkcheckbutton(frameCNA)
tkconfigure(cbThrCNA,variable=cbValThrCNA, text="")

ScaleCNA<-tkscale(frameCNA,length=200,from=0.5,to=1,showvalue=TRUE,
        variable=CNA_Entry,resolution=.05,orient='horiz')

tkgrid(ScaleCNA, cbThrCNA)
tkgrid.configure(cbThrCNA,sticky="s", column=1)

## Methylation HM450 frame
HM450_Entry  = tclVar(.8)
frameHM450<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameHM450, text="Threshold of Met. HM450 cor :"))
tkgrid(frameHM450)
tkgrid.configure(frameHM450, sticky="new")

cbValThrHM450 <- tclVar("0")
cbThrHM450 <- tkcheckbutton(frameHM450)
tkconfigure(cbThrHM450,variable=cbValThrHM450, text="")

ScaleHM450 <- tkscale(frameHM450,length=200,from=0.2,to=1,showvalue=TRUE,
                      variable=HM450_Entry,resolution=.05,orient='horiz')
tkgrid(ScaleHM450, cbThrHM450)
tkgrid.configure(cbThrHM450, sticky="s",column=1)

## Methylation HM27 frame
HM27_Entry  = tclVar(.8)
frameHM27<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameHM27, text="Threshold of Met. HM27 cor :"))
tkgrid(frameHM27)
tkgrid.configure(frameHM27, sticky="new")

cbValThrHM27 <- tclVar("0")
cbThrHM27 <- tkcheckbutton(frameHM27)
tkconfigure(cbThrHM27,variable=cbValThrHM27, text="")

ScaleHM27 <- tkscale(frameHM27,length=200,from=0.2,to=1,showvalue=TRUE,
                     variable=HM27_Entry,resolution=.05,orient='horiz')
tkgrid(ScaleHM27, cbThrHM27)
tkgrid.configure(cbThrHM27,sticky="s",column=1)

##  RPPA frame
RPPA_Entry  = tclVar(.8)
frameRPPA<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameRPPA, text="Threshold of RPPA correlation :"))
tkgrid(frameRPPA)
tkgrid.configure(frameRPPA, sticky="new")

cbValThrRPPA <- tclVar("0")
cbThrRPPA <- tkcheckbutton(frameRPPA)
tkconfigure(cbThrRPPA,variable=cbValThrRPPA, text="")

ScaleRPPA <- tkscale(frameRPPA,length=200,from=0.5,to=1,showvalue=TRUE,
                     variable=RPPA_Entry,resolution=.05,orient='horiz')

tkgrid(ScaleRPPA,cbThrRPPA)
tkgrid.configure(cbThrRPPA,sticky="s",column=1)

##  miRNA frame
miRNA_Entry  = tclVar(.8)
framemiRNA<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(framemiRNA, text="Threshold of miRNA correlation :"))
tkgrid(framemiRNA)
tkgrid.configure(framemiRNA, sticky="new")

cbValThrmiRNA <- tclVar("0")
cbThrmiRNA <- tkcheckbutton(framemiRNA)
tkconfigure(cbThrmiRNA,variable=cbValThrmiRNA, text="")

ScalemiRNA <- tkscale(framemiRNA,length=200,from=0.5,to=1,showvalue=TRUE,
                      variable=miRNA_Entry,resolution=.05,orient='horiz')

tkgrid(ScalemiRNA, cbThrmiRNA)
tkgrid.configure(cbThrmiRNA,sticky="s",column=1)

## Mutation frequency frame
#Mut_Entry  = tclVar(median(ENV$Freq_DfMutData))
Mut_Entry  = tclVar("14")
frameMut<- tkframe(ttCircos,relief="groove",borderwidth=2)
tkgrid(tklabel(frameMut, text="Threshold of Mutation Frequency:"))
tkgrid(frameMut)
tkgrid.configure(frameMut, sticky="new")

cbValThrMut <- tclVar("0")
cbThrMut <- tkcheckbutton(frameMut)
tkconfigure(cbThrMut,variable=cbValThrMut, text="")

ScaleMut <- tkscale(frameMut,length=200,from=0,to=max(ENV$Freq_DfMutData, na.rm=TRUE),showvalue=TRUE,
                    variable=Mut_Entry,resolution=1,orient='horiz')

tkgrid(ScaleMut, cbThrMut)
tkgrid.configure(cbThrMut,sticky="s",column=1)


okOn <- function(){
    ## checkBox for Dimensions
    cbValuemRNA <- as.character(tclvalue(cbValmRNA))
    cbValueCNA <- as.character(tclvalue(cbValCNA))
    cbValueHM450 <- as.character(tclvalue(cbValHM450))
    cbValueHM27 <- as.character(tclvalue(cbValHM27))
    cbValueRPPA <- as.character(tclvalue(cbValRPPA))
    cbValuemiRNA <- as.character(tclvalue(cbValmiRNA))
    cbValueMut <- as.character(tclvalue(cbValMut))
    
    
    if(cbValuemRNA=="1"){mRNA <- 1}else if(cbValuemRNA=="0"){mRNA <- 0}
    if(cbValueCNA=="1"){CNA <- 1}else if(cbValueCNA=="0"){CNA <- 0}
    if(cbValueHM450=="1"){HM450 <- 1}else if(cbValueHM450=="0"){HM450 <- 0}    
    if(cbValueHM27=="1"){HM27 <- 1}else if(cbValueHM27=="0"){HM27 <- 0}
    if(cbValueRPPA=="1"){RPPA <- 1}else if(cbValueRPPA=="0"){RPPA <- 0}
    if(cbValuemiRNA=="1"){miRNA <- 1}else if(cbValuemiRNA=="0"){miRNA <- 0}
    if(cbValueMut=="1"){Mut <- 1}else if(cbValueMut=="0"){Mut <- 0}
    
    ENV$ReturnCBoxCircos <-c(mRNA,CNA,HM450,HM27,RPPA,miRNA,Mut)
    
    
    ## CheckBox for Thrshold
    cbvaluemRNA <- as.character(tclvalue(cbValThrmRNA))
    cbvalueCNA <- as.character(tclvalue(cbValThrCNA))
    cbvalueHM450 <- as.character(tclvalue(cbValThrHM450))
    cbvalueHM27 <- as.character(tclvalue(cbValThrHM27))
    cbvalueRPPA <- as.character(tclvalue(cbValThrRPPA))
    cbvaluemiRNA <- as.character(tclvalue(cbValThrmiRNA))
    cbvalueMut <- as.character(tclvalue(cbValThrMut))
    
    
    if(cbvaluemRNA=="1"){ThrmRNA <- 1}else if(cbvaluemRNA=="0"){ThrmRNA <- 0}
    if(cbvalueCNA=="1"){ThrCNA <- 1}else if(cbvalueCNA=="0"){ThrCNA <- 0}
    if(cbvalueHM450=="1"){ThrHM450 <- 1}else if(cbvalueHM450=="0"){ThrHM450 <- 0}    
    if(cbvalueHM27=="1"){ThrHM27 <- 1}else if(cbvalueHM27=="0"){ThrHM27 <- 0}
    if(cbvalueRPPA=="1"){ThrRPPA <- 1}else if(cbvalueRPPA=="0"){ThrRPPA <- 0}
    if(cbvaluemiRNA=="1"){ThrmiRNA <- 1}else if(cbvaluemiRNA=="0"){ThrmiRNA <- 0}
    if(cbvalueMut=="1"){ThrMut <- 1}else if(cbvalueMut=="0"){ThrMut <- 0}
    
    ENV$ReturnCBoxThrCircos <-c(ThrmRNA,ThrCNA,ThrHM450,ThrHM27,ThrRPPA,ThrmiRNA,ThrMut)
    
    
   #Entry Threshold of mRNA correlation
    ThreshmRNA <- as.numeric(tclvalue(mRNA_Entry))
    
    #Entry Threshold of CNA correlation
    ThreshCNA <- as.numeric(tclvalue(CNA_Entry))
    
    #Entry Threshold of Met HM450 correlation
    ThreshHM450 <- as.numeric(tclvalue(HM450_Entry))
    
    #Entry Threshold of Met HM27 correlation
    ThreshHM27 <- as.numeric(tclvalue(HM27_Entry))
    
    #Entry Threshold of RPPA correlation
    ThreshRPPA <- as.numeric(tclvalue(RPPA_Entry))
    
    #Entry Threshold of miRNA correlation
    ThreshmiRNA <- as.numeric(tclvalue(miRNA_Entry))
    
    #Entry Threshold of miRNA correlation
    ThreshMut <- as.numeric(tclvalue(Mut_Entry))
    
    ENV$ReturnThreshCircos <-c(ThreshmRNA,ThreshCNA,ThreshHM450,ThreshHM27,ThreshRPPA, ThreshmiRNA,ThreshMut) 
   
    tkdestroy(ttCircos)
}

Ok.but <- tkbutton(ttCircos, text= "OK", command= okOn)


tkgrid(Ok.but)
tkgrid.configure(Ok.but, sticky="n", column=0)

tkwait.window(ttCircos)

}

#' Built Expression Set (eSet) from profile data.
#' @usage geteSet()
#' @export
#' @return ExpressionSet
#' @examples 
#'  f <- 9
#'  \dontrun{
#'  readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#'  geteSet()
#'  }
#' 
geteSet <- function(){
    
    #function to replace blanks with missing
    
    blank2na <- function(x){ 
        z <- gsub("\\s+", "", x)  #make sure it's "" and not " " etc
        x[z==""] <- NA 
        return(x)
    } 
        ##Test checked Cases and Genetic Profiles
        testCheckedCaseGenProf()
       
        
        ProfData=0
        i<-0
        for (s in ENV$checked_Studies_id){
            i<-i+1
            Si = ENV$checked_StudyIndex[i]
            progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                                      max = length(ENV$curselectGenProfs), width = 400)
            
            
            study_desc_position_in_genProfs <- 0
            for (k in 1:length(ENV$curselectCases)){
                
                Sys.sleep(0.1)
                setTkProgressBar(progressBar_ProfilesData, k, 
                                 label=paste(round(k/length(ENV$curselectGenProfs)*100, 0),
                                             "% of Profiles Data"))
    
                
                # Avoid to select study description  
                if (ENV$curselectGenProfs[k] <= ENV$n_GenProfs[i] && 
                    ENV$curselectGenProfs[k] > study_desc_position_in_genProfs){    
                    
                    study_desc_position_in_genProfs <- study_desc_position_in_genProfs + ENV$n_GenProfs[i]
                    
                    GenProf <- ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
            
                    print(GenProf)
                    print(s)
                    ## verify if GenProf has expression data
                    if (length(grep("mrna", GenProf, ignore.case = TRUE))==0 && 
                        length(grep("gistic", GenProf, ignore.case = TRUE))==0 &&
                        length(grep("CNA", GenProf, ignore.case = TRUE))==0){
                        msgNoExp <- "Select Expression data from Genetics Profiles"
                        tkmessageBox(message = msgNoExp, icon='info')
                        break
                    }  

                    ProfData <- getDataByGenes(
                        api =  ENV$cgds,
                        studyId = s,
                        genes = ENV$GeneList,  #c("NF1", "TP53", "ABL1")
                        by = "hugoGeneSymbol",
                        molecularProfileIds = GenProf)  |>
                        unname() |>
                        as.data.frame() |>
                        select("hugoGeneSymbol","sampleId", "value") |>
                        tidyr::spread("hugoGeneSymbol", "value") |>
                        mutate(across(-c("sampleId"), \(x)round(x, digits = 3)))
                        #data.frame(row.names = 1)

                    ProfData <<- ProfData 
                    print("getting Profile Data and removing all NAs rows...")
                    ##remove all NAs rows
                    ProfData <- ProfData[which( apply(!( apply(ProfData,1,is.na) ),2,sum)!=0 ),]

                    ## Display AssyData with Tcl Table
                    title <- paste0(ENV$StudyRefGenProf[k],": ", ENV$GenProfChoice[k])
                    getInTable(ProfData, title)
                    
                    #####nicData_MultipleCases function
                    #Case<- ENV$CasesRefStudies[ENV$curselectCases[k]]
                    
                    ClinicalData <- cBioPortalData::clinicalData(ENV$cgds, s) |>
                        select("sampleId", dplyr::everything()) 
                        #data.frame(row.names = 1)
                   
                    #ClinicalData <<- ClinicalData
                    
                    if(nrow(ClinicalData)==0){
                        msgNoClinData=paste("No Clinical Data are Available for\n", CasesStudies[curselectCases[k]+1])
                        tkmessageBox(message=msgNoClinData, title= paste("Study: ",ENV$StudyRefCase[k]))
                        close(progressBar_ProfilesData)
                        break
                    }else{
                        ## getClinicalData generate CHARACTER class if is there "NA" value in any column
                        ## Convert character value to numeric if grep [0-9] != 0
                        # for(i in 1:ncol(ClinicalData)){
                        #     ## substitute  NO digital value to NA
                        #     ClinicalData[,i]<- gsub("^\\D.*",NA, ClinicalData[,i], ignore.case=TRUE)
                        #     ## substitte "NA" charcter to NA 
                        #     ClinicalData[,i]<- gsub("NA",NA, ClinicalData[,i], ignore.case=TRUE)
                        #     ## substiture space by NA
                        #     ClinicalData[,i]<-gsub("\\s+", NA,  ClinicalData[,i], ignore.case=TRUE) 
                        #     if(length(grep("-?[0-9]*\\.[0-9]*",ClinicalData[,i]))!=0){
                        #         ClinicalData[,i] <- as.numeric(ClinicalData[,i])
                        #     }
                        # }
                        
                        title <- paste(ENV$StudyRefCase[k],": ", ENV$CaseChoice[k])
                        getInTable(ClinicalData,title)
                    }       
                    
                    
                    
                    names_Clinical_Data <- colnames(ClinicalData)
                    names_ProfData <- colnames(ProfData)
                    
                    clinical_profData <- ClinicalData |>
                        left_join(ProfData, by="sampleId")
                    
                    
                    AssayData <- clinical_profData |>
                        select(all_of(names_ProfData)) |>
                        data.frame(row.names = 1) |>
                        t() |>
                        as.matrix()
                    
                    ClinicalData <- ClinicalData |>
                                    data.frame(row.names = 1)


                    # matrix <-rbind(colnames(ClinicalData), ClinicalData)
                    # rnames <- rownames(ClinicalData)
                    # cnames <- colnames(ClinicalData)
                    
                    #apply blank2na function
                    # ClinicalData <- data.frame(lapply(ClinicalData,  blank2na))
                    # rownames(ClinicalData) <- rnames
                    # names(ClinicalData) <- cnames
                  
                    ## Select only Cases (rownames) that exist in ClinicalDataSub and ProfData
                    #merge <- merge(ClinicalData, ProfData, by="row.names")
                    #print("merge Clinical and Profile Data")
                    #ClinicalData <- merge[,1:(length(ClinicalData)+1)]
                    
                    # rownames(ClinicalData) <- ClinicalData[,1]
                    # ClinicalData <- ClinicalData[-1]
                    # ProfData <- merge[,!(merge %in% ClinicalData)]
        
                    # AssayData<- t(ProfData)
                    # colnames(AssayData) <- AssayData[1,]
                    # AssayData <- AssayData[-1,]
                    # rnames <- rownames(AssayData) 
                    # AssayData <- as.matrix(apply(t(ProfData),2 ,function(x) as.numeric(x)))
                    # rownames(AssayData) <- rnames
                    
                    
                    ##Convert column with digital values from factor to numeric
                    for(i in 1:ncol(ClinicalData)){
                        ClinicalData[,i] <- sapply(ClinicalData[,i], 
                                                   function(x) if(length(grep("[a-z'-'+A-Z'/'' ']", 
                                as.character(ClinicalData[,i])))==0) { as.numeric(as.character(x)) } else {x})
                    }
                    
                
                    ENV$ClinicalData <- ClinicalData
                    ENV$ProfData <- t(AssayData)
                    ENV$AssayData <- AssayData
                    
                    
                    #Test if the same length cases for phenoData and AssayData
                    if (all(rownames(ClinicalData)==colnames(AssayData))){

                        ## create labelDescription for columns of phenoData. 
                        ## labeldescription is used by Biobase packages
                        ## In our case labelDescription is Equal to column names
                        metaData <- data.frame(labelDescription= colnames(ClinicalData), row.names=colnames(ClinicalData))        ## Bioconductor’s Biobase package provides a class called AnnotatedDataFrame   
                        ##that conveniently stores and manipulates 
                        ##the phenotypic data and its metadata in a coordinated fashion. 
                        phenoData <- new("AnnotatedDataFrame", data=ClinicalData, varMetadata=metaData)    
                        
                        ##Assembling an ExpressionSet  
                        ENV$eSet<-Biobase::ExpressionSet(assayData=AssayData, phenoData=phenoData, annotation="GO") 
                        print(paste("End of building eSet..."))
                        
                        #             for (i in 1:length(names(pData(eSet)))){
                        #             pData(eSet)[i] <- as.matrix(na.omit(pData(eSet)[i]))
                        #             }
                    }else {tkmessageBox( message= "The expression Gene Set and the Clinical Data do not have the same samples", icon="warning")}
                } 
            } 
            close(progressBar_ProfilesData)
        } 
        
}

#'Dialog Box to specify phenotype (variable) used in last GSEA-R to get Summary Results. This function ask the user to specify the phenotype (variable).
#' @usage dialogSummary_GSEA(Variable,returnValOnCancel ="ID_CANCEL")
#' 
#' @param Variable phenotype
#' @param returnValOnCancel  "ID_CANCEL"
#' 
#' @return variables
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' #Run.GSEA()
#' #getSummaryGSEA()
#' }
dialogSummary_GSEA <- function(Variable, returnValOnCancel = "ID_CANCEL") {
    
    dlg <- tktoplevel()
    tkwm.deiconify(dlg)
    tkgrab.set(dlg)
    tkfocus(dlg)
    title="GSEA Summary Results"
    tkwm.title(dlg, title)
    
    tkgrid(tklabel(dlg, text = " "))  
    ###select GSEA summary Report of  Phenotype 1 
    tl1<-tklistbox(dlg,height=1, width= 50 ,background="white")
    #tklabel(ttDialogGSEA, text="dd")
    
    if(length(table(Variable))==2){
        Classes <- names(table(Variable))
    }
    if(names(table(Variable))[1]==""){
        Classes <- names(table(Variable))[-1]
    }
    
    getSummary_location1 <- function(Variable){
        tkfocus(dlg)
        if(exists("location1", envir = ENV)){
            tkdelete(tl1,0,1)
        } 
        
        ENV$location1 <- tclvalue(tkgetOpenFile(filetypes = "{{TXT Files} {.txt}} {{All files} *}", 
                                                title="Select Summary Results file"))
        
        if(length(grep(paste(".SUMMARY.RESULTS.REPORT.", Classes[1], sep=""), ENV$location1))==0){
            tkmessageBox(message= paste("Select Summary Results Report of",
                                        names(table(ENV$PhenoData[,ENV$curselect])[1])), 
                         icon="warning")
            stop(paste("Select Summary Results Report of ",Classes[1]))
            tkfocus(dlg)
        } else
            tkinsert(tl1,"end",ENV$location1)
        tkfocus(dlg)
    }
    
    location1.but <- tkbutton(dlg, text = "browse", command = getSummary_location1)
    
    tklabellocation1 <-tklabel(dlg,text=paste("Phenotype 1: ",Classes[1]))
    
    tkgrid(tklabellocation1,tl1,location1.but, columnspan=1)
    tkgrid.configure(tklabellocation1,rowspan=20, columnspan=1,sticky="nsw")
    
    
    ###select GSEA summary Report of  Phenotype 1 
    tl2<-tklistbox(dlg,height=1, width= 50 ,background="white")
    
    
    getSummary_location2 <- function(){
        tkfocus(dlg)
        if(exists("location2", envir = ENV)){
            tkdelete(tl2,0,1)
        }
        ENV$location2 <- tclvalue(tkgetOpenFile(filetypes = "{{TXT Files} {.txt}} {{All files} *}", 
                                                title="Select Summary Results file")) # Very simple, isn't it?
        
        if(length(grep(paste(".SUMMARY.RESULTS.REPORT.", Classes[2], sep=""), ENV$location2))==0){
            tkmessageBox(message= paste("Select Summary Results Report of",Classes[2]), icon="warning")
            stop(paste("Select Summary Results Report of ",Classes[2]))
            tkfocus(dlg)
        } else
            tkinsert(tl2,"end",ENV$location2)
        tkfocus(dlg)
    }
    
    location2.but <- tkbutton(dlg, text = "browse", command = getSummary_location2)
    
    tklabellocation2 <-tklabel(dlg,text=paste("Phenotype 2: ",Classes[2]))
    
    tkgrid(tklabellocation2,tl2,location2.but, columnspan=1)
    tkgrid.configure(tklabellocation2,rowspan=20, columnspan=1,sticky="nsw")
    

    entryWidth = 20
    entryInit <- "0.25"
    
    textEntryVarTcl <- tclVar(paste(entryInit))
    textEntryWidget <- tkentry(dlg, width = paste(entryWidth),
                               textvariable = textEntryVarTcl)
    tkgrid(tklabel(dlg, text = " "))
    tkgrid(tklabel(dlg, text = "Specify FDR threshold"), textEntryWidget)
    tkgrid(tklabel(dlg, text = "       "))
    ReturnVal <- returnValOnCancel
    
    onOK <- function() {
        ENV$ReturnVal <- tclvalue(textEntryVarTcl)
        tkgrab.release(dlg)
        
        tkdestroy(dlg)
        
    }
    onCancel <- function() {
        ENV$vReturnVal <- returnValOnCancel
        tkgrab.release(dlg)
        tkdestroy(dlg)
        
    }
    OK.but <- tkbutton(dlg, text = "   OK   ", command = onOK)
    Cancel.but <- tkbutton(dlg, text = " Cancel ", command = onCancel)
    tkgrid(Cancel.but, OK.but)
    tkgrid(tklabel(dlg, text = "    "))
    
    tkwait.window(dlg)
    
    return(ENV$ReturnVal)
    
}

#' Search MSigDb that overlap gene list
#' @usage Match_GeneList_MSigDB
#' @return GeneList
#' @export
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#' \dontrun{
#' Match_GeneList_MSigDB()
#' }
Match_GeneList_MSigDB <- function(){
    
    
    ttdialogMSigDB<-tktoplevel()
    tktitle(ttdialogMSigDB) <- "Setting MSigDB Directory"
    
    fromDir <- function(){
        ##Select directory which has MSigDB
        ENV$dirMSigDBPath <- tk_choose.dir()
        tkdestroy(ttdialogMSigDB)
        #return(dirMSigDBPath)
    }
    
    fromExample <- function(){
        ##Or select MSigDB from package
        ENV$dirMSigDBPath <- paste(path.package("canceR"),"/extdata/MSigDB/",sep="")
        tkdestroy(ttdialogMSigDB)
        #return(dirMSigDBPath)
    }
    
    text <- tklabel(ttdialogMSigDB,text="Select Directory with MSigDB.gmt files")
    tkgrid(text)
    fromDir.but <- tkbutton(ttdialogMSigDB, text = "Directory", command = fromDir)
    fromExample.but <- tkbutton(ttdialogMSigDB, text = "Example", command = fromExample)
    tkgrid(fromDir.but,fromExample.but)
    tkgrid.configure(fromDir.but,columnspan=1,sticky="w")
    
    tkwait.window(ttdialogMSigDB)
    
    MSigDBList <-  list.files(ENV$dirMSigDBPath)  
    MSigDB_Rate <- 0
    for(i in 1 :length(MSigDBList)){
        
        MSigDBPath <- paste(ENV$dirMSigDBPath,"/",MSigDBList[i] ,sep="")
        
        mSigDB<-readLines(MSigDBPath)
        
        #split each line at the tabs
        mSigDB<-strsplit(mSigDB,'\t')
        
        
        #use the first entry of each gene set as name
        names(mSigDB)<-sapply(mSigDB,function(x)x[1])
        
        
        #remove the first two entries
        mSigDB<-sapply(mSigDB,function(x)x[3:length(x)])
        
        
        ##get a unique list of all genes
        #allGenes<-unique(unlist(mSigDB))
        
        
        Matched_GeneSets <- sapply(ENV$GeneList, function(x) which(sapply(x, regexpr, mSigDB)!=-1))
        
        MeanOfGeneSetsHavingGene <- round(length(unlist(Matched_GeneSets))/length(Matched_GeneSets),digits=2)
        
        
        MSigDB_Rate <- rbind(MSigDB_Rate,cbind(MSigDBList[i],MeanOfGeneSetsHavingGene))
        
        
        print(paste(MSigDBList[i],MeanOfGeneSetsHavingGene, sep=": "))
        
    }
    MSigDB_Rate <- MSigDB_Rate[-1,]
    colnames(MSigDB_Rate) <- c("MSigDB","Matched(Genes/Gene Sets)")
    #rownames(MSigDB_Rate) <- MSigDB_Rate[,1]
    title<- "Mean of Gene Sets overlapping Gene List"
    getInTable(MSigDB_Rate, title)
    
    
}

#' Setting work Directory and output folders.At starting window, user needs to set work directory for output data. The function is foud in File menu.
#' @usage setWorkspace()
#' @return paths of output files
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' setWorkspace()
#' }
#' 
setWorkspace <- function(){
    
    
    ttWorkspace<-tktoplevel()
    tktitle(ttWorkspace) <- "Setting Workspace"
    
    
    ###select GSEA program file
    tlWorkspace<-tklistbox(ttWorkspace,height=1, width= 50 ,background="white")
    
    
    
    getWorkspace <- function(){
        tkfocus(ttWorkspace)
        if(exists("path_workspace", envir = ENV)){
            tkdelete(tlWorkspace,0,1)
        }
        ENV$path_workspace <- tk_choose.dir()
        
        if(!file.exists(ENV$path_workspace)){
            Sys.chmod(dirname(ENV$path_workspace), mode = "0777", use_umask = TRUE)
            dir.create(file.path(ENV$path_workspace), showWarnings = FALSE)
            
        }
        
        
        tkinsert(tlWorkspace,"end",ENV$path_workspace)
        tkfocus(ttWorkspace)
    }
    
    workspace.but <- tkbutton(ttWorkspace, text = "Browse", command = getWorkspace)
    
    tklabelWorkspace <-tklabel(ttWorkspace,text="Workspace")
    
    tkgrid(tklabelWorkspace,tlWorkspace,workspace.but, columnspan=1)
    tkgrid.configure(tklabelWorkspace,rowspan=20, columnspan=1,sticky="nsw")
    
    
    ### Set Results Folder
    
    tlResults<-tklistbox(ttWorkspace,height=1, width= 50 ,background="white")
    getResultsFolder <- function(){
        if(exists("pathResultsFolder")){
            tkdelete(tlResults,0,1)
        }
        pathResultsFolder <- paste(ENV$path_workspace, "/Results/", sep="")
        
        if(!file.exists(pathResultsFolder)){
            Sys.chmod(dirname(ENV$path_workspace), mode = "0777", use_umask = TRUE)
            dir.create(file.path(pathResultsFolder), showWarnings = FALSE)
            dir.create(file.path(paste(pathResultsFolder,"ProfileData",sep="")), showWarnings = FALSE)
            dir.create(file.path(paste(pathResultsFolder,"GSEAlm",sep="")), showWarnings = FALSE)
            dir.create(file.path(paste(pathResultsFolder,"Classifier",sep="")), showWarnings = FALSE)
            dir.create(file.path(paste(pathResultsFolder,"GSEA",sep="")), showWarnings = FALSE)
        }
        tkinsert(tlResults,"end",pathResultsFolder)
        tkfocus(ttWorkspace)
    }
    
    getResultsFolder.but <- tkbutton(ttWorkspace, text = "Set", command = getResultsFolder)
    
    tklabelResults <- tklabel(ttWorkspace,text="Results")
    
    tkgrid(tklabelResults,tlResults,getResultsFolder.but, columnspan=1)
    tkgrid.configure(tklabelResults,rowspan=20, columnspan=1,sticky="nsw")
    
    
    
    ### Set MSigDB folder
    tlMSigDB<-tklistbox(ttWorkspace,height=1, width= 50 ,background="white")
    
    getMSigDBFolder <- function(){
        if(exists("pathMSigDBFolder")){
            tkdelete(tlMSigDB,0,1)
        }
        
        pathMSigDBFolder <- paste(ENV$path_workspace, "/Results/MSigDB/", sep="")
        
        
        if(!file.exists(pathMSigDBFolder)){
            Sys.chmod(dirname(ENV$path_workspace), mode = "0777", use_umask = TRUE)
            dir.create(file.path(pathMSigDBFolder), showWarnings = FALSE)
            
        }
        
        
        tkinsert(tlMSigDB,"end",pathMSigDBFolder)
        tkfocus(ttWorkspace)
    }
    
    getMSigDBFolder.but <- tkbutton(ttWorkspace, text = "Set", command = getMSigDBFolder)
   
    tklabelMSigDB <-tklabel(ttWorkspace,text="MSigDB")
    
    tkgrid(tklabelMSigDB,tlMSigDB,getMSigDBFolder.but, columnspan=1)
    tkgrid.configure(tklabelMSigDB,rowspan=20, columnspan=1,sticky="nsw")
    
    
    ### Set gct_cls Folder
    
    tlgct_cls<-tklistbox(ttWorkspace,height=1, width= 50 ,background="white")
    
    getgct_clsFolder <- function(){
        if(exists("pathgct_clsFolder")){
            tkdelete(tlgct_cls,0,1)
        }
        
        pathgct_clsFolder <- paste(ENV$path_workspace, "/Results/gct_cls/", sep="")
        
        
        if(!file.exists(pathgct_clsFolder)){
            Sys.chmod(dirname(ENV$path_workspace), mode = "0777", use_umask = TRUE)
            dir.create(file.path(pathgct_clsFolder), showWarnings = FALSE)
            
        }
        tkinsert(tlgct_cls,"end",pathgct_clsFolder)
    }
    
    getgct_clsFolder.but <- tkbutton(ttWorkspace, text = "Set", command = getgct_clsFolder)
    
    tklabelgct_cls <- tklabel(ttWorkspace,text="gct,cls files")
    
    tkgrid(tklabelgct_cls,tlgct_cls,getgct_clsFolder.but, columnspan=1)
    tkgrid.configure(tklabelgct_cls,rowspan=20, columnspan=1,sticky="nsw")
    
   
    ##
    OK <- function(){
        
        setwd(ENV$path_workspace)
        tkdestroy(ttWorkspace)
    }
    
    Ok.but <- tkbutton(ttWorkspace, text = " OK ", command = OK)
    
    Cancel <- function(){
        tkdestroy(ttWorkspace)
        
    }
    Cancel.but <- tkbutton(ttWorkspace, text= "Cancel", command = Cancel)
    tkgrid(Cancel.but, Ok.but)
    tkgrid.configure(Cancel.but, sticky="n",columnspan=2)
    tkgrid.configure(Ok.but, sticky="n",columnspan=2)
    
    tkwait.window(ttWorkspace)
    
}

#' get Mutation data for multiple genes
#' @usage
#' getMutData()
#'
#' @return a a dataframe with mutation informations
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' getMutData()
#' }
#' 
getMutData <- function(){
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    testCheckedCaseGenProf()
    
    ProfData = 0
    i <- 0
    for (s in ENV$checked_Studies_id){
        i<-i+1
        Si =ENV$checked_StudyIndex[i]
        progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                                  max = length(ENV$curselectGenProfs), width = 400)
        
        study_desc_position_in_genProfs <- 0
        for(c in 1:length(ENV$curselectCases)){
            Sys.sleep(0.1)
            setTkProgressBar(progressBar_ProfilesData, c, 
                             label=paste( round(c/length(ENV$curselectGenProfs)*100, 0),
                                          "% of Mutation Data"))
            
            # Avoid to select study description  
            if (ENV$curselectGenProfs[c] <= ENV$n_GenProfs[i] && 
                ENV$curselectGenProfs[c] > study_desc_position_in_genProfs){ 
                
                study_desc_position_in_genProfs <- study_desc_position_in_genProfs + ENV$n_GenProfs[i]
                
                GenProf<-ENV$GenProfsRefStudies[ENV$curselectGenProfs[c]]
                
                if (length(grep("mutation", GenProf))==0){
                    msgNoMut <- "Select Mutation data from Genetics Profiles"
                    tkmessageBox(message = msgNoMut, icon='info')
                    break
                }
                
                
                # MutData <- getMutationData(ENV$cgds,Study_id, GenProf, ENV$GeneList)
                
                MutData <- getDataByGenes(
                    api = ENV$cgds,
                    studyId = s,
                    genes = ENV$GeneList,
                    by = "hugoGeneSymbol",
                    molecularProfileIds = GenProf) |>
                    unname() |>
                    as.data.frame() |>
                    select(-c("uniqueSampleKey", "uniquePatientKey", "molecularProfileId", "sampleId", "studyId"))
                
                
                if(length(MutData[,1])==0){
                    msgNoMutData=paste("No Mutation Data are Available for\n",
                                       ENV$CasesStudies[ENV$curselectCases[c]+2])
                    tkmessageBox(message=msgNoMutData, 
                                 title= paste(ENV$StudyRefCase[c],ENV$CasesStudies[ENV$curselectCases[c]+2], 
                                              ENV$GenProfsStudies[ENV$curselectGenProfs[c]+1], sep=": "))
                    
                    
                } else{
                    ttMutData_cb <- tktoplevel()
                    tktitle(ttMutData_cb) <- paste(ENV$StudyRefCase[c],ENV$CasesStudies[ENV$curselectCases[c]+2],
                                                   ENV$GenProfsStudies[ENV$curselectGenProfs[c]+2], sep=": ")
                    
                    cbAll <- tkcheckbutton(ttMutData_cb)
                    cbAllValue <- tclVar("0")
                    tkconfigure(cbAll,variable=cbAllValue)
                    labelAll<- tklabel(ttMutData_cb,text= "All")
                    tkgrid(labelAll, cbAll)
                    
                    cbIValue=0
                    for(i in 1: length(names(MutData))){
                        
                        cbi <- paste ("cb", i, sep="")  
                        cbi <- tkcheckbutton(ttMutData_cb)
                        cbiValue <- paste("cb", i, "Value", sep="")
                        cbIValue[i] <- cbiValue
                        cbIValue[i] <- tclVar("0")
                        
                        tkconfigure(cbi,variable=cbiValue)
                        labeli <- paste ("label", i , sep="") 
                        labelI <- labeli
                        labelI <- tklabel(ttMutData_cb,text= names(MutData[i]))
                        tkgrid(labelI,cbi)
                    }
                    
                    OnOK <- function(){
                        
                        cbAllVal <- as.character(tclvalue(cbAllValue))
                        if(cbAllVal =="1"){
                            
                            title=paste(ENV$StudyRefCase[c],ENV$CasesStudies[ENV$curselectCases[c]+2], 
                                        ENV$GenProfsStudies[ENV$curselectGenProfs[c]+2], sep=": ")
                            getInTable(MutData, title=title)
                            
                        } else{
                            
                            
                            for (i in 1: length(names(MutData))){
                                cbiValue <- paste("cb", i, "Value", sep="")
                                cbIValue <- cbiValue
                                cbiVal <- paste("cb", i, "Val", sep="")
                                cbIVal<-cbiVal
                                
                                cbIVal[i] <- as.character(tclvalue(cbIValue))
                                if (cbIVal[i]=="1"){
                                    
                                    ## convert metacharacter "[""]" not supported by tclarray()
                                    #MutData_All[i]<- gsub("\\[Not Available\\]","NA", MutData[i])
                                    MutDataSub <- cbind(MutDataSub, MutData[i])
                                    
                                }
                                
                            }
                            MutDataSub<-MutDataSub[-1]
                            #MutDataSub_All[[c]]<<- MutDataSub
                            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                            #MutDataSub_All[[c]]<<- as.data.frame.array(MutDataSub, envir=.GlobalEnv)
                            #ENV$MutDataSub_All[[c]] <- as.list.data.frame(MutDataSub, envir=ENV)
                            
                            
                            name <- paste("MutDataSub", c, sep="")
                            #Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                            assign(name, MutDataSub, envir=ENV)
                            
                            title=paste(ENV$StudyRefCase[c],ENV$CasesStudies[ENV$curselectCases[c]+1],
                                        ENV$GenProfsStudies[ENV$curselectGenProfs[c]+1], sep=": ")
                            getInTable(MutDataSub, title=title)
                            
                        }
                        
                        tkdestroy(ttMutData_cb)
                    }         
                    
                    OK.but <- tkbutton(ttMutData_cb,text="OK",command=OnOK)
                    tkgrid(OK.but)
                    
                    ##Waiting to checkbox before to access to the next clinical data
                    tkwait.window(ttMutData_cb)
                }
            } 
        }
        close(progressBar_ProfilesData)
    }
}

#' Dialog box to specify Gene Symbol.
#' @usage modalDialog(title, question, entryInit, entryWidth = 40,returnValOnCancel = "ID_CANCEL")
#' @param title string
#' @param question string
#' @param entryInit entryInit
#' @param entryWidth 40
#' @param returnValOnCancel "ID_CANCEL"
#' 
#' @return dialog box
#' @export
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' ## Select Case from Breast Cancer
#' ENV <- new.env(parent = emptyenv())
#' ENV$curselectCases <- 9
#' ##Select Genetic Profile from Breast Cancer
#' ENV$curselectGenProfs <- 4
#' ## get Specific Mutation data for 73 Genes list
#' \dontrun{
#' getProfilesDataSingleGene()
#' }
modalDialog <- function(title, question, entryInit, entryWidth = 40,
                        returnValOnCancel = "ID_CANCEL") {
    dlg <- tktoplevel()
    tkwm.deiconify(dlg)
    tkgrab.set(dlg)
    tkfocus(dlg)
    tkwm.title(dlg, title)
    
    textEntryVarTcl <- tclVar(paste(entryInit))
    textEntryWidget <- tkentry(dlg, width = paste(entryWidth),
                               textvariable = textEntryVarTcl)
    tkgrid(tklabel(dlg, text = "       "))
    tkgrid(tklabel(dlg, text = question), textEntryWidget)
    tkgrid(tklabel(dlg, text = "       "))
    ReturnVal <- returnValOnCancel
    
    onOK <- function() {
        ENV$ReturnVal <- tclvalue(textEntryVarTcl)
        tkgrab.release(dlg)
        tkdestroy(dlg)
        #tkfocus(ttMain)
    }
    onCancel <- function() {
        ENV$ReturnVal <- returnValOnCancel
        tkgrab.release(dlg)
        tkdestroy(dlg)
        #tkfocus(ttMain)
    }
    OK.but <- tkbutton(dlg, text = "   OK   ", command = onOK)
    Cancel.but <- tkbutton(dlg, text = " Cancel ", command = onCancel)
    tkgrid(Cancel.but,OK.but)
    tkgrid(tklabel(dlg, text = "    "))
    
    tkfocus(dlg)
    tkbind(dlg, "<Destroy>", function() {tkgrab.release(dlg); tkfocus(ENV$ttMain)})
    tkbind(textEntryWidget, "<Return>", onOK)
    tkwait.window(dlg)
    
    return(ENV$ReturnVal)
}

#' Checkbox to select variables for plotting
#' @usage
#' dialogPlotOption_SkinCor(s)
#' @param s  integer number of Studies
#' 
#' @return Dialog box with setting of correlation method
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
#' \dontrun{
#' dialogPlotOption_SkinCor(1)
#' }
#' @import tkrplot
#' 
dialogPlotOption_SkinCor <- function(s){
    
    ENV$ttDialSkinCor <- tktoplevel()
    tktitle(ENV$ttDialSkinCor) <- paste("Plot Option: Skin, Correlation, X/Y...")
    #tkwm.geometry(ENV$ttDialSkinCor, "300x170")
    
    frameSkin <- tkframe(ENV$ttDialSkinCor,relief="groove",borderwidth=2)
    
    
    frameCorrelation <- tkframe(ENV$ttDialSkinCor,relief="groove",borderwidth=2)
    
    frameAxes <- tkframe(ENV$ttDialSkinCor,relief="groove",borderwidth=2)
    
    tkgrid(frameSkin)
    tkgrid.configure(frameSkin, sticky="new")
    tkgrid(tklabel(frameSkin, text="Specify which plotting layout skin to use: "), columnspan=3)
    
    rbValue1 <- tclVar("cont") # le choix par défaut
    # création des 3 boutons radio
    rb1 <- tkradiobutton(frameSkin)
    ##rb2 works with one case and one gen.profile
    #rb2 <- tkradiobutton(frameSkin)
    rb3 <- tkradiobutton(frameSkin)
    rb4 <- tkradiobutton(frameSkin)
    
    # config des boutons radio. Une seule variable tcl pour 3 boutons
    tkconfigure(rb1,variable=rbValue1,value="cont", text="continu")
    #tkconfigure(rb2,variable=rbValue1,value="disc", text="discret")
    tkconfigure(rb3,variable=rbValue1,value="disc_cont", text="disc_cont")
    tkconfigure(rb4,variable=rbValue1,value="cna_mrna_mut", text="cna_mrna_mut")
    tkgrid(rb1, rb3, rb4)
    
    
    rbValue2 <- tclVar("NULL") # le choix par défaut
    
    tkgrid(frameCorrelation)
    tkgrid.configure(frameCorrelation, sticky="new")
    tkgrid(tklabel(frameCorrelation, text=" Specify correlation method:"), columnspan=4)
    rb5 <- tkradiobutton(frameCorrelation)
    rb6 <- tkradiobutton(frameCorrelation)
    rb7 <- tkradiobutton(frameCorrelation)
    rb8 <- tkradiobutton(frameCorrelation)
    #rbValue <- tclVar("pearson") # le choix par défaut
    # config des boutons radio. Une seule variable tcl pour 3 boutons
    tkconfigure(rb5,variable=rbValue2,value="pearson", text="pearson")
    tkconfigure(rb6,variable=rbValue2,value="kendall", text="kendall")
    tkconfigure(rb7,variable=rbValue2,value="spearman", text="spearman")
    tkconfigure(rb8,variable=rbValue2,value="NULL", text="NULL")
    tkgrid(rb8, rb5, rb6, rb7, columnspan=3)
    
    
    
    rbValue3 <- tclVar("0") # le choix par défaut
    
    tkgrid(frameAxes)
    tkgrid.configure(frameAxes, sticky="new")
    tkgrid(tklabel(frameAxes, text=" Choose wich genetic profile in X axe:"), columnspan=2)
    rb9 <- tkradiobutton(frameAxes)
    rb10 <- tkradiobutton(frameAxes)
    
    #rbValue <- tclVar("pearson") # le choix par défaut
    # config des boutons radio. Une seule variable tcl pour 3 boutons
    tkconfigure(rb9,variable=rbValue3,value="0", text=ENV$GenProfsStudies[(ENV$curselectGenProfs)+1][(s*2)+1])
    tkconfigure(rb10,variable=rbValue3,value="1", text=ENV$GenProfsStudies[(ENV$curselectGenProfs)+1][(s*2)+2])
    
    tkgrid(rb9, rb10, columnspan=3)
    
    
    
    OnOK <- function()
    {
        rbVal1 <- as.character(tclvalue(rbValue1))
        rbVal2 <- as.character(tclvalue(rbValue2))
        rbVal3 <- as.character(tclvalue(rbValue3))
        tkdestroy(ENV$ttDialSkinCor)
        if (rbVal1=="cont")
            #tkmessageBox(message=rbVal1)
            ENV$skin<-paste(rbVal1)
        
        if (rbVal1=="disc_cont")
            #tkmessageBox(message=rbVal1)
            ENV$skin<-paste(rbVal1)
        if (rbVal1=="cna_mrna_mut")
            #tkmessageBox(message=rbVal1)
            ENV$skin<-paste(rbVal1)
        if (rbVal2=="pearson")
            #tkmessageBox(message=rbVal2)
            ENV$correlation<-paste(rbVal2)
        if (rbVal2=="kendall")
            #tkmessageBox(message=rbVal2)
            ENV$correlation<-paste(rbVal2)
        if (rbVal2=="spearman")
            #tkmessageBox(message=rbVal2)
            ENV$correlation<-paste(rbVal2)
        if (rbVal2=="NULL")
            #tkmessageBox(message=rbVal2)
            ENV$correlation<-paste("NULL")
        
        if (rbVal3=="0")
            #tkmessageBox(message=rbVal2)
            ENV$Axes<-paste("0")
        
        if (rbVal3=="1")
            #tkmessageBox(message=rbVal2)
            ENV$Axes<-paste("1")
    }
    
    OK.but <- tkbutton(ENV$ttDialSkinCor,text="OK",command=OnOK)
    tkgrid(OK.but)
    
}

#' get matrix with gene expression from file
#' @usage getGeneExpMatrix()
#' @export
#' @return dataframe of gene expression
#' @examples 
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' \dontrun{
#' getGeneExpMatrix()
#' }
getGeneExpMatrix <- function(){
    
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(exists("GeneExpMatrixfile", envir = ENV)){
        rm(ENV$GeneExpMatrixfile)
    }
    ENV$GeneExpMatrixfile <- tclvalue(tkgetOpenFile(filetypes = "{{txt Files} {.txt}} {{All files} *}", title="Choose Gene Expression Matrix from File")) 
    
    if (!nchar(ENV$GeneExpMatrixfile)) {
        tkmessageBox(message = "No file was selected!")
    } else {
        Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        ENV$ProfData<-read.table(ENV$GeneExpMatrixfile)
        
        tkmessageBox(message = paste("The file selected was", ENV$GeneExpMatrixfile))
        tkfocus(ttMain)
    }
    
}

#' Get profile data for more than 500 genes list.
#' @usage getMegaProfData(MegaGeneList,k)
#' @param MegaGeneList Genelist >500
#' @param k integer number of studies
#' @return dataframewith profile data
#' @export
#' @examples 
#' myGlobalEnv <- new.env(parent = emptyenv())
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcgaGSEAlm1021.rds", sep=""))
#' \dontrun{
#' getMegaProfData(myGlobaEnv$MegaGeneList,1) 
#' }
#' 
getMegaProfData <- function(MegaGeneList,k)
{
    
    if(is.integer(length(MegaGeneList)/500)){
        
        G <- lenght(MegaGeneList)/500
    }else{
        G <- as.integer(length(MegaGeneList)/500) + 1
        
    }
    
    GenProf<-myGlobalEnv$GenProfsRefStudies[myGlobalEnv$curselectGenProfs[k]]
    
    Case<-myGlobalEnv$CasesRefStudies[myGlobalEnv$curselectCases[k]]
    
    
    MegaProfData <- 0
    SubMegaGeneList <- 0
    for(g in 1: G){
        
        if (length(MegaGeneList) - length(SubMegaGeneList) > 500){
            SubMegaGeneList <- MegaGeneList[(((g-1)*500)+1):((g)*500)]
        } else{
            SubMegaGeneList <- MegaGeneList[((g*500)+1):(length(MegaGeneList) - length(SubMegaGeneList))]
        }
        
        
        print(paste("Getting Profile Data of Genes from: ", (((g-1)*500)+1), "to",((g)*500), sep= " "))
        
        ProfData<-getProfileData(myGlobalEnv$cgds,SubMegaGeneList, GenProf,Case)
        MegaProfData <- cbind(MegaProfData, ProfData)
    }
    MegaProfData <- MegaProfData[,-1]
    
    
    return(MegaProfData)
    
    
}

#' get Genes Classifier
#' @usage
#' getGenesClassifier()
#'
#' @return a data frma with genes classes
#' @export
#'
#' @examples
#' x <- 0
#' \dontrun{
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
#' getGenesClassifier()
#' }
#' @importFrom geNetClassifier calculateGenesRanking
#' @importFrom geNetClassifier genesDetails
#' @importFrom Biobase ExpressionSet
#' @importFrom Biobase exprs
#' @importFrom Biobase openPDF
#' @importFrom Biobase pData
#' @importFrom Biobase varLabels
#' @importFrom GSEABase GeneSet
#' @importFrom GSEABase GeneSetCollection

getGenesClassifier <- function(){
    
    ## function to remove existant object
    ifrm <- function(obj, env = ENV()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            rm(list = obj, envir = env)
        }
    }
    
    
    
    ####Create a new directory "Results/Classifier" for output: graph of significant genes and Table of genes details
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/")){
        dir.create(file.path(paste(getwd(), "/Results", sep="")), showWarnings = FALSE)
        dir.create(file.path(paste(getwd(), "/Results/Classifier", sep="")), showWarnings = FALSE)
    } else if (!file.exists("Results/Classifier/")){
        dir.create(file.path(paste(getwd(), "/Results/Classifier", sep="")), showWarnings = FALSE)
    }
            testCheckedCaseGenProf()
        
        Lchecked_Studies <- ENV$lchecked_Studies_forCases
        Lchecked_Cases <- length(ENV$curselectCases)
        Lchecked_GenProf <- length(ENV$curselectGenProfs)
        
        #dialog function to select the number of samples
        ENV$ReturnDialogGeneClasses<- dialogGeneClassifier(Lchecked_Cases)
        if(ENV$ReturnDialogGeneClasses[1] == "ID_CANCEL"){
            stop()
        } else{
            
            
            DiseasesType <- 0
            SamplingProfsData<-0
            ProfDataAll<-0
            ProfData<-0
            LengthGenProfs<-0
            LengthCases<-0
            for (i in 1:Lchecked_Studies){
                Si <- ENV$checked_StudyIndex[i]
                progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies[Si], min = 0,
                                                          max = Lchecked_GenProf, width = 400)
                
                #tkfocus(progressBar_ProfilesData)
                LastLengthGenProfs <- LengthGenProfs
                LengthGenProfs <- LengthGenProfs + ENV$LGenProfs[i]+1
                LastLengthCases <- LengthCases
                LengthCases <- LengthCases + ENV$LCases[i]+1
                
                for (k in 1:Lchecked_Cases){
                    
                    Sys.sleep(0.1)
                    setTkProgressBar(progressBar_ProfilesData, k, label=paste( round(k/Lchecked_GenProf*100, 0),
                                                                               "% of Expression Set"))
                    
                    if (ENV$curselectGenProfs[k] <= LengthGenProfs && ENV$curselectGenProfs[k]>LastLengthGenProfs){    
                        
                        GenProf<- ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                        
                        Case<- ENV$CasesRefStudies[ENV$curselectCases[k]]
                        
                        ProfData<- getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                        
                        ProfData <- t(ProfData)
                        
                        ##remove all NAs rows
                        ProfData<- ProfData[which( apply( !( apply(ProfData,1,is.na) ),2,sum)!=0 ),]
                        
                        ##Convert data frame to numeric structure
#                         print("converting data frame of Profile data to numeric stucture...")
#                         
#                         for(i in 1:ncol(ProfData)){
#                             
#                             ProfData[,i] <- as.numeric(ProfData[,i])
#                         }
                        ## for loop is faster than apply fonction
                        #rnames <- rownames(ProfData)
                        #ProfData <- as.data.frame(apply(ProfData,2 ,function(x) as.numeric(x)))
                        #rownames(ProfData) <- rnames
                        
                        
                                            
                        title <- paste(ENV$StudyRefGenProf[k],":",ENV$GenProfChoice[k])
                        
                        if(ncol(ProfData)<ENV$ReturnDialogGeneClasses[1]){
                            msgBigSampl <- paste(ENV$StudyRefCase[k], "has only", ncol(ProfData),"samples.","\nSelect at Max: ",ncol(ProfData), "samples")
                            tkmessageBox(message=msgBigSampl, icon="info")
                            close(progressBar_ProfilesData)
                            stop(msgBigSampl)
                        }
                        set.seed(1234)
                        SamplingProfData <- t(apply(ProfData, 1,function(x)sample(x[!is.na(x)],ENV$ReturnDialogGeneClasses[1])))
                        
                        SamplingColnamesProfData <- sample(colnames(ProfData), ENV$ReturnDialogGeneClasses[1])
                        
                        colnames(SamplingProfData) <- SamplingColnamesProfData
                        SamplingProfsData <- cbind.na(SamplingProfsData,SamplingProfData)
                        print(paste("Sampling from ",ENV$StudyRefCase[k]))
                        ##Extracting Disease Type
                        DiseaseType<- as.matrix(rep(ENV$StudyRefGenProf[k],times=ENV$ReturnDialogGeneClasses[1]))
                        DiseasesType <- c(DiseasesType, DiseaseType)  
                        
                       } 
                } 
                close(progressBar_ProfilesData)
            } 
            
            SamplingProfsData<- SamplingProfsData[,-1]
            DiseasesType <-DiseasesType[-1]
            DiseasesType <- as.data.frame(DiseasesType)
            print("converting DiseaseType as DataFrame...")
            
            if (inherits(try(rownames(DiseasesType) <- colnames(SamplingProfsData) , silent=TRUE),"try-error"))
            {
                msgDuplicateSamples <- paste("Duplicate sample names are not allowed. Do no select two studies from the same disease.")
                tkmessageBox(message=msgDuplicateSamples , icon="warning")
                
                stop(msgDuplicateSamples)
            } else{ 
                
                rownames(DiseasesType) <- colnames(SamplingProfsData)
            }
            
            
            print("adding rownames to DiseasesType...")
            ## create labelDescription for columns of phenoData. 
            ## labeldescription is used by Biobase packages
            ## In our case labelDescription is Equal to column names
            ##metaData <- data.frame(labelDescription= colnames(ClinicalData), row.names=colnames(ClinicalData))        ## Bioconductor’s Biobase package provides a class called AnnotatedDataFrame   
            metaData <- data.frame(labelDescription= "DiseasesType", row.names="DiseasesType")        ## Bioconductor’s Biobase package provides a class called AnnotatedDataFrame   
            
            print("getting metaData...")
            ##that conveniently stores and manipulates 
            ##the phenotypic data and its metadata in a coordinated fashion. 
            phenoData<-new("AnnotatedDataFrame", data=DiseasesType, varMetadata=metaData)    
            print("getting phenoData...")
            ##Assembling an ExpressionSet  
            
            
            eSetClassifier<-Biobase::ExpressionSet(assayData=SamplingProfsData, phenoData=phenoData, annotation="GO")
            print("getting eSetClassifier...")
            if(min(Biobase::exprs(eSetClassifier), na.rm=TRUE)<0){
                print("There are negative values. Translating values by adding the absolute of minimum value to all matrix")
                Biobase::exprs(eSetClassifier) <- Biobase::exprs(eSetClassifier)+(abs(min(Biobase::exprs(eSetClassifier), na.rm=TRUE)))
            }
            
            ENV$eSetClassifier <- eSetClassifier
            
            if (inherits(try(signGenesRank_DiseaseType<- geNetClassifier::calculateGenesRanking(ENV$eSetClassifier[,1:(ENV$ReturnDialogGeneClasses[1]*Lchecked_Cases)], sampleLabels="DiseasesType", lpThreshold=ENV$ReturnDialogGeneClasses[2], returnRanking="significant", plotLp = FALSE), silent=TRUE),"try-error"))
            {
                msgNoSignificantDiff <- paste("The current genes don't differentiate the classes..")
                tkmessageBox(message=msgNoSignificantDiff , icon="warning")
                
                stop(msgNoSignificantDiff )
            } else{ 
                
                signGenesRank_DiseaseType<- geNetClassifier::calculateGenesRanking(ENV$eSetClassifier[,1:(ENV$ReturnDialogGeneClasses[1]*Lchecked_Cases)], sampleLabels="DiseasesType", lpThreshold=ENV$ReturnDialogGeneClasses[2], returnRanking="significant", plotLp = FALSE)
            }
            
            plotCommand<- function(){
                signGenesRank_DiseaseType<- geNetClassifier::calculateGenesRanking(ENV$eSetClassifier[,1:(ENV$ReturnDialogGeneClasses[1]*Lchecked_Cases)], sampleLabels="DiseasesType", lpThreshold=ENV$ReturnDialogGeneClasses[2], returnRanking="significant", plotLp = TRUE)
                
                
            }
            plotModel(plotCommand, title="Ranking Genes By Diseases",hscale=1, vscale=1)
            
            print("plotting model...")
            
            
            ENV$GenesDetails <- geNetClassifier::genesDetails(signGenesRank_DiseaseType)
            print("getting Genes Details...")
            
            GenesDetailsTab <- do.call(rbind.data.frame, ENV$GenesDetails)
            GenesDetailsTab <- t(t(as.data.frame.matrix(GenesDetailsTab)))
            
            
            title=paste("Ranking of significant Genes By Classes")
            getInTable(GenesDetailsTab, title)
            
            
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            
            name1 <- paste(ENV$StudyRefCase, collapse='_', sep="_")
            name2 <- basename(gsub(".txt","",ENV$GeneListfile))
            name3 <- paste(name2,"_" ,name1,".txt",sep="")
            path  <- paste("./Results/Classifier/", name3, sep="")
            write.table(GenesDetailsTab,file=path, col.names=TRUE, quote=FALSE, row.names=TRUE, sep="\t")
            
            msgSavedGenesDetailsClassifier="The table of the ranking genes was saved to /Results/Classifier/"
            tkmessageBox(message=msgSavedGenesDetailsClassifier, icon="info")
            
           
        }
    
    
    ifrm(GenesDetails, ENV)
    
}

#' The  main function to run GSEA-R from Broad Institute
#' @usage
#' Run.GSEA()
#' 
#' @return A vector with sampling size
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
#' \dontrun{
#' Run.GSEA()
#' }
Run.GSEA <- function(){
#   if (!require("GSEABase")) {
#     if (!requireNamespace("BiocManager", quietly=TRUE))
#    install.packages("BioManager")
#    BiocManager::install("GSEABase")
#     library("GSEABase")
#   }
  #source("dialogSelectFiles_GSEA.R")
  
# GSEA 1.0 -- Gene Set Enrichment Analysis / Broad Institute 
#source("dialogSelectFiles_GSEA.R")

#workspace <- getwd()

dialogSelectFiles_GSEA()

##stop the Run when cancelling dialogSelectFiles()
#if(!exists("fname.GSEA")){
#  stop("The GSEA.1.0.R file is not found.")
#}

GSEA.program.location <- paste(path.package("canceR"),"/extdata/GSEA.1.0.R", sep="")
#GSEA.program.location <- fname.GSEA   #  R source program (change pathname to the rigth location in local machine)
source(GSEA.program.location, verbose=TRUE, max.deparse.length=9999)

ENV$prefix <- basename(ENV$fname.Output)

GSEA(                                                                    # Input/Output Files :-------------------------------------------
 input.ds  = ENV$fname.GCT,           # Input gene expression Affy dataset file in RES or GCT format
 input.cls = ENV$fname.CLS,           # Input class vector (phenotype) file in CLS format
 gs.db     = ENV$fname.GMT,         # Gene set database in GMT format
 output.directory      = ENV$fname.Output,        # Directory where to store output and results (default: "")
##  Program parameters :-------------------------------------------------------------------------------------------------------------------------
 doc.string            = ENV$prefix,          # Documentation string used as a prefix to name result files (default: "GSEA.analysis")
 non.interactive.run   = FALSE,               # Run in interactive (i.e. R GUI) or batch (R command line) mode (default: F)
 reshuffling.type      = "sample.labels", # Type of permutation reshuffling: "sample.labels" or "gene.labels" (default: "sample.labels" 
 nperm                 = 1000,            # Number of random permutations (default: 1000)
 weighted.score.type   =  1,              # Enrichment correlation-based weighting: 0=no weight (KS), 1= weigthed, 2 = over-weigthed (default: 1)
 nom.p.val.threshold   = -1,              # Significance threshold for nominal p-vals for gene sets (default: -1, no thres)
 fwer.p.val.threshold  = -1,              # Significance threshold for FWER p-vals for gene sets (default: -1, no thres)
 fdr.q.val.threshold   = 0.25,            # Significance threshold for FDR q-vals for gene sets (default: 0.25)
 topgs                 = 20,              # Besides those passing test, number of top scoring gene sets used for detailed reports (default: 10)
 adjust.FDR.q.val      = FALSE,               # Adjust the FDR q-vals (default: F)
 gs.size.threshold.min = 15,              # Minimum size (in genes) for database gene sets to be considered (default: 25)
 gs.size.threshold.max = 500,             # Maximum size (in genes) for database gene sets to be considered (default: 500)
 reverse.sign          = FALSE,               # Reverse direction of gene list (pos. enrichment becomes negative, etc.) (default: F)
 preproc.type          = 0,               # Preproc.normalization: 0=none, 1=col(z-score)., 2=col(rank) and row(z-score)., 3=col(rank). (def: 0)
 random.seed           = 3338,            # Random number generator seed. (default: 123456)
 perm.type             = 0,               # For experts only. Permutation type: 0 = unbalanced, 1 = balanced (default: 0)
 fraction              = 1.0,             # For experts only. Subsampling fraction. Set to 1.0 (no resampling) (default: 1.0)
 replace               = FALSE,               # For experts only, Resampling mode (replacement or not replacement) (default: F)
 save.intermediate.results = FALSE,           # For experts only, save intermediate results (e.g. matrix of random perm. scores) (default: F)
 OLD.GSEA              = FALSE,               # Use original (old) version of GSEA (default: F)
 use.fast.enrichment.routine = TRUE          # Use faster routine to compute enrichment for random permutations (default: T)
)
#-----------------------------------------------------------------------------------------------------------------------------------------------

# Overlap and leading gene subset assignment analysis of the GSEA results

GSEA.Analyze.Sets(
  directory           = ENV$fname.Output,        # Directory where to store output and results (default: "")
   topgs = 20,                              # number of top scoring gene sets used for analysis
   non.interactive.run = FALSE,
   height = 16,
   width = 16
)

}

#' Dialogbox to select variables from Clinical data
#' @usage
#' dialogOptionGSEAlm(k,ClinicalData)
#' @param k  integer 1
#' @param ClinicalData dataframe with clinical variables
#'
#' @return permutaion value, p-value, coVariables
#' @export
#'
#' @examples
#' #data(ClinicalData)
#' \dontrun{
#' getOptionGSEAlm()
#' }
dialogOptionGSEAlm <- function(k, ClinicalData){
    Lchecked_Cases <- length(ENV$curselectCases)
    ttClinData_cb <- tktoplevel()
    #tkwm.geometry(ttClinData_cb,"180x250")

    tktitle(ttClinData_cb) <- paste(ENV$StudyRefCase[k],ENV$CasesStudies[ENV$curselectCases[k]+1], sep=" ")

    permutEntry <- tclVar(1000)
    rEntry  <- tclVar(.05)

    #tkgrid(tklabel(ttClinData_cb, text="Select CoVariables:"))
    frameCoVariables <- tkframe(ttClinData_cb,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameCoVariables, text="Select Variables:"))
    tkgrid(frameCoVariables)
    tkgrid.configure(frameCoVariables, sticky="new")
    #font <- tkfont.create(size=12,weight="bold")
    #tkgrid(tklabel(frameCoVariables, text="Numerics OR Factors", font=font),columnspan=3)



    ####################
    tkwm.geometry(ttClinData_cb, "430x220")
    yscr1 <- tkscrollbar(ttClinData_cb, repeatinterval=2,
                         command=function(...)tkyview(ttc,...))
    xscr1 <- tkscrollbar(ttClinData_cb, repeatinterval=2,orient="horizontal",
                         command=function(...)tkxview(ttc,...))
    ttc<-tklistbox(ttClinData_cb,height=20, width= 50 ,selectmode="multiple",xscrollcommand=function(...)tkset(xscr1,...),yscrollcommand=function(...)tkset(yscr1,...),background="white")

    tkgrid(ttc,yscr1, columnspan=1)
    tkgrid.configure(yscr1,rowspan=20, columnspan=1,sticky="nsw")
    tkgrid(xscr1,columnspan=2)
    tkgrid.configure(xscr1,rowspan=2,columnspan=2,sticky="ew")

    #tkinsert(ttc,"end","All")
    cbIValue=0
    for(j in 1: length(names(ClinicalData))){
        tkinsert(ttc,"end",names(ClinicalData)[j])

    }



    #####################

#     tkgrid(frameCoVariables)
#     tkgrid.configure(frameCoVariables, sticky="new")
#
#     cbIValue=0
#     for(i in 1: length(names(ClinicalData))){
#
#         cbi <- paste ("cb", i, sep="")
#         cbi <- tkcheckbutton(frameCoVariables)
#         cbiValue <- paste("cb", i, "Value", sep="")
#         cbIValue[i] <- cbiValue
#         cbIValue[i] <- tclVar("0")
#
#         tkconfigure(cbi,variable=cbiValue)
#         labeli <- paste ("label", i , sep="")
#         labelI <- labeli
#         labelI <- tklabel(frameCoVariables,text= names(ClinicalData[i]))
#         tkgrid(labelI,cbi)
#     }
    OnOK <- function(){

        ######################

        ENV$curselect <- as.numeric(tkcurselection(ttc))+1
        ENV$ClinicalData <- ClinicalData[ENV$curselect-1]

        namesClinicalData <- paste("0",names(ClinicalData[ENV$curselect]), sep="+")
        ENV$class2 <- names(table(ClinicalData[ENV$curselect]))[2]
        coVariables <- names(ClinicalData[ENV$curselect])
        #######################


        ### Listen permutation Value
        ENV$permutVal <- as.numeric(tclvalue(permutEntry))

        ##Listen pValue
        ENV$seuilpVal <- as.numeric(tclvalue(rEntry))

        print(paste("permutation: ",ENV$permutVal, sep=" "))
        print (paste("p-Value: ", ENV$seuilpVal, sep=" "))


#         namesClinicalData<-0
#         for (i in 1: length(names(ClinicalData))){
#             cbiValue <- paste("cb", i, "Value", sep="")
#             cbIValue <- cbiValue
#             cbiVal <- paste("cb", i, "Val", sep="")
#             cbIVal<-cbiVal
#
#             cbIVal[i] <- tclvalue(cbIValue)
#
#             if (cbIVal[i]=="1"){
#
#                 namesClinicalData <- paste(namesClinicalData,names(ClinicalData[i]), sep="+")
#
#                 ENV$class2 <- names(table(ClinicalData[i]))[2]
#             }
#         }

        ENV$namesClinicalData <- namesClinicalData

        ### only for Survival plot
        ENV$variable <- as.formula(sprintf("Surv(OS_MONTHS,OS_STATUS)~%s", coVariables))

        #coVariables<-paste0(strsplit(namesClinicalData, '')[[1]][c(-1,-2)], collapse = '')
        coVariables <- sprintf("~%s", coVariables)
        ENV$coVariables <- as.formula(coVariables)
        print(paste("coVariables:", ENV$coVariables, sep=" ")[2])
        tkdestroy(ttClinData_cb)
    }

    framePermutation<- tkframe(ttClinData_cb,relief="groove",borderwidth=2)
    tkgrid(tklabel(framePermutation, text="Specify Permutation number:"))
    tkgrid(framePermutation)
    tkgrid.configure(framePermutation, sticky="new")
    tkgrid(tkentry(framePermutation,width=12,textvariable=permutEntry))



    frameTHRESHOLD<- tkframe(ttClinData_cb,relief="groove",borderwidth=2)
    tkgrid(tklabel(frameTHRESHOLD, text="Specify Threshold pValue:"))
    tkgrid(frameTHRESHOLD)
    tkgrid.configure(frameTHRESHOLD, sticky="new")

    tkgrid(tkscale(frameTHRESHOLD,from=0.01,to=0.1,showvalue=TRUE,
                   variable=rEntry,resolution=.005,orient='horiz'))

    OK.but <- tkbutton(ttClinData_cb,text="OK",command=OnOK)
    tkgrid(OK.but)

    tkwait.window(ttClinData_cb)

}

#' Reduce MSigDB size for only gene list
#' @usage getMSigDB(eSet, k)
#' @param eSet Expression Set
#' @param k integer Number of studies
#' 
#' @return MSigDB for user gene List
#' @export
#' 
#' @examples
#'  d <- 7
#' \dontrun{
#' setWorkspace()
#' getMSigDB(eSet = ENV$eSetClassifier,k = 1)
#' }
#' 
getMSigDB <- function(eSet,k){
     
    ## Selecting GeneList from getGeneList.R or getGenesClasses.R functions.
    ifExist <- function(obj, env = ENV()) {
        obj <- deparse(substitute(obj))
        if(exists(obj, envir = env)) {
            ENV$GeneList <- rownames(ENV$GenesDetails[[k]])
            ENV$GeneListfile <- paste(names(ENV$GenesDetails)[k])
            return(ENV$GeneListfile)
        }
    }
 
    ifExist(ENV$GenesDetails, ENV)
    
    #list of GO terms (biological processes) acquired from MSigDB
    #http://www.broadinstitute.org/gsea/downloads.jsp
    mSigDBPath <- tclvalue(tkgetOpenFile(filetypes = "{{GMT Files} {.gmt}} {{All files} *}",
                                         title="load MSigDB for Gene Symbol List ")) # Very simple, isn't it?
    if (!nchar(mSigDBPath)) {
        tkmessageBox(message = "No file was selected!")
    } else {
        tkmessageBox(message = paste("The file selected was", mSigDBPath))
    }
    ##verify if the mSigDB_forGenelist  exists or not If yes skip if no do the remain codes
    ####Changing the gene list file with genes classes
    mSigDB_SubName <- paste(basename(mSigDBPath), "_",basename(ENV$GeneListfile),".RData",sep="")
    mSigDB_SubName <- gsub(".txt","", mSigDB_SubName)
    ENV$mSigDB_SubName <- gsub(".gmt","", mSigDB_SubName)
    
    #########################
    ####Create a new directory "MSigDB" from MSigDB and All specific MSigDB created
    ## for specific Gene List (MSigDB_GeneList.RData)
    ##Set MSigDB as tempary work directory
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/MSigDB/")){
        dir.create(file.path(paste(getwd(), "/Results/MSigDB", sep="")), showWarnings = FALSE)
    } 
    ###########
    
    workspace <- getwd()
    setwd("./Results/MSigDB")
    if(file.exists(ENV$mSigDB_SubName)&& exists("mSigDB_forGeneList",envir = ENV)){
        setwd(workspace)
        print("skip getMSigDB: The SubMSigDB existe in /Results/MSigDB folder.")
        ENV$mSigDB_SubName <-ENV$mSigDB_SubName
    } else{
        setwd(workspace)
        print("Getting SubMSigDB for requested eSet and genes list.")
        mSigDB<-readLines(mSigDBPath)
        
        #split each line at the tabs
        mSigDB<-strsplit(mSigDB,'\t')
        
        
        #use the first entry of each gene set as name
        names(mSigDB)<-sapply(mSigDB,function(x)x[1])
               
        #remove the first two entries
        mSigDB<-sapply(mSigDB,function(x)x[3:length(x)])
        
        
        mSigDB_forGeneList <- sapply(rownames(Biobase::exprs(eSet)), function(x) as.numeric(sapply(mSigDB, '%in%', x=x)))
        rownames(mSigDB_forGeneList) <- names(mSigDB)
        
        #save the processed gene sets for further use
        #save(mSigDB_forGeneList,file='mSigDB_forGeneList.RData')
        mSigDB_SubName <-paste(basename(mSigDBPath), "_",basename(ENV$GeneListfile),".RData",sep="")
        mSigDB_SubName <- gsub(".txt","", mSigDB_SubName)
        mSigDB_SubName <- gsub(".gmt","", mSigDB_SubName)
        ENV$mSigDB_SubName <- mSigDB_SubName
        ENV$mSigDB_forGeneList<-mSigDB_forGeneList
        
        #assign(mSigDB_SubName,mSigDB_SubName, envir=.GlobalEnv)
        workspace <- getwd()
        setwd("./Results/MSigDB")
        Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
        save(mSigDB_forGeneList,file=ENV$mSigDB_SubName, envir = ENV)
        setwd(workspace)
    }
}

#' Associate phenotype to Studies (cancers)
#' @usage
#' getPhenoTest()
#' 
#' @return a dataframe with disease/ variables association
#' @export
#'
#' @examples
#' readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#' \dontrun{
#' getPhenoTest(ENV$eSet)
#' }
#' @importFrom phenoTest ExpressionPhenoTest
#' @importFrom phenoTest getSignif
#' @importFrom phenoTest getFc
#' 

getPhenoTest <- function(){
    
    print("Getting eSet from selected Case and Genetic Profile...")
    geteSet()
    
    
    Biobase::pData(ENV$eSet)<- as.data.frame(lapply(Biobase::pData(ENV$eSet),
                                                    function(x) if(is.character(x)|is.factor(x)) gsub("N/A","NA",x,  ignore.case = TRUE) else x))
    
    
    ##popup select variable
    
    dialogOptionPhenoTest(ENV$eSet)
    tkwait.window(ENV$ttPheno)
    
    if(length(table(Biobase::pData(ENV$eSet)[ENV$Category]))>4){
        
        msgBadCategory <- paste("Category variable has more than 4 Levels. Select another category.")
        tkmessageBox(message= msgBadCategory, icon="info")
        stop(msgBadCategory)
    }
    
    
    if (inherits(try(epheno <- phenoTest::ExpressionPhenoTest(ENV$eSet, ENV$var2test,
                                                              p.adjust.method=ENV$p.adjustChoice, approach="frequentist"), silent=TRUE),"try-error"))
    {
        msgBadCovariables <- paste("There is incompatible variables. Select another Formula.")
        tkmessageBox(message=msgBadCovariables, icon="warning")
        stop(msgBadCovariables)
    } else{         
        epheno <- phenoTest::ExpressionPhenoTest(ENV$eSet, ENV$var2test,
                                                 p.adjust.method=ENV$p.adjustChoice, approach="frequentist", continuousCategories= length(table(pData(ENV$eSet)[ENV$Category])))
        
        
    }
    
    ## add continuousCategories and approach options
    #epheno <- ExpressionPhenoTest(ENV$eSet, var2test, p.adjust.method=p.adjustChoice, continuousCategories= 3)
    #epheno <- ExpressionPhenoTest(ENV$eSet, var2test, p.adjust.method=p.adjustChoice, continuousCategories= 3, approach="frequentist")
    
    # ##################################
    ephenoSign<- phenoTest::getSignif(epheno)
    fChange <- phenoTest::getFc(epheno)
    PhenoTestResults <- merge(ephenoSign, fChange, by="row.names")
    
    row.names(PhenoTestResults) <- PhenoTestResults[,1]
    PhenoTestResults <- PhenoTestResults[,-1]
    
    ## Round numeric values
    for (i in 1:length(PhenoTestResults)){
        if(is.numeric(PhenoTestResults[1,i])){
            PhenoTestResults[,i] <- round(PhenoTestResults[,i], digits=3)
        } 
        
    }
    ## rearange row by Significant pval
    if(exists("EventTime", envir = ENV)){
        ENV$PhenoTestResults <- PhenoTestResults[order(PhenoTestResults[,length(table(pData(ENV$eSet)[ENV$Category]))]),]
    } else{
        PhenoTestResults <- PhenoTestResults[order(PhenoTestResults[,2]),]
    }
    
    PhenoTestResults <-rbind(colnames(PhenoTestResults), PhenoTestResults)
    PhenoTestResults<- cbind(as.character(row.names(PhenoTestResults)), PhenoTestResults)
    
    Title <- paste(ENV$GenProfChoice,"Significant pVal <0.05", sep=":")
    getInTable(PhenoTestResults, Title)
    
    
    
    ### filtering only pVal < 0.05.
    if(exists("EventTime", envir = ENV)){
        PhenoTestResultsFilter <- subset(PhenoTestResults, 
                                         PhenoTestResults[,length(table(pData(ENV$eSet)[ENV$Category]))] <= 0.05)
        
    }else{
        PhenoTestResultsFilter <- subset(PhenoTestResults, PhenoTestResults[,3] <= 0.05)
    }
    
    title<- paste(ENV$GenProfChoice," ONLY Significant pVal <0.05", sep=":")
    getInTable(PhenoTestResultsFilter, title)
    
     
}

#' get Profles Data of multiple genes
#' 
#' @usage
#' getProfilesDataMultipleGenes(getSummaryGSEAExists)
#' @param getSummaryGSEAExists if equal to 0, the clinical data is displayed in table. if the argument is equal to 1, the clinical data is used to summarise GSEA analysis results.
#' 
#' @return a file with a dataframe of profle data
#' @export
#'
#' @examples
#'readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
#'\dontrun{
#' getProfilesDataMultipleGenes(getSummaryGSEAExists = 0)
#' }
#' 
getProfilesDataMultipleGenes <-function(getSummaryGSEAExists){
    
    tclRequire("BWidget")
    tclRequire("Tktable")
    
    
    testCheckedCaseGenProf()
    
    ####Create a new directory "Results" for All Profiles Data
    ##Set Results as tempary work directory
    Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
    if(!file.exists("Results/")){
        dir.create(file.path(paste(getwd(), "/Results", sep="")), showWarnings = FALSE)
        dir.create(file.path(paste(getwd(), "/Results/ProfileData", sep="")), showWarnings = FALSE)
    } else if (!file.exists("Results/ProfileData/")){
        dir.create(file.path(paste(getwd(), "/Results/ProfileData", sep="")), showWarnings = FALSE)
    }
    
    ###
    if(exists("ttProfData")){
        tkdestroy(ttProfData)
    }
    
    ttProfData <- tktoplevel()
    tkwm.title(ttProfData,"The list of Profiles Data")
    tkwm.geometry(ttProfData, "580x470")
    
    xScr       <- tkscrollbar(ttProfData,command=function(...)tkxview(treeWidget,...),orient="horizontal")
    yScr       <- tkscrollbar(ttProfData,command=function(...)tkyview(treeWidget,...))
    police <- tkfont.create(family="arial", size=11)
    treeWidget <- tkwidget(ttProfData,"Tree",xscrollcommand=function(...)tkset(xScr,...),
                           yscrollcommand=function(...)tkset(yScr,...),width=68,height=25, bg="white")
    
    
    tkgrid(treeWidget,yScr)
    tkgrid.configure(yScr,stick="nsw")
    tkgrid(xScr)
    tkgrid.configure(xScr,stick="new")
    ###
    
    ProfData=0
    i<-0
    for (s in ENV$checked_Studies_id){
        i<-i+1
        Si = ENV$checked_StudyIndex[i]
        progressBar_ProfilesData <- tkProgressBar(title = ENV$Studies$name[Si], min = 0,
                                                  max = length(ENV$curselectGenProfs), width = 400)
        
        #tkfocus(progressBar_ProfilesData)
        # do not select study_desc_position_in_genProfs
        
        #study_desc_position_in_Cases <- ENV$n_Cases[i]+1
        
        #print("Studies"); print(i)
        
        ####Tree ROOT==Studies
        StudyiChoice <- paste("Study",i,"Choice", sep="")
        
        tkinsert(treeWidget, "end", "root", StudyiChoice, text= ENV$StudyChoice[i])
        

        study_desc_position_in_genProfs <- 0
        
        for (k in 1:length(ENV$curselectGenProfs)){
            
            Sys.sleep(0.1)
            setTkProgressBar(progressBar_ProfilesData, k, 
                             label=paste(round(k/length(ENV$curselectGenProfs)*100, 0),
                                                                       "% of Profiles Data"))
            # print(paste(ENV$curselectGenProfs[k], "<=", ENV$n_GenProfs[i]))
            # print(paste(ENV$curselectGenProfs[k], ">", study_desc_position_in_genProfs))
            
            # Avoid to select study description  
            if (ENV$curselectGenProfs[k] <= ENV$n_GenProfs[i] && 
                 ENV$curselectGenProfs[k] > study_desc_position_in_genProfs){    
                
                study_desc_position_in_genProfs <- study_desc_position_in_genProfs + ENV$n_GenProfs[i]
                
                GenProf <- ENV$GenProfsRefStudies[ENV$curselectGenProfs[k]]
                
                ## verify if GenProf has expression data
                if (length(grep("mrna", GenProf, ignore.case = TRUE))==0 && 
                    length(grep("gistic", GenProf, ignore.case = TRUE))==0 &&
                    length(grep("CNA", GenProf, ignore.case = TRUE))==0){
                    msgNoExp <- "Select Expression data from Genetics Profiles"
                    tkmessageBox(message = msgNoExp, icon='info')
                    break
                }
                
                    #ProfData <- getProfileData(ENV$cgds,ENV$GeneList, GenProf,Case)
                    ProfData <- getDataByGenes(
                                    api =  ENV$cgds,
                                    studyId = s,
                                    genes = ENV$GeneList,  #c("NF1", "TP53", "ABL1")
                                    by = "hugoGeneSymbol",
                                    molecularProfileIds = GenProf)  |>
                                    unname() |>
                                    as.data.frame() |>
                                    select("hugoGeneSymbol","sampleId", "value") |>
                                    tidyr::spread("hugoGeneSymbol", "value") |>
                                    data.frame(row.names = 1)
                        
                 
                ##Convert data frame to numeric structure
#                 print("converting data frame of Profile data to numeric stucture...")
#                 cidx <- !(sapply(ProfData, is.numeric))
#                 ProfData[cidx] <- lapply(ProfData[cidx], as.numeric)
                #for(p in 1:ncol(ProfData)){
                    
                 #   ProfData[,p] <- as.numeric(ProfData[,p])
                #}
                
                print(paste("Genes number:", ncol(ProfData), sep=" "))
                ##cleaning data.framme from NaN and NA
                ## substitute "NaN" by "NA"
                
                if(length(grep("NaN",ProfData))!=0 && getSummaryGSEAExists==1){
                    
                    print("This step takes a while to remove NaN string and convert them to NA (12000 genes takes about 10 min)")
                    for (j in 1:ncol(ProfData)){
                        #ProfData[,i]<- as.numeric(gsub("\\[Not Available\\]","na", ProfData[,i]))
                        #ProfData <- sapply(ProfData, function(x) gsub("NA", "na", x))
                        ProfData[,j]<- as.numeric(gsub("NaN",NA, ProfData[,j]))
                        #ProfData[,i]<- as.numeric(gsub(NA,"na", ProfData[,i]))
                        
                    }
                }
                print("Removing genes without data... ")
                ##remove all NAs rows
                print(paste("Genes number:", ncol(ProfData), sep=" "))
            
                if(ncol(ProfData)==0){
                    tkmessageBox(message=paste("There is no", 
                                               GenProf, " for used gene list",icon="info"))
                    break(paste("There is no", 
                                GenProf, " for used gene list"))
                    
                }else{
                    ProfData<- ProfData[which( apply( !( apply(ProfData,1,is.na) ),2,sum)!=0 ),]
                    print(paste("dimension of Profile Data:"))
                    print(dim(ProfData))
                }
                

                ## When matrix has negative values,  simple space are generated before every positive value. 
                ## This space cause deleting all columns from matrix when we try to remove the columns without values NAs
                if(min(ProfData, na.rm = TRUE) < 0){
                    for (j in 1:ncol(ProfData)){
                        ProfData[,j] <- as.numeric(gsub(" ","", ProfData[,j] ))
                    }
                }
                
                print(paste("dimension of Profile Data:"))
                print(dim(ProfData))
                print("end cleaning...")
                
                # if(ncol(ProfData)==0){
                #     tkmessageBox(message=paste("There is no", 
                #                                ENV$GenProfChoice[k]," for used gene list",icon="info"))
                #     #tkdestroy(ttProfData)
                #     break 
                #     
                # }
                print(paste("dimension of Profile Data:"))
                print(dim(ProfData))
                
                ProfData <- round(ProfData, digits=3)
                print("rounding Profile Data ...")
                fileName <- paste("SD_", ENV$checked_StudyIndex[i],"_","GenProf_",
                                ENV$curselectGenProfs[k],"_",
                                "CASE_",ENV$curselectCases[k],".txt")
                
                workspace <- getwd()
                Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
                setwd(paste(getwd(),"/Results/ProfileData", sep=""))
                
                write.table(ProfData,file=fileName, col.names=TRUE, 
                            quote=FALSE, row.names=TRUE, sep="\t")
                
                setwd(workspace)
                
                ###Tree suit
                GenkProfChoice <- paste("Gen", k,"Prof", sep="")
                CasekChoice <- paste("Case", k, "Choice", sep="")
                ProfkData <- paste("Prof", k, "Data", sep="")
                
                tkinsert(treeWidget, "end", StudyiChoice, GenkProfChoice, 
                         text = paste("Genetic Profile: ", GenProf, sep=" "))
                tkinsert(treeWidget, "end", GenkProfChoice,CasekChoice,
                         text = paste("Case: ", ENV$CaseChoice[k], sep=" "))
                tkinsert(treeWidget, "end",CasekChoice, ProfkData,
                         text= paste("Profile Data file: ", fileName, sep=" "))
                
                print("END!")
                
            }
        } 
        close(progressBar_ProfilesData) 
    }
    
    OnOK <- function()
    { 
        ###view WIDGETOOL
        file <- tclvalue(tkgetOpenFile()) 
        if (!nchar(file)) {
            tkmessageBox(message = "No file was selected!")
        } else {
            Sys.chmod(getwd(), mode = "0777", use_umask = TRUE)
            #w<-2
            if(exists("ProfDataTable")){
                LastProfDataTable<-  ProfDataTable 
            }
            ProfDataTable<-read.table(file, header=TRUE, row.names=1)
            assign("ProfDataTable", ProfDataTable, envir = ENV)
            
            
            getInTable(ProfDataTable, file)
            
        }
        
    }
    OnCancel <- function(){
        tkdestroy(ttProfData)
    }
    
    OK.but <-tkbutton(ttProfData,text="   OK   ", command= OnOK)
    tkgrid(OK.but,  row=2,column=0, sticky="e")
    
    Cancel.but <- tkbutton(ttProfData, text="Cancel", command=OnCancel )
    # Place the two buttons on the same row in their assigned window (ttProfData)
    tkgrid(Cancel.but,row=2, column=0, sticky="w")
    tkfocus(ttProfData)
    
}

testthat::context("canceR functions")

#test_that("set_class", {
# foo <- . %>% .^2 %>% set_class(c("foo", class(.)))
#expect_equal(3 %>% foo %>% class,c("foo","numeric"))
#})

testthat::test_that("cbioportal connection",
                    {
                       # mycgds <- CGDS("http://www.cbioportal.org/api/v2/api-docs")
                        cgds <- cBioPortalData::cBioPortal(
                            hostname = "www.cbioportal.org",
                            protocol = "https",
                            api = "/api/v2/api-docs"
                        )
                        result <- test.CGDS(cgds)
                    })

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modalDialog.R
\name{modalDialog}
\alias{modalDialog}
\title{Dialog box to specify Gene Symbol.}
\usage{
modalDialog(title, question, entryInit, entryWidth = 40,returnValOnCancel = "ID_CANCEL")
}
\arguments{
\item{title}{string}

\item{question}{string}

\item{entryInit}{entryInit}

\item{entryWidth}{40}

\item{returnValOnCancel}{"ID_CANCEL"}
}
\value{
dialog box
}
\description{
Dialog box to specify Gene Symbol.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
## Select Case from Breast Cancer
ENV <- new.env(parent = emptyenv())
ENV$curselectCases <- 9
##Select Genetic Profile from Breast Cancer
ENV$curselectGenProfs <- 4
## get Specific Mutation data for 73 Genes list
\dontrun{
getProfilesDataSingleGene()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGSEAlm_Diseases.R
\name{getGSEAlm_Diseases}
\alias{getGSEAlm_Diseases}
\title{get GSEA linear modeling by studies (diseases)}
\usage{
getGSEAlm_Diseases()
}
\value{
a dataframe with annotation (GO, BP)
}
\description{
get GSEA linear modeling by studies (diseases)
}
\examples{
 readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
 \dontrun{
 getGSEAlm_Diseases
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/displayInTable.R
\name{displayInTable}
\alias{displayInTable}
\title{Display matrix in tcltk table}
\usage{
displayInTable(tclarray,title="",height=-1,width=-1,nrow=-1,ncol=-1)
}
\arguments{
\item{tclarray}{a dataframe}

\item{title}{title of the table}

\item{height}{-1}

\item{width}{-1}

\item{nrow}{-1}

\item{ncol}{-1}
}
\value{
display a Table
}
\description{
Display matrix in tcltk table
}
\examples{
#data(ClinicalData)
\dontrun{
getInTable(Table= ClinicalData, title= "Clinical Data")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getListProfData.R
\name{getProfData}
\alias{getProfData}
\title{Search and get genetic profiles (CNA,mRNA, Methylation, Mutation...)}
\usage{
getProfData(study,genProf, listGenProf, GeneList, Mut)
}
\arguments{
\item{study}{Study ID}

\item{genProf}{Genetic Profile id (cancer_study_id_[mutations, cna, methylation, mrna ]).}

\item{listGenProf}{A list of Genetic Profiles for one study.}

\item{GeneList}{A list of genes}

\item{Mut}{Condition to set if the genetic profile is mutation or not (0,1)}
}
\value{
A data frame with Genetic profile
}
\description{
Search and get genetic profiles (CNA,mRNA, Methylation, Mutation...)
}
\details{
See \url{https://github.com/kmezhoud/bioCancer/wiki}
}
\examples{
cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geteSet.R
\name{geteSet}
\alias{geteSet}
\title{Built Expression Set (eSet) from profile data.}
\usage{
geteSet()
}
\value{
ExpressionSet
}
\description{
Built Expression Set (eSet) from profile data.
}
\examples{
 f <- 9
 \dontrun{
 readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
 geteSet()
 }

}

\name{GSEA.GeneRanking}
\alias{GSEA.GeneRanking}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.GeneRanking
}
\description{
GSEA.GeneRanking
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A}{
%%     ~~Describe \code{A} here~~
}
  \item{class.labels}{
%%     ~~Describe \code{class.labels} here~~
}
  \item{gene.labels}{
%%     ~~Describe \code{gene.labels} here~~
}
  \item{nperm}{
%%     ~~Describe \code{nperm} here~~
}
  \item{permutation.type}{
%%     ~~Describe \code{permutation.type} here~~
}
  \item{sigma.correction}{
%%     ~~Describe \code{sigma.correction} here~~
}
  \item{fraction}{
%%     ~~Describe \code{fraction} here~~
}
  \item{replace}{
%%     ~~Describe \code{replace} here~~
}
  \item{reverse.sign}{
%%     ~~Describe \code{reverse.sign} here~~
}
}
\value{
GSEA.GeneRanking
}

\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGeneList.R
\name{getGeneList}
\alias{getGeneList}
\title{User needs to specify which gene is interesting to get genomic cancer data. The gene must be with Symbol and one gene by line.}
\usage{
getGeneList()
}
\value{
Gene list path of file
}
\description{
User needs to specify which gene is interesting to get genomic cancer data. The gene must be with Symbol and one gene by line.
}
\examples{
ENV <- new.env(parent = emptyenv())
\dontrun{
getGeneList()
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGenesClassifier.R
\name{getGenesClassifier}
\alias{getGenesClassifier}
\title{get Genes Classifier}
\usage{
getGenesClassifier()
}
\value{
a data frma with genes classes
}
\description{
get Genes Classifier
}
\examples{
x <- 0
\dontrun{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
getGenesClassifier()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGenesTree_SingleCase.R
\name{getGenesTree_SingleCase}
\alias{getGenesTree_SingleCase}
\title{classify genes in tree for two phenotypes in the same case(disease).}
\usage{
getGenesTree_SingleCase()
}
\value{
tree plot
}
\description{
classify genes in tree for two phenotypes in the same case(disease).
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
\dontrun{
getGenesTree_SingleCase()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogSelectFiles_GSEA.R
\name{dialogSelectFiles_GSEA}
\alias{dialogSelectFiles_GSEA}
\title{Dialog Box to  Select GCT, CLS, GMT and output Files for GSEA-R (Broad Institute)}
\usage{
dialogSelectFiles_GSEA()
}
\value{
A vector with files paths
}
\description{
Dialog Box to  Select GCT, CLS, GMT and output Files for GSEA-R (Broad Institute)
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
dialogSelectFiles_GSEA()
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGenProfs.R
\name{getGenProfs}
\alias{getGenProfs}
\title{Get Genetic Profile from selected Studies}
\usage{
getGenProfs()
}
\value{
dataframe with genetic profil
}
\description{
Get Genetic Profile from selected Studies
}
\examples{

cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogSpecificMut.R
\name{dialogSpecificMut}
\alias{dialogSpecificMut}
\title{dialog box to Specify Mutation using Regular Expression. Search specific mutation using regular expression.}
\usage{
getSpecificMut()
}
\value{
a a dataframe with specific mutation informations
}
\description{
dialog box to Specify Mutation using Regular Expression. Search specific mutation using regular expression.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getSpecificMut()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbind.na.R
\name{rbind.na}
\alias{rbind.na}
\title{bind non equal row}
\usage{
rbind.na(..., deparse.level = 1)
}
\arguments{
\item{...}{(generalized) vectors or matrices.}

\item{deparse.level}{integer controlling the construction of labels in 
the case of non-matrix-like arguments (for the default method):
deparse.level = 0 constructs no labels; the default,
deparse.level = 1 or 2 constructs labels from the argument names.}
}
\value{
a data frame with merged rows
}
\description{
bind non equal row
}
\examples{
\dontrun{
row1 <- c("a","b","c","d")
row2 <- c("A", "B", "C")
row3 <- rbind.na(row1, row2)
}

}

\name{GSEA.Threshold}
\alias{GSEA.Threshold}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.Threshold
}
\description{
GSEA.Threshold
}
\usage{
GSEA.Threshold(V, thres, ceil)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{V}{
%%     ~~Describe \code{V} here~~
}
  \item{thres}{
%%     ~~Describe \code{thres} here~~
}
  \item{ceil}{
%%     ~~Describe \code{ceil} here~~
}
}

\value{
GSEA.Threshold
}
\examples{
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{

##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogOptionCircos.R
\name{dialogOptionCircos}
\alias{dialogOptionCircos}
\title{Checkbox to select dimensions}
\usage{
dialogOptionCircos()
}
\value{
a checkbox with all dimensions
}
\description{
Checkbox to select dimensions
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
\dontrun{
dialogOptionCircos()
#getCircos(dimension ="All")
}
}

\name{GSEA.VarFilter}
\alias{GSEA.VarFilter}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.VarFilter
}
\description{
GSEA.VarFilter
}
\usage{
GSEA.VarFilter(V, fold, delta, gene.names = "NULL")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{V}{
%%     ~~Describe \code{V} here~~
}
  \item{fold}{
%%     ~~Describe \code{fold} here~~
}
  \item{delta}{
%%     ~~Describe \code{delta} here~~
}
  \item{gene.names}{
%%     ~~Describe \code{gene.names} here~~
}
}
\value{
GSEA.VarFilter
}

\examples{
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialoggetGeneListMSigDB.R
\name{dialoggetGeneListMSigDB}
\alias{dialoggetGeneListMSigDB}
\title{Multi-select choice of gene sets from loaded MSigDB}
\usage{
dialoggetGeneListMSigDB(MSigDB)
}
\arguments{
\item{MSigDB}{object with MSigDB. A list of genesets}
}
\value{
a dataframe with genes classes
}
\description{
Multi-select choice of gene sets from loaded MSigDB
}
\examples{
z <- 7
\dontrun{
##MSigDB <- readLines(paste(.libPaths(),"/canceR/extdata/MSigDB/c5.bp.v4.0.symbols.gmt", sep=""))
dialoggetGeneListMSigDB(MSigDB)
}



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogPlotOption_SkinCor.R
\name{dialogPlotOption_SkinCor}
\alias{dialogPlotOption_SkinCor}
\title{Checkbox to select variables for plotting}
\usage{
dialogPlotOption_SkinCor(s)
}
\arguments{
\item{s}{integer number of Studies}
}
\value{
Dialog box with setting of correlation method
}
\description{
Checkbox to select variables for plotting
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
\dontrun{
dialogPlotOption_SkinCor(1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getProfilesDataSingleGene.R
\name{getProfilesDataSingleGene}
\alias{getProfilesDataSingleGene}
\title{get Profiles Data  for a Single Gene.}
\usage{
getProfilesDataSingleGene()
}
\value{
dataframe with profiles data for a single gene
}
\description{
get Profiles Data  for a Single Gene.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
## Select Case from Breast Cancer
 ENV <- new.env(parent = emptyenv())
ENV$curselectCases <- 9
##Select Genetic Profile from Breast Cancer
ENV$curselectGenProfs <- 4
## get Specific Mutation data for 73 Genes list
\dontrun{
getProfilesDataSingleGene()
}
}

\name{GSEA.Analyze.Sets}
\alias{GSEA.Analyze.Sets}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.Analyze.Sets
}
\description{
http://www.broadinstitute.org/cancer/software/gsea/wiki/index.php/R-GSEA_Readme
}
\value{
GSEA.Analyze.Sets
}
\usage{
GSEA.Analyze.Sets(directory,topgs="",non.interactive.run= FALSE,height=12,width=17)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{directory}{
\code{directory= fname.Output} 
}
  \item{topgs}{
\code{topgs = 20} 
}
  \item{non.interactive.run}{
      \code{non.interactive.run= FALSE} 
}
  \item{height}{
      \code{height=16} 
}
  \item{width}{
      \code{width=16} 
}
}


\references{
 
 http://www.broadinstitute.org/cancer/software/gsea/wiki/index.php/Main_Page.
}
\author{
Subramanian, Tamayo, et al. (2005, PNAS 102, 15545-15550) and Mootha,
Lindgren, et al. (2003, Nat Genet 34, 267-273)
}



\examples{
\dontrun{
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPhenoTest.R
\name{getPhenoTest}
\alias{getPhenoTest}
\title{Associate phenotype to Studies (cancers)}
\usage{
getPhenoTest()
}
\value{
a dataframe with disease/ variables association
}
\description{
Associate phenotype to Studies (cancers)
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
\dontrun{
getPhenoTest(ENV$eSet)
}
}

\name{OLD.GSEA.EnrichmentScore}
\alias{OLD.GSEA.EnrichmentScore}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ 
OLD.GSEA.EnrichmentScore
}
\description{
OLD.GSEA.EnrichmentScore
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gene.list}{
%%     ~~Describe \code{gene.list} here~~
}
  \item{gene.set}{
%%     ~~Describe \code{gene.set} here~~
}
}
\value{
OLD.GSEA.EnchmentScore
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgdsr.R
\name{test.CGDS}
\alias{test.CGDS}
\title{S3 method to test cBioPortal connection}
\usage{
\method{test}{CGDS}(x, ...)
}
\arguments{
\item{x}{connection object}

\item{...}{not used}
}
\description{
S3 method to test cBioPortal connection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogMetOption.R
\name{dialogMetOption}
\alias{dialogMetOption}
\title{Dialog Box to set methylation options}
\usage{
dialogMetOption(ProfData, k)
}
\arguments{
\item{ProfData}{adataframe with methylation data}

\item{k}{threshold of silencing gene 0:1}
}
\value{
a dialog box to set methylation option (threshold of silencing gene)
}
\description{
Dialog Box to set methylation options
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getMetDataMultipleGenes()
#dialogMetOption(ProfData,0.7)
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Match_GeneList_MSigDB.R
\name{Match_GeneList_MSigDB}
\alias{Match_GeneList_MSigDB}
\title{Search MSigDb that overlap gene list}
\usage{
Match_GeneList_MSigDB
}
\value{
GeneList
}
\description{
Search MSigDb that overlap gene list
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
\dontrun{
Match_GeneList_MSigDB()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_2Genes_1GenProf.R
\name{plot_2Genes_1GenProf}
\alias{plot_2Genes_1GenProf}
\title{plot correlation of two genes expressions.}
\usage{
plot_2Genes_1GenProf()
}
\value{
plot
}
\description{
plot correlation of two genes expressions.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
plot_2Genes_1GenProf()
}
}

\name{GSEA.EnrichmentScore2}
\alias{GSEA.EnrichmentScore2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.EnrichmentScore2
}
\description{
GSEA.EnrichmentScore2
}
\usage{
GSEA.EnrichmentScore2(gene.list, gene.set, weighted.score.type = 1, correl.vector = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gene.list}{
%%     ~~Describe \code{gene.list} here~~
}
  \item{gene.set}{
%%     ~~Describe \code{gene.set} here~~
}
  \item{weighted.score.type}{
%%     ~~Describe \code{weighted.score.type} here~~
}
  \item{correl.vector}{
%%     ~~Describe \code{correl.vector} here~~
}
}
\value{
GSEA.EnrichmentScore2
}
\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCircos.R
\name{getCircos}
\alias{getCircos}
\title{get Circos Layout for selected studies and selected dimensions}
\usage{
getCircos(dimension)
}
\arguments{
\item{dimension}{string (All,mRNA, CNA, Met,RPPA, miRNA, Mut)}
}
\value{
a plot with Circos style
}
\description{
get Circos Layout for selected studies and selected dimensions
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
\dontrun{
getCircos(dimension ="All")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGenesTree_MultipleCases.R
\name{getGenesTree_MultipleCases}
\alias{getGenesTree_MultipleCases}
\title{Get  successively trees of genes list for multiple cases}
\usage{
getGenesTree_MultipleCases(entryWidth = 10)
}
\arguments{
\item{entryWidth}{10}
}
\value{
plot tree
}
\description{
Get  successively trees of genes list for multiple cases
}
\examples{
 q <- readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
\dontrun{
readRDS(paste(.libPaths(),"/canceR/data/brca_tcga73genes.rds", sep=""))
getGenesTree_MultipleCases(entryWidth = 10)
}
}

\name{GSEA.Res2Frame}
\alias{GSEA.Res2Frame}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.Res2Frame
}
\description{
GSEA.Res2Frame
}
\usage{
GSEA.Res2Frame(filename = "NULL")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{filename}{
%%     ~~Describe \code{filename} here~~
}
}
\value{
GSEA.NormalizeCols
}

\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
##Run.GSEA()
}
}

\name{GSEA.NormalizeRows}
\alias{GSEA.NormalizeRows}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.NormalizeRows
}
\description{
GSEA.NormalizeRows
}
\usage{
GSEA.NormalizeRows(V)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{V}{
%%     ~~Describe \code{V} here~~
}
}

\value{
GSEA.NormalizeRows
}

\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

\name{myGlobalEnv}
\alias{myGlobalEnv}
\docType{data}
\title{
myGlobalEnv
}
\description{
Global environment to store canceR variables.
}

\format{
  The format is:
<environment: 0xb3eb240> 
}

\keyword{datasets}


\examples{
myGlobalEnv <- new.env(parent = emptyenv())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getInTable.R
\name{getInTable}
\alias{getInTable}
\title{get dataframe in TK/TCL table}
\usage{
getInTable(table,title)
}
\arguments{
\item{table}{Dataframe}

\item{title}{string a title of the table}
}
\value{
display a Table
}
\description{
get dataframe in TK/TCL table
}
\examples{
#data(ClinicalData)
\dontrun{
getInTable(Table= ClinicalData, title= "Clinical Data")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSurvival.R
\name{getSurvival}
\alias{getSurvival}
\title{Survival plot}
\usage{
getSurvival(Coxph)
}
\arguments{
\item{Coxph}{if Coxph = 0 : plot Kaplan-Meier curves else Coxph= 1 : plot Cox Proportional Hazard Model}
}
\value{
Survival plot
}
\description{
Survival plot
}
\examples{
surv <- 11
\dontrun{
readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
getSurvival(Coxph = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMSigDB.R
\name{getMSigDB}
\alias{getMSigDB}
\title{Reduce MSigDB size for only gene list}
\usage{
getMSigDB(eSet, k)
}
\arguments{
\item{eSet}{Expression Set}

\item{k}{integer Number of studies}
}
\value{
MSigDB for user gene List
}
\description{
Reduce MSigDB size for only gene list
}
\examples{
 d <- 7
\dontrun{
setWorkspace()
getMSigDB(eSet = ENV$eSetClassifier,k = 1)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getClinicalDataMatrix.R
\name{getClinicalDataMatrix}
\alias{getClinicalDataMatrix}
\title{get matrix with clinical from file}
\usage{
getClinicalDataMatrix()
}
\value{
dataframe of clinicaldata
}
\description{
get matrix with clinical from file
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
\dontrun{
getClinicalDataMatrix()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canceR_vignette.R
\name{canceR_Issue}
\alias{canceR_Issue}
\title{canceR Report Issue}
\usage{
canceR_Issue()
}
\value{
link to github issues
}
\description{
canceR Report Issue
}
\examples{
\dontrun{
canceR_Issue()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGCTCLSExample.R
\name{getGCTCLSExample}
\alias{getGCTCLSExample}
\title{get GCT and CLS example files.}
\usage{
getGCTCLSExample()
}
\value{
GCT and CLS files
}
\description{
get GCT and CLS example files.
}
\examples{
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getGCTCLSExample()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCasesGenProfs.R
\name{getCasesGenProfs}
\alias{getCasesGenProfs}
\title{get Cases and Genetic Profiles of selected Studies.}
\usage{
getCasesGenProfs()
}
\value{
This function is run by the "Get Cases and Genetic Profiles for selected Studies in starting window. This function needs to select at least one study and display Cases and genetic profiles in the main window.
}
\description{
get Cases and Genetic Profiles of selected Studies.
}
\examples{
cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setWorkspace.R
\name{setWorkspace}
\alias{setWorkspace}
\title{Setting work Directory and output folders.At starting window, user needs to set work directory for output data. The function is foud in File menu.}
\usage{
setWorkspace()
}
\value{
paths of output files
}
\description{
Setting work Directory and output folders.At starting window, user needs to set work directory for output data. The function is foud in File menu.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
\dontrun{
setWorkspace()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSummaryGSEA.R
\name{getSummaryGSEA}
\alias{getSummaryGSEA}
\title{get Summary  results from GSEA-R (Broad Institute)}
\usage{
getSummaryGSEA()
}
\value{
Dataframe with summary results
}
\description{
get Summary  results from GSEA-R (Broad Institute)
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
Run.GSEA()
getSummaryGSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCor_ExpCNAMet.R
\name{getCor_ExpCNAMet}
\alias{getCor_ExpCNAMet}
\title{Get gene correlation for multiple dimensions.}
\usage{
getCor_ExpCNAMet(ListMatrix, dimension)
}
\arguments{
\item{ListMatrix}{is a List of numeric matrices}

\item{dimension}{Exp,CNA, Met , miRNA , RPPA}
}
\value{
correlation matrix
}
\description{
Get gene correlation for multiple dimensions.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/Circos.rds", sep=""))
\dontrun{
getListProfData()
getCor_ExpCNAMet(ENV$ListProfData$Expression, dimension="mRNA")
head(ENV$Cor_Exp)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogSamplingGSEA.R
\name{dialogSamplingGSEA}
\alias{dialogSamplingGSEA}
\title{Dialog  Box for Sampling patients from expression profile data used for GSEA-R (Broad Institute)}
\usage{
dialogSamplingGSEA(n_checked_GenProf, entryWidth = 10,returnValOnCancel = "ID_CANCEL")
}
\arguments{
\item{n_checked_GenProf}{Number of checked genetic profiles}

\item{entryWidth}{10}

\item{returnValOnCancel}{"ID_CANCEL"}
}
\value{
A vector with sampling size
}
\description{
Dialog  Box for Sampling patients from expression profile data used for GSEA-R (Broad Institute)
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
Run.GSEA()
#dialogSamplingGSEA(1,entryWidth=10,returnValOnCancel = "ID_CANCEL")
}
 
}

\name{GSEA.Gct2Frame2}
\alias{GSEA.Gct2Frame2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.Gct2Frame2
}
\description{
GSEA.Gct2Frame2
}
\usage{
GSEA.Gct2Frame2(filename = "NULL")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{filename}{
%%     ~~Describe \code{filename} here~~
}
}
\value{
GSEA.GCT2Frame2
}

\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTextWin.R
\name{getTextWin}
\alias{getTextWin}
\title{get text in tcltk windows}
\usage{
getTextWin(text)
}
\arguments{
\item{text}{string}
}
\value{
tcltk windows with text
}
\description{
get text in tcltk windows
}
\examples{
text <- "mytext"
\dontrun{
getTextWin(text) 
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testCheckedCaseGenProf.R
\name{testCheckedCaseGenProf}
\alias{testCheckedCaseGenProf}
\title{Testing checked appropriate Cases for appropriate Genetic profiles.}
\usage{
testCheckedCaseGenProf(singleGene=0)
}
\arguments{
\item{singleGene}{specify if the check for querying genetic profile for a specific gene or not (0,1).}
}
\value{
dialog box with warning message
}
\description{
Testing checked appropriate Cases for appropriate Genetic profiles.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
\dontrun{
testCheckedCaseGenProf(singleGene=0)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGSEAlm_Variables.R
\name{getGSEAlm_Variables}
\alias{getGSEAlm_Variables}
\title{get GSEA linear modeling by variables (phenotype)}
\usage{
getGSEAlm_Variables()
}
\value{
a dataframe with annotation (GO, BP)
}
\description{
get GSEA linear modeling by variables (phenotype)
}
\examples{
 x <- 3
\dontrun{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
getGSEAlm_Variables()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getClinicData_MultipleCases.R
\name{getClinicData_MultipleCases}
\alias{getClinicData_MultipleCases}
\title{get Clinical Data for Multiple Cases. User needs to select at least one case to run this function. Get clinical data for more one or multiple cases.}
\usage{
getClinicData_MultipleCases(getSummaryGSEAExists)
}
\arguments{
\item{getSummaryGSEAExists}{if equal to 0, the clinical data is displayed in table. if the argument is equal to 1, the clinical data is used to summarise GSEA analysis results.}
}
\value{
dataframe with clinical data
}
\description{
get Clinical Data for Multiple Cases. User needs to select at least one case to run this function. Get clinical data for more one or multiple cases.
}
\examples{
##Load Session
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
## Select Case
ENV <- new.env(parent = emptyenv())
ENV$curselectCases <- 2
## get Clinical data
\dontrun{
getClinicData_MultipleCases(getSummaryGSEAExists = 0)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/about.R
\name{about}
\alias{about}
\title{about canceR}
\usage{
about()
}
\value{
dialig box with text
}
\description{
about canceR
}
\examples{
\dontrun{
about()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMSigDBfile.R
\name{getMSigDBfile}
\alias{getMSigDBfile}
\title{Dialog Box to  Select MSigDB Files from drive}
\usage{
getMSigDBfile()
}
\value{
A path of MSigDB file
}
\description{
Dialog Box to  Select MSigDB Files from drive
}
\examples{
f <- 5+2
\dontrun{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
geteSet()
getMSigDBfile()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getListProfData.R
\name{UnifyRowNames}
\alias{UnifyRowNames}
\title{Unify row names in data frame with the same order of gene list.}
\usage{
UnifyRowNames(x,geneList)
}
\arguments{
\item{x}{data frame with gene symbol in the row name}

\item{geneList}{a gene list}
}
\value{
a data frame having the gene in row name ordered as in gene list.
}
\description{
Unify row names in data frame with the same order of gene list.
}
\examples{
cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGeneListExample.R
\name{getGeneListExample}
\alias{getGeneListExample}
\title{get Gene List from examples. User can select one from available gene list}
\usage{
getGeneListExample()
}
\value{
Gene list path of file
}
\description{
get Gene List from examples. User can select one from available gene list
}
\examples{
ENV <- new.env(parent = emptyenv())
\dontrun{
getGeneListExample() 
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSpecificMut.R
\name{getSpecificMut}
\alias{getSpecificMut}
\title{get specific Mutation data for multiple genes}
\usage{
getSpecificMut()
}
\value{
a a dataframe with specific mutation informations
}
\description{
get specific Mutation data for multiple genes
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata//ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getSpecificMut()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Run.GSEA.1.0.R
\name{Run.GSEA}
\alias{Run.GSEA}
\title{The  main function to run GSEA-R from Broad Institute}
\usage{
Run.GSEA()
}
\value{
A vector with sampling size
}
\description{
The  main function to run GSEA-R from Broad Institute
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMegaProfData.R
\name{getMegaProfData}
\alias{getMegaProfData}
\title{Get profile data for more than 500 genes list.}
\usage{
getMegaProfData(MegaGeneList,k)
}
\arguments{
\item{MegaGeneList}{Genelist >500}

\item{k}{integer number of studies}
}
\value{
dataframewith profile data
}
\description{
Get profile data for more than 500 genes list.
}
\examples{
myGlobalEnv <- new.env(parent = emptyenv())
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcgaGSEAlm1021.rds", sep=""))
\dontrun{
getMegaProfData(myGlobaEnv$MegaGeneList,1) 
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbind.na.R
\name{cbind.na}
\alias{cbind.na}
\title{bind non equal colunm}
\usage{
cbind.na(..., deparse.level = 1)
}
\arguments{
\item{...}{(generalized) vectors or matrices.}

\item{deparse.level}{integer controlling the construction of labels in 
the case of non-matrix-like arguments (for the default method):
deparse.level = 0 constructs no labels; the default,
deparse.level = 1 or 2 constructs labels from the argument names.}
}
\value{
a data frame with merged columns
}
\description{
bind non equal colunm
}
\examples{
\dontrun{
col1 <- c("a","b","c","d")
col2 <- c("A", "B", "C")
col3 <- cbind.na(col1, col2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMutData.R
\name{getMutData}
\alias{getMutData}
\title{get Mutation data for multiple genes}
\usage{
getMutData()
}
\value{
a a dataframe with mutation informations
}
\description{
get Mutation data for multiple genes
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getMutData()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGeneListFromMSigDB.R
\name{getGeneListFromMSigDB}
\alias{getGeneListFromMSigDB}
\title{get gene list from MSigDB}
\usage{
getGeneListFromMSigDB()
}
\value{
a vector with gene list
}
\description{
get gene list from MSigDB
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcgaGSEAlm1021.rds", sep=""))
\dontrun{
getGeneListFromMSigDB()
}

}

\name{GSEA}
\alias{GSEA}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA-R (Broad Institute)
}
\description{
 See \url{http://www.broadinstitute.org/cancer/software/gsea/wiki/index.php/R-GSEA_Readme}
}

\author{
Subramanian, Tamayo, et al. (2005, PNAS 102, 15545-15550) and Mootha,
Lindgren, et al. (2003, Nat Genet 34, 267-273)
}
\value{
GSEA
}


\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getProfilesDataMultipleGenes.R
\name{getProfilesDataMultipleGenes}
\alias{getProfilesDataMultipleGenes}
\title{get Profles Data of multiple genes}
\usage{
getProfilesDataMultipleGenes(getSummaryGSEAExists)
}
\arguments{
\item{getSummaryGSEAExists}{if equal to 0, the clinical data is displayed in table. if the argument is equal to 1, the clinical data is used to summarise GSEA analysis results.}
}
\value{
a file with a dataframe of profle data
}
\description{
get Profles Data of multiple genes
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
\dontrun{
getProfilesDataMultipleGenes(getSummaryGSEAExists = 0)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canceR.R
\name{canceR}
\alias{canceR}
\title{main function}
\usage{
canceR()
}
\value{
open the starting windows with cancer studies
}
\description{
main function
}
\examples{
ENV <- new.env(parent = emptyenv())
\dontrun{
canceR()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMSigDBExample.R
\name{getMSigDBExample}
\alias{getMSigDBExample}
\title{get example of .gmt file from MSigDB (Broad Institute)}
\usage{
getMSigDBExample()
}
\value{
path of GMT file
}
\description{
get example of .gmt file from MSigDB (Broad Institute)
}
\examples{
 readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
 \dontrun{
 getMSigDBExample()
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_1Gene_2GenProfs.R
\name{plot_1Gene_2GenProfs}
\alias{plot_1Gene_2GenProfs}
\title{Plotting two genetic profiles for  one Gene}
\usage{
plot_1Gene_2GenProfs()
}
\value{
plot
}
\description{
Plotting two genetic profiles for  one Gene
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
\dontrun{
plot_1Gene_2GenProfs()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getListProfData.R
\name{getListProfData}
\alias{getListProfData}
\title{Get list of data frame with profiles data (CNA,mRNA, Methylation, Mutation...)}
\usage{
getListProfData(checked_Studies, geneList)
}
\arguments{
\item{checked_Studies}{checked studies in corresponding panel (input$StudiesIDCircos, input$StudiesIDReactome).}

\item{geneList}{GeneList with Hugo Symbol}
}
\value{
A LIST of profiles data (CNA, mRNA, Methylation, Mutation, miRNA, RPPA).
        Each dimension content a list of studies.
}
\description{
Get list of data frame with profiles data (CNA,mRNA, Methylation, Mutation...)
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/brca_tcga73genes.rds", sep=""))
\dontrun{
getListProfData()
head(ENV$ProfData$Expression)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogOptionGSEAlm.R
\name{dialogOptionGSEAlm}
\alias{dialogOptionGSEAlm}
\title{Dialogbox to select variables from Clinical data}
\usage{
dialogOptionGSEAlm(k,ClinicalData)
}
\arguments{
\item{k}{integer 1}

\item{ClinicalData}{dataframe with clinical variables}
}
\value{
permutaion value, p-value, coVariables
}
\description{
Dialogbox to select variables from Clinical data
}
\examples{
#data(ClinicalData)
\dontrun{
getOptionGSEAlm()
}
}

\name{GSEA.ConsPlot}
\alias{GSEA.ConsPlot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.ConsPlot
}
\description{
GSEA.ConsPlot
}
\usage{
GSEA.ConsPlot(V, col.names, main = " ", sub = " ", xlab = " ", ylab = " ")
}
%GSEA.ConsPlot(Itable, col.names = set.table.phen1[1:max.sets.phen1, 2], main = " ", sub=paste("Leading Subsets Overlap ", doc.string, " - ", phen1, sep=""), xlab=" ", ylab=" ")
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{V}{
   \code{V="Itable"} 
}
  \item{col.names}{
     \code{col.names = colnames} 
}
  \item{main}{
     \code{main= " "} 
}
  \item{sub}{
\code{sub = " "} 
}
  \item{xlab}{
    \code{xlab= " "} 
}
  \item{ylab}{
    \code{ylab = " "} 
}
}
\value{
GSEA.ConsPlot
}
\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogSummary_GSEA.R
\name{dialogSummary_GSEA}
\alias{dialogSummary_GSEA}
\title{Dialog Box to specify phenotype (variable) used in last GSEA-R to get Summary Results. This function ask the user to specify the phenotype (variable).}
\usage{
dialogSummary_GSEA(Variable,returnValOnCancel ="ID_CANCEL")
}
\arguments{
\item{Variable}{phenotype}

\item{returnValOnCancel}{"ID_CANCEL"}
}
\value{
variables
}
\description{
Dialog Box to specify phenotype (variable) used in last GSEA-R to get Summary Results. This function ask the user to specify the phenotype (variable).
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
#Run.GSEA()
#getSummaryGSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetDataMultipleGenes.R
\name{getMetDataMultipleGenes}
\alias{getMetDataMultipleGenes}
\title{get Methylation data for multiple genes}
\usage{
getMetDataMultipleGenes()
}
\value{
a a dataframe with mean and median of methylation rate (threshold of silencing gene)
}
\description{
get Methylation data for multiple genes
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getMetDataMultipleGenes()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGCT_CLSfiles.R
\name{getGCT_CLSfiles}
\alias{getGCT_CLSfiles}
\title{get Profile (GCT file) and Phenotype (CLS file) Data from Disease.}
\usage{
getGCT_CLSfiles()
}
\value{
GCT and CLS files paths
}
\description{
get Profile (GCT file) and Phenotype (CLS file) Data from Disease.
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
getGCT_CLSfiles()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogGeneClassifier.R
\name{dialogGeneClassifier}
\alias{dialogGeneClassifier}
\title{Dialogue Box for gene classifier setting: sample size and postprob threshold}
\usage{
dialogGeneClassifier(Lchecked_Cases,entryWidth = 10,returnValOnCancel = "ID_CANCEL")
}
\arguments{
\item{Lchecked_Cases}{integer with a number of checked cases}

\item{entryWidth}{integer default 10}

\item{returnValOnCancel}{"ID_CANCEL"}
}
\value{
a dataframe with genes classes
}
\description{
Dialogue Box for gene classifier setting: sample size and postprob threshold
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
\dontrun{
getGenesClassifier()
dialogGeneClassifier(1,10,returnValOnCancel = "ID_CANCEL")
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCases.R
\name{getCases}
\alias{getCases}
\title{Get cases for selected Studies. The Cases are the descrption of the samples from patients. The samples can be subdivided by the type of assays as, sequencing, CNA, Mutation, Methylation.}
\usage{
getCases()
}
\value{
a dataframe with cases
}
\description{
Get cases for selected Studies. The Cases are the descrption of the samples from patients. The samples can be subdivided by the type of assays as, sequencing, CNA, Mutation, Methylation.
}
\examples{

cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/canceR_vignette.R
\name{canceR_Vignette}
\alias{canceR_Vignette}
\title{open pdf vignette}
\usage{
canceR_Vignette()
}
\value{
open pdf vignette
}
\description{
open pdf vignette
}
\examples{
\dontrun{
canceR_Vignette()
}

}

\name{GSEA.HeatMapPlot2}
\alias{GSEA.HeatMapPlot2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
GSEA.HeatMapPlot2
}

\description{
GSEA.HeatMapPlot2
}
\value{
GSEA.HeatMapPlot2
}
\examples{
\dontrun{
library(canceR)
## Load workspace
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
##Run.GSEA()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogOptionPhenoTest.R
\name{dialogOptionPhenoTest}
\alias{dialogOptionPhenoTest}
\title{Checkbox to select variables from clinical data}
\usage{
dialogOptionPhenoTest(eSet)
}
\arguments{
\item{eSet}{Expression Set}
}
\value{
vectors: variables to test Survival status, AGE, p-value
}
\description{
Checkbox to select variables from clinical data
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/prad_michPhenoTest1021.rds", sep=""))
\dontrun{
dialogOptionPhenoTest(ENV$eSet)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgdsr.R
\name{CGDS}
\alias{CGDS}
\title{CGDS  connect object to cBioPortal}
\usage{
CGDS(url,verbose=FALSE,ploterrormsg='',token=NULL)
}
\arguments{
\item{url}{A CGDS URL (required).}

\item{verbose}{A boolean variable specifying verbose output (default FALSE)}

\item{ploterrormsg}{An optional message to display in plots if an error occurs (default ”)}

\item{token}{An optional 'Authorization: Bearer' token to connect to cBioPortal instances that require authentication (default NULL)}
}
\description{
Creates a CGDS connection object from a CGDS endpoint URL. This object must be passed on to the methods which query the server.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotModel.R
\name{plotModel}
\alias{plotModel}
\title{model plotting with tcltk}
\usage{
plotModel(plotCommand, title= "TITLE",hscale=1, vscale=1 )
}
\arguments{
\item{plotCommand}{plotcommand}

\item{title}{title of plot}

\item{hscale}{horizintal scale}

\item{vscale}{vertical scale}
}
\value{
plot
}
\description{
model plotting with tcltk
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/gbm_tcgaPlotTwoGenProf.rds", sep=""))
\dontrun{
plot_1Gene_2GenProfs()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getListProfData.R
\name{getFreqMutData}
\alias{getFreqMutData}
\title{get mutation frequency}
\usage{
getFreqMutData(list, GeneList)
}
\arguments{
\item{list}{a list of data frame with mutation data. Each data frame is for one study}

\item{GeneList}{file name of GeneList examples: "73"}
}
\value{
a data frame with mutation frequency. gene is in rows and study is in column
}
\description{
get mutation frequency
}
\examples{
cgds <- cBioPortal(
hostname = "www.cbioportal.org",
protocol = "https",
api = "/api/v2/api-docs"
)
\dontrun{
getDataByGenes( api =  cgds,
studyId = "gbm_tcga_pub",
genes = c("NF1", "TP53", "ABL1"),
by = "hugoGeneSymbol",
molecularProfileIds = "gbm_tcga_pub_mrna"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dialogMut.R
\name{dialogMut}
\alias{dialogMut}
\title{Dialog bos to set returned Mutation information}
\usage{
dialogMut(title, question, entryInit, entryWidth = 40,returnValOnCancel = "ID_CANCEL")
}
\arguments{
\item{title}{title of the table}

\item{question}{question}

\item{entryInit}{entryInit}

\item{entryWidth}{40}

\item{returnValOnCancel}{"ID_CANCEL"}
}
\value{
a check box with mutations variables
}
\description{
Dialog bos to set returned Mutation information
}
\examples{
readRDS(paste(path.package("canceR"),"/extdata/rdata/ucec_tcga_pubGSEA1021.rds", sep=""))
\dontrun{
dialogMut("title", "question", "entryInit", entryWidth = 40, returnValOnCancel = "ID_CANCEL")
}
}
